/*----------------------------------------------------------------------------*\
**                              akima_ex.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions:
**       akima_ex
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)akima_ex.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                akima_ex
** input parameters
**
**     x_arr : x values
**     y_arr : values of function
**     n_source : number of values used in  x_arr, y_arr
**
** output parameters
**
**     st : slopes of curve
**
**  Description:
**    extrapolates the function two points left and right
**  References:
**    This function is described in the Module ...
**  Libraries:
**    libm
**  Created:	14.7.94
**  Author:	Akima, Implemented in C by Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void akima_ex (short n_source, double x_arr[], double y_arr[], double st[])
{
	/* local variables */

	register int i;
	double tmp;

	/* first part is the interpolation */

	/* auxiliary points are moved 2 indices up */

	for ( i = 0; i < n_source; i++)
	{
		 x_arr[(n_source+1)-i] = x_arr[(n_source-1)-i];
		 y_arr[(n_source+1)-i] = y_arr[(n_source-1)-i];
	}

	/* calculation of extrapolated x-values left and right */

	x_arr[1] = x_arr[2] + x_arr[3] - x_arr[4];
	x_arr[0] = x_arr[1] + x_arr[2] - x_arr[3];

	x_arr[n_source+2] = x_arr[n_source+1] + x_arr[n_source] - x_arr[n_source-1];
	x_arr[n_source+3] = x_arr[n_source+2] + x_arr[n_source+1] - x_arr[n_source];

	/* calculation of slopes of curves */

	for ( i = 2; i < n_source+1; i++)
	{
		tmp = x_arr[i+1] - x_arr[i];

		if ( tmp < 1e-10)
			st[i] = 0.0;
		else
			st[i] = (y_arr[i+1] - y_arr[i]) / tmp;
	}

	/* calculation of extrapolated y-values left and right */

        tmp = x_arr[2] - x_arr[1];

	y_arr[1] = tmp * (st[3] - 2.0 * st[2]) + y_arr[2];

	if ( tmp < 1e-10)
		st[1] = 0.0;
	else
		st[1]    = (y_arr[2] - y_arr[1]) / tmp;

        tmp = x_arr[1] - x_arr[0];

	y_arr[0] = tmp * (st[2] - 2.0 * st[1]) + y_arr[1];

	if ( tmp < 1e-10)
		st[0] = 0.0;
	else
		st[0]    = (y_arr[1] - y_arr[0]) / tmp;

        tmp = x_arr[n_source+2] - x_arr[n_source+1];

	y_arr[n_source+2] = tmp * (2.0 * st[n_source] - st[n_source-1]) + y_arr[n_source+1];

	if ( tmp < 1e-10)
		st[n_source+1] = 0.0;
	else
		st[n_source+1]    = (y_arr[n_source+2] - y_arr[n_source+1]) / tmp;

	tmp = x_arr[n_source+3] - x_arr[n_source+2];

	y_arr[n_source+3] = tmp * (2.0 * st[n_source+1] - st[n_source]) + y_arr[n_source+2];

	if ( tmp < 1e-10)
		st[n_source+2] = 0.0;
	else
		st[n_source+2]    = (y_arr[n_source+3] - y_arr[n_source+2]) / tmp;

	/* end of extrapolation */

	return;
}
