/*----------------------------------------------------------------------------*\
**                              akima_po.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions:
**    akima_po
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)akima_po.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"

/*----------------------------------------------------------------------------*\
**                                akima_po
** input parameters
**
**     x_arr : x values
**     x : the abscissa at which you want y
**     n_source : dimension of x_arr, y_arr
**     a, b, c, d : arrays of polynomial coefficients
**
** output parameters
**
**     y : the function value at x
**
**  Description:
**	  Given an n-dim. array of abscissa x_arr and arrays a, b, c, d of
**	  polynomial coefficients as returned by the akima_su, this
**	  subroutine returns the interpolated value y at abscissa x
**  References:
**    This function is described in the Module ...
**  Libraries:
**    libm
**  Created:	14.7.94
**  Author:	Akima, Implemented in C by Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void akima_po (short n_source, double x_arr[], double a[], double b[], double c[], double d[], double x, double *y)
{
	register short klo, khi, k;

      	double delta;

	klo = 0;
	khi = n_source - 1;
	while (khi-klo > 1) 
	{
		k=(khi+klo) >> 1;
		if (x_arr[k] > x) khi=k;
		else klo=k;
	}

	k = klo;

	delta = x - x_arr[k];

	*y = 	a[k] + 
		(b[k] * delta) + 
		(c[k] * delta * delta) + 
		(d[k] * delta * delta * delta);

	return;
}
