/*----------------------------------------------------------------------------*\
**                              akima_su.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions:
**       akima_ex
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)akima_su.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                akima_su
** input parameters
**
**     x_arr : x values
**     y_arr : values of function
**     n_source : dimension of x_arr, y_arr
**
** output parameters
**
**    a_arr : polynomial constants
**    b_arr : polynomial constants
**    c_arr : polynomial constants	
**    d_arr : polynomial constants
**
**  Description:
**    fits a polynome through a given input function for inter- and extrapolation 
**    by akima_po function later
**  References:
**    This function is described in the Module ...
**  Libraries:
**    libm
**  Created:	14.7.94
**  Author:	H. Akima, Implemented in C by Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void akima_su (double x_arr[], double y_arr[], short n_source, double a_arr[], double b_arr[], double c_arr[], double d_arr[])
{
	static double		t[MAX_AKIMA_POINTS + AKIMA_EXTRA_POINTS], 
				st[MAX_AKIMA_POINTS + AKIMA_EXTRA_POINTS];
	register int	i;
        double          tmp1,tmp2;


	/* calculate auxiliary points left and right */
	
	akima_ex (n_source, x_arr, y_arr, st);

	/* calculation of all polynomial slopes */

	for (i = 2; i < n_source+2; i++)
	{
		if ( st[i-2] == st[i-1] && st[i] == st[i+1] )
	    		t[i] = 0.5 * (st[i+1] + st[i]);
	 	else
		{
                        tmp1 = fabs( st[i+1] - st[i]   );
                        tmp2 = fabs( st[i-1] - st[i-2] );

			if ( (tmp1 + tmp2 ) != 0.0 )
			{
	  	  		t[i] = ( tmp1 * st[i-1] + tmp2 * st[i] ) / ( tmp1 + tmp2 );
			}
			else t[i] = 0.0;
		}
	}

	/* back-indexing auxiliary points by 2 entries */

	for (i = 0; i < n_source; i++)
	{
	 	x_arr[i] = x_arr[i+2];
		y_arr[i] = y_arr[i+2];
	 	t[i]     = t[i+2];
	}

	/* calculation of polynomial coefficients */

	for (i = 0; i < n_source-1; i++)
	{
	 	a_arr[i] = y_arr[i];
	 	b_arr[i] = t[i];

                tmp1 = x_arr[i+1] - x_arr[i];

		if ( tmp1 > 1e-10 )
		 	c_arr[i] = ( ((3.0 * st[i+2]) - (2.0 * t[i])) - t[i+1] ) / tmp1;
	 	else 
			c_arr[i] = 0.0;

		tmp1 *= (x_arr[i+1] - x_arr[i]);

		if ( tmp1 > 1e-10 )
			d_arr[i] = ( (t[i] + t[i+1]) - (2.0 * st[i+2]) ) / tmp1;
		else
			d_arr[i] = 0.0;
	}

	/* end of routine */

        return;
}
