/*----------------------------------------------------------------------------*\
**                                BandRecord.h
**
**  Class BandRecord declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _BandRecord_h
#define _BandRecord_h

#pragma ident "@(#)BandRecord.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"
#include "sciedata.h"

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
** String definitions of input format specifications
\*----------------------------------------------------------------------------*/
static  const char	*BandStrTab[SCIENCE_DATA_DEFINED] = {
			"Band 1a", "Band 1b", "Band 2a", "Band 2b", "Band 3",
			"Band 4", "Blind 1a", "Straylight 1a",
			"Straylight 1b", "Straylight 2a"};

/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class BandRecord
\*============================================================================*/
typedef struct BandRecord {
    short Flag;			/* Quality Flags */
    short CalibrationIndx;	/* Index to Ground Pixel Specific Calibration
				   Parameters Record */
    short KeyDataCalibrationIndx; /* Index to KeyData Calibration Parameters */
    float IntgTime;		/* Integration Time */
    float *Data;		/* Array Data Record */
} BandRecord_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
BandRecord_s*
BandRecord__New(FILE *Stream, short Begin, short End,
		    FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
BandRecord__Delete(BandRecord_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/

FunctionStatus_e
Band__read(BandRecord_s *self,  FILE* Stream, long DataSize);

void
Band__show(const BandRecord_s* self,  FILE* Stream, long DataSize);

FunctionStatus_e
WriteBandParameter(FILE *Stream, int Band, const BandRecord_s *BandRecord,
		   short Nsamp, float FirstWaveLength, float LastWaveLength,
		   int ReflectivityJump, double Error);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
