/*----------------------------------------------------------------------------*\
**                                BDR.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)BDR.c	5.1 02/26/99, Copyright DLR 1998"

#include "bdr.h"
#include "sciedata.h"
#include "log.h"


#define	SccsId	("@(#)BDR.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("BDR")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_bdr_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/
/*----------------------------------------------------------------------------*\
**                                BDR__Read
**  Input parameters:
**    Stream		input stream for Band data to read
**    BDR_Rec_Size	length of the BDR record in Level 1 Product
**    BandSize		Size of the Band data to be read from the BDR record
**				of the Level 1 Product
**
**  Output parameters:
**    BandData		Band data read from Level 1 Product
**    OrigBandRecord	Parameter of the BDR record of the Level 1 Product
**    BDR__Read		SUCCESS	= Reading ok
**			other	= Reading failed
**
**  Other interfaces:
**    Input file of Level 1 Product specified by the given input Stream
**
**  Description:
**    Function the read one complete band from the specified Level 1 Product.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
BDR__read(OrigBandRecord_s* self, FILE* Stream, long BDR_Rec_Size,
	  GOME_Word_t *BandData, short BandSize, Bool convert_sun_2_host)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    Science_s		ScienceRecord;
#ifdef DEBUG
    FILE* debug_file_ptr;
#endif

    /*------------------------------------------------------------------------*\
    ** read Science record
    \*------------------------------------------------------------------------*/
    ScienceRecord.Data = BandData;
    if (ScienceRecord.Data == 0) {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__77", Status, 
		 LOG_CANNOT_ALLOCATE, "Band Data", "");
	return Status;
    }

    if (Science__read(&ScienceRecord, Stream, BandSize, convert_sun_2_host)
	!= SUCCESS)
    {
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__85", Status,
		 LOG_CANNOT_READ, "Science record from Level 1 Product",
		 "");
	return Status;
    }

    /*----------------------------------------------------------------*\
    ** set the BDR record Parameter of self
    \*----------------------------------------------------------------*/
    self->Flag = ScienceRecord.Flag;
    self->CalibrationIndx = ScienceRecord.CalibrationIndx;
    self->KeyDataCalibrationIndx =
	ScienceRecord.KeyDataCalibrationIndx;
    self->IntgTime = ScienceRecord.IntgTime;
#ifdef DEBUG
    if ( (debug_file_ptr = Debug__getStream(debug_bdr_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL_BDR)) != 0)
    {
	Science__show(&ScienceRecord, debug_file_ptr, BandSize);
    }
#endif

    return SUCCESS;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
