/*----------------------------------------------------------------------------*\
**                                BSDF.h
**
**  Class BSDF declaration
** 
**  Version:    5.2
**  Date:       08/02/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _BSDF_h
#define _BSDF_h 

#pragma ident "@(#)BSDF.h	5.2 08/02/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"
#include "sciedata.h"
#include "utc.h"

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	BSDF_FITT_ORDER	(8)

#define	BSDF_SIZE	((3 + BSDF_FITT_ORDER) * FLOAT_SIZE)


#define	MAX_N_BSDF_A1	(100)

/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
**  structures
\*----------------------------------------------------------------------------*/
typedef struct BSDF_azimuth_corr_s
{
  float BSDF_a;
  float BSDF_a0;
  int   n_BSDF_a1;
  float BSDF_a1_grid[MAX_N_BSDF_A1];
  float BSDF_a1[MAX_N_BSDF_A1];
}
BSDF_azimuth_corr_s;

/*============================================================================*\
**  class BSDF
\*============================================================================*/
typedef struct BSDF {
	float BSDF_0;
	float Elevation;
	float Azimuth;
	float Coefficients[BSDF_FITT_ORDER];
} BSDF_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
BSDF__LoadAzimuthCorrectionParameters(const char filename[], BSDF_s BSDF, 
  char SoftwareVersion[LEVEL_1_PRODUCT_VERSION_SIZE]);

FunctionStatus_e
BSDF__CalculateMeanSunAzimuth(float* Azimuth, const UTC_s Date);

FunctionStatus_e 
BSDF__CorrectAzimuth (float Azimuth,
		    ScienceData_e Band, float Reticon[], const float LambdaGrid[], 
		    short Begin, short End, const UTC_s *Date);

void 
BSDF__ComputeAngles(float PropagatorZenith, float PropagatorAzimuth,
		    float *BSDF_Zenith, float *BSDF_Azimuth);
    
float 
BSDF__Calculate(const BSDF_s *Self, float Lambda, float Elevation,
		float Azimuth);
void 
BSDF__Apply(const BSDF_s* Self, float PropagatorZenith,
	    float PropagatorAzimuth, ScienceData_e Band, float Reticon[],
	    const float LambdaGrid[], float BSDF[ DETECTOR_PIXELS],
	    short Begin, short End, const char *PixelDate);

FunctionStatus_e
BSDF__read(BSDF_s* self, FILE* file_ptr, Bool convert_sun_2_host);

FunctionStatus_e
BSDF__write(const BSDF_s* self, FILE* Stream);

void
BSDF__show(const BSDF_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/
void
BSDF__Sun2Host(BSDF_s *Param);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
