/*----------------------------------------------------------------------------*\
**                                CalibrationFilter.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)CalibrationFilter.c	5.1 02/26/99, Copyright DLR 1998"

#include "calilter.h"


#define	SccsId	("@(#)CalibrationFilter.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("CalibrationFilter")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                CalibrationTypeGet
**  Input parameters:
**    ProductType		product type
**    calibrationFilter		calibration filter
**
**  Output parameters:
**    CalibrationTypeGet	CAL_UNDEF = illegal calibration mode
**				else calibration mode
**
**  Other interfaces:
**
**  Description:
**    Function to return the calibration mode as defined in enumeration
**    CalibrationType_e
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
CalibrationType_e
CalibrationTypeGet(const ProductType_e ProductType,
		   const Bool calibrationFilter[CALIBRATION_DEFINED])
{
    CalibrationType_e	CalType;

    switch (ProductType) {
    case PR_EARTHSHINE: {
	if (calibrationFilter[CAL_POLARIZATION] &&
	    calibrationFilter[CAL_INTENSITY]) {
	    CalType = CAL_RAD_POL_INTENS;
	}
	else {
	    if (calibrationFilter[ CAL_POLARIZATION]) {
		CalType = CAL_RAD_POL;
	    }
	    else {
		if (calibrationFilter[ CAL_INTENSITY]) {
		    CalType = CAL_RAD_INTENS;
		}
		else {
		    CalType = CAL_RAD;
		}
	    }
	}
	break;
    }
    case PR_MOONSHINE: {
	if (calibrationFilter[ CAL_INTENSITY]) {
	    CalType = CAL_RAD_INTENS;
	}
	else {
	    CalType = CAL_RAD;
	}
	break;
    }
    case PR_SUNSHINE: {
	if (calibrationFilter[ CAL_BSDF] && calibrationFilter[ CAL_INTENSITY]) {
	    CalType = CAL_IRR_INTENS_BSDF;
	}
	else {
	    if (calibrationFilter[ CAL_BSDF]) {
		CalType = CAL_IRR_BSDF;
	    }
	    else {
		if (calibrationFilter[ CAL_INTENSITY]) {
		    CalType = CAL_IRR_INTENS;
		}
		else {
		    CalType = CAL_IRR;
		}
	    }
	}
	break;
    }
    default: {
	CalType = CAL_UNDEF;
	break;
    }
    }
    return CalType;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
