/*----------------------------------------------------------------------------*\
**                                Config.c
**
**  Description:
** 
**  Version:    5.7
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Config.c	5.7 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "file.h"
#include "leveduct.h"
#include "qa.h"
#include "smcd.h"
#include "sfs.h"
#include "eps.h"
#include "log.h"
#include "wat2phot.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)Config.c 5.7 08/19/02, Copyright DLR 1998")


#define	SS_ID	("Config")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*----------------------------------------------------------------------------*\
**  extern global variables
\*----------------------------------------------------------------------------*/
extern int EDR_WrittenCounter;


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                Config__new
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	17.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Config_s*
Config__new(void)
{
#ifdef __cplusplus
    return new Config_s;
#else
      return (Config_s *) malloc( sizeof(Config_s) );
#endif
}

/*----------------------------------------------------------------------------*\
**                                Config__delete
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Config__delete(Config_s* self)
{
#ifdef __cplusplus
   delete self;
#else
   free(self);
#endif
}


/*----------------------------------------------------------------------------*\
**                                Config__Show
**  Input parameters:
**    Stream		output stream
**    Config		Control structure of extracting program
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**
**  Description:
**    Function to write the data structure Config_s to the output stream
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Config__Show(FILE *Stream, const Config_s *Config)
{
    int	i;

    /*------------------------------------------------------------------------*\
    ** Show Config
    \*------------------------------------------------------------------------*/
    fprintf(Stream, "Filter:\n");
    Filter__show(&Config->Filter, Stream);
    if (Config->Level_1_ProductStream == NULL) {
	fprintf( Stream, "Level_1_ProductStream = NULL\n");
    }
    else {
	fprintf( Stream, "Level_1_ProductStream open\n");
    }
    if (Config->Extract_1_ProductStream == NULL) {
	fprintf( Stream, "Extract_1_ProductStream = NULL\n");
    }
    else {
	fprintf( Stream, "Level_1_ProductStream open\n");
    }
    if (Config->Extract_1_MeanSun_ProductStream == NULL) {
	fprintf( Stream, "Extract_1_MeanSun_ProductStream = NULL\n");
    }
    else {
	fprintf( Stream, "Extract_1_MeanSun_ProductStream open\n");
    }

    fprintf( Stream, "SpectrumString      : %s\n", Config->SpectrumString);
    fprintf( Stream, "ExtractFileExt      : %s\n", Config->ExtractFileExt);
    fprintf( Stream, "PixelParString      : %s\n", Config->PixelParString);
    fprintf( Stream, "ProductSize         : %d\n", Config->ProductSize);
    fprintf( Stream, "ProductType         : %d\n", Config->ProductType);
    fprintf( Stream, "SDR_StartBand       : %d\n", Config->SDR_StartBand);
    fprintf( Stream, "SDR_EndBand         : %d\n", Config->SDR_EndBand);
    fprintf( Stream, "StartBand           : %d\n", Config->StartBand);
    fprintf( Stream, "EndBand             : %d\n", Config->EndBand);
    fprintf( Stream, "FirstMeasPixel      : %d\n", Config->FirstMeasurementPixel);
    fprintf( Stream, "LastMeasPixel       : %d\n", Config->LastMeasurementPixel);
    fprintf( Stream, "FirstMeasPixelFile  : %d\n",
	     Config->FirstMeasurementPixelFile);
    fprintf( Stream, "LastMeasPixelFile   : %d\n",
	     Config->LastMeasurementPixelFile);
    fprintf( Stream, "BeginTime           : %d, %d\n",
	     Config->BeginTime.Days, Config->BeginTime.Msecs);
    fprintf( Stream, "EndTime             : %d, %d\n",
	     Config->EndTime.Days, Config->EndTime.Msecs);
    fprintf( Stream, "Time first PCD      : %d, %d\n",
	     Config->TimeFirst_PCD.Days, Config->TimeFirst_PCD.Msecs);
    fprintf( Stream, "Time last PCD       : %d, %d\n",
	     Config->TimeLast_PCD.Days, Config->TimeLast_PCD.Msecs);
    fprintf( Stream, "FilePosPCD          : %d\n", Config->FilePosPCD);
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
	fprintf( Stream, "FilePos_SD[ %2d]    : %d\n", i, Config->FilePos_SD[ i]);
}


/*----------------------------------------------------------------------------*\
**                                Config__Init
**  Input parameters:
**    ProductType		product type (PR_EARTHSHINE, PR_SUNSHINE,
**						PR_MOONNSHINE)
**    Config			Extract Configuration structure
**    xCD__ReadTime		function to read the PCD or SCD or MCD
**    FilePosPCD		file position of the PCD or SCD or MCD
**    NoOf_PCD_Rec		number of PCD or SCD or MCD records
**    PCD_Size			size of PCD or SCD or MCD
**    FilePos_SD		file position of the BDR
**    NoOfBDR_Rec		numbers of band records
**    BDR_Rec_Size		Sizes of band records
**
**  Output parameters:
**    Config			Extract Configuration structure, initialized
**				with the Level 1 Product Data
**    Config__Init		SUCCESS = Initialization is ok, the extracted
**						pixel range is valid
**				INFO	= Initialization ok, but no valid
**						extracted pixel range possible,
**						because of number of PCD or
**						SMCDs zero
**				FATAL	= initialization failed, pixel
**						extraction not possible
**
**  Other interfaces:
**    Input file Level 1 Product specified in Config
**
**  Description:
**    Function to initialize the parameter of the Config_s struct which depends
**    on the Level 1 Product itself or on a combination of the Level 1 product
**    and the given extract switches (Filter_s). These parameter are:
**	- specific extract product type parameter (Earth, Sun or Moon product)
**	- time of the first and the last PCD, SCD or MCD (TimeFirst_PCD,
**							 TimeLast_PCD)
**	- file position of the PCD, SCD or MCD (FilePosPCD)
**	- file positions of the different Science data( FilePos_SD)
**	- the first and the last possible measurement pixel which has to be
**	  extracted (FirstMeasurementPixel, LastMeasurementPixel)
**	  and the according PCD, SCD or MCD times (BeginTime, EndTime).
**     The first and the last possible measurement pixel is derived from the
**     following parameter:
**	- the given time range (switch -t)
**	- the given pixel range (switch -p)
**	- the given data range within the level 1 product
**
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__init(Config_s* self, ProductType_e ProductType,
	     FunctionStatus_e (*xCD__ReadTime)
	     (FILE* Stream, long FilePos, long PCD_Size, UTC_s* PCD_Time,
	      PCD_s* PCD, Bool convert_sun_2_host),
	     long FilePosPCD,
	     short NoOf_PCD_Rec,
	     long PCD_Size,
	     long FilePos_SD,
	     short NoOfBDR_Rec[SCIENCE_DATA_DEFINED],
	     long BDR_Rec_Size[SCIENCE_DATA_DEFINED],
	     Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    int	i;
    double First_PCD_Time;
    double BeginTime;
    double EndTime;
    char ASCII_Time[ 30];
    char Begin_ASCII_Time[ 30];
    char End_ASCII_Time[ 30];
    UTC_s PixelTime;
    char LogBuffer[LOG_FIELD_SIZE];
    char LogAction[LOG_FIELD_SIZE];
#ifdef DEBUG
    char debug_message[255];
#endif

    /*--------------------------------------------------------------------*\
    ** initialize product specific parameter
    \*--------------------------------------------------------------------*/
    switch (ProductType) {
    case PR_EARTHSHINE:
    case PR_SUNSHINE:
    case PR_MOONSHINE: {
	self->ProductType	= ProductType;
	if (ProductType == PR_EARTHSHINE) {
	    self->SpectrumString = EARTHSHINE_SPEC;
	    self->ExtractFileExt = EXTRACT_FILE_EXTENSION;
	    self->PixelParString = PCD_EXTRACT_FILE_EXTENSION;
	}
	if (ProductType == PR_SUNSHINE) {
	    self->SpectrumString = SUN_SPEC;
	    self->ExtractFileExt = SUN_EXTRACT_FILE_EXTENSION;
	    self->PixelParString = SCD_EXTRACT_FILE_EXTENSION;
	}
	if (ProductType == PR_MOONSHINE) {
	    self->SpectrumString = MOON_SPEC;
	    self->ExtractFileExt = MOON_EXTRACT_FILE_EXTENSION;
	    self->PixelParString = MCD_EXTRACT_FILE_EXTENSION;
	}
	break;
    }
    default: {
	self->ProductType = PR_OTHER;
	Status = FATAL;
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__317", Status, LOG_NUM,
		 "illegal product type for struct config", "");
	return Status;
	break;
    }
    }
 
    /*--------------------------------------------------------------------*\
    ** initialize PCD or SCD or MCD fileposition, record size and the file
    ** position of the bands
    \*--------------------------------------------------------------------*/
    self->FilePosPCD	= FilePosPCD;
    self->PCD_Size	= PCD_Size;
    self->FilePos_SD[ 0] = FilePos_SD;
    for ( i = 1; i < SCIENCE_DATA_DEFINED; i++) {      
	self->FilePos_SD[ i] = self->FilePos_SD[ i - 1] +
	    NoOfBDR_Rec[ i - 1] * BDR_Rec_Size[  i - 1];
    }

    /*----------------------------------------------------------------------*\
    ** message if no PCDs, SCDs or MCDs
    \*----------------------------------------------------------------------*/
    if (NoOf_PCD_Rec == 0) 
    {
	self->FirstMeasurementPixel	= 0;
	self->LastMeasurementPixel	= -1;
	Status = WARN;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__344", Status,
		  LOG_NUM, "Level 1 Product contains no PCD, SCD or MCD", "");
    }
    else
    {
	/*--------------------------------------------------------------------*\
	** read the first PCD or SMCD and the last PCD or SMCD and get the time 
	** of the first PCD or SMCD and the last PCD or SMCD
	\*--------------------------------------------------------------------*/
	if ((*xCD__ReadTime)(self->Level_1_ProductStream, self->FilePosPCD,
			     PCD_Size, &PixelTime, 0, convert_sun_2_host)
	    != SUCCESS){
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__357", Status, LOG_CANNOT_READ,
		      "first PCD or SMCD from Level 1 Product file", "");
	    return Status;
	}
	else {     
	    self->TimeFirst_PCD = PixelTime;
	    /*----------------------------------------------------------------*\
	    **  read the last PCD or SMCD to get the end of the time of the
	    **  product
	    \*----------------------------------------------------------------*/
	    if (NoOf_PCD_Rec > 1) {
		if ((*xCD__ReadTime)(self->Level_1_ProductStream, 
				     self->FilePosPCD + PCD_Size 
				     * (NoOf_PCD_Rec - 1), PCD_Size,
				     &PixelTime, 0, convert_sun_2_host) 
		    != SUCCESS){
		    Status = FATAL;
		    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__374", Status,
			      LOG_CANNOT_READ,
			      "last PCD or SMCD from Level 1 Product file", "");
		    return Status;
		}
	    }
	    self->TimeLast_PCD = PixelTime;
   
	}

#ifdef DEBUG  
	UTC_2_ASCII( self->TimeFirst_PCD, ASCII_Time);
	sprintf(debug_message, "First PCD or SMCD[ 0] %s Days: %u, Msecs: %u\n",
	       ASCII_Time, self->TimeFirst_PCD.Days,
	       self->TimeFirst_PCD.Msecs);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);

	UTC_2_ASCII( self->TimeLast_PCD, ASCII_Time);
	sprintf(debug_message, "Last  PCD or SMCD[ %d] %s Days: %u, Msecs: %u\n",
	       NoOf_PCD_Rec - 1, ASCII_Time, self->TimeLast_PCD.Days,
	       self->TimeLast_PCD.Msecs);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif

	/*--------------------------------------------------------------------*\
	**  determine the first and the last PCD or SMCD to be processed 
	**  according the given time range (switch -t)
	\*--------------------------------------------------------------------*/
	if (self->Filter.timeFlag) {
	    /*----------------------------------------------------------------*\
	    ** check the time range
	    \*----------------------------------------------------------------*/
	    if ( UTC_Comp( '<', self->Filter.timeStop, self->TimeFirst_PCD)){
		Status = FATAL;
		UTC_2_ASCII( self->TimeFirst_PCD, Begin_ASCII_Time);
		UTC_2_ASCII( self->TimeLast_PCD, End_ASCII_Time);
		UTC_2_ASCII( self->Filter.timeStop, ASCII_Time);
		sprintf( LogBuffer, "Stop Time %s out of range of Level 1 Product Data",
			 ASCII_Time);
		sprintf(LogAction, "valid time range is %s to %s", Begin_ASCII_Time,
			 End_ASCII_Time);
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__415", Status, LOG_NUM,
			  LogBuffer, LogAction);
		return Status;
	    }
	    else {
		if ( UTC_Comp( '>', self->Filter.timeStart, self->TimeLast_PCD)){
		    Status = FATAL;
		    UTC_2_ASCII( self->TimeFirst_PCD, Begin_ASCII_Time);
		    UTC_2_ASCII( self->TimeLast_PCD, End_ASCII_Time);
		    UTC_2_ASCII( self->Filter.timeStart, ASCII_Time);
		    sprintf(LogBuffer,
			    "Start Time %s out of range of Level 1 Product Data",
			    ASCII_Time);
		    sprintf(LogAction, "valid time range is %s to %s", Begin_ASCII_Time,
			    End_ASCII_Time);
		    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__430", Status, LOG_NUM,
			      LogBuffer, LogAction);
		    return Status;
		}
		else {
		    /*---------------------------------------------------------------*\
		    ** determine time range and 
		    \*---------------------------------------------------------------*/
		    if ( UTC_Comp( '<', self->Filter.timeStart,
				   self->TimeFirst_PCD))
			self->BeginTime = self->TimeFirst_PCD;
		    else
			self->BeginTime = self->Filter.timeStart;
		    if ( UTC_Comp( '>', self->Filter.timeStop, self->TimeLast_PCD))
			self->EndTime = self->TimeLast_PCD;
		    else
			self->EndTime = self->Filter.timeStop;

		    /*---------------------------------------------------------------*\
		    ** calculate first and last PCD or SMCD index
		    \*---------------------------------------------------------------*/
		    BeginTime = (double)self->BeginTime.Days * (double)864000.0 +
			(double)self->BeginTime.Msecs / (double)100.0;

		    EndTime = (double)self->EndTime.Days * (double)864000.0 +
			(double)self->EndTime.Msecs / (double)100.0;

		    First_PCD_Time = (double)self->TimeFirst_PCD.Days *
			(double)864000.0 +
			(double)self->TimeFirst_PCD.Msecs / (double)100.0;

		    self->FirstMeasurementPixel = (int)((BeginTime - First_PCD_Time + 
							 (double)INTG_TIME_CONV_FACTOR)/
							(double)(GROUND_PIXEL_TIME * 10.0));

		    if (self->FirstMeasurementPixel < 0)
			self->FirstMeasurementPixel = 0;
		    else
			if (self->FirstMeasurementPixel >= NoOf_PCD_Rec)
			    self->FirstMeasurementPixel = NoOf_PCD_Rec - 1;

		    self->LastMeasurementPixel = (int)((EndTime - First_PCD_Time)/
						       (double)(GROUND_PIXEL_TIME * 10.0));

		    if (self->LastMeasurementPixel < 0)
			self->LastMeasurementPixel = 0;
		    else
			if (self->LastMeasurementPixel >= NoOf_PCD_Rec)
			    self->LastMeasurementPixel = NoOf_PCD_Rec - 1;

		    /*--------------------------------------------------------*\
		    **  search the measurement pixel with the defined BeginTime.
		    **  The search starts with the calculated 
		    **  'FirstMeasurementPixel'
		    \*--------------------------------------------------------*/
		    if (PCD__SearchTime(self->Level_1_ProductStream,
					xCD__ReadTime,
					self->FilePosPCD, NoOf_PCD_Rec,
					PCD_Size, self->BeginTime,
					self->FirstMeasurementPixel,
					&self->FirstMeasurementPixel,
					&self->BeginTime,convert_sun_2_host) 
			!= SUCCESS){
			Status = FATAL;
			UTC_2_ASCII( self->BeginTime, ASCII_Time);
			sprintf(LogBuffer,
				"the PCD or SMCD of the given Start time %s from the Level 1 Product",
				ASCII_Time);
			Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__498", Status,
				  LOG_CANNOT_READ, LogBuffer,
				  "Check the pixel time of on Level 1 Product");
			return Status;
		    }

		    /*--------------------------------------------------------*\
		    **  search the measurement pixel with the defined 'EndTime'.
		    **  The search starts with the calculated 
		    **  'LastMeasurementPixel'
		    \*--------------------------------------------------------*/
		    if (PCD__SearchTime(self->Level_1_ProductStream,
					xCD__ReadTime, self->FilePosPCD,
					NoOf_PCD_Rec, PCD_Size,	self->EndTime,
					self->LastMeasurementPixel,
					&self->LastMeasurementPixel,
					&self->EndTime, convert_sun_2_host)
			!= SUCCESS){
			Status = FATAL;
			UTC_2_ASCII( self->EndTime, ASCII_Time);
			sprintf(LogBuffer,
				"the PCD or SMCD of the given Stop time %s from the Level 1 Product",
				ASCII_Time);
			Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__521", Status,
				  LOG_CANNOT_READ, LogBuffer,
				  "Check the pixel time in the Level 1 Product");
			return Status;
		    }
		    /*-----------------------------------------------------------------*\
		    ** case albedoFlag:
		    ** set pixelStart to the first pixel, after all bands were completed
		    ** and pixelStop to the next pixel, when all bands were completed
		    \*-----------------------------------------------------------------*/
		    if (self->Filter.albedoFlag) 
			PCD__FindAllBandsCompleted(self, NoOf_PCD_Rec, 
						   convert_sun_2_host);
		}
	    }
	}
	/*----------------------------------------------------------------------*\
	** determine the first and the last PCD or SMCD to be processed according
	** the given pixel range (switch -p)
	\*----------------------------------------------------------------------*/
	else {
	    /*-----------------------------------------------------------------*\
	    ** check pixel range parameter
	    \*-----------------------------------------------------------------*/
	    if (self->Filter.pixelFlag) {
		/*-----------------------------------------------------------------*\
		** case albedoFlag:
		** set pixelStart to the first pixel, after all bands were completed
		** and pixelStop to the last pixel, before all bands were completed
		\*-----------------------------------------------------------------*/
		if (self->Filter.albedoFlag) 
		    PCD__FindAllBandsCompleted(self, NoOf_PCD_Rec, 
					       convert_sun_2_host);

		/*-----------------------------------------------------------------*\
		** check the lower pixel limit if out of range
		\*-----------------------------------------------------------------*/
		if (self->Filter.pixelStart > (NoOf_PCD_Rec - 1)) {
		    Status = FATAL;
		    sprintf(LogBuffer,
			    "lower Pixel interval (%2d) out of range of Level 1 Product Data",
			    self->Filter.pixelStart + 1);
		    sprintf(LogAction,
			    "the Pixel range of the Level 1 Product is 1 to %2d",
			    NoOf_PCD_Rec);
		    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__566", Status, LOG_NUM,
			      LogBuffer, LogAction);
		    return Status;
		}
		else {
		    if (self->Filter.pixelStart < 0)
			self->FirstMeasurementPixel = 0;
		    else
			self->FirstMeasurementPixel = self->Filter.pixelStart;
		}

		/*-----------------------------------------------------------------*\
		** check the upper pixel limit if out of range
		\*-----------------------------------------------------------------*/
		if (self->Filter.pixelStop < 0) {
		    Status = FATAL;
		    sprintf(LogBuffer,
			    "upper Pixel interval (%2d) out of range of Level 1 Product Data",
			    self->Filter.pixelStop + 1);
		    sprintf(LogAction,
			    "the Pixel range of the Level 1 Product is 1 to %2d",
			    NoOf_PCD_Rec);
		    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__588", Status, LOG_NUM,
			      LogBuffer, LogAction);
		    return Status;
		}
		else {
		    if (self->Filter.pixelStop > (NoOf_PCD_Rec - 1))
			self->LastMeasurementPixel = NoOf_PCD_Rec - 1;
		    else
			self->LastMeasurementPixel = self->Filter.pixelStop;
		}

		if (Status == SUCCESS) {
		    /*--------------------------------------------------------*\
		    ** read the time of the first pixel to be processed
		    \*--------------------------------------------------------*/
		    if ((*xCD__ReadTime)(self->Level_1_ProductStream, 
					 self->FilePosPCD + PCD_Size 
					 * self->FirstMeasurementPixel,
					 PCD_Size, &PixelTime, 0, 
					 convert_sun_2_host) != SUCCESS){
			Status = FATAL;
			Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__609", Status,
				  LOG_CANNOT_READ,
				  "first PCD or SMCD from level 1 product file", "");
			return Status;
		    }
		    else {     
			self->BeginTime = PixelTime;

			/*----------------------------------------------------*\
			** read the time of the last pixel to be processed
			\*----------------------------------------------------*/
			if (self->FirstMeasurementPixel != self->LastMeasurementPixel){
			    if ((*xCD__ReadTime)(self->Level_1_ProductStream, 
						 self->FilePosPCD + PCD_Size
						 * self->LastMeasurementPixel,
						 PCD_Size, &PixelTime, 0,
						 convert_sun_2_host)
				!= SUCCESS){
				Status = FATAL;
				Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__628", Status,
					  LOG_CANNOT_READ,
					  "last PCD or SMCD from level 1 product file", "");
				return Status;
			    }
			}
			self->EndTime = PixelTime;
		    }
		}
	    }
	    /*-----------------------------------------------------------------*\
	    ** set default value for pixel range
	    \*-----------------------------------------------------------------*/
	    else{
		self->FirstMeasurementPixel	= 0;
		self->LastMeasurementPixel	= self->FirstMeasurementPixel - 1;

		/*-----------------------------------------------------------------*\
		** case albedoFlag:
		** set pixelStart to the first pixel, after all bands were completed
		** and pixelStop to the next pixel, when all bands were completed
		\*-----------------------------------------------------------------*/
		if (self->Filter.albedoFlag) 
		    PCD__FindAllBandsCompleted(self, NoOf_PCD_Rec, 
					       convert_sun_2_host);
	    }
	}
    }

    /*----------------------------------------------------------------------*\
    ** determine the first and the last PCD or SMCD to be processed according
    ** the geolocation range (switch -r)
    \*----------------------------------------------------------------------*/
    if (self->Filter.regionFlag)
    {
	if (PCD__SearchGeolocation(self, xCD__ReadTime, NoOf_PCD_Rec, 
				   PCD_Size, &(self->FirstMeasurementPixel),
				   &(self->BeginTime),
				   &(self->LastMeasurementPixel),
				   &(self->EndTime), convert_sun_2_host)
	    != SUCCESS){
	    Status = FATAL;
	    sprintf(LogBuffer,
		    "the PCD or SMCD of the given geolocation range from the Level 1 Product");
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__672", Status,
		      LOG_CANNOT_READ, LogBuffer,
		      "Check the given geolocation range");
	    return Status;
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Config__openLevel_1_ProductStream
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Open the level 1 product file and test right product size
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__openLevel_1_ProductStream(Config_s* self)
{
    char LogBuffer[LOG_FIELD_SIZE];
    
    if ( (self->Level_1_ProductStream = fopen(self->Filter.inputFilename,
					      "rb")) == NULL)
    {
	sprintf(LogBuffer, "input file '%s'", self->Filter.inputFilename);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__719", FATAL, LOG_CANNOT_OPEN,
		 LogBuffer, "permissions and try again");
	return FATAL;
    }
    else if (File__getSize(self->Level_1_ProductStream, &(self->ProductSize))
	     != SUCCESS) 
    {
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__726", FATAL, LOG_NUM,
		 "Error getting file size of Level 1 Product", "");
	return FATAL;
    }
    else 
    {
	return SUCCESS;
    }
}


/*----------------------------------------------------------------------------*\
**                                PCD__SearchAllBandsCompleted
**  Input parameters:
**	Config			containing input file stream, filter... 
**				
**      NoOf_PCD_Rec		total number of earthshine pixels
**      PCD_Size		size of the PCD
**	direction		which direction to search, up or down the file
**	SearchStart		pixel number, where to start searching
**
**  Output parameters:
**      PCD_Time		the time of the PCD found
**      PCD_Time_prev		the time of the previous PCD found
**	FoundPosition		the pixel number of the PCD found
**	Status			if search was successful or not
**
**  Other interfaces:
**    Input file Level 1 Product defined via config->Stream
**
**  Description:
**    find the next pixel starting from SearchStart going to direction direction, 
**    where all integrations are completed
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	1.9.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
**			adopted from PCD__ReadTime
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__SearchAllBandsCompleted(Config_s *Config, short NoOf_PCD_Rec, 
			     Search_e direction, int SearchStart, 
			     UTC_s *PCD_Time, UTC_s *PCD_Time_prev,
			     int* FoundPosition, Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    PCD_s			PCD_Local;
    long 			FilePos;
    int 			Pixelindex;
    int 			pixelStart;
    int 			pixelStop;
    int 			loop;
    Bool 			IntgStatus;
    int			increment;
    UTC_s			cur_time = {0, 0};
    UTC_s			prev_time = {0, 0};
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** set increment
    \*--------------------------------------------------------------------*/
    if (direction == SEARCH_DOWN)
	increment = -1;
    else
	increment = 1;

    /*--------------------------------------------------------------------*\
    ** search
    \*--------------------------------------------------------------------*/
    IntgStatus = False;
    Pixelindex = SearchStart;

    while (Status == SUCCESS && Pixelindex > 0 && Pixelindex < NoOf_PCD_Rec) 
    {
	/*----------------------------------------------------------------*\
	** calculate file position
	\*----------------------------------------------------------------*/
	FilePos = Config->FilePosPCD + Config->PCD_Size * Pixelindex;

	/*----------------------------------------------------------------*\
	** read PCD
	\*----------------------------------------------------------------*/
	if (fseek ( Config->Level_1_ProductStream, FilePos, SEEK_SET) == -1) 
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__814", Status,
		     LOG_CANNOT_READ,
		     "PCD of history from Level 1 Product file (fseek)", "");
	    return Status;
	}
	
	if ( (Status = PCD__read(&PCD_Local, Config->Level_1_ProductStream,
				 convert_sun_2_host)) != SUCCESS)
	{
	    sprintf(LogBuffer,
		    "the PCD of the given FilePos %d from the Level 1 Product",
		    FilePos);
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__826", Status,
		     LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** save times
	\*--------------------------------------------------------------------*/
	prev_time = cur_time;
	cur_time = PCD_Local.Geolocation.DateUTC;
	
	/*--------------------------------------------------------------------*\
	** check integration status
	\*--------------------------------------------------------------------*/
	IntgStatus = True;
	for (loop = 0; loop < SCIENCE_DATA_DEFINED; loop++) {
	    if (Config->Filter.bandFilter[loop] 
		&& (PCD_Local.ScienceIndex[loop]) == INVALID_INDEX) 
	    {
		IntgStatus = False;
	    }
	}
	/*--------------------------------------------------------------------*\
	** check return requirements
	\*--------------------------------------------------------------------*/
	if (IntgStatus == True) 
	{
	    if ( (direction == SEARCH_DOWN && Pixelindex < SearchStart) 
		 || (direction == SEARCH_UP && Pixelindex 
		     >= SearchStart) )	
	    {
		/*--------------------------------------------------------------------*\
		** return the previous pixel number and time but only, if the pixel at 
		** search start is not a complete one. Otherwise continue search to the
		** previous completed pixel
		\*--------------------------------------------------------------------*/
		*PCD_Time = cur_time;
		*PCD_Time_prev = prev_time;
		*FoundPosition = Pixelindex;
		return SUCCESS;
	    }
	}
	Pixelindex += increment;
    }
    
    /*--------------------------------------------------------------------*\
    ** nothing found
    \*--------------------------------------------------------------------*/
    return ERROR;
}


/*----------------------------------------------------------------------------*\
**                                PCD__FindAllBandsCompleted
**  Input parameters:
**	Config			containing input file stream, filter... 
**				
**      NoOf_PCD_Rec		total number of earthshine pixels
**
**  Output parameters:
**	Config			containing input file stream, filter... 
**	Status			if search was successful or not
**
**  Other interfaces:
**
**  Description:
**	Function to modify 
**	    Config->BeginTime
**	    Config->Filter.pixelStart
**	    Config->FirstMeasurementPixel
**	    Config->EndTime
**	    Config->Filter.pixelStop
**	    Config->LastMeasurementPixel
**	so that they point on a range of pixels, beginning with a pixel following
**	one where all bands were completed and ending with one where all bands are
**	completed     
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	1.9.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__FindAllBandsCompleted(Config_s *Config, short NoOf_PCD_Rec,
			   Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    int 			Pixelindex;
    int 			pixelStart;
    int 			pixelStop;
    UTC_s			PCD_Time = {0, 0};
    UTC_s			PCD_Time_prev = {0, 0};

    /*--------------------------------------------------------------------*\
    ** set start/stop indizes; do that routine only in case of pixelFlag
    ** or timeFlag
    \*--------------------------------------------------------------------*/
    if (Config->Filter.pixelFlag) {
	pixelStart = Config->Filter.pixelStart;
	pixelStop = Config->Filter.pixelStop;
    }
    else if (Config->Filter.timeFlag) {
	pixelStart = Config->FirstMeasurementPixel;
	pixelStop = Config->LastMeasurementPixel;
    }
    else {
	pixelStart = Config->FirstMeasurementPixel;
	pixelStop = NoOf_PCD_Rec - 1;
    }

    if (pixelStart < 1) {
	pixelStart = 1;
    }
    if (pixelStop > NoOf_PCD_Rec) {
	pixelStop = NoOf_PCD_Rec - 1;
    }

    /*--------------------------------------------------------------------*\
    ** search from pixel start downwards in time
    \*--------------------------------------------------------------------*/
    Status = PCD__SearchAllBandsCompleted(Config, NoOf_PCD_Rec, SEARCH_DOWN,
					  pixelStart, &PCD_Time, &PCD_Time_prev,
					  &Pixelindex, convert_sun_2_host);

    /*--------------------------------------------------------------------*\
    ** if search was successful, set new pixelStart after the pixel, where
    ** all bands were completed
    \*--------------------------------------------------------------------*/
    if (Status == SUCCESS) {
	Config->BeginTime = PCD_Time_prev;
	Config->Filter.pixelStart = Pixelindex + 1;
	Config->FirstMeasurementPixel = Config->Filter.pixelStart;
    }
    else {
	/*--------------------------------------------------------------------*\
	** if not found, search back from pixel start upwards in time
	\*--------------------------------------------------------------------*/
	Status = PCD__SearchAllBandsCompleted(Config, NoOf_PCD_Rec,
					      SEARCH_UP, pixelStart,
					      &PCD_Time, &PCD_Time_prev,
					      &Pixelindex, convert_sun_2_host);

	if (Status == SUCCESS) {
	    Config->BeginTime = PCD_Time;
	    Config->BeginTime.Msecs += (unsigned long)(GROUND_PIXEL_TIME 
						       * 1000.0);
	    Config->Filter.pixelStart = Pixelindex + 1;
	    Config->FirstMeasurementPixel = Config->Filter.pixelStart;
	}
    }

    /*--------------------------------------------------------------------*\
    ** search from pixel end upwards in time
    \*--------------------------------------------------------------------*/
    Status = PCD__SearchAllBandsCompleted(Config, NoOf_PCD_Rec, SEARCH_UP, 
					  pixelStop, &PCD_Time, &PCD_Time_prev,
					  &Pixelindex, convert_sun_2_host);

    /*--------------------------------------------------------------------*\
    ** if search was successful, set new pixelStart at the pixel, where
    ** all bands were completed
    \*--------------------------------------------------------------------*/
    if (Status == SUCCESS) {
	Config->EndTime = PCD_Time;
	Config->Filter.pixelStop = Pixelindex;
	Config->LastMeasurementPixel = Config->Filter.pixelStop;
    }
    else {
	/*--------------------------------------------------------------------*\
	** if not found, search back from pixel stop downwards in time
	\*--------------------------------------------------------------------*/
	Status = PCD__SearchAllBandsCompleted(Config, NoOf_PCD_Rec, SEARCH_DOWN,
					      pixelStop, &PCD_Time,
					      &PCD_Time_prev, &Pixelindex,
					      convert_sun_2_host);
	if (Status == SUCCESS) {
	    Config->EndTime = PCD_Time;
	    Config->Filter.pixelStop = Pixelindex;
	    Config->LastMeasurementPixel = Config->Filter.pixelStop;
	}
    }

    if (Config->Filter.pixelStart >= Config->Filter.pixelStop) {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__1012", Status,
		  LOG_CANNOT_FIND,
		  " complete set of pixel in specified range", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PCD__SearchGeolocation
**  Input parameters:
**    Stream		Input stream
**    xCD__ReadTime	function to read either the PCD or SMCD
**    FirstFilePosPCD	the file position of the first PCD or SMCD
**    NoOf_PCD_Rec	number of PCD or SMCD in the file
**    PCD_Size		the size of the PCD or SMCD
**    SearchTime	Time to be searched within the PCD list of the Level 1
**			Product
**    MeasurementPixel	the number of measurement pixel
**
**  Output parameters:
**    FoundMeasPixel	PCD or SMCD index (Measurement pixel number) of the
**				given SearchTime
**    FoundTime		the time of the PCD or SMCD which corresponds to the
**				returned PCD or SMCD index (Measurement pixel
				number)
**    PCD__SearchTime		SUCCESS == time found in Level 1 Product
**				FATAL	== time not found in Level 1 Product
**
**  Other interfaces:
**    Input file Level 1 Product defined in Config
**
**  Description:
**	Function to search a given Time within the PCD or SMCD list of the Level
**	1 Product. If the search was successful 'FoundMeasurementPixel' contains
**	the corresonding PCD or SMCD index, 'FoundTime' the corresponding PCD
**      or SMCD time.
**	The search run is started by the given Pixel index ('MeasurementPixel').
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__SearchGeolocation(Config_s* Config,
		       FunctionStatus_e (*xCD__ReadTime)
		       (FILE* Stream, long FilePos, long PCD_Size, 
			UTC_s* PCD_Time, PCD_s* PCD, Bool convert_sun_2_host),
		       short NoOf_PCD_Rec, long PCD_Size,
		       int *StartMeasurementPixel, UTC_s *StartTime, 
		       int *EndMeasurementPixel, UTC_s *EndTime,
		       Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long FirstFilePosPCD = Config->FilePosPCD;
    int MeasurementPixel = Config->FirstMeasurementPixel;
    int MaxMeasurementPixel = *EndMeasurementPixel;
    int	counter;
    Search_e Search = SEARCH_UP;
    char ASCII_Time[ 30];
    UTC_s PixelTime;
    PCD_s PCD;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** Search first pixel inside geolocation range
    \*--------------------------------------------------------------------*/
    while (Search == SEARCH_UP && MeasurementPixel <= MaxMeasurementPixel)
    {
	/*--------------------------------------------------------------------*\
	** Read PCD or SMCD
	\*--------------------------------------------------------------------*/
	if ((*xCD__ReadTime)(Config->Level_1_ProductStream, FirstFilePosPCD
			     + PCD_Size * MeasurementPixel, PCD_Size,
			     &PixelTime, &PCD, convert_sun_2_host)
	    != SUCCESS)
	{
	    Status = FATAL;
	    sprintf(LogBuffer,
		    "the PCD or SMCD Nr %d from the Level 1 Product",
		    MeasurementPixel);
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__1098", Status,
		      LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
	}

	if (CheckIfInsideGeolocation(&(PCD.Geolocation), Config)) Search = SEARCH_FOUND;

	*StartMeasurementPixel = MeasurementPixel;
	*StartTime = PixelTime;
	*EndMeasurementPixel = MeasurementPixel;
	*EndTime = PixelTime;

	MeasurementPixel++;
    }

    if (Search == SEARCH_UP)
    {
      /* nothing was found */
      return ERROR;
    }

    /* Undo last increment */
    MeasurementPixel--;

    /*--------------------------------------------------------------------*\
    ** Search last pixel inside geolocation range, allow 10 outside
    \*--------------------------------------------------------------------*/
    counter = 0;
    while (MeasurementPixel <= MaxMeasurementPixel && counter < 10)
    {
	    /*----------------------------------------------------------------*\
	    ** Read PCD or SMCD
	    \*----------------------------------------------------------------*/
	    if ((*xCD__ReadTime)(Config->Level_1_ProductStream, 
				 FirstFilePosPCD + PCD_Size * MeasurementPixel,
				 PCD_Size, &PixelTime, &PCD, convert_sun_2_host)
		!= SUCCESS)
            {
		Status = FATAL;
		sprintf(LogBuffer,
			"the PCD or SMCD Nr %d from the Level 1 Product",
			MeasurementPixel);
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__1140", Status,
			  LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }

	    if ( !CheckIfInsideGeolocation(&(PCD.Geolocation), Config) ) counter++;

	    *EndMeasurementPixel = MeasurementPixel;
	    *EndTime = PixelTime;

	    MeasurementPixel++;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Extracting
**  Input parameters:
**    Config		Extract configuration structure
**
**  Output parameters:
**    Config		Extract configuration structure
**    Extracting	SUCCESS	= extraction finished without problems
**    			else	= extraction failed
**
**  Other interfaces:
**    Output file of Extract Level 1 Product specified in Config
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function to generate a Extract Level 1 Product according the given
**    input parameter and the given input switches stored in Filter of Config.
**    The actions are:
**	- read the PIR from Level 1 Product
**	- read FSR1
**	- read SPH1
**	- read FCD and check the indices
**	- open the Calibration files if necessary according the product type
**	- get the file positions of the PCD, SCD, MCD and BDR
**	- initialize the pixel range to be processed (Config_Init) according
**		the given product type
**	- generate different outputs:
**		- case extract earth spectrum data
**		- case extract sun spectrum data
**		- case extract Moon spectrum data
**		- case show info of Level 1 Product
**		- case show geolocation info of Level 1 Product
**	- close all open files
**	- deallocate memory
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.5.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__extract(Config_s*self, Bool convert_sun_2_host, Bool new_name_flag_albedo, 
		Bool write_sun_flag_albedo, Bool sun_extra_file_written, 
		Bool do_write_eps_flag)
{
    FunctionStatus_e Status = SUCCESS;
    Level_1_Product_s* level_1_product_ptr = Level_1_Product__new();
    char FCD_outputFilename[MAX_FILENAME_LENGTH];
    char PCD_outputFilename[MAX_FILENAME_LENGTH];
    char QA_outputFilename[MAX_FILENAME_LENGTH];
    long FilePosPCD;
    long FilePosSCD;
    long FilePosMCD;
    long FilePos_SD;
    ProductType_e ProductType;

    /*------------------------------------------------------------------------*\
    **  Open the level 1 product file and test right product size
    \*------------------------------------------------------------------------*/
    if (Config__openLevel_1_ProductStream(self) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1221", Status,
		 LOG_CANNOT_READ, "Error reading level 1 product", "");
	return Status;
    }


    /*------------------------------------------------------------------------*\
    ** read the static parts of the Level 1 Product (PIR, FSR1, SPH1, FCD)
    \*------------------------------------------------------------------------*/
    if (Level_1_Product__readStatic(level_1_product_ptr,
				    self->Level_1_ProductStream,
				    convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1235", Status, LOG_CANNOT_READ,
		 " PIR, FSR1, SPH1, FCD from Level 1 Product", "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** open QA (Quality Assurance) product file 
    \*------------------------------------------------------------------------*/
    if (self->Filter.qaInfoFlag) 
    {
	/*--------------------------------------------------------------------*\
	** extend the QA outputfile name 
	\*--------------------------------------------------------------------*/
	sprintf(QA_outputFilename, "%s.%s", self->Filter.outputFilename,
		QA_ExtractFileExt);
	/*--------------------------------------------------------------------*\
	** open the QA outputfile name
	\*--------------------------------------------------------------------*/
	if ((self->QA_ProductStream = fopen(QA_outputFilename, "w")) == NULL) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    Status = FATAL;
	    sprintf(LogBuffer, "output file '%s'", QA_outputFilename);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1258", Status,
		      LOG_CANNOT_OPEN, LogBuffer, "permissions and try again");
	    return Status;
	}
	
	/*--------------------------------------------------------------------*\
	** Write extract Header to QA outputfile
	\*--------------------------------------------------------------------*/
	if (Config__writeExtractHeader(self, self->QA_ProductStream) 
	    != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1270", Status,
		      LOG_CANNOT_WRITE, "Extract Header in extract "
		      "level 1 product", "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** Write QA info
	\*--------------------------------------------------------------------*/
	QA__WriteSpectral(self->QA_ProductStream, &(level_1_product_ptr->FCD));
    }
    else
    {
	self->QA_ProductStream = NULL;
    }

    /*------------------------------------------------------------------------*\
    ** open FCD and PCD or SCD or MCD product file 
    \*------------------------------------------------------------------------*/
    if (self->Filter.calFileFlag) 
    {
	/*--------------------------------------------------------------------*\
	** open the FCD outputfile name
	\*--------------------------------------------------------------------*/
	sprintf(FCD_outputFilename, "%s.%s", self->Filter.outputFilename,
	       FCD_EXTRACT_FILE_EXTENSION);
	if ((self->FCD_ProductStream = fopen(FCD_outputFilename, "wb")) ==
	    NULL) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    Status = FATAL;
	    sprintf(LogBuffer, "output file '%s'", FCD_outputFilename);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1302", Status,
		     LOG_CANNOT_OPEN, LogBuffer, "permissions and try again");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** Write extract Header
	\*--------------------------------------------------------------------*/
	if (Config__writeExtractHeader(self, self->FCD_ProductStream)
	    != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1314", Status,
		      LOG_CANNOT_WRITE, "Extract Header in extract "
		      "level 1 product", "");
	    return Status;
	}
	FCD__show(&(level_1_product_ptr->FCD), self->FCD_ProductStream, False);

	/*--------------------------------------------------------------------*\
	** extend the PCD or SCD or MCD outputfile name 
	\*--------------------------------------------------------------------*/
	Config__getXcdFilename(self, PCD_outputFilename);

	/*--------------------------------------------------------------------*\
	** open the PCD or SCD or MCD outputfile name
	\*--------------------------------------------------------------------*/
	if ((self->PCD_ProductStream = fopen(PCD_outputFilename, "wb")) ==
	    NULL) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    Status = FATAL;
	    sprintf(LogBuffer, "output file '%s'", PCD_outputFilename);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1335", Status,
		     LOG_CANNOT_OPEN, LogBuffer, "permissions and try again");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** Write extract Header
	\*--------------------------------------------------------------------*/
	if (Config__writeExtractHeader(self, self->PCD_ProductStream)
	     != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1347", Status,
		     LOG_CANNOT_WRITE, "Extract Header in extract "
		     "level 1 product", "");
	    return Status;
	}
    }
    else
    {
	self->FCD_ProductStream = NULL;
	self->PCD_ProductStream = NULL;
    }

    /*-----------------------------------------------------------------------*\
    ** get the product type
    \*-----------------------------------------------------------------------*/
    if (Filter__getProductType(&self->Filter, &ProductType) != SUCCESS) 
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1365", Status, LOG_NUM,
		  "undefined extracted product type", "");
	return Status;
    }

    /*-----------------------------------------------------------------------*\
    ** get the file pointer positions of PCD, SCD, MCD and BDR
    \*-----------------------------------------------------------------------*/
    FSR1__GetFilePos(&(level_1_product_ptr->FSR1), &FilePosPCD, &FilePosSCD,
		     &FilePosMCD, &FilePos_SD);

#ifdef DEBUG  
    {
	char debug_message[LOG_FIELD_SIZE];
	sprintf(debug_message, "ProductType: %d\n", ProductType);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
	sprintf(debug_message, "FilePosPCD %d FilePosSCD %d FilePosMCD %d\n",
	       FilePosPCD, FilePosSCD, FilePosMCD);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
    }
#endif

    switch (ProductType) 
    {
    case PR_EARTHSHINE:
	/*--------------------------------------------------------------------*\
	** initialize the pixel range to be processed and extract sun spectrum
	\*--------------------------------------------------------------------*/
	if (Config__init(self, ProductType, PCD__ReadTime, FilePosPCD,
			 level_1_product_ptr->FSR1.NoOf_PCD_Rec,
			 level_1_product_ptr->FSR1.PCD_Size,
			 FilePos_SD, level_1_product_ptr->FSR1.NoOfBDR_Rec,
			 level_1_product_ptr->FSR1.BDR_Rec_Size, 
			 convert_sun_2_host) != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1401", Status, LOG_NUM,
		      "Cannot find the required pixel range in the "
		      "level 1 product", "");
	    return Status;
	}

	if (Config__extractSpectra(self,level_1_product_ptr, convert_sun_2_host,
				   new_name_flag_albedo, write_sun_flag_albedo,
				   sun_extra_file_written, do_write_eps_flag)
	     != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1413", Status,
		      LOG_CANNOT_WRITE, "Extract Product", "");
	    return Status;
	}
	break;

    case PR_SUNSHINE:
	/*--------------------------------------------------------------------*\
	** initialize the pixel range to be processed and extract sun spectrum
	\*--------------------------------------------------------------------*/
	if (Config__init(self, ProductType, SMCD__ReadTime, FilePosSCD,
			 level_1_product_ptr->FSR1.NoOf_SCD_Rec, 
			 level_1_product_ptr->FSR1.SCD_Size, FilePos_SD,
			 level_1_product_ptr->FSR1.NoOfBDR_Rec,
			 level_1_product_ptr->FSR1.BDR_Rec_Size, 
			 convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1431", Status, LOG_NUM,
		      "Cannot find the required pixel range in the level "
		      "1 product", "");
	    return Status;
	}

	if (Config__extractSpectra(self, level_1_product_ptr,
				   convert_sun_2_host, new_name_flag_albedo, 
				   write_sun_flag_albedo, sun_extra_file_written,
				   do_write_eps_flag) != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1443", Status,
		     LOG_CANNOT_WRITE, "Sun Product", "");
	    return Status;
	}
	break;

    case PR_MOONSHINE:
	/*--------------------------------------------------------------------*\
	** initialize the pixel range to be processed and extract moon spectrum
	\*--------------------------------------------------------------------*/
	if (Config__init(self, ProductType, SMCD__ReadTime, 
			 FilePosMCD, level_1_product_ptr->FSR1.NoOf_MCD_Rec,
			 level_1_product_ptr->FSR1.MCD_Size, FilePos_SD, 
			 level_1_product_ptr->FSR1.NoOfBDR_Rec, 
			 level_1_product_ptr->FSR1.BDR_Rec_Size,
			 convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1461", Status, LOG_NUM,
		     "Cannot find the required pixel range in the level 1 "
		     "product", "");
	    return Status;
	}

	if (Config__extractSpectra(self, level_1_product_ptr,
				   convert_sun_2_host, new_name_flag_albedo, 
				   write_sun_flag_albedo,
				   sun_extra_file_written, do_write_eps_flag)
	    != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1474", Status,
		      LOG_CANNOT_WRITE, "Moon Product", "");
	    return Status;
	}
	break;

    case PR_INFO:
	/*---------------------------------------------------------------*\
	** extract info data
	\*---------------------------------------------------------------*/
	if (Level_1_Product__showExtractInfo(level_1_product_ptr, stdout,
					     self->Level_1_ProductStream,
					     convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1489", Status,
		      LOG_CANNOT_WRITE, "INFO of Extract Level 1 Product", "");
	    return Status;
	}
	break;

    case PR_INFO_GEOLOCATION:
	/*-------------------------------------------------------------------*\
	** initialize the pixel range to be processed
	\*-------------------------------------------------------------------*/
	if (Config__init(self, PR_EARTHSHINE, PCD__ReadTime, FilePosPCD,
			 level_1_product_ptr->FSR1.NoOf_PCD_Rec,
			 level_1_product_ptr->FSR1.PCD_Size, FilePos_SD,
			 level_1_product_ptr->FSR1.NoOfBDR_Rec,
			 level_1_product_ptr->FSR1.BDR_Rec_Size,
			 convert_sun_2_host) != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1507", Status,
		     LOG_NUM, "Cannot find the required pixel range in the "
		     "level 1 product", "");
	    return Status;
	}

	if (Config__extractGLRInfoShow(stdout, self, 
				       &(level_1_product_ptr->SPH1),
				       PR_EARTHSHINE, convert_sun_2_host)
	     != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1519", Status,
		      LOG_CANNOT_WRITE,
		      "Geolocation INFO of Extract Level 1 Product", "");
	    return Status;
	}
	break;

    case PR_INFO_QA:
	break;
    }

    /*--------------------------------------------------------------------*\
    ** close FCD, PCD and QA file
    \*--------------------------------------------------------------------*/
    if (self->FCD_ProductStream != NULL) 
    {
	if (fclose(self->FCD_ProductStream) != 0) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    sprintf(LogBuffer, "file '%s'", FCD_outputFilename);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1539", FATAL,
		     LOG_CANNOT_CLOSE, LogBuffer, "");
	    return Status;
	}
    }
    if (self->PCD_ProductStream != NULL) 
    {
	if (fclose(self->PCD_ProductStream) != 0) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    sprintf(LogBuffer, "file '%s'", PCD_outputFilename);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1550", FATAL,
		     LOG_CANNOT_CLOSE, LogBuffer, "");
	    return Status;
	}
    }
    if (self->QA_ProductStream != NULL) 
    {
	if (fclose(self->QA_ProductStream) != 0) 
	{
	    char LogBuffer[LOG_FIELD_SIZE];
	    sprintf(LogBuffer, "file '%s'", QA_outputFilename);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1561", FATAL,
		     LOG_CANNOT_CLOSE, LogBuffer, "");
	    return Status;
	}
    }


    /*--------------------------------------------------------------------*\
    ** deallocate the total allocated memory
    \*--------------------------------------------------------------------*/
#ifdef __cplusplus
    if (level_1_product_ptr->SPH1.Nref > 0)
	delete [] level_1_product_ptr->SPH1.PIR_Of_SDH;
    if (level_1_product_ptr->FCD.Nleak > 0)
	delete [] level_1_product_ptr->FCD.LeakagePar;
    if (level_1_product_ptr->FCD.Nhot > 0)
	delete [] level_1_product_ptr->FCD.HotPixelPar;
    if (level_1_product_ptr->FCD.Nspec > 0)
	delete [] level_1_product_ptr->FCD.SpectralPar;
    if (level_1_product_ptr->FCD.Nang > 0)
	delete [] level_1_product_ptr->FCD.KeyDataCalibrationL1_Par;
#else
    if (level_1_product_ptr->SPH1.Nref > 0)
	free((void *) (level_1_product_ptr->SPH1.PIR_Of_SDH));
    if (level_1_product_ptr->FCD.Nleak > 0)
	free((void *) (level_1_product_ptr->FCD.LeakagePar));
    if (level_1_product_ptr->FCD.Nhot > 0)
	free((void *) (level_1_product_ptr->FCD.HotPixelPar));
    if (level_1_product_ptr->FCD.Nspec > 0)
	free((void *) (level_1_product_ptr->FCD.SpectralPar));
    if (level_1_product_ptr->FCD.Nang > 0)
	free((void *) (level_1_product_ptr->FCD.KeyDataCalibrationL1_Par));
#endif

    Level_1_Product__delete(level_1_product_ptr);

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Config__extractSpectra
**  Input parameters:
**    Config			Extract configuration structure
**    PIR			PIR of Level 1 Product
**    FSR1			File structure record of Level 1 Product
**    SPH1			Specific Product Header of Level 1 Product
**    FCD			Fixed calibration record
**
**  Output parameters:
**    Config__extractSpectra	SUCCESS	= extraction finished without problems
**    			else	= extraction failed
**
**  Other interfaces:
**    Output file of Extract Level 1 Product specified in Config
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function to generate a Extract Level 1 Product according the given
**    input parameter and the given input switches stored in Filter in Config.
**    The actions are:
**	- initialize the pixel range to be processed (Config_Init)
**	- case extract data
**		- calculate grids of all Science Data
**		- initialize the Solar description record
**		- calculate errors and flags of the SDR
**		- for all extract files do
**			- Write extract Header
**			- Write PIR
**			- Write SFS
**			- Write SDR
**			- Init EDR
**			- Write EDR summary (EPS__Write)
**			- for all spectra belonging to the processed
**			  extract file do
**				- get the Spectrum data (EDR__Process)
**				- calibrate the spectrum data (EDR__Process)
**				- calculate the errors and flags (EDR__Process)
**				- write the spectrum (EDR__Write)
**	- case show info of Level 1 Product
**	- case show geolocation info of Level 1 Product
**
**    Split Mode:
**    The Config parameter FirstMeasurementPixelFile and
**    LastMeasurementPixelFile are
**    initialized in such a way, that every pixel (PCD or SCD or MCD) of the
**    Level 1 Product generates one extract file indicated by the pixel number.
**    In this case the EDR parameter First_PCD and Last_PCD are initialized
**    by the EDR parameter Current_PCD.
**
**    Non Split Mode:
**    The Config parameter FirstMeasurementPixelFile and
**    LastMeasurementPixelFile are
**    initialized in such a way, so that only one extract file will be
**    generated. All pixels (PCDs or SCDs or MCDs) to be processed will be
**    written into this extract file.
**    In this case the EDR parameter First_PCD and Last_PCD are initialized
**    by the pixel range to be processed.
**
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	28.9.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__extractSpectra(Config_s* self, Level_1_Product_s* level_1_product_ptr,
		       Bool convert_sun_2_host,
		       Bool new_name_flag_albedo, Bool write_sun_flag_albedo,
		       Bool sun_extra_file_written, Bool do_write_eps_flag)
{
    FunctionStatus_e Status = SUCCESS;
    FunctionStatus_e BSDF_Status = SUCCESS;
    SDR_s SDR;
    float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS];
    EDR_s EDR;
    GRID_s GRID;
    int current_file;
    int	FirstPixel;
    int	index;
    char open_type[3];
    char outputFilename[MAX_FILENAME_LENGTH];
    char OutputMeanSunFileName[MAX_FILENAME_LENGTH];
    char AlbedoLogFileName[MAX_FILENAME_LENGTH];
    FILE* albedo_file_ptr;
    int PixelIncrement;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    **  initialize the dynamic data structures
    \*------------------------------------------------------------------------*/
    GRID.NumberOfGrids = 0;
    GRID.Grid = 0;

    EDR.N_PCD_History = 0;
    EDR.N_PCD_HistoryRead = 0;
    EDR.PCD_History = 0;

    /*------------------------------------------------------------------------*\
    **  set the file parameter according the split and albedo flag
    \*------------------------------------------------------------------------*/
    if (self->Filter.splitFlag || self->Filter.noSunFlag 
	|| self->Filter.albedoFlag) {
	self->FirstMeasurementPixelFile = self->FirstMeasurementPixel;
	self->LastMeasurementPixelFile = self->LastMeasurementPixel;
    }
    else {
	self->FirstMeasurementPixelFile = -1;
	self->LastMeasurementPixelFile = -1;
    }

#ifdef DEBUG
    {
	FILE* debug_file_ptr;
	if ( (debug_file_ptr = Debug__getStream(debug_ptr, DBG_EXTRACTING,
						DEBUG_LEVEL)) != 0)
	{
	    fprintf(debug_file_ptr, "Config init:\n");
	    Config__Show(debug_file_ptr, self);
	}
    }
#endif

    /*------------------------------------------------------------------------*\
    **  Calculate all Grids of the FCD
    \*------------------------------------------------------------------------*/
    if (FCD__calculateGrids(&(level_1_product_ptr->FCD), &GRID) != SUCCESS)
    {
	Status = FATAL;
	return Status;
    }

    /*----------------------------------------------------------------*\
    **  if correctDegradationFlag, load degradation parameters from the
    **  degradation parameter file
    **  These are needed by SDR__init first time
    \*----------------------------------------------------------------*/
    if (self->Filter.correctDegradationFlag == True)
    {
      if (DegradationParameter__new(self->Filter.DegradationParameterFile) != SUCCESS)
      {
        Status = FATAL;
        Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1745", Status,
          LOG_CANNOT_CALCULATE, "Degradation parameters", "");

        return Status;
      }
    }

    /*----------------------------------------------------------------*\
    ** if correctDegradationFlag or correct_BSDF_Flag is set, load BSDF 
    ** azimuth dependancy correction parameters from given file
    \*----------------------------------------------------------------*/
    if (self->Filter.correctDegradationFlag == True || self->Filter.correct_BSDF_Flag == True)
    {
        BSDF_Status = BSDF__LoadAzimuthCorrectionParameters(self->Filter.DegradationParameterFile,
          level_1_product_ptr->FCD.BSDF,
          level_1_product_ptr->SPH1.SoftwareVersion);

      if (BSDF_Status == FATAL)
      {
        Status = FATAL;
        Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.4__1745", Status,
          LOG_CANNOT_READ, "BSDF azimuth dependancy parameters", "");

        return Status;
      }
      if (BSDF_Status == ERROR)
      {
        Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.4__1745", Status,
          LOG_CANNOT_CALCULATE, "BSDF azimuth dependancy parameters - aymmetric BSDF correction denied", "");

        self->Filter.correctDegradationFlag = False;
      }
    }

    /*------------------------------------------------------------------------*\
    **  initialize the solar record
    \*------------------------------------------------------------------------*/
    if (SDR__init(&SDR, SolarGrid, self, &(level_1_product_ptr->FCD)) != SUCCESS)
    {
	Status = FATAL;
	return Status;
    }

    /*------------------------------------------------------------------------*\
    **  calculate errors and flags of the SDR
    \*------------------------------------------------------------------------*/
    SDR__calculateErrorsAndFlags(&SDR, self, &(level_1_product_ptr->FCD));

    /*------------------------------------------------------------------------*\
    **  get all peltier
    \*------------------------------------------------------------------------*/
    if (self->Filter.calibrationFilter[ CAL_FPA] && 
	((self->ProductType == PR_EARTHSHINE) ||
	 (self->ProductType == PR_SUNSHINE) ||
	 (self->ProductType == PR_MOONSHINE))) {
	Config__getPeltierNoise(self, &(level_1_product_ptr->FSR1), 
				&(level_1_product_ptr->SPH1), 
				&(level_1_product_ptr->FCD), convert_sun_2_host);
    }

    /*------------------------------------------------------------------------*\
    **  open albedo log file (containing file names of all created 
    **  albedo files
    \*------------------------------------------------------------------------*/
    if (self->Filter.albedoFlag) 
    {
	strcpy(AlbedoLogFileName, self->Filter.outputFilename);
	strcat(AlbedoLogFileName, ".afn");
	albedo_file_ptr = fopen(AlbedoLogFileName, "w");
	if (albedo_file_ptr == NULL) 
	{
	    Status = FATAL;
	    sprintf(LogBuffer, "output file '%s'", AlbedoLogFileName);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1818", Status,
		     LOG_CANNOT_OPEN, LogBuffer, "permissions and try again");
	    return Status;
	}
    }

    /*------------------------------------------------------------------------*\
    **  open mean sun reference file (if noSunFlag is true)
    \*------------------------------------------------------------------------*/
    if (self->Filter.noSunFlag)
    {
	strcpy(OutputMeanSunFileName, self->Filter.outputFilename);
	strcat(OutputMeanSunFileName, ".sun");
	self->Extract_1_MeanSun_ProductStream = 
	    fopen(OutputMeanSunFileName, "w");
	if (self->Extract_1_MeanSun_ProductStream == NULL)
	{ 
	    Status = FATAL;
	    sprintf(LogBuffer, "output file '%s'", OutputMeanSunFileName);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1837", Status,
		      LOG_CANNOT_OPEN, LogBuffer, "permissions and try again");
	    return Status;
	}
    }

    /*------------------------------------------------------------------------*\
    **  loop over all output product files to be generated
    \*------------------------------------------------------------------------*/
    for (current_file = self->FirstMeasurementPixelFile;
	 current_file <= self->LastMeasurementPixelFile; current_file++)
    {
	EDR_WrittenCounter = 0;
	Config__generateOutputFilename(self, current_file, 
				       &new_name_flag_albedo,
				       &write_sun_flag_albedo, albedo_file_ptr,
				       outputFilename, open_type);
	
	/*--------------------------------------------------------------------*\
	**  write QA (Quality Assurance) output
	\*--------------------------------------------------------------------*/
	if (self->Filter.qaInfoFlag)
	{
	    if (QA__WriteSun(self, &(level_1_product_ptr->FCD), &SDR,
			     SolarGrid) != SUCCESS)	
	    {
		Status = FATAL;
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1864", Status,
			 LOG_CANNOT_WRITE, "SDR in extract level 1 product",
			 "");
		return Status;
	    }
	}
	else 
	{
	    /*----------------------------------------------------------------*\
	    **  normal mode (no Quality Assurance)
	    \*----------------------------------------------------------------*/
#ifdef DEBUG
	    {
		char debug_message[255];
		sprintf(debug_message, "open extract file: %s\n",
			outputFilename);
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
			     debug_message);
	    }
#endif

	    if (strncmp(outputFilename, "-", 1) == 0)
	    {
	      self->Extract_1_ProductStream = stdout; 
	    }
            else
	    {
	      if ( (self->Extract_1_ProductStream = fopen(outputFilename, 
							open_type)) == NULL)
	      {
		Status = FATAL;
		sprintf(LogBuffer, "output file '%s'", outputFilename);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1896", Status,
			 LOG_CANNOT_OPEN, LogBuffer,
			 "permissions and try again");
		return Status;
		break;
	      }
	    }
	}

	/*--------------------------------------------------------------------*\
	**  Write extract Header, PIR, ERS Information, SFS and SDR,
	**  only once in an output file at the beginning
	\*--------------------------------------------------------------------*/
	if (!self->Filter.albedoFlag || write_sun_flag_albedo) 
	{
	    write_sun_flag_albedo = False;

	    if (!self->Filter.qaInfoFlag) 
	    {
		/*------------------------------------------------------------*\
		**  Write extract Header, PIR and ERS Information
		\*------------------------------------------------------------*/
		if (Config__writeExtractHeader(self, 
					       self->Extract_1_ProductStream)
		    != SUCCESS)	
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1923", Status,
			     LOG_CANNOT_WRITE, 
			     "Extract Header in extract level 1 product", "");
		    return Status;
		}

#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "PIR__WriteExtr\n");
#endif
		if (PIR__writeExtractLevel_1_Product(level_1_product_ptr->PIR,
					     self->Extract_1_ProductStream) 
		    != SUCCESS)
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1938", Status,
			     LOG_CANNOT_WRITE, 
			     "PIR in extract level 1 product", "");
		    return Status;
		}

#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "ERS_Information_Show\n");
#endif
		if (SPH1__showERSInformation(&(level_1_product_ptr->SPH1), 
					     self->Extract_1_ProductStream)
		    != SUCCESS)	
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__1953", Status, 
			     LOG_CANNOT_WRITE,
			     "ERS Information in extract level 1 product", "");
		    return Status;
		}
	    }

	    /*----------------------------------------------------------------*\
	    **  Write SFS, SDR
	    \*----------------------------------------------------------------*/
	    if (!self->Filter.qaInfoFlag && !self->Filter.noSunFlag) 
	    {
#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "SFS__Write\n");
#endif
		if (SFS__Write(self->Extract_1_ProductStream, self,
			       &(level_1_product_ptr->FCD)) != SUCCESS)
		{
		    Status = FATAL;
		    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1973", Status,
			      LOG_CANNOT_WRITE, 
			      "SFS in extract level 1 product", "");
		    return Status;
		}

#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "SDR__write\n");
#endif

		if (SDR__write(&SDR, SolarGrid, self->Extract_1_ProductStream,
			       self, &(level_1_product_ptr->FCD))
		    != SUCCESS)
		{
		    Status = FATAL;
		    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__1989", Status,
			      LOG_CANNOT_WRITE, 
			      "SDR in extract level 1 product", "");
		    return Status;
		}
	    }

	    /*----------------------------------------------------------------*\
	    **  Write extract Header to extra file if noSunFlag is true
	    \*----------------------------------------------------------------*/
	    if (!self->Filter.qaInfoFlag && self->Filter.noSunFlag
		&& !sun_extra_file_written) 
	    {
		if (Config__writeExtractHeader(self, 
				       self->Extract_1_MeanSun_ProductStream)
		    != SUCCESS) 
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2007", Status,
			     LOG_CANNOT_WRITE, "Extract Header in extract "
			     "level 1 sun product", "");
		    return Status;
		}

		/*------------------------------------------------------------*\
		**  Write SFS to extra file
		\*------------------------------------------------------------*/
#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "SFS__Write sun product\n");
#endif
		if (SFS__Write(self->Extract_1_MeanSun_ProductStream, 
			       self, &(level_1_product_ptr->FCD)) != SUCCESS) 
		{
		    Status = FATAL;
		    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__2024", Status,
			      LOG_CANNOT_WRITE, 
			      "SFS in extract level 1 sun product", "");
		    return Status;
		}

		/*------------------------------------------------------------*\
		**  Write SDR to extra file if noSunFlag is true
		\*------------------------------------------------------------*/
#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "SDR__write sun product\n");
#endif

		if (SDR__write(&SDR, SolarGrid,
			       self->Extract_1_MeanSun_ProductStream, self,
			       &(level_1_product_ptr->FCD)) != SUCCESS) 
		{
		    Status = FATAL;
		    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__2043", Status,
			      LOG_CANNOT_WRITE,
			      "SDR in extract level 1 sun product", "");
		    return Status;
		}
		sun_extra_file_written = True;
	    }
	}

	/*--------------------------------------------------------------------*\
	**  Init EDR
	\*--------------------------------------------------------------------*/
#ifdef DEBUG  
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, "EDR__Init\n");
#endif
	if (EDR__Init(self, current_file, &EDR) != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__2061", Status,
		      LOG_NUM, "can't initialize EDR", "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	**  Write EDR
	\*--------------------------------------------------------------------*/
	if (EDR.Last_PCD < EDR.First_PCD) 
	{
	    /*----------------------------------------------------------------*\
	    ** Write EPS (Extracted Product Summary) only
	    \*----------------------------------------------------------------*/
	    if (!(self->Filter.splitFlag || self->Filter.noSunFlag 
		  || self->Filter.albedoFlag || self->Filter.qaInfoFlag)) 
	    {
#ifdef DEBUG  
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     "EPS__Write\n");
#endif
		if (EPS__Write(self, &EDR, False) != SUCCESS) 
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2084", Status,
			     LOG_CANNOT_WRITE, 
			     "EDR in extract level 1 product", "");
		    return Status;
		}
	    }
	}
	else 
	{
	    /*----------------------------------------------------------------*\
	    **  Write EPS and all EDR
	    \*----------------------------------------------------------------*/
#ifdef DEBUG
	    {
		char debug_message[255];  
		sprintf(debug_message, "process PCD or SCD or MCD[%d] to PCD "
			"or SCD or MCD[%d]\n", EDR.First_PCD, EDR.Last_PCD);
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
			     debug_message);
	    }
#endif
	    /*----------------------------------------------------------------*\
	    **  Process/calibrate data
	    \*----------------------------------------------------------------*/
	    /*----------------------------------------------------------------*\
	    **  if QA mode process only every e.g. 100th pixel (passed as 
	    **  parameter)
	    \*----------------------------------------------------------------*/
	    if (self->Filter.qaInfoFlag)
	    {
		/* PixelIncrement = QA_increment */;
		PixelIncrement = self->Filter.qa_increment;
	    }
	    else
	    {
		PixelIncrement = 1;
	    }

	    /*----------------------------------------------------------------*\
	    **  loop over all measurement pixels in one output file
	    \*----------------------------------------------------------------*/
	    for (EDR.Spectrum.Current_PCD = EDR.First_PCD;
		 (EDR.Spectrum.Current_PCD <= EDR.Last_PCD);
		  EDR.Spectrum.Current_PCD += PixelIncrement)
	    {
		if (EDR__process(&EDR, &GRID, self, level_1_product_ptr,
				 (const float(*)[DETECTOR_PIXELS])
				 SDR.Irradiance, convert_sun_2_host)
		    != SUCCESS) 
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2135", Status,
			     LOG_NUM, "Cannot process EDR in extract level 1 "
			     "product", "");
		    return Status;
		}

		/*--------------------------------------------------------*\
		**  check for geolocation restrictions
		\*--------------------------------------------------------*/
		if (!self->Filter.regionFlag 
		    || CheckIfInsideGeolocation(&(EDR.PixelPar.Specific.Earth.Geolocation), 
						self))
		{
		    /*----------------------------------------------------*\
		    **  Check if all bands have integration completed, 
		    **  if yes set flag to use new file next time
		    \*----------------------------------------------------*/
		    new_name_flag_albedo = True;
		    for (index = 0; index < SCIENCE_DATA_DEFINED; index++) 
		    {
			if ((self->Filter.bandFilter[index])
			    && (EDR.PixelPar.ScienceIndex[index] 
				== INVALID_INDEX)) 
			{
			    new_name_flag_albedo = False;
			}
		    }
			
		    /* if (EDR.Spectrum.BandRecord[ index].IntgTime == 0.0)
		       NewNameFlagAlbedo = False; */

		    /*--------------------------------------------------------*\
		    **  Write EPS if the first measurement pixel
		    \*--------------------------------------------------------*/
		    if ((EDR.Spectrum.Current_PCD == EDR.First_PCD
			 && !self->Filter.qaInfoFlag) ||
			(self->Filter.regionFlag && do_write_eps_flag)) 
		    {
#ifdef DEBUG  
			Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
				     "EPS__Write\n");
#endif
			if (EPS__Write(self, &EDR, False) != SUCCESS) 
			{
			    Status = FATAL;
			    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__2180",
				      Status, LOG_CANNOT_WRITE,
				      "EDR in extract level 1 product", "");
			    return Status;
			}
			do_write_eps_flag = False;
		    }            

		    /*--------------------------------------------------------*\
		    **  Write EDR
		    \*--------------------------------------------------------*/
#ifdef DEBUG
		    {
			char debug_message[255];  
			sprintf(debug_message, "process and write Spectrum "
				"of PCD or SCD or MCD[ %d]\n",
				EDR.Spectrum.Current_PCD);
			Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
				     debug_message);
		    }
#endif
		    if (EDR__write(&EDR, &GRID, self, 
				   &(level_1_product_ptr->FCD)) != SUCCESS) 
		    {
			Status = FATAL;
			Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2205", 
				 Status, LOG_CANNOT_WRITE,
				 "EDR in extract level 1 product", "");
			return Status;
		    }
		}

		/*------------------------------------------------------------*\
		**  delete memory for PCD_History
		\*------------------------------------------------------------*/
		/*
		  if ((self->Filter.splitFlag 
		  || self->Filter.noSunFlag) && (EDR.N_PCD_History > 0))
		  */
		if (EDR.N_PCD_History > 0)
		{
#ifdef __cplusplus
		    delete [] EDR.PCD_History;
#else
		    free((void *)EDR.PCD_History);
#endif
		    EDR.N_PCD_History = 0;
		    EDR.N_PCD_HistoryRead = 0;
		    EDR.PCD_History	= 0;		
		}
	    }
	}

	/*--------------------------------------------------------------------*\
	**  close the product file
	\*--------------------------------------------------------------------*/
	if (!self->Filter.qaInfoFlag) 
	{
	    /*----------------------------------------------------------------*\
	    **  Update EPS, set the correct written number of ground pixels
	    \*----------------------------------------------------------------*/
	    if (EPS__Write(self, &EDR, True) != SUCCESS) 
	    {
		Status = FATAL;
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2244", Status,
			 LOG_CANNOT_WRITE, 
			 "EDR in extract level 1 product", "");
		return Status;
	    }

#ifdef DEBUG
	    {
		char debug_message[255];  
		sprintf(debug_message, "close extract file: %s\n",
			outputFilename);
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
			     debug_message);
	    }
#endif
	    if (strncmp(outputFilename, "-", 1) == 0)
	    {
	      /* Extract_1_ProductStream is stdout, no need to close */
	    }
            else
            {
	      if (fclose(self->Extract_1_ProductStream) != 0) 
	      {
		Status = FATAL;
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.7__2268", Status,
			 LOG_CANNOT_CLOSE, outputFilename, "");
		return Status;
	      }
            }
	}
    }

    /*----------------------------------------------------------------*\
    **  if correctDegradationFlag, free buffer for degradation pars
    \*----------------------------------------------------------------*/
    if (self->Filter.correctDegradationFlag == True) DegradationParameter__delete();

    /*------------------------------------------------------------------------*\
    **  close the albedo log file
    \*------------------------------------------------------------------------*/
    if (self->Filter.albedoFlag && albedo_file_ptr != NULL) 
    {
	fclose(albedo_file_ptr);
    }

    /*------------------------------------------------------------------------*\
    **  close the mean sun file (if noSunFlag true)
    \*------------------------------------------------------------------------*/
    if (self->Filter.noSunFlag 
	&& self->Extract_1_MeanSun_ProductStream != NULL) 
    {
	fclose(self->Extract_1_MeanSun_ProductStream);
    }
#ifdef __cplusplus
    if (GRID.NumberOfGrids > 0) 
    {
	delete [] GRID.Grid;
    }
#else
    if (GRID.NumberOfGrids > 0) 
    {
	free((void *)GRID.Grid);
    }
#endif

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Extract_GLR_Info_Show
**  Input parameters:
**    Stream			output stream of info
**    Config		Extract Configuration structure
**    SPH1		Specific Product Header of Level 1 Product
**    ProductType	PR_EARTHSHINE, PR_SUNSHINE, PR_MOONSHINE
**
**  Output parameters:
**    Extract_GLR_Info_Show	SUCCESS	= output of info ok
**				else	= any error occured during output
**
**  Other interfaces:
**    Input file of Level 1 Product specified in Config
**    Output file for Info specified by stream
**
**  Description:
**    Function to write all State Vectors of the SPH1 of the Level 1 Product
**    and some geolocation information of the whole Level 1 Product
**    Product to the output stream according the product type.
**
**  References:
**    This function ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__extractGLRInfoShow(FILE *Stream, const Config_s *Config,
			   const SPH1_s *SPH1, ProductType_e ProductType,
			   Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    int i;
    long FilePosPCD;
    int Current_PCD;
    float CenterLatitude;
    float CenterLongitude;
    PixelPar_s PixelPar;
    char ASCII_Time[ 25];

    /*----------------------------------------------------------------------*\
    ** Show the StateVector of the ERS_PropagatorState
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "State Vector:  Date,  Orbit  Number,  Position  Vectors  (X,  Y,  Z),  Velocity  Vector  (dx/dt, dy/dt, dz/dt)\n");

    /*----------------------------------------------------------------------*\
    ** convert ERS_PropagatorState UTC time to ASCII time
    \*----------------------------------------------------------------------*/
    if (UTC_2_ASCII( SPH1->ERS_PropagatorState.StateVector.DateUTC,
		     ASCII_Time) != SUCCESS) {
	Status = ERROR;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.7__2366", Status, LOG_NUM,
		  "error converting UTC time of ERS_PropagatorState to ASCII", "");
	ASCII_Time[ 0] = 0;
    }

    fprintf( Stream,
	     "%16s  %05u  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f\n",
	     ASCII_Time,
	     SPH1->ERS_PropagatorState.StateVector.OrbitNumber,
	     SPH1->ERS_PropagatorState.StateVector.Position.X,
	     SPH1->ERS_PropagatorState.StateVector.Position.Y,
	     SPH1->ERS_PropagatorState.StateVector.Position.Z,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.X,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.Y,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.Z);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** Show Corner Coordinates of all Geolocations
    \*----------------------------------------------------------------------*/
    fprintf( Stream,
	     "Geolocation Information: SubsetCounter, 4 Corner and Center Coordinates (Lat., Long.)\n");
    for ( Current_PCD = Config->FirstMeasurementPixel;
	  (Current_PCD <= Config->LastMeasurementPixel) 
	      && ((Status == SUCCESS) || (Status == WARN)); Current_PCD++) 
    {
	/*--------------------------------------------------------------------*\
	** read PCD
	\*--------------------------------------------------------------------*/
	FilePosPCD = Config->FilePosPCD + Config->PCD_Size * Current_PCD;
	if (PixelPar__Get(Config->Level_1_ProductStream, FilePosPCD,
			  Config->PCD_Size, SPH1, ProductType, Current_PCD,
			  &PixelPar, convert_sun_2_host) != SUCCESS)
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__2401", Status,
		     LOG_CANNOT_READ,
		     "PCD from level 1 product file", "");
	    return Status;
	}
	else
	{
	    switch (ProductType) 
	    {
	    case PR_EARTHSHINE: {
				/*-----------------------------------------------------------------*\
				** get the CenterCoordinates
				\*-----------------------------------------------------------------*/
		CenterLatitude = 
		    PixelPar.Specific.Earth.Geolocation.CornerCoord[ 4].Latitude;
		CenterLongitude =
		    PixelPar.Specific.Earth.Geolocation.CornerCoord[ 4].Longitude;

				/*-----------------------------------------------------------------*\
				** Show Corner coordinates of Geolocation of PixelPar
				\*-----------------------------------------------------------------*/
		fprintf( Stream,
			 "%1d %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f\n",
			 PixelPar.AuxiliaryPar.SubsetCounter,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[0].Latitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[0].Longitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[1].Latitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[1].Longitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[2].Latitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[2].Longitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[3].Latitude,
			 PixelPar.Specific.Earth.Geolocation.CornerCoord[3].Longitude,
			 CenterLatitude,
			 CenterLongitude);
		break;
	    }
	    }
	    if ( ferror( Stream) != 0) {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__2440", Status,
			  LOG_CANNOT_WRITE, "Geolocation record", "");
		return Status;
	    }
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                FPA__GetPeltierNoise
**  Input parameters:
**    Config			Extract configuration structure
**    FSR1			File structure record of Level 1 Product
**    SPH1			Specific Product Header of Level 1 Product
**
**  Output parameters:
**    FCD			initializes the PeltierNoise in the FCD.FPA
**    FPA__GetPeltierNoise	SUCCESS	=	ok
**				else	=	any error occured 
**
**  Other interfaces:
**
**  Description:
**    Function to get the Peltier of Channel 0 from all PCDs, SCDs or MCDs.
**    The function calculates the according PeltierNoise based on the peltier
**    values.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	8.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__getPeltierNoise(const Config_s *Config, const FSR1_s *FSR1, 
		     const SPH1_s *SPH1, FCD_s *FCD, Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    short			PeltierDataSize;
    float			*PeltierData;
    long			xCD_Size;
    long			FilePos_xCD;
    long			FilePosPCD;
    long			FilePosSCD;
    long			FilePosMCD;
    long			FilePosSD;

    /*-------------------------------------------------------------------------*\
    ** create memory for peltier if PeltierDataSize > 0
    \*-------------------------------------------------------------------------*/
    FSR1__GetFilePos( FSR1, &FilePosPCD, &FilePosSCD, &FilePosMCD, &FilePosSD);
    switch( Config->ProductType) {
    case PR_EARTHSHINE: {
	FilePos_xCD	= FilePosPCD;
	xCD_Size 		= FSR1->PCD_Size;
	PeltierDataSize	= FSR1->NoOf_PCD_Rec;
	break;
    }
    case PR_SUNSHINE: {
	FilePos_xCD	= FilePosSCD;
	xCD_Size 		= FSR1->SCD_Size;
	PeltierDataSize	= FSR1->NoOf_SCD_Rec;
	break;
    }
    case PR_MOONSHINE: {
	FilePos_xCD	= FilePosMCD;
	xCD_Size 		= FSR1->MCD_Size;
	PeltierDataSize	= FSR1->NoOf_MCD_Rec;
	break;
    }
    default: {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.7__2516", Status,
		  LOG_NUM, "illegal Product Type", "");
	return Status;
	break;
    }
    }
    if (PeltierDataSize > 0) {
#ifdef __cplusplus
	PeltierData = new float[ PeltierDataSize];
#else
	PeltierData = (float*)malloc( sizeof( float) * PeltierDataSize);
#endif
	if (PeltierData == 0) {
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__2530", Status,
		     LOG_CANNOT_ALLOCATE, "memory for peltier", "");
	    return Status;
	}
	else {
	    /*----------------------------------------------------------------*\
	    ** get the peltier over all packages (SCANNING)
	    \*----------------------------------------------------------------*/
	    if (FPA__GetPeltier(Config->Level_1_ProductStream, 
				Config->ProductType, SPH1, FilePos_xCD,
				xCD_Size, PeltierDataSize, PeltierData,
				convert_sun_2_host) != SUCCESS) {
		Status = FATAL;
		Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__2543", Status, LOG_NUM,
			 "error getting Peltier value of BAND_1A", "");
		return Status;
	    }
	    else {
		/*------------------------------------------------------------*\
		** compute the peltier noise
		\*------------------------------------------------------------*/
		FPA__Compute( &FCD->FPA, PeltierData, PeltierDataSize);
	    }
	    
	    /*---------------------------------------------------------------*\
	    ** delete memory
	    \*---------------------------------------------------------------*/
#ifdef __cplusplus
	    delete [] PeltierData;
#else
	    free((void *)PeltierData);
#endif
	}
    }
    return Status;
}



/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                CheckIfInsideGeolocation
**  Input parameters:
**    EDR		Extract description record structure
**    Config		Extract Configuration structure
**
**  Output parameters:
**    CheckIfInsideGeolocation		True or False
**
**  Other interfaces:
**
**  Description:
**    Function to check if the center coordinate of the ground pixel is inside
**    the given limits.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.10.97
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
CheckIfInsideGeolocation(const GeolocationC_s *Geolocation, 
			 const Config_s *Config)
{
    Bool inside_latitudes = False;
    Bool inside_longitudes = False;

    float top_latitude;
    float bottom_latitude;

    float left_longitude;
    float right_longitude;

    float Latitude  = Geolocation->CornerCoord[4].Latitude;
    float Longitude = Geolocation->CornerCoord[4].Longitude;

    /* find minima and maxima of latitude */
    if (Config->Filter.cornerCoordinate[0].Latitude > Config->Filter.cornerCoordinate[1].Latitude)
    {
      top_latitude    = Config->Filter.cornerCoordinate[0].Latitude;
      bottom_latitude = Config->Filter.cornerCoordinate[1].Latitude;
    }
    else
    {
      top_latitude    = Config->Filter.cornerCoordinate[1].Latitude;
      bottom_latitude = Config->Filter.cornerCoordinate[0].Latitude;
    }

    /* find minima and maxima of longitude */
    if (Config->Filter.cornerCoordinate[0].Longitude > Config->Filter.cornerCoordinate[1].Longitude)
    {
      right_longitude  = Config->Filter.cornerCoordinate[0].Longitude;
      left_longitude = Config->Filter.cornerCoordinate[1].Longitude;
    }
    else
    {
      right_longitude  = Config->Filter.cornerCoordinate[1].Longitude;
      left_longitude = Config->Filter.cornerCoordinate[0].Longitude;
    }

    /* longitudes around 0 : */
    /* e.g -10..10 degr wanted */
    if (left_longitude < 0.0 && Longitude >= 180.0) Longitude -= 360.0;

    /* e.g 350..360 degr wanted */
    if (right_longitude > 360.0 && Longitude < 180.0) Longitude += 360.0;

    /* do comparison */
    if (Latitude  >= bottom_latitude  && Latitude  <= top_latitude)  inside_latitudes = True;
    if (Longitude >= left_longitude   && Longitude <= right_longitude) inside_longitudes = True;

    if (inside_latitudes && inside_longitudes)
	return True;
    else
	return False;
}

 
/*----------------------------------------------------------------------------*\
**                                Config__getXcdFilename
**  Input parameters:
**    ProductType
**    outputFilename
**
**  Output parameters:
**    Filename
**    Get_xCD_Filename		SUCCESS = ok
**				else	FATAL
**
**  Other interfaces:
**
**  Description:
**    Function to get the file name of the pixel specific file.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Config__getXcdFilename(Config_s* self, char* filename)
{
    FunctionStatus_e Status = SUCCESS;
    const char* extractFileExtension;

    switch (self->ProductType)
    {
    case PR_EARTHSHINE:
	{
	    extractFileExtension = PCD_EXTRACT_FILE_EXTENSION;
	    break;
	}
    case PR_SUNSHINE: 
	{
	    extractFileExtension = SCD_EXTRACT_FILE_EXTENSION;
	    break;
	}
    case PR_MOONSHINE: 
	{
	    extractFileExtension = MCD_EXTRACT_FILE_EXTENSION;
	    break;
	}
    default: 
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.7__2702", Status, LOG_NUM,
		     "illegal product type", "");
	    return Status;
	    break;
	}
    }

    /*------------------------------------------------------------------------*\
    ** extend the PCD or SCD or MCD outputfile name 
    \*------------------------------------------------------------------------*/
    sprintf(filename, "%s.%s", self->Filter.outputFilename,
	    extractFileExtension);

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Config__generateOutputFilename
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	24.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Config__generateOutputFilename(Config_s* self,
			       int current_file,
			       Bool* new_name_flag_albedo_ptr,
			       Bool* write_sun_flag_albedo_ptr,
			       FILE* albedo_file_ptr,
			       char outputFilename[MAX_FILENAME_LENGTH],
			       char open_type[3])
{
    /*--------------------------------------------------------------------*\
    **  if splitFlag extend the outputfile name by the measurement pixel
    **  number.
    **  if Albedo flag extend the outputfile name by the 
    **  measurement pixel number where all bands are completed
    \*--------------------------------------------------------------------*/
    if (self->Filter.splitFlag || self->Filter.noSunFlag) 
    {
	sprintf(outputFilename, "%s_%.4d.%s", self->Filter.outputFilename,
		current_file + 1, self->ExtractFileExt);
	    strcpy(open_type, "wb");
    }
    else if (self->Filter.albedoFlag) 
    {
	if (*new_name_flag_albedo_ptr) 
	{
	    sprintf(outputFilename, "%s_%.4d.%s", 
		    self->Filter.outputFilename,
		    current_file + 1, self->ExtractFileExt);		
	    strcpy(open_type, "wb");
	    *new_name_flag_albedo_ptr = False;
	    *write_sun_flag_albedo_ptr = True;
	    
	    if (albedo_file_ptr != NULL)
	    {
		fprintf(albedo_file_ptr, "%s\n", outputFilename);
	    }
	}
	else 
	{
	    strcpy(open_type, "ab");
	    *write_sun_flag_albedo_ptr = False;
	}
    }
    else
    {
	sprintf(outputFilename, "%s.%s", self->Filter.outputFilename,
		self->ExtractFileExt);
	    strcpy(open_type, "wb");
    }
}
