/*----------------------------------------------------------------------------*\
**                                Config.h
**
**  Class Config declaration
** 
**  Version:    5.2
**  Date:       08/02/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Config_h
#define _Config_h

#pragma ident "@(#)Config.h	5.2 08/02/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include <stdio.h>
#include "defs.h"
#include "filter.h"
#include "pcd.h"
#include "utc.h"
#include "fsr1.h"
#include "fcd.h"
#include "leveduct.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	EARTHSHINE_SPEC	("Earthshine Spectrum")

#define	MOON_SPEC	("Moonshine Spectrum")

#define	SUN_SPEC	("Sunshine Spectrum")


#define	EXTRACT_FILE_EXTENSION	("el1")

#define	SUN_EXTRACT_FILE_EXTENSION	("sl1")

#define	MOON_EXTRACT_FILE_EXTENSION	("ml1")

#define	PCD_EXTRACT_FILE_EXTENSION	("pcd")

#define	FCD_EXTRACT_FILE_EXTENSION	("fcd")

#define	SCD_EXTRACT_FILE_EXTENSION	("scd")

#define	MCD_EXTRACT_FILE_EXTENSION	("mcd")


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class Config
\*============================================================================*/
typedef struct Config {
    Filter_s Filter;
    FILE* Level_1_ProductStream;
    long ProductSize;
    FILE* Extract_1_ProductStream;
    FILE* Extract_1_MeanSun_ProductStream;
    FILE* FCD_ProductStream;
    FILE* PCD_ProductStream;
    FILE* QA_ProductStream;
    ScienceData_e SDR_StartBand;
    ScienceData_e SDR_EndBand;
    ScienceData_e StartBand;
    ScienceData_e EndBand;
    int FirstMeasurementPixel;
    int LastMeasurementPixel;
    int	FirstMeasurementPixelFile;
    int	LastMeasurementPixelFile;
    long FilePos_SD[ SCIENCE_DATA_DEFINED];
    long FilePosPCD;
    long PCD_Size;
    UTC_s BeginTime;
    UTC_s EndTime;
    UTC_s TimeFirst_PCD;
    UTC_s TimeLast_PCD;
    ProductType_e ProductType;
    const char*	SpectrumString;
    const char*	ExtractFileExt;
    const char*	PixelParString;
} Config_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
Config_s*
Config__new(void);

/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
Config__delete(Config_s* self);

/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/
#ifdef __cplusplus
inline
void
Config__setSDRBand(Config_s* self, ScienceData_e SDR_StartBand, 
		   ScienceData_e SDR_EndBand)
{
    self->SDR_StartBand = SDR_StartBand;
    self->SDR_EndBand = SDR_EndBand;
}
#else
#define \
Config__setSDRBand(self, SDR_StartBand1, SDR_EndBand1) \
    self->SDR_StartBand = SDR_StartBand1; self->SDR_EndBand = SDR_EndBand1
#endif


#ifdef __cplusplus
inline
void
Config__setBand(Config_s* self, ScienceData_e StartBand, ScienceData_e EndBand)
{
    self->StartBand = StartBand;
    self->EndBand = EndBand;
}
#else
#define \
Config__setBand(self, StartBand1, EndBand1) \
    self->StartBand = StartBand1; self->EndBand = EndBand1
#endif

/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/

FunctionStatus_e
Config__init(Config_s*self, ProductType_e ProductType,
	     FunctionStatus_e (*xCD__ReadTime)
	     (FILE* Stream, long FilePos, long PCD_Size, UTC_s* PCD_Time,
	      PCD_s* PCD, Bool CONVERT_SUN_2_HOST),
	     long FilePosPCD,
	     short NoOf_PCD_Rec,
	     long PCD_Size,
	     long FilePos_SD,
	     short NoOfBDR_Rec[SCIENCE_DATA_DEFINED],
	     long BDR_Rec_Size[SCIENCE_DATA_DEFINED],
	     Bool CONVERT_SUN_2_HOST);

#ifdef __cplusplus
inline
FunctionStatus_e
Config__parseUserParameters(Config_s* self, int argc, char* const argv[])
{
    return Filter__parseUserParameters(&(self->Filter), &(self->ProductType), argc, argv);
}
#else
#define \
Config__parseUserParameters(self, argc, argv) \
	Filter__parseUserParameters(&(self->Filter), &(self->ProductType), argc, argv)
#endif

void
Config__Show(FILE* Stream, const Config_s* Config);

#ifdef __cplusplus
inline
FunctionStatus_e
Config__writeExtractHeader(Config_s* self, FILE* stream)
{
    return Filter__writeExtractHeader(&(self->Filter), stream, self->ProductType);
}
#else
#define \
Config__writeExtractHeader(self, stream) \
	Filter__writeExtractHeader(&(self->Filter), stream, self->ProductType)
#endif

FunctionStatus_e
Config__openLevel_1_ProductStream(Config_s* self);

FunctionStatus_e
Config__extract(Config_s*self, Bool convert_sun_2_host, Bool new_name_flag_albedo, 
	   Bool write_sun_flag_albedo, Bool sun_extra_file_written, 
	   Bool do_write_eps_flag);








FunctionStatus_e
PCD__SearchAllBandsCompleted(Config_s *Config, short NoOf_PCD_Rec, 
			     Search_e direction, int SearchStart, 
			     UTC_s *PCD_Time, UTC_s *PCD_Time_prev, 
			     int* FoundPosition, Bool CONVERT_SUN_2_HOST);

FunctionStatus_e
PCD__FindAllBandsCompleted(Config_s *Config, short NoOf_PCD_Rec,
			   Bool CONVERT_SUN_2_HOST);

FunctionStatus_e
PCD__SearchGeolocation(Config_s* Config,
		       FunctionStatus_e (*xCD__ReadTime)
		       (FILE* Stream, long FilePos, long PCD_Size, 
			UTC_s *PCD_Time, PCD_s* PCD, Bool convert_sun_2_host),
		       short NoOf_PCD_Rec, long PCD_Size,
		       int *StartMeasurementPixel, UTC_s *StartTime, 
		       int *EndMeasurementPixel, UTC_s *EndTime,
		       Bool CONVERT_SUN_2_HOST);

FunctionStatus_e
Config__extractSpectra(Config_s* self, Level_1_Product_s* level_1_product_ptr,
		       Bool convert_sun_2_host,
		       Bool new_name_flag_albedo, Bool write_sun_flag_albedo,
		       Bool sun_extra_file_written, Bool do_write_eps_flag);


FunctionStatus_e
Config__extractGLRInfoShow(FILE *Stream, const Config_s *Config,
			   const SPH1_s *SPH1, ProductType_e ProductType,
			   Bool convert_sun_2_host);

FunctionStatus_e
Config__getPeltierNoise(const Config_s *Config, const FSR1_s *FSR1, 
		     const SPH1_s *SPH1, FCD_s *FCD, Bool convert_sun_2_host);


/*============================================================================*\
**  private operations
\*============================================================================*/
Bool
CheckIfInsideGeolocation(const GeolocationC_s *Geolocation, 
			 const Config_s *Config);

FunctionStatus_e
Config__getXcdFilename(Config_s* self, char* filename);

void
Config__generateOutputFilename(Config_s* self,
			       int current_file,
			       Bool* new_name_flag_albedo_ptr,
			       Bool* write_sun_flag_albedo_ptr,
			       FILE* albedo_file_ptr,
			       char outputFilename[MAX_FILENAME_LENGTH],
			       char open_type[3]);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
