/*----------------------------------------------------------------------------*\
**                              convlv.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function convlv
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)convlv.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"
#include "log.h"


#define	SS_ID	("convlv")

static double sqrarg;
#define SQR(a) (sqrarg=(a),sqrarg*sqrarg)

/*----------------------------------------------------------------------------*\
**                                convlv
**  Input parameters:
**    vector data1[1..n], respns[1..n]
**    short n, m, isign
**  Output parameters:
**    vector ans[1..2*n]
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Convolves or deconvolves a real data set data1[1..n]
**    with a response function respns[1..n]. The response function must
**    be stored in wrap around order in the first m elements of respns
**    where m is an off short <= n. Wrap around order means that the
**    first half of the array respons contains the impulse response
**    function at positive times, while the second half of the array
**    contains the impulse response function at negative times, coun-
**    ting down from the highest element respns[m]. On input isign is
**    +1 for convolution, -1 for deconvolution. The answer is returned
**    in the first n componenets of ans. Howere, ans must be supplied
**    in the calling program with dimensions [1..2*n], for consistency
**    with twofft.. n MUST be an shor power of 2.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    27.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void convlv(double data[], short n, double respns[], short m, short isign, double ans[], FunctionStatus_e * Status)
{
	short i,no2;
	double dum,mag2,*fft,*vector();

	*Status = SUCCESS;
	fft=vector__new(1,2*n);
	for (i=1;i<=(m-1)/2;i++)
		respns[n+1-i]=respns[m+1-i];
	for (i=(m+3)/2;i<=n-(m-1)/2;i++)
		respns[i]=0.0;
	twofft(data,respns,fft,ans,n);
	no2=n/2;
	for (i=2;i<=n+2;i+=2) {
		if (isign == 1) {
			ans[i-1]=(fft[i-1]*(dum=ans[i-1])-fft[i]*ans[i])/no2;
			ans[i]=(fft[i]*dum+fft[i-1]*ans[i])/no2;
		} else if (isign == -1) {
			if ((mag2=SQR(ans[i-1])+SQR(ans[i])) == 0.0) {
				Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_69", ERROR, LOG_NUMRECIP, "Deconvolving at response zero", "");
				*Status = ERROR;
				return;
			}
			ans[i-1]=(fft[i-1]*(dum=ans[i-1])+fft[i]*ans[i])/mag2/no2;
			ans[i]=(fft[i]*dum-fft[i-1]*ans[i])/mag2/no2;
		} else {
			Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_76", ERROR, LOG_NUMRECIP, "No meaning for ISIGN", "");
			*Status = ERROR;
			return;
		}
	}
	ans[2]=ans[n+1];
	realft(ans,no2,-1);
	vector__delete(fft,1,2*n);
}

#undef SQR
