/*----------------------------------------------------------------------------*\
**                               dataconv.c
**
**  GDP Level 0 to 1 functions to convert different data formats to the right
**  format of the running machine.
**
**  Version:	4.1
**  Date:		29 Jul 1996
**  Contents:
**    This file contains the following library functions:
**
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)dataconv.c	4.1 29 Jul 1996, Copyright DLR 1994"

#include <stdio.h>
#ifndef __MSDOS__
#include <unistd.h>
#endif
#include <limits.h>
#include <math.h>

#include "dataconv.h"


#define	SS_Id	("dataconv")

/*----------------------------------------------------------------------------*\
**                                SunShort2Short
**  Input parameters:
**    Sunshort		byte array consisting of 2 bytes which represent a
**			Short value in SUN memory representation.
**  Output parameters:
**    SunShort2Short	Short value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a Short value which has the Short representation
**	of the running system. The calculation attempts a short value with 
**	SUN memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
short
SunShort2Short( const unsigned char* buffer)
{
  return buffer[ 0] * 0x100 + buffer[ 1];	
}

/*----------------------------------------------------------------------------*\
**                                DecShort2Short
**  Input parameters:
**    Decshort		byte array consisting of 2 bytes which represent a
**			Short value in DEC memory representation.
**  Output parameters:
**    DecShort2Short	Short value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a Short value which has the Short representation
**	of the running system. The calculation attempts a short value with 
**	DEC memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
short
DecShort2Short( const unsigned char* buffer)
{
  return (unsigned short)buffer[ 1] * (unsigned short)0x100 +
	(unsigned short)buffer[ 0];
}

/*----------------------------------------------------------------------------*\
**                                Short2SunShort
**  Input parameters:
**    short		short in memory representation of running system
**			
**  Output parameters:
**    Short2SunShort	Short value in form of Sun representation
**
**  Other interfaces:
**  Description:
**	Function to convert a Short value which has the Short representation
**	of the running system into  a short value with 
**	Sun memory representation.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
short
Short2SunShort( unsigned short S_Value)
{
  unsigned short	ShortValue;
  unsigned char*	U_Char_pt;

  U_Char_pt = (unsigned char*)&ShortValue;
  *U_Char_pt = (unsigned char)((unsigned short)S_Value / (unsigned short)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((unsigned short)S_Value % (unsigned short)0x100);

  return ShortValue;
}

/*----------------------------------------------------------------------------*\
**                                Short2DecShort
**  Input parameters:
**    short		short in memory representation of running system
**			
**  Output parameters:
**    Short2DecShort	Short value in form of DEC representation
**
**  Other interfaces:
**  Description:
**	Function to convert a Short value which has the Short representation
**	of the running system into  a short value with 
**	DEC memory representation.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
short
Short2DecShort( unsigned short S_Value)
{
  unsigned short	ShortValue;
  unsigned char*	U_Char_pt;

  U_Char_pt = (unsigned char*)&ShortValue;
  *U_Char_pt = (unsigned char)((unsigned short)S_Value % (unsigned short)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((unsigned short)S_Value / (unsigned short)0x100);
  return ShortValue;
}

/*----------------------------------------------------------------------------*\
**                                SunLong2Long
**  Input parameters:
**    Sunlong		byte array consisting of 2 bytes which represent a
**			Long value in SUN memory representation.
**  Output parameters:
**    SunLong2Long	Long value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a Long value which has the Long representation
**	of the running system. The calculation attempts a long value with 
**	SUN memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
long
SunLong2Long( const unsigned char* buffer)
{
  return (long)((unsigned long)buffer[ 0] * (unsigned long)0x1000000L +
	(unsigned long)buffer[ 1] * (unsigned long)0x10000L +
	(unsigned long)buffer[ 2] * (unsigned long)0x100 +
	(unsigned long)buffer[ 3]);
}

/*----------------------------------------------------------------------------*\
**                                Long2SunLong
**  Input parameters:
**    long		long in memory representation of running system
**			
**  Output parameters:
**    Long2SunLong	long value in form of DEC representation
**
**  Other interfaces:
**  Description:
**	Function to convert a long value which has the long representation
**	of the running system into  a long value with 
**	DEC memory representation.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
long
Long2SunLong( unsigned long L_Value)
{
  unsigned long		LongValue;
  unsigned char*	U_Char_pt;

  U_Char_pt = (unsigned char*)&LongValue;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x1000000L) %
			(unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x10000L) %
			(unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x100) % 
			(unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)(L_Value % (unsigned long)0x100);

  return LongValue;
}


/*----------------------------------------------------------------------------*\
**                                DecLong2Long
**  Input parameters:
**    Declong		byte array consisting of 2 bytes which represent a
**			Long value in DEC memory representation.
**  Output parameters:
**    DecLong2Long	Long value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a Long value which has the Long representation
**	of the running system. The calculation attempts a long value with 
**	DEC memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
long
DecLong2Long( const unsigned char* buffer)
{
  return (long)((unsigned long)buffer[ 3] * (unsigned long)0x1000000L +
	(unsigned long)buffer[ 2] * (unsigned long)0x10000L +
	(unsigned long)buffer[ 1] * (unsigned long)0x100 +
	(unsigned long)buffer[ 0]);
}

/*----------------------------------------------------------------------------*\
**                                Long2DecLong
**  Input parameters:
**    long		long in memory representation of running system
**			
**  Output parameters:
**    Long2DecLong	long value in form of DEC representation
**
**  Other interfaces:
**  Description:
**	Function to convert a long value which has the long representation
**	of the running system into  a long value with 
**	DEC memory representation.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
long
Long2DecLong( unsigned long L_Value)
{
  unsigned long		LongValue;
  unsigned char*	U_Char_pt;

  U_Char_pt = (unsigned char*)&LongValue;
  *U_Char_pt = (unsigned char)(L_Value % (unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x100) % 
			(unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x10000L) %
			(unsigned long)0x100);
  U_Char_pt++;
  *U_Char_pt = (unsigned char)((L_Value / (unsigned long)0x1000000L) %
			(unsigned long)0x100);
  return LongValue;
}

/*----------------------------------------------------------------------------*\
**                                SunFloat2Float
**  Input parameters:
**    Sunfloat		byte array consisting of 4 bytes which represent a
**			float value in SUN memory representation according the
**			"ANSI IEEE 754-1985" standard.
**  Output parameters:
**    SunFloat2Float	float value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a float value which has the float representation
**	of the running system. The calculation attempts an "ANSI IEEE 754-1985"
**	standard float value with SUN memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
**  Updated:    17.04.2000 Bernd Aberle, DLR/IMF, case exponent ge 128 considered
\*----------------------------------------------------------------------------*/
float
SunFloat2Float( const unsigned char* buffer)
{
  int		sign;
  int		exponent;
  float 	fraction;
  char*		buf;
  float		val;

  buf = ( char*)&val;
  sign  = ((int)buffer[ 0] & 0x80)/0x80;

  exponent = (((int)buffer[ 0] & 0x7f) * 2) + ((int)buffer[ 1] & 0x80)/0x80 - 127;

  if (exponent >= 128) exponent = 127; /* shrink to maximum possible */

  fraction = (float)buffer[ 3] / (float)0x800000L +
			(float)buffer[ 2] / (float)0x8000 +
			(float)((int)buffer[ 1] & 0x7f) / (float)0x80;

  val = (float)pow( (double)2, (double)exponent) *
		 (float) pow( (double)-1.0, (double)sign);

  val = val * (1.0 + fraction);
  return val;
}

/*----------------------------------------------------------------------------*\
**                                SunDouble2Double
**  Input parameters:
**    Sundouble		byte array consisting of 8 bytes which represent a
**			double value in SUN memory representation according the
**			"ANSI IEEE 754-1985" standard.
**  Output parameters:
**    SunDouble2Double	Double value in form of the running system
**
**  Other interfaces:
**  Description:
**	Function to calculate a double value which has the double representation
**	of the running system. The calculation attempts an "ANSI IEEE 754-1985"
**	standard double value with SUN memory representation as input.
**
**  References:
**  Libraries:
**  Created:    22.7.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
double
SunDouble2Double( const unsigned char* buffer)
{
  int		sign;
  int		exponent;
  double	fraction;

  sign  = ((int)buffer[ 0] & 0x80)/0x80;
  exponent = (((int)buffer[ 0] & 0x7f) * 0x10) + ((int)buffer[ 1] & 0xf0) /
							0x10 - 1023;
  fraction = (double)buffer[ 7] / (double)0x10000000L/(double)0x1000000L +
		(double)buffer[ 6] / (double)0x10000000L/(double)0x10000L +
		(double)buffer[ 5] / (double)0x10000000L/(double)0x100 +
		(double)buffer[ 4] / (double)0x10000000L +
		(double)buffer[ 3] / (double)0x100000L +
		(double)buffer[ 2] / (double)0x1000 +
		(double)((int)buffer[ 1] & 0x0f) / (double)0x10;
  return (( (double)1.0 + fraction) *
		 pow( (double)2.0, (double)exponent)) *
		 pow( (double)-1.0, (double)sign);
}
