/*----------------------------------------------------------------------------*\
**                                debug.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)debug.c	5.1 02/26/99, Copyright DLR 1998"

#include <string.h>
#include "debug.h"
#include "utc.h"
#include "log.h"


#define	SccsId	("@(#)debug.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("gdp01_ex")

/*============================================================================*\
**  static debug settings
\*============================================================================*/
static Debug_s debug_local;

/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                               Debug__new
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Debug_s*
Debug__new()
{
/*
#ifdef __cplusplus
    Debug_s* self = new Debug_s;
#else
    Debug_s* self = (Debug_s *) malloc( sizeof(Debug_s) );
#endif
    debug_local.module = 0;
    debug_local.level = 0;
    debug_local.to_standardout = False;
    debug_local.dbg_stream = NULL;
    return self;
*/
    return &debug_local;
}


/*----------------------------------------------------------------------------*\
**                              Debug_delete
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__delete(Debug_s* self)
{
/*
#ifdef __cplusplus
    delete self;
#else
    free(self);
#endif
*/
}


/*----------------------------------------------------------------------------*\
**                             Debug__set
**  Input parameters:
**    NewDbg the debug to be set
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Set the dbg Module, Level and Stdout using NewDbg.
**    NOTE: The codification is done using a 32 bits integer
**
**       210987653210987653210987653210210987653210
**      +------------------------------------------+
**      |S MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM LLLL|
**      +------------------------------------------+
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    21.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Debug__set(Debug_s* self, int NewDbg)
{
    if (NewDbg < 0)
    {
	debug_local.to_standardout = True;
	NewDbg = -NewDbg;
    }
    debug_local.module = NewDbg & 2147483640;	/* 0x7FFFFFF8 */
    debug_local.level = NewDbg & 7;
}


/*----------------------------------------------------------------------------*\
**                                Debug__open
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__open(Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	    const char* filename_ptr)
{
    char log_buffer[LOG_FIELD_SIZE];

    if ( (debug_local.dbg_stream == NULL) 
	 && (Debug__isActive(self, module, level)))
    {
	if (debug_local.to_standardout == True) 
	{
	    debug_local.dbg_stream = stdout;
	}
	else 
	{
	    if ((debug_local.dbg_stream = fopen(filename_ptr, "w")) == NULL) 
	    {
		debug_local.dbg_stream = stdout;
		sprintf(log_buffer, "debug file '%s'", filename_ptr);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_199", INFO,
			 LOG_CANNOT_OPEN, log_buffer, "permissions");
	    }
	}
    }
}


/*----------------------------------------------------------------------------*\
**                                Debug__append
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	31.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__append(Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	      const char* filename_ptr)
{
    char log_buffer[LOG_FIELD_SIZE];

    if ( (debug_local.dbg_stream == NULL) 
	 && (Debug__isActive(self, module, level)))
    {
	if (debug_local.to_standardout == True) 
	{
	    debug_local.dbg_stream = stdout;
	}
	else
	{
	    if ((debug_local.dbg_stream = fopen(filename_ptr, "a")) == NULL) 
	    {
		debug_local.dbg_stream = stdout;
		sprintf(log_buffer, "debug file '%s'", filename_ptr);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_253", INFO,
			 LOG_CANNOT_OPEN, log_buffer, "permissions");
	    }
	}
    }
}


/*----------------------------------------------------------------------------*\
**                             Debug__close
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__close(Debug_s* self)
{
    char log_buffer[LOG_FIELD_SIZE];

    if ( (debug_local.dbg_stream != stdout) && (debug_local.dbg_stream != NULL)) {
	if (fclose(debug_local.dbg_stream) != 0) {
	    sprintf(log_buffer, "debug file ");
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__296", INFO, 
		     LOG_CANNOT_CLOSE, log_buffer, "");
	}
	debug_local.dbg_stream = NULL;
    }
}


/*----------------------------------------------------------------------------*\
**                           Debug__getStream
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FILE*
Debug__getStream(const Debug_s* self, DbgModule_e module, DbgLevel_e level)
{
    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	return debug_local.dbg_stream;
    }
    else
    {
	return NULL;
    }
}


/*----------------------------------------------------------------------------*\
**                               Debug__print
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__print(const Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	     const char* message_ptr)
{
    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	fprintf(debug_local.dbg_stream, message_ptr);
    }
}


/*----------------------------------------------------------------------------*\
**                               Debug__println
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__println(const Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	       const char* message_ptr)
{
    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	fprintf(debug_local.dbg_stream, message_ptr);
	fprintf(debug_local.dbg_stream, "\n");
    }
}


/*----------------------------------------------------------------------------*\
**                      Debug__printTable / -Double / -Float
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	31.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Debug__printTable(const Debug_s* self, DbgModule_e module, DbgLevel_e level,
		  const char *Description, const PixelIndex_s *PixelIndex,
		  const float Lambda[OPTICAL_CHANNELS][DETECTOR_PIXELS],
		  const float Detector[OPTICAL_CHANNELS][DETECTOR_PIXELS])
{
    int loop;
    int BandIndx;
    int OpticalChannel;

    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	fprintf(debug_local.dbg_stream, "\n%s\n", Description);
        fprintf(debug_local.dbg_stream, "Band       Index      Lambda     Detector\n");
        fprintf(debug_local.dbg_stream, "[]         []         [nm]       []\n");
       
        for (BandIndx = BLIND_1A; BandIndx <= STRAYLIGHT_1A; BandIndx++)
        {
	    OpticalChannel = Band2Channel((ScienceData_e)BandIndx); 
	    for (loop = PixelIndex[BandIndx].First; 
		 loop <= PixelIndex[BandIndx].Last; loop++)
	    {
		fprintf(debug_local.dbg_stream, "%10d %10d %*.*g %*.*g\n", BandIndx,
			loop + (OpticalChannel * DETECTOR_PIXELS),
			DEBUG_WIDTH, DEBUG_PRECISION,
			Lambda [OpticalChannel] [loop],
			DEBUG_WIDTH, DEBUG_PRECISION,
			Detector [OpticalChannel] [loop]);
	    }
        }
 
        for (BandIndx = BAND_1A; BandIndx <= BAND_1B; BandIndx++)
        {
	    OpticalChannel = Band2Channel((ScienceData_e)BandIndx);
	    for (loop = PixelIndex[BandIndx].First; 
		 loop <= PixelIndex[BandIndx].Last; loop++)
	    {
		fprintf(debug_local.dbg_stream, "%10d %10d %*.*g %*.*g\n",
			BandIndx, loop + (OpticalChannel * DETECTOR_PIXELS),
			DEBUG_WIDTH, DEBUG_PRECISION,
			Lambda [OpticalChannel] [loop],
			DEBUG_WIDTH, DEBUG_PRECISION,
			Detector [OpticalChannel] [loop]);
	    }
        }
 
        for (BandIndx = STRAYLIGHT_1B; BandIndx <= STRAYLIGHT_2A; BandIndx++)
        {
	    OpticalChannel = Band2Channel((ScienceData_e)BandIndx);	    
	    for (loop = PixelIndex[BandIndx].First;
		 loop <= PixelIndex[BandIndx].Last; loop++)
	    {
		fprintf(debug_local.dbg_stream, "%10d %10d %*.*g %*.*g\n",
			BandIndx, loop + (OpticalChannel * DETECTOR_PIXELS),
			DEBUG_WIDTH, DEBUG_PRECISION, 
			Lambda [OpticalChannel] [loop],
			DEBUG_WIDTH, DEBUG_PRECISION,
			Detector [OpticalChannel] [loop]);
	    }
        }
	
        for (BandIndx = BAND_2A; BandIndx <= BAND_4; BandIndx++)
        {
	    OpticalChannel = Band2Channel((ScienceData_e)BandIndx);
	    for (loop = PixelIndex[BandIndx].First; 
		 loop <= PixelIndex[BandIndx].Last; loop++)
	    {
		fprintf(debug_local.dbg_stream, "%10d %10d %*.*g %*.*g\n",
			BandIndx, loop + (OpticalChannel * DETECTOR_PIXELS),
			DEBUG_WIDTH, DEBUG_PRECISION,
			Lambda [OpticalChannel] [loop],
			DEBUG_WIDTH, DEBUG_PRECISION,
			Detector [OpticalChannel] [loop]);
	    }
        }
    }
}


void
Debug__printTableDouble(const Debug_s* self, DbgModule_e module,
			DbgLevel_e level, 
			const char* Description, const double* X_Axis, 
			const double* Y_Data, short Size)
{
    int loop;

    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	fprintf(debug_local.dbg_stream, "\n%s\n", Description);
	fprintf(debug_local.dbg_stream, "Index      X_Axis    Y_Data\n");
	fprintf(debug_local.dbg_stream, "[]         []        []    \n");
	for (loop = 0; loop < Size; loop++)
	{
	    fprintf(debug_local.dbg_stream, "%10d", loop);
	    fprintf(debug_local.dbg_stream, " %*.*g", DEBUG_WIDTH, DEBUG_PRECISION,
		    X_Axis[loop]);
	    fprintf(debug_local.dbg_stream, " %*.*g", DEBUG_WIDTH, DEBUG_PRECISION,
		    Y_Data[loop]);	
	    fprintf(debug_local.dbg_stream, "\n");
	}
    }
}

void
Debug__printTableFloat(const Debug_s* self, DbgModule_e module,
		       DbgLevel_e level, 
		       const char* Description, const float* X_Axis, 
		       const float* Y_Data, short Size)
{
    int loop;

    if (Debug__isActive(self, module, level) && (debug_local.dbg_stream != NULL)) 
    {
	fprintf(debug_local.dbg_stream, "\n%s\n", Description);
	fprintf(debug_local.dbg_stream, "Index      X_Axis    Y_Data\n");
	fprintf(debug_local.dbg_stream, "[]         []        []    \n");
        for (loop = 0; loop < Size; loop++)
        {
	    fprintf(debug_local.dbg_stream, "%10d %*.*g %*.*g",
		    loop,
		    DEBUG_WIDTH, DEBUG_PRECISION, X_Axis[loop],
		    DEBUG_WIDTH, DEBUG_PRECISION, Y_Data[loop]);
	    fprintf(debug_local.dbg_stream, "\n");
        }
 
    }
}


/*----------------------------------------------------------------------------*\
**                             DbgIsActive
**  Input parameters:
**    FuncName the initial part of the debug function name
**    Date a string with date in UTC format
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Returns TRUE if Module is set with at least Level
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    21.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
Debug__isActive(const Debug_s* self, DbgModule_e module, DbgLevel_e level)
{
    return ((debug_local.module & module) && (debug_local.level >= level))? True: False;
}











/*
  ATTENTION:
  The following functions are independant of class Debug_s but left in this 
  file.
  They should be seperated in future.
*/

/*----------------------------------------------------------------------------*\
**                             DbgFileName
**  Input parameters:
**    FuncName the initial part of the debug function name
**    Date a string with date in UTC format
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Appends and returns the FuncName and the UTC Date 
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    06.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
const char*
DbgFileName(const char *FuncName, const char *Date)
{
    static char DbgFileName[MAX_FILENAME_LENGTH];
    char ASCII_Date[ASCII_UTC_TIME_LENGTH];

    strcpy(ASCII_Date, Date);
    ASCII_Date[11] = '_';
    sprintf(DbgFileName, "%s.%s.dbg", FuncName, ASCII_Date);
    return DbgFileName;
}








/*============================================================================*\
**  private operations
\*============================================================================*/

