/*----------------------------------------------------------------------------*\
**                                debug.h
**
**  Class ... declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Debug_h
#define _Debug_h

#pragma ident "@(#)debug.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include <stdio.h>
#include "defs.h"
#include "pixendex.h"


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum { 
    DBG_LEVEL_0 = 0,
    DBG_LEVEL_1,
    DBG_LEVEL_2,
    DBG_LEVEL_3
} DbgLevel_e;

typedef enum { 
    DBG_CALIBRATION = 8,
    DBG_DATAPACKAGE = 16,
    DBG_EXTRACTING = 32,
    DBG_FIXED = 64,
    DBG_GEOLOCATION = 128,
    DBG_INTENSITY = 256,
    DBG_LEAKAGE = 512,
    DBG_LEVEL_O_TAPE = 1024,
    DBG_ORBIT = 2048,
    DBG_ORBIT_PROCESS = 4096,
    DBG_POLARIZATION = 8192,
    DBG_POLARIZATION_ANGLES = 16384,
    DBG_POLARIZATION_APPLY = 32768,
    DBG_POLARIZATION_APPLY_PMD  = 65536,
    DBG_POLARIZATION_OVERLAP = 131072,
    DBG_POLARIZATION_PMD = 262144,
    DBG_POLARIZATION_RESULTS = 524288,
    DBG_RAWDATAPACKAGE = 1048576,
    DBG_RAWDATAPACKAGE_HISTORY = 2097152,
    DBG_SPECTRAL = 4194304,
    DBG_STRAYLIGHT = 8388608,
    DBG_TEST = 16777216,
    DBG_VENI = 33554432,
    DBG__SPARE_1 = 67108864,
    DBG__SPARE_2 = 134217728,
    DBG__SPARE_3 = 268435456,
    DBG__SPARE_4 = 536870912,
    DBG__SPARE_5 = 1073741824
} DbgModule_e;


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	DEBUG_WIDTH	(10)

#define	DEBUG_PRECISION	(5)


#define	DEBUG_LEVEL	(DBG_LEVEL_0)

#define	DEBUG_LEVEL_BDR	(DBG_LEVEL_3)

#define	DEBUG_LEVEL_FCD	(DBG_LEVEL_3)

#define	DEBUG_LEVEL_PCD	(DBG_LEVEL_3)


/*============================================================================*\
**  class Debug
\*============================================================================*/
typedef struct Debug {
    int module;
    int level;
    Bool to_standardout;
    FILE* dbg_stream;
} Debug_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
Debug_s*
Debug__new();


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
Debug__delete(Debug_s* self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/
#ifdef __cplusplus
inline
Bool
Debug__toStandardout(Debug_s* self)
{
    return self->to_standardout;
}
#else
#define \
Debug__toStandardout(self) \
    (self->to_standardout)
#endif


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  private operations
\*============================================================================*/
void
Debug__set(Debug_s* self, int NewDbg);

void
Debug__open(Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	    const char* filename_ptr);

void
Debug__append(Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	      const char* filename_ptr);

FILE*
Debug__getStream(const Debug_s* self, DbgModule_e module, DbgLevel_e level);

void
Debug__print(const Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	     const char* message_ptr);

void
Debug__println(const Debug_s* self, DbgModule_e module, DbgLevel_e level, 
	       const char* message_ptr);

void
Debug__printTable(const Debug_s* self, DbgModule_e module, DbgLevel_e level,
		  const char *Description, const PixelIndex_s *PixelIndex,
		  const float Lambda[OPTICAL_CHANNELS][DETECTOR_PIXELS],
		  const float Detector[OPTICAL_CHANNELS][DETECTOR_PIXELS]);

void
Debug__printTableDouble(const Debug_s* self, DbgModule_e module, 
			DbgLevel_e level, 
			const char *Description, const double* X_Axis, 
			const double* Y_Data, short Size);
void
Debug__printTableFloat(const Debug_s* self, DbgModule_e module, 
		       DbgLevel_e level, 
		       const char *Description, const float*X_Axis, 
		       const float*Y_Data, short Size);

void
Debug__close(Debug_s* self);

const char*
DbgFileName(const char *FuncName, const char *Date);

Bool
Debug__isActive(const Debug_s* self, DbgModule_e module, DbgLevel_e level);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif

