/*----------------------------------------------------------------------------*\
**                                DegradationParameter.c
**
**  Description:
** 
**  Version:    5.5
**  Date:       09/20/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)DegradationParameter.c	5.5 09/20/99, Copyright DLR 1998"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "degreter.h"
#include "log.h"
#include "utils.h"



#define	SccsId	("@(#)DegradationParameter.c 5.5 09/20/99, Copyright DLR 1998")


#define	SS_ID	("DegradationParameter")

static DegradationParameter_s self;
static short n_time_range = 0;
static Bool file_read = False;

static UTC_s JULY_3_1995;

/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                             DegradationParameter__new
**  Input parameters:
**      filename: name of the file to initialize the b[i][j]
**
**  Return value:
**      pointer to the allocated memory for class DegradationParameter,
**      0 if an error occures.
**
**  Description:
**      ....
**
**  Created:	25.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__new(const char filename[])
{
  FunctionStatus_e Status = SUCCESS;
  char line[MAX_FILENAME_LENGTH];
  FILE *deg_stream;
  float xtmp;
  short i_time_range = -1;

  char ascii_time[ASCII_UTC_TIME_LENGTH];

  int atmp, btmp;
  int degree[OPTICAL_CHANNELS][2];
  short iline;
  short icol;
  int dimension;
  
  float ftmp[B_NUMBER_OF_COLUMNS];

  /*------------------------------------------------------------------------*\
  **  if not already read, read the degradation parameter file
  \*------------------------------------------------------------------------*/
  if (file_read == False)
  {

    deg_stream = fopen(filename, "r");

    if (deg_stream == NULL)
    {
      Status = FATAL;
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__80", Status,
        LOG_CANNOT_OPEN, "Degradation parameter file - download the latest version from http://auc.dfd.dlr.de/GOME/", "");
      printf("Degradation parameter file missing - download the latest version from\nhttp://auc.dfd.dlr.de/GOME/\n");
      return Status;
    }

    /* Count number of different versions */
    while (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
      if (strncmp(line, "Time_range", 10) == 0) n_time_range++;

    rewind(deg_stream);

#ifdef __cplusplus
    self.section = new DegradationSection_s[n_time_range];
#else
    self.section = (DegradationSection_s *) calloc( n_time_range, sizeof(DegradationSection_s) );
#endif

    /*------------------------------------------------------------------------*\
    **  read line by line, search keywords, copy data to parameter structure
    \*------------------------------------------------------------------------*/
    while (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
    {
      if (strncmp(line, "Version", 7) == 0)
      {
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n ", &self.version);
        }
      }

      if (strncmp(line, "Time_range", 10) == 0)
      {
        /* get start time and end time */

        i_time_range++;

        self.section[i_time_range].date_from.Msecs = 0;
        self.section[i_time_range].date_to.Msecs = 0;

        strcpy(ascii_time, "");
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          strncpy(ascii_time, line, 11);
          ascii_time[11] = 0;
          strcat(ascii_time, " 12:01:21.445");  /* extent minutes and seconds of reference time */
          ascii_time[ASCII_UTC_TIME_LENGTH - 1] = 0;

          if (!ASCII_2_UTC(ascii_time, &(self.section[i_time_range].date_from)))
          {
            fclose(deg_stream);
            Status = FATAL;
            Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__132", Status,
              LOG_CANNOT_CALCULATE, "start time for degradation parameters", "check times in degradation parameter file");
            return Status;
          }
        }

        strcpy(ascii_time, "");
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          strncpy(ascii_time, line, 11);
          ascii_time[11] = 0;
          strcat(ascii_time, " 12:01:21.445"); /* extent minutes and seconds of reference time */
          ascii_time[ASCII_UTC_TIME_LENGTH - 1] = 0;

          if (!ASCII_2_UTC(ascii_time, &(self.section[i_time_range].date_to)))
          {
            fclose(deg_stream);
            Status = FATAL;
            Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__150", Status,
              LOG_CANNOT_CALCULATE, "end time for degradation parameters", "check times in degradation parameter file");
            return Status;
          }
        }
      }

      if (strncmp(line, "Lambda_0", 8) == 0)
      {
        /* get center wavelength */

        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n ", &xtmp);
          self.section[i_time_range].lambda[0] = xtmp;
        }
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n ", &xtmp);
          self.section[i_time_range].lambda[1] = xtmp;
        }

        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n ", &xtmp);
          self.section[i_time_range].lambda[2] = xtmp;
        }

        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n ", &xtmp);
          self.section[i_time_range].lambda[3] = xtmp;
        }      
      }

      /* get Reference solar measurement */
      if (strncmp(line, "T_0", 3) == 0)
      {
        /*------------------------------------------------------------------------*\
        ** set reference time (probably 03-JUL-1995 12:01:21.445)
        ** JULY_3_1995.Days  = 16619;
        ** JULY_3_1995.Msecs = 43281445;
	\*------------------------------------------------------------------------*/
        JULY_3_1995.Days = 0;
        JULY_3_1995.Msecs = 0;

        strcpy(ascii_time, "");
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          strncpy(ascii_time, line, ASCII_UTC_TIME_LENGTH - 1);
          ascii_time[ASCII_UTC_TIME_LENGTH - 1] = 0;

          if (!ASCII_2_UTC(ascii_time, &JULY_3_1995) )
          {
            fclose(deg_stream);
            Status = FATAL;
            Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__206", Status,
              LOG_CANNOT_CALCULATE, "time of reference solar measurement", "check degradation parameter file");
            return Status;
          }
        }
        /* printf("ascii_time : %d %d\n", JULY_3_1995.Days, JULY_3_1995.Msecs); */
      }

      if (strncmp(line, "Dim", 3) == 0)
      {
        /* get order of polynomes */

        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%d %d", &atmp, &btmp);
          self.section[i_time_range].dimension[OPTICAL_CH_1][0] = atmp;
          self.section[i_time_range].dimension[OPTICAL_CH_1][1] = btmp;
        }
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%d %d", &atmp, &btmp);
          self.section[i_time_range].dimension[OPTICAL_CH_2][0] = atmp;
          self.section[i_time_range].dimension[OPTICAL_CH_2][1] = btmp;
        }
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%d %d", &atmp, &btmp);
          self.section[i_time_range].dimension[OPTICAL_CH_3][0] = atmp;
          self.section[i_time_range].dimension[OPTICAL_CH_3][1] = btmp;
        }
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%d %d", &atmp, &btmp);
          self.section[i_time_range].dimension[OPTICAL_CH_4][0] = atmp;
          self.section[i_time_range].dimension[OPTICAL_CH_4][1] = btmp;
        }
      }

      /* get parameters p_ij */

      if (strncmp(line, "Channel_1", 9) == 0)
      {
        for (iline = 0; iline < self.section[i_time_range].dimension[OPTICAL_CH_1][1]; iline++)
        {
          if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
          {
            dimension = self.section[i_time_range].dimension[OPTICAL_CH_1][0];
            
            if (count_words_in_string(line) != dimension)
            {
              Status = FATAL;
              Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__388", Status,
                LOG_CANNOT_OPEN, "Degradation parameter file inconsistent - Dimension of matrix and Channel_1 don't match", "");
              return Status;
            }
            sscanf (line, "%f %f %f %f %f %f %f %f %f %f",
              &(ftmp[0]), &(ftmp[1]), &(ftmp[2]), &(ftmp[3]), &(ftmp[4]),
              &(ftmp[5]), &(ftmp[6]), &(ftmp[7]), &(ftmp[8]), &(ftmp[9]) );
  
            for (icol = 0; icol < self.section[i_time_range].dimension[OPTICAL_CH_1][0]; icol++)
            {
              self.section[i_time_range].b[OPTICAL_CH_1][iline][icol] = ftmp[icol];
            }
          }
        }
      }
      if (strncmp(line, "Channel_2", 9) == 0)
      {
        for (iline = 0; iline < self.section[i_time_range].dimension[OPTICAL_CH_2][1]; iline++)
        {
          if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
          {
            dimension = self.section[i_time_range].dimension[OPTICAL_CH_2][0];
            
            if (count_words_in_string(line) != dimension)
            {
              Status = FATAL;
              Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__388", Status,
                LOG_CANNOT_OPEN, "Degradation parameter file inconsistent - Dimension of matrix and Channel_2 don't match", "");
              return Status;
            }
            sscanf (line, "%f %f %f %f %f %f %f %f %f %f",
              &(ftmp[0]), &(ftmp[1]), &(ftmp[2]), &(ftmp[3]), &(ftmp[4]),
              &(ftmp[5]), &(ftmp[6]), &(ftmp[7]), &(ftmp[8]), &(ftmp[9]) );
  
            for (icol = 0; icol < dimension; icol++)
            {
              self.section[i_time_range].b[OPTICAL_CH_2][iline][icol] = ftmp[icol];
            }
          }
        }
      }
      if (strncmp(line, "Channel_3", 9) == 0)
      {
        for (iline = 0; iline < self.section[i_time_range].dimension[OPTICAL_CH_3][1]; iline++)
        {
          if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
          {
            dimension = self.section[i_time_range].dimension[OPTICAL_CH_3][0];
            
            if (count_words_in_string(line) != dimension)
            {
              Status = FATAL;
              Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__388", Status,
                LOG_CANNOT_OPEN, "Degradation parameter file inconsistent - Dimension of matrix and Channel_3 don't match", "");
              return Status;
            }
            sscanf (line, "%f %f %f %f %f %f %f %f %f %f",
              &(ftmp[0]), &(ftmp[1]), &(ftmp[2]), &(ftmp[3]), &(ftmp[4]),
              &(ftmp[5]), &(ftmp[6]), &(ftmp[7]), &(ftmp[8]), &(ftmp[9]) );
  
            for (icol = 0; icol < dimension; icol++)
            {
              self.section[i_time_range].b[OPTICAL_CH_3][iline][icol] = ftmp[icol];
            }
          }
        }
      }
      if (strncmp(line, "Channel_4", 9) == 0)
      {
        for (iline = 0; iline < self.section[i_time_range].dimension[OPTICAL_CH_4][1]; iline++)
        {
          if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
          {
            dimension = self.section[i_time_range].dimension[OPTICAL_CH_4][0];
            
            if (count_words_in_string(line) != dimension)
            {
              Status = FATAL;
              Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__388", Status,
                LOG_CANNOT_OPEN, "Degradation parameter file inconsistent - Dimension of matrix and Channel_4 don't match", "");
              return Status;
            }
            sscanf (line, "%f %f %f %f %f %f %f %f %f %f",
              &(ftmp[0]), &(ftmp[1]), &(ftmp[2]), &(ftmp[3]), &(ftmp[4]),
              &(ftmp[5]), &(ftmp[6]), &(ftmp[7]), &(ftmp[8]), &(ftmp[9]) );
  
            for (icol = 0; icol < dimension; icol++)
            {
              self.section[i_time_range].b[OPTICAL_CH_4][iline][icol] = ftmp[icol];
            }
          }
        }
      }
    }
  
    fclose(deg_stream);
  
    file_read = True;
  }
    
  return Status;
}


/*----------------------------------------------------------------------------*\
**                        DegradationParameter__delete
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
DegradationParameter__delete()
{
#ifdef __cplusplus
  delete [] self.section;
#else
  free(self.section);
#endif

  file_read = False;
  n_time_range = 0;
  self.section = NULL;
}


/*----------------------------------------------------------------------------*\
**                             DegradationParameter__ReadVersion
**  Input parameters:
**      filename: name of the file containing parameters and version
**
**  Output parameters:
**
**  Return value:
**
**  Description:
**      Just read the version in the degradation parameter file
**
**  Created:	25.6.99
**
**  Author:	Bernd Aberle, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__ReadVersion(const char filename[])
{
  FunctionStatus_e Status = SUCCESS;
  char line[MAX_FILENAME_LENGTH];
  FILE *deg_stream;

  deg_stream = fopen(filename, "r");

  if (deg_stream == NULL)
  {
    Status = FATAL;
    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__388", Status,
      LOG_CANNOT_OPEN, "Degradation parameter file - download the latest version from http://auc.dfd.dlr.de/GOME/", "");
    printf("Degradation parameter file missing - download the latest version from\nhttp://auc.dfd.dlr.de/GOME/\n");
    return Status;
  }

  /*------------------------------------------------------------------------*\
  **  read line by line, search keyword, copy data to parameter structure
  \*------------------------------------------------------------------------*/
  while (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
  {
    if (strncmp(line, "Version", 7) == 0)
    {
      if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
      {
        sscanf (line, "%f\n ", &self.version);
        break;
      }
    }
  }
 
  fclose(deg_stream);
  return Status;
}


/*----------------------------------------------------------------------------*\
**                             DegradationParameter__GetVersion
**  Input parameters:
**
**  Output parameters:
**      version: as float
**
**  Return value:
**
**  Description:
**      Just read the version in the degradation parameter file
**
**  Created:	25.6.99
**
**  Author:	Bernd Aberle, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__GetVersion(float* version)
{
  FunctionStatus_e Status = SUCCESS;
  *version = self.version;
  return Status;
}


/*----------------------------------------------------------------------------*\
**                         DegradationParameter__getP
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	27.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__getP(OpticalChannel_e channel, double wavelength, 
			   const UTC_s* utc_ptr,
			   double* p)
{
    FunctionStatus_e Status = SUCCESS;
    double a[B_NUMBER_OF_LINES];
    int i_time_range = 0;
    short ncolumns;
    short nlines;
    int i;
    int j;
    UTC_s utc_difference;
    double time_difference;
    double lambda_difference;
    
    /*------------------------------------------------------------------------*\
    **  search for the right parameter version, depending on the date
    \*------------------------------------------------------------------------*/
    while (i_time_range < n_time_range)
    {
      if ( (UTC_Comp('<', self.section[i_time_range].date_from, *utc_ptr) == True) &&
		(UTC_Comp('>', self.section[i_time_range].date_to,   *utc_ptr) == True) )
        break;
      else
        i_time_range++;
    }

    if (i_time_range >= n_time_range)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__498", Status,
		 LOG_NUM, "No degradation parameters found for given time", "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    **  compute the difference in time between july 3 1995 and the date of
    **  the spectrum
    \*------------------------------------------------------------------------*/
    if (UTC_Sub(*utc_ptr, JULY_3_1995, &utc_difference) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__510", Status,
		 LOG_NUM, "Cannot compute time difference t - t0", "");
	return Status;
    }

    /* convert UTC to one single double number in days for processing */

    time_difference = utc_difference.Days 
	+ (double)utc_difference.Msecs / (double)MSECS_PER_DAY;

    /*------------------------------------------------------------------------*\
    **  compute a_0, a_1, a_2
    \*------------------------------------------------------------------------*/
    ncolumns = self.section[i_time_range].dimension[channel][0];
    nlines   = self.section[i_time_range].dimension[channel][1];

    for (i=0; i < nlines; i++)
    {
	a[i] = 0.0;
	for (j=0; j < ncolumns; j++)
	{
	    a[i] += self.section[i_time_range].b[channel][i][j]
		* pow(time_difference, (double)j);
	}
    }

    /*------------------------------------------------------------------------*\
    **  compute P_Degradation
    \*------------------------------------------------------------------------*/
    *p = 0.0;
    lambda_difference = wavelength - self.section[i_time_range].lambda[channel];
    for (i=0; i < nlines; i++)
    {
	*p += a[i] * pow(lambda_difference, (double)i);
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                DegradationParameter__Apply
**  Input parameters:
**      reticon_ptr:  Reticon data (1024)
**      lambda_grid_ptr: Lambda grid for Reticon data (1024)
**
**  Output parameters:
**      reticon_ptr
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Do degradation correction for quality improvement.
**
**  References:
**      ask Ernst Hegels (DLR-DFD/AV)
**
**
**  Created:	24.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__Apply(const float LambdaGrid[], float *Reticon,
	const UTC_s* utc_ptr, const ScienceData_e Band, short Begin, short End)
{
	FunctionStatus_e Status = SUCCESS;
	int j;
	double p;

	OpticalChannel_e channel = Band2Channel( Band);

	for (j = Begin; j <= End; j++)
	{
		if (DegradationParameter__getP(channel, LambdaGrid[j], utc_ptr, &p) != SUCCESS)
		{
		    Status = FATAL;
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "1.5__2207", Status,
			     LOG_NUM, "cannot get P_Degradation", "");

		    return Status;
		}

		Reticon[j] /= p;
	}
    
	return SUCCESS;
}

/*============================================================================*\
**  private operations
\*============================================================================*/

 








