/*----------------------------------------------------------------------------*\
**                                DegradationParameter.h
**
**  Class ... declaration
** 
**  Version:    5.3
**  Date:       01/22/04
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _DEGRADATIONPARAMETER_h
#define _DEGRADATIONPARAMETER_h

#pragma ident "@(#)DegradationParameter.h	5.3 01/22/04, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sciedata.h"
#include "utc.h"

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	B_NUMBER_OF_LINES	(7)

#define	B_NUMBER_OF_COLUMNS	(10)

/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/

/*============================================================================*\
**  class DegradationParameter
\*============================================================================*/
typedef struct DegradationSection_s
{
    UTC_s date_from;
    UTC_s date_to;
    double lambda[OPTICAL_CHANNELS];
    short dimension[OPTICAL_CHANNELS][2];
    double b[OPTICAL_CHANNELS][B_NUMBER_OF_LINES][B_NUMBER_OF_COLUMNS];
}
DegradationSection_s;

typedef struct DegradationParameter_s
{
	float version; /* Degradation parameter file version */
	DegradationSection_s* section;
}
DegradationParameter_s;

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__new(const char filename[]);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
DegradationParameter__delete();

/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DegradationParameter__ReadVersion(const char filename[]);

FunctionStatus_e
DegradationParameter__GetVersion(float* version);

FunctionStatus_e
DegradationParameter__getP(OpticalChannel_e channel, double wavelength, 
			   const UTC_s* utc_ptr, double* p);

FunctionStatus_e
DegradationParameter__Apply(const float LambdaGrid[], float *Reticon,
  const UTC_s* utc_ptr, const ScienceData_e Band, short Begin, short End);

/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
