/*----------------------------------------------------------------------------*\
**                                EDR.h
**
**  Class EDR declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _EDR_h
#define _EDR_h 

#pragma ident "@(#)EDR.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "pixelpar.h"
#include "dr.h"
#include "config.h"
#include "fcd.h"
#include "grid.h"
#include "fsr1.h"
#include "sph1.h"
#include "degreter.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	NE	(937.0)

#define	MIN_VAL	(1.0e-20)

#define	WAVELENGTH_OVERL_3_4	(600.0)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class EDR
\*============================================================================*/
typedef struct EDR {
    UTC_s BeginTime;
    UTC_s EndTime;
    int	First_PCD;
    int	Last_PCD;
    ProductType_e ProductType;
    short N_PCD_History;
    short N_PCD_HistoryRead;
    PixelPar_s *PCD_History;
    PixelPar_s PixelPar;
    DR_s Spectrum;
} EDR_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__Init(const Config_s* Config, int Current_PCD, EDR_s* EDR);

FunctionStatus_e
EDR__CalculateErrors(const Bool bandFilter[SCIENCE_DATA_DEFINED],
		     const Bool calibrationFilter[CALIBRATION_DEFINED],
		     const FCD_s* FCD, const GRID_s* GRID, EDR_s* EDR,
		     const float Signal[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
		     const float BSDF[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
		     Bool P7_BAND_1A_MeanCalculated,
		     const PolarizationC_s *PolarizationParP7_Mean);

FunctionStatus_e
EDR__calculateErrorsAndFlags(EDR_s* EDR, const GRID_s* GRID, 
			     const Bool bandFilter[SCIENCE_DATA_DEFINED],
			     const Bool calibrationFilter[CALIBRATION_DEFINED],
			     const FCD_s* FCD, 
			     const float ErrCorrRadiance[OPTICAL_CHANNELS][DETECTOR_PIXELS],
			     const float BSDF[OPTICAL_CHANNELS][DETECTOR_PIXELS],
			     Bool P7_BAND_1A_MeanCalculated,
			     const PolarizationC_s *PolarizationParP7_Mean);

FunctionStatus_e
EDR__process(EDR_s *EDR, const GRID_s *GRID,
	     Config_s* Config, const Level_1_Product_s* level_1_product_ptr,
	     const float Irradiance[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	     Bool convert_sun_2_host);

FunctionStatus_e
EDR__write(const EDR_s* self, const GRID_s *GRID,
	   const Config_s *Config, const FCD_s *FCD);

FunctionStatus_e
PolarizationC__CalculatePolParBand1a(PolarizationC_s* PolarizationParP7_Mean,
				     const EDR_s* EDR, float OrigIntgTime);

FunctionStatus_e
EDR__getSpectrumDataLevel_1_Product(EDR_s* self, const Config_s *Config, 
				    const Level_1_Product_s* 
				    level_1_product_ptr,
				    Bool convert_sun_2_host);

FunctionStatus_e
WriteAllSpectrumParameter(const Config_s *Config, const EDR_s *EDR);

FunctionStatus_e
WriteSpectrum(const Config_s *Config, const GRID_s *GRID, const FCD_s *FCD,
	      const EDR_s *EDR);

FunctionStatus_e
EDR__calibratePMD(EDR_s* self,
		  const Bool calibrationFilter[CALIBRATION_DEFINED],
		  const FCD_s *FCD);

FunctionStatus_e
EDR__calibrate(EDR_s* EDR, const GRID_s* GRID,
	       Config_s *Config, const FCD_s *FCD, 
	       float Signal[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
	       const float Irradiance[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
	       float BSDF[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
	       Bool *P7_BAND_1A_MeanCalculated,
	       PolarizationC_s *PolarizationParP7_Mean);


/*============================================================================*\
**  private operations
\*============================================================================*/
FunctionStatus_e
Get_PCD_History(const Config_s *Config, const SPH1_s *SPH1, EDR_s *EDR,
		Bool convert_sun_2_host);

/*------------------------------------------------------------------------*\
**  Normalization of the data to 1 second
\*------------------------------------------------------------------------*/
void
Normalize(float IntegrationTime, float Reticon[], short Begin, short End);

#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
