/*----------------------------------------------------------------------------*\
**                                EngConversion.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)EngConversion.c	5.1 02/26/99, Copyright DLR 1998"

#include "engcsion.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)EngConversion.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("EngConversion")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            EngConversion__Sun2Host
**  Input parameters:
**    EngConversion		in Sun memory representation
**  Output parameters:
**    EngConversion		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the EngConversion struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
EngConversion__Sun2Host( EngConversion_s *Param)
{
    int	i;

    for ( i = 0; i < 2; i++)
	{
	    /* float */
	    Param->Coefficient[ i] = SunFloat2Float( (unsigned char*)
						     &Param->Coefficient[ i]);
	}
}


/*----------------------------------------------------------------------------*\
**                                EngConversion__Read
**  Input parameters:
**	Stream			input stream
**  Output parameters:
**	EngConversion		Engineering Conversion factors
**
**  Other interfaces:
**
**  Description:
**	Function to read all Engineering Conversion entries from the given 
**	input stream into the Engineering Conversion structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EngConversion__read(EngConversion_s* self, FILE* file_ptr, 
		    Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;

    /*----------------------------------------------------------------------*\
    ** read engineering conversion parameter
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread(self->Coefficient, 1,
		       sizeof( self->Coefficient), file_ptr); 
    if (NoOfBytes != ENG_CONVERSION_SIZE)
    {
	char LogBuffer[LOG_FIELD_SIZE];
	
	Status = FATAL;
	sprintf(LogBuffer, "Conversion Parameter, #read bytes: %d",
		NoOfBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__98", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	EngConversion__Sun2Host(self);
    }

    return SUCCESS;
} 


/*----------------------------------------------------------------------------*\
**                                EngConversion__Write
**  Input parameters:
**	Stream			output stream
**	EngConversion		Engineering Conversion Parameter
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the Engineering Conversion parameter to the given 
**	output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EngConversion__Write( FILE * Stream, const EngConversion_s *EngConversion)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** write engineering conversion parameter
    \*----------------------------------------------------------------------*/
    NoOfBytes += fwrite( EngConversion->Coefficient, 1,
			 sizeof( EngConversion->Coefficient), Stream); 
    if (NoOfBytes != ENG_CONVERSION_SIZE)
	{
	    Status = FATAL;
	    sprintf( LogBuffer, "Conversion Parameter, #written bytes: %d",
		     NoOfBytes);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__150", Status,
		      LOG_CANNOT_WRITE, LogBuffer, "");
	}
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                EngConversion__Show
**  Input parameters:
**	Stream			output stream
**	EngConversion		Engineering Conversion structure
**
**  Output parameters:
**
**  Other interfaces:
**	Engineering Conversion structure written to the output stream
**
**  Description:
**	Function to write the Engineering Conversion structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
EngConversion__Show( FILE * Stream, const EngConversion_s *EngConversion)
{
    fprintf( Stream, "\nEngConversion 0 and 1\n");
    fprintf( Stream, "%f %f\n", EngConversion->Coefficient[ 0],
	     EngConversion->Coefficient[ 1]);
}


/*============================================================================*\
**  private operations
\*============================================================================*/


