/*----------------------------------------------------------------------------*\
**                                EPS.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)EPS.c	5.1 02/26/99, Copyright DLR 1998"

#include "eps.h"
#include "log.h"


#define	SccsId	("@(#)EPS.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("EPS")


extern int EDR_WrittenCounter;


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                EPS__Write
**  Input parameters:
**    Config		Extract configuration structure
**    EDR		Extract description record structure
**
**  Output parameters:
**    EPS__Write	SUCCESS	= ouput ok
**			else	= any error during output
**
**  Other interfaces:
**    output file of Extract Level 1 Product specified in Config
**
**  Description:
**    Function to write the Extracted Product Summary on the
**    Extract Level 1 Product file
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EPS__Write(const Config_s *Config, const EDR_s *EDR, Bool DoUpdate)
{
    FunctionStatus_e Status = SUCCESS;
    unsigned long FirstTimeHour;
    unsigned long FirstTimeMin;
    unsigned long FirstTimeSec;
    unsigned long FirstTimeMSec;
    unsigned long LastTimeHour;
    unsigned long LastTimeMin;
    unsigned long LastTimeSec;
    unsigned long LastTimeMSec;
    UTC_s BeginTime;
    UTC_s UTC_MeasurementPixel;
    int	n_ground_pixels;
    static long EPS_FilePosition = 0;
    
    UTC_MeasurementPixel.Days	= 0;
    UTC_MeasurementPixel.Msecs	= (unsigned long)(GROUND_PIXEL_TIME * 1000.0);
    UTC_Sub( EDR->BeginTime, UTC_MeasurementPixel, &BeginTime);
    FirstTimeHour = BeginTime.Msecs / 3600000;
    FirstTimeMin = (BeginTime.Msecs % 3600000) / 60000;
    FirstTimeSec = ((BeginTime.Msecs % 3600000) % 60000) / 1000;
    FirstTimeMSec = BeginTime.Msecs % 1000;

    LastTimeHour = EDR->EndTime.Msecs / 3600000;
    LastTimeMin = (EDR->EndTime.Msecs % 3600000) / 60000;
    LastTimeSec = ((EDR->EndTime.Msecs % 3600000) % 60000) / 1000;
    LastTimeMSec = EDR->EndTime.Msecs % 1000;

    if (!DoUpdate) 
    {
	EPS_FilePosition = ftell(Config->Extract_1_ProductStream);
	n_ground_pixels = EDR->Last_PCD - EDR->First_PCD + 1 + EDR->N_PCD_HistoryRead;
    }
    else
    {
	fseek (Config->Extract_1_ProductStream, EPS_FilePosition, SEEK_SET);
	n_ground_pixels = EDR_WrittenCounter;
    }

    fprintf(Config->Extract_1_ProductStream,
	    "%s %02d:%02d:%02d.%03d %02d:%02d:%02d.%03d %04d\n",
	    Config->SpectrumString,
	    FirstTimeHour,
	    FirstTimeMin,
	    FirstTimeSec,
	    FirstTimeMSec,
	    LastTimeHour,
	    LastTimeMin,
	    LastTimeSec,
	    LastTimeMSec,
	    n_ground_pixels);
    if (ferror(Config->Extract_1_ProductStream) != 0) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__108", Status, LOG_CANNOT_WRITE,
		  "EPS into exract level 1 file", "");
    }

#ifdef DEBUG
    fflush(Config->Extract_1_ProductStream);
#endif
    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
