/*----------------------------------------------------------------------------*\
**                                ERS_PropagatorState.h
**
**  Class ERS_PropagatorState declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _ERS_PropagatorState_h
#define _ERS_PropagatorState_h 

#pragma ident "@(#)ERS_PropagatorState.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "stattorc.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/
    static const short ERS_PROPAGATOR_STATE_SIZE = STATE_VECTOR_SIZE 
    + 2 * (ATTITUDE_SIZE * DOUBLE_SIZE) + LONG_SIZE + LONG_SIZE + DOUBLE_SIZE 
    + KEPLER_STATE_SIZE * DOUBLE_SIZE;
/* NOTE: ERS_PROPAGATOR_STATE_SIZE doesn't take into account OFFI size */


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class ERS_PropagatorState
\*============================================================================*/
    typedef struct ERS_PropagatorState {
	StateVectorC_s StateVector;
	/* ESOC Supplied State Vector */
	double ATT[ATTITUDE_SIZE];
	/* Combined Effect of AOCS Mispointing, Mispointing and
	   Misalignment of GOME Scan Mirror Axis, where
	   ATT(1) yaw around z
	   ATT(2) pitch around -x
	   ATT(3) roll around -y */
	double DATT[ATTITUDE_SIZE];
	/* AOCS Mispointing Rate, deg/sec, w.r.t. Nominal */
	long IATT;
	/* Attitude Flag, where
	   0 Nominal Yaw Steering Mode (+Local Normal)
	   1 Local Normal Pointing Mode
	   2 S/C Z = Radial, -Y = Within Velocity & Z Plane */
	long StatusInit;
	/* Status Returned from GOME_INIT, where
	   0 Nominal Node Determination   
	   1 No Convergence in Mean Elements Iteration
	   2 No Convergence in True Node Crossing Iteration
	   3 State Vector is further than +/- 5 deg from True Node
	   4 State Vector for an Orbit Outside Nominal ERS
	   5 State Vector for an Orbit Far Outside Nominal ERS */
	double RTIME;
	/* Modified Julian Day (MJD50) at True Ascending Node Crossing */
	double XM[KEPLER_STATE_SIZE];
	/* Reference Mean Kepler State at True Ascending Node Crossing,
	   where
	   XM(1) Semi-major Axis (Km)
	   XM(2) Excentricity
	   XM(3) Inclination (deg)
	   XM(4) Right Asc. of Asc. Node (deg)
	   XM(5) Argument of Perigee (deg)
	   XM(6) Mean anomaly (deg) */
	double OFFI[ANGLES_DEFINED];
	/* Pitch Rotation of Scan Mirror (DEG) at Pixel Begin, Mid and End */
    } ERS_PropagatorState_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*- --------------------------------------------------------------------------*/
    FunctionStatus_e
    ERS_PropagatorState__read(ERS_PropagatorState_s* self, FILE* file_ptr,
			      Bool convert_sun_2_host);

    FunctionStatus_e 
    ERS_PropagatorState__Write(FILE* Stream, const ERS_PropagatorState_s* Param);

    void 
    ERS_PropagatorState__Show(FILE* Stream, const ERS_PropagatorState_s *Param);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
