/*----------------------------------------------------------------------------*\
**                                FCD.h
**
**  Class FCD declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _FCD_h
#define _FCD_h

#pragma ident "@(#)FCD.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "fixedc.h"
#include "fpa.h"
#include "hotpxelc.h"
#include "inteityc.h"
#include "keydrror.h"
#include "strastic.h"
#include "leakagec.h"
#include "keydonl1.h"
#include "specralc.h"
#include "grid.h"
#include "bcr.h"
#include "bsdf.h"
#include "debug.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	FCD_DETECTOR_CONFIDENCE_SIZE	(SHORT_SIZE)

#define	FCD_BCR_SIZE	(SCIENCE_DATA_DEFINED * BCR_SIZE)

#define	FCD_PREFLIGHT_ERRORS_SIZE	(KEYDATA_ERRORS_SIZE)

#define	FCD_BSDF_SIZE	(BSDF_SIZE)

#define	FCD_STRAYLIGHT_CHARACTERISTIC_SIZE	(STRAYLIGHT_CHARACTERISTIC_SIZE)

#define	FCD_FPA_SIZE	(FPA_SIZE)

#define	FCD_NLEAK_SIZE	(SHORT_SIZE)

#define	FCD_LEAKAGE_PAR_SIZE	(LEAKAGE_SIZE)

#define	FCD_IT_PATTERN_PAR_SIZE	(FCD_LEAKAGE_PAR_SIZE)

#define	FCD_FIXED_PAR_SIZE	(FIXED_SIZE)

#define	FCD_NHOT_SIZE	(SHORT_SIZE)

#define	FCD_HOT_PIXEL_OCCURRENCE_SIZE	(HOT_PIXEL_SIZE)

#define	FCD_NSPEC_SIZE	(SHORT_SIZE)

#define	FCD_SPECTRAL_CAL_PAR_SIZE	(SPECTRAL_SIZE)

#define	FCD_INDEX_SPECTRAL_CAL_PAR_SIZE	(SHORT_SIZE)

#define	FCD_INTENSITY_CAL_PAR_SIZE	(INTENSITY_SIZE)
/*

#define	FCD_NSUNREF_SIZE	(SHORT_SIZE)

#define	FCD_SUN_REF_SPECTRUM_MEAS_SIZE	(OPTICAL_CHANNELS * DETECTOR_PIXELS * FLOAT_SIZE)
  */

#define	FCD_NANG_SIZE	(SHORT_SIZE)

#define	PREFLIGHT_CALIBRATION_L1_SIZE	(FLOAT_SIZE * DETECTOR_PIXELS + FLOAT_SIZE * DETECTOR_PIXELS)

#define	FCD_PREFLIGHT_CALIBRATION_L1_SIZE	(PREFLIGHT_CALIBRATION_L1_SIZE)

#define	FCD_STATIC_SIZE	(FCD_DETECTOR_CONFIDENCE_SIZE + FCD_BCR_SIZE + FCD_PREFLIGHT_ERRORS_SIZE + FCD_BSDF_SIZE + FCD_STRAYLIGHT_CHARACTERISTIC_SIZE + FCD_FPA_SIZE + FCD_NLEAK_SIZE + FCD_FIXED_PAR_SIZE + FCD_NHOT_SIZE + FCD_NSPEC_SIZE + FCD_INDEX_SPECTRAL_CAL_PAR_SIZE + FCD_INTENSITY_CAL_PAR_SIZE + FCD_NANG_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class FCD
\*============================================================================*/
typedef struct FCD {
    short DetectorConfFlags ;
    BCR_s BCR[SCIENCE_DATA_DEFINED];
    KeyDataError_s KeyDataError;
    BSDF_s BSDF;
    StraylightCharacteristic_s StraylightCharacteristic;
    FPA_s FPA;
    short Nleak;
    LeakageC_s* LeakagePar;
    FixedC_s FixedPar;
    short Nhot;
    HotPixelC_s* HotPixelPar;
    short Nspec;
    SpectralC_s* SpectralPar;
    short IndexSpectralCalPar;
    IntensityC_s IntensityPar;
    short Nang;
    KeyDataCalibrationL1_s* KeyDataCalibrationL1_Par;
} FCD_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FCD__calculateGrids(FCD_s* self, GRID_s* grid_ptr);

FunctionStatus_e
FCD__read(FCD_s* self, FILE* file_ptr, Bool convert_sun_2_host);

void
FCD__show(const FCD_s* self, FILE* file_ptr, Bool FullInfo);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
