/*----------------------------------------------------------------------------*\
**                                File.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)File.c	5.1 02/26/99, Copyright DLR 1998"

#include <string.h>
#include "file.h"


#define	SccsId	("@(#)File.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("File")


#define	MAX_LINE_LENGTH_1	(81)

#define	MAX_LINE_LENGTH_2	(256)

/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                MoreDataInFile
**  Input parameters:
**    Stream		File stream
**  Output parameters:
**    MoreDataInFile	True	= behind the file pointer are at minimum 
**				  one byte(s)
**    			False	= behind the file pointer is nothing (EOF)
**  Description:
**	Function to test if behind the file pointer data is available.
**      Returns True if there are more Bytes in the file.
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
File__moreData(FILE* self)
{

    fgetc(self);
    if (feof(self) != 0)
    {
	return False;
    }
    else
    {
	fseek(self, -sizeof(char), SEEK_CUR);
	return True;
    }
}


/*----------------------------------------------------------------------------*\
**                                FileSize
**  Input parameters:
**    Stream		File stream
**  Output parameters:
**    F_Size		Size of file in bytes
**    FileSize		SUCCESS	: determination of the file size was possible
**			else	: determination of the file size was not
**					 possible
**  Description:
**	Function to determine the size of the given file.
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
File__getSize(FILE* self, long* filesize_ptr)
{
    FunctionStatus_e Status = SUCCESS;
    long FilePosition;

    *filesize_ptr = -1;
    /*------------------------------------------------------------------------*\
    ** save the file position
    \*------------------------------------------------------------------------*/
    FilePosition = ftell(self);

    /*------------------------------------------------------------------------*\
    ** get the file length
    \*------------------------------------------------------------------------*/
    if (fseek (self, 0L, SEEK_END) != 0)
    {
	Status = FATAL;
    }
    else
    {
	*filesize_ptr = ftell( self);

	/*------------------------------------------------------------------------*\
	** reposition the file pointer
	\*------------------------------------------------------------------------*/
	if (fseek (self, FilePosition, SEEK_SET) != 0)
	{
	    Status = FATAL;
	}
    }
    return Status;
}


/*----------------------------------------------------------------------*\
**				FindString
**  Input:
**	fd		File pointer
**	Name		string to be found
**
**  Output:
**	FindString	SUCCESS if string found
**			ERROR if not found
**
**  Description:
**	Returns true if the string is found. The filepointer is on the
**	beginning of the next line.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__findString(FILE* self, const char *string_ptr)
{
    char Buff[ MAX_LINE_LENGTH_2];
    Bool StringFound = False;
    Bool EOF_OrError = False;

    rewind(self);
    do
    {
	if (fgets(Buff, MAX_LINE_LENGTH_2, self) != NULL)
	{
	    if (strcasecmp(Buff, string_ptr) == 0)
		StringFound = True;
	}
	else
	    EOF_OrError = True;
    }
    while ((StringFound == False) && (EOF_OrError == False));

    if (EOF_OrError == True)
	return ERROR;
    else
	return SUCCESS;
}

/*----------------------------------------------------------------------*\
**				ReadUntilNextLine
**  Input:
**	self			File pointer
**
**  Output:
**	ReadUntilNextLine	SUCCESS =	end of line is detected and
**						file is positioned to the next
**						line.
**				ERROR	=	EOF or error occurred
**
**  Description:
**	Reads the current file until end of line, or EOF, or error. If
**	If end of line is detected the function returns with success.
**	Otherwise ERROR is returned.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__readUntilNextLine(FILE* self)
{
    char Buff[MAX_LINE_LENGTH_1];

    while ( (fgets(Buff, MAX_LINE_LENGTH_1, self) != NULL) &&
	    (strlen( Buff) == (MAX_LINE_LENGTH_1 - 1))) {}
    if ( (feof( self) != 0) || (ferror( self) != 0))
	return ERROR;
    else
	return SUCCESS;
}


/*----------------------------------------------------------------------*\
**				ReadArrayUnsignedShort
**  Input:
**	self			File pointer
**	ArrayLeng		max length of array
**
**  Output:
**	ReturnedArrayLeng	number of read array elements
**	Array			array containing the read elements
**      ReadArrayUnsignedShort	SUCCESS	=	ArrayLeng values read
**				WARN	=	ReturnedArrayLeng < ArrayLeng
**				ERROR	=	Error during read
**
**  Description:
**	Reads a maximum of ArrayLeng values into the Array. Reading is 
**	stopped if input value is not an unsigned short or EOF or i/o error is
**	detected.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__readArrayUnsignedShort(FILE* self, int array_length, 
			     int* returned_array_length_ptr,
			     unsigned short* array_ptr)
{
    unsigned short i_value;

    *returned_array_length_ptr = 0;
    while( (feof(self) == 0) && (ferror(self) == 0) 
	   && (fscanf( self,"%hu", &i_value) == 1) 
	   && ( *returned_array_length_ptr < array_length))
    {
	array_ptr[*returned_array_length_ptr] = i_value;
	(*returned_array_length_ptr)++;
    }

    if (ferror( self) != 0)
	return ERROR;
    else
	if (array_length != *returned_array_length_ptr)
	    return WARN;
	else
	    return SUCCESS;
}


/*----------------------------------------------------------------------*\
**				Readarray_ptrShort
**  Input:
**	self			File pointer
**	array_length		max length of array
**
**  Output:
**	returned_array_length_ptr	number of read array elements
**	array_ptr			array containing the read elements
**      ReadArrayShort		SUCCESS	=	array_length values read
**				WARN	=	returned_array_length_ptr < array_length
**				ERROR	=	Error during read
**
**  Description:
**	Reads a maximum of array_length values into the Array. Reading is 
**	stopped if input value is not an unsigned short or EOF or i/o error is
**	detected.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__readArrayShort(FILE* self, int array_length, 
		     int* returned_array_length_ptr, short* array_ptr)
{
    short i_value;

    *returned_array_length_ptr = 0;
    while((feof(self) == 0) && (ferror(self) == 0) &&
	  ( fscanf( self,"%hd", &i_value) == 1) &&
	  ( *returned_array_length_ptr < array_length))
    {
	array_ptr[ *returned_array_length_ptr] = i_value;
	(*returned_array_length_ptr)++;
    }
    if (ferror( self) != 0)
	return ERROR;
    else
	if (array_length != *returned_array_length_ptr)
	    return WARN;
	else
	    return SUCCESS;
}


/*----------------------------------------------------------------------*\
**				ReadArrayUnsignedLong
**  Input:
**	self			File pointer
**	array_length		max length of array
**
**  Output:
**	returned_array_length_ptr	number of read array elements
**	Array			array containing the read elements
**      ReadArrayUnsignedLong	SUCCESS	=	array_length values read
**				WARN	=	returned_array_length_ptr < array_length
**				ERROR	=	Error during read
**
**  Description:
**	Reads a maximum of array_length values into the Array. Reading is 
**	stopped if input value is not an unsigned long or EOF or i/o error is
**	detected.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__readArrayUnsignedLong(FILE* self, int array_length, 
			     int* returned_array_length_ptr,
			    unsigned long* array_ptr)
{
    unsigned long	i_value;

    *returned_array_length_ptr = 0;
    while((feof( self) == 0) && (ferror( self) == 0) &&
	  ( fscanf( self,"%lu", &i_value) == 1) &&
	  ( *returned_array_length_ptr < array_length))
    {
	array_ptr[ *returned_array_length_ptr] = i_value;
	(*returned_array_length_ptr)++;
    }
    if (ferror( self) != 0)
	return ERROR;
    else
	if (array_length != *returned_array_length_ptr)
	    return WARN;
	else
	    return SUCCESS;
}

/*----------------------------------------------------------------------*\
**				ReadArrayLong
**  Input:
**	self			File pointer
**	array_length		max length of array
**
**  Output:
**	returned_array_length_ptr	number of read array elements
**	Array			array containing the read elements
**      ReadArrayLong		SUCCESS	=	array_length values read
**				WARN	=	returned_array_length_ptr < array_length
**				ERROR	=	Error during read
**
**  Description:
**	Reads a maximum of array_length values into the Array. Reading is 
**	stopped if input value is not an unsigned short or EOF or i/o error is
**	detected.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__readArrayLong(FILE* self, int array_length, 
		    int* returned_array_length_ptr, long* array_ptr)
{
    long	i_value;

    *returned_array_length_ptr = 0;
    while((feof( self) == 0) && (ferror( self) == 0) &&
	  ( fscanf( self,"%ld", &i_value) == 1) &&
	  ( *returned_array_length_ptr < array_length))
    {
	array_ptr[ *returned_array_length_ptr] = i_value;
	(*returned_array_length_ptr)++;
    }
    if (ferror( self) != 0)
	return ERROR;
    else
	if (array_length != *returned_array_length_ptr)
	    return WARN;
	else
	    return SUCCESS;
}


/*----------------------------------------------------------------------*\
**				ReadArrayFloat
**  Input:
**	self			File pointer
**	array_length		max length of array
**
**  Output:
**	returned_array_length_ptr	number of read array elements
**	Array			array containing the read elements
**      ReadArrayFloat		SUCCESS	=	array_length values read
**				WARN	=	returned_array_length_ptr < array_length
**				ERROR	=	Error during read
**
**  Description:
**	Reads a maximum of array_length values into the Array. Reading is 
**	stopped if input value is not an unsigned short or EOF or i/o error is
**	detected.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	29.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------*/
FunctionStatus_e
File__ReadArrayFloat(FILE* self, int array_length, 
		     int* returned_array_length_ptr, float* array_ptr)
{
    float		f_value;

    *returned_array_length_ptr = 0;
    while((feof( self) == 0) && (ferror( self) == 0) &&
	  ( fscanf( self,"%f", &f_value) == 1) &&
	  ( *returned_array_length_ptr < array_length))
    {
	array_ptr[ *returned_array_length_ptr] = f_value;
	(*returned_array_length_ptr)++;
    }
    if (ferror( self) != 0)
	return ERROR;
    else
	if (array_length != *returned_array_length_ptr)
	    return WARN;
	else
	    return SUCCESS;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
