/*----------------------------------------------------------------------------*\
**                                Filter.h
**
**  Class Filter (user filter parameters) declaration
** 
**  Version:    5.5
**  Date:       12/21/01
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Filter_h
#define _Filter_h 

#pragma ident "@(#)Filter.h	5.5 12/21/01, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "utc.h"
#include "coornate.h"
#include "sciedata.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	CALIBRATION_DEFINED	(10)

#define	SUBSET_COUNTER_DEFINED	(4)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum { 
    QF_GOOD = 0,
    QF_DEAD = 1,
    QF_HOT = 2,
    QF_SATURATED = 3,
    QF_OTHER = 9
} Q_Flag_e;

typedef enum { 
    CAL_LEAKAGE = 0,
    CAL_FPA,
    CAL_FIXED,
    CAL_STRAYLIGHT,
    CAL_NORMALIZE,
    CAL_POLARIZATION,
    CAL_INTENSITY,
    CAL_BSDF,
    CAL_UNITS
} Calibration_e;


/*============================================================================*\
**  class Filter
\*============================================================================*/
typedef struct Filter {
    Bool pixelFlag;
    Bool timeFlag;
    Bool regionFlag;
    Bool calFileFlag;
    Bool albedoFlag;
    Bool radianceJumpFlag;
    Bool xCorrelationFlag;
    Bool hdfOutputFlag;		 /* True if output is written in HDF format */
    Bool correctDegradationFlag; /* True if degradation and BSDF should be corrected */
    Bool correct_BSDF_Flag;      /* True if BSDF should be corrected */
    Bool splitFlag;
    Bool noSunFlag;
    Bool infoFlag;
    Bool geolocationInfoFlag;
    Bool qaInfoFlag;
    Bool sunFlag;
    Bool moonFlag;	
    int pixelStart;
    int pixelStop;
    UTC_s timeStart;
    UTC_s timeStop;
    Coordinate_s cornerCoordinate[2];
    Bool bandFilter[SCIENCE_DATA_DEFINED];
    Bool calibrationFilter[CALIBRATION_DEFINED];
    Bool subsetCounterFilter[SUBSET_COUNTER_DEFINED];
    char *inputFilename;
    char *outputFilename;
    char DegradationParameterFile[MAX_FILENAME_LENGTH];
    int qa_increment;
    int qa_pixelnumber;
    int qa_meannumber;
} Filter_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Filter__parseUserParameters(Filter_s *self, ProductType_e* ProductType, 
			    int argc, char* const argv[]);

void
Filter__show(const Filter_s* self, FILE* Stream);

FunctionStatus_e
Filter__writeDLR_Header(FILE* stream);

FunctionStatus_e
Filter__writeExtractHeader(Filter_s* self, FILE* stream, 
			   ProductType_e product_type);

FunctionStatus_e
Filter__getProductType(const Filter_s* self, ProductType_e *ProductType);


/*============================================================================*\
**  private operations
\*============================================================================*/
Bool 
Filter__setYesNoPreferences(Bool Preference[], short N_Pref, const char *Arg);

Bool 
Filter__setCalibration(Filter_s* self, const char *Arg, 
		       ProductType_e ProductType);
void 
Filter__setDefaultValues(Filter_s* self, ProductType_e ProductType);

void 
Filter__printUsage(const char *program_name);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
