/*----------------------------------------------------------------------------*\
**                              fit_akim.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions: 
**    fit_grid_akima
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)fit_akim.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                fit_grid_akima
**  Input parameters:
**    xin: x_axis of the given spectrum
**    yin: values of the given spectrum
**    nin: number of values of the given spectrum
**    xres: x_axis of the desired spectrum
**
**  Output parameters:
**    yres: values of the spectrum fitted to the desired spectrum
**    Begin: begin of values of the desired spectrum, normally 0
**    End: end of values of the desired spectrum, normally DETECTOR_PIXELS-1
**
**  Description:
**    interpolates the values of a given spectrum to another wavelength grid
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	07.4.94
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void fit_grid_akima (const float xin[], const float yin[], int nin, 
		     const float xres[], float yres[], int Begin, int End)
{
    register int loop;
    int nleft = 0;
    int nright = 0;
    short n_source = nin;
    double delta;

    static double	x_arr[MAX_AKIMA_POINTS + AKIMA_EXTRA_POINTS], 
	y_arr[MAX_AKIMA_POINTS + AKIMA_EXTRA_POINTS];
    static double	a_arr[MAX_AKIMA_POINTS], b_arr[MAX_AKIMA_POINTS]; 
    static double	c_arr[MAX_AKIMA_POINTS], d_arr[MAX_AKIMA_POINTS];


    /*------------------------------------------------------------------------*\
    ** copy data because x_arr and y_arr have four points more than input array
    \*------------------------------------------------------------------------*/
    for (loop = 0; loop < n_source; loop++)
	{
	    x_arr[loop] = (double) xin [loop];
	    y_arr[loop] = (double) yin [loop];
	}

    /*------------------------------------------------------------------------*\
    ** calculate polynomes
    \*------------------------------------------------------------------------*/
    akima_su (x_arr, y_arr, n_source, a_arr, b_arr, c_arr, d_arr);

    /*------------------------------------------------------------------------*\
    ** interpolate
    \*------------------------------------------------------------------------*/
    nleft = 0;
    nright = 0;
    n_source --;

    for (loop = Begin; loop <= End; loop++)
	{
	    while (xres[loop] > xin[nright] && nright < n_source) 
		{
		    nright++;
		    nleft = nright - 1;
		}

	    delta = xres[loop] - xin[nleft];

	    yres[loop] = a_arr[nleft] + (b_arr[nleft] * delta) + (c_arr[nleft] * delta*delta) + (d_arr[nleft] * delta*delta*delta);
	}
}
