/*----------------------------------------------------------------------------*\
**                              fit_line.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions:
**    fit_grid_linear
**    LinearInterpolation
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)fit_line.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                fit_grid_linear
**  Input parameters:
**    x_axis of the given spectrum, x_axis_in: float array []
**    values of the given spectrum, y_axis_in: float array []
**    number of values of the given spectrum, n_in: short, mostly DETECTOR_PIXELS
**  Output parameters:
**    x_axis of the desired spectrum: float array [TOTAL_DETECTOR_PIXELS]
**    values of the spectrum fitted to the desired spectrum: float array [DETECTOR_PIXELS]
**    number of values of the desired spectrum, from Begin to End, normally 0 to DETECTOR_PIXELS-1
**  Other interfaces:
**  Description:
**    interpolates the values of a given spectrum to another wavelength grid by linear interpolation
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	07.4.94
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void fit_grid_linear(const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End)
{
	int loop = 0;

	int nleft = 0;
	int nright = 1;

	for (loop = Begin; loop <= End; loop++)
	{
		while (xres[loop] > xin[nright] && nright < nin - 1) 
		{
			nright++;
			nleft = nright - 1;
		}

		yres[loop] = LinearInterpolation( xres[loop], xin[nleft], xin[nright], yin[nleft], yin[nright]);
	}
}

/*----------------------------------------------------------------------------*\
**                          LinearInterpolation
**  Input parameters: X,  X_left,  X_right,  Y_left,  Y_right
**  Output parameters: Y
**  Other interfaces:
**  Description:
**    interpolate between X_left and X_right at abscissa X
**  References:
**  Libraries:
**  Created:	3.2.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float LinearInterpolation(float X, float X_left, float X_right, float Y_left, float Y_right)
{
	float delta;
	float Y;

	if (X_left >= X_right) return Y_right;

	delta = ( Y_left - Y_right ) / ( X_left - X_right );
	Y = Y_left + delta * (X - X_left);

	return Y;
}
