/*----------------------------------------------------------------------------*\
**                              fit_poly.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following functions : 
**    CalculatePolynome
**    CalculatePolynome1
**    fit_grid_polyn
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)fit_poly.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                CalculatePolynome
**  Input parameters:
**    xin : Abscissa, yin : y values, nin : number of points of given function
**  Output parameters:
**    pol_coeff : polynomial coefficients of the curve
**  Other interfaces:
**  Description:
**    calls Numerical Recipes Function svdfit to fit a polynome with order order
**    through the function given by xin and yin
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	11.10.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void CalculatePolynome(const float xin[], const float yin[], short nin, double pol_coeff[], short order)
{
	int loop;
	static const float SIG = 1e-10;  /* Precision error */

	double *X;
	double *Y;
	double *Sig;
	double *A;
	double *W;
	double **U;
	double **V;
	double ChisQ;
	FunctionStatus_e Status;

	X = vector__new(1, nin); 
	Y = vector__new(1, nin);
	Sig = vector__new(1, nin);
	A = vector__new(1, order);
	W = vector__new(1, order);

	U=matrix__new(1, nin, 1, order);
	V=matrix__new(1, order, 1, order);

	for (loop=1; loop<=nin; loop++)
	{
		Sig[loop] = SIG;
		X[loop] = (double) xin[loop - 1];
		Y[loop] = (double) yin[loop - 1];
	}

 	svdfit(X, Y, Sig, nin, A, order, U, V, W, &ChisQ, CalculatePolynome1, &Status);
	matrix__delete(U, 1, nin, 1, order);
	matrix__delete(V, 1, order, 1, order);

	for (loop = 0; loop < order; loop++)
	{
		pol_coeff[loop] = A[loop + 1];
	}

	vector__delete(X,1,nin);
	vector__delete(Y,1,nin);
	vector__delete(Sig,1,nin);
	vector__delete(A,1,order);
	vector__delete(W,1,order);
}


/*----------------------------------------------------------------------------*\
**                                CalculatePolynome1
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    called by svdfit, which is called by CalculatePolynome
**  Description:
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	
**  Author:
\*----------------------------------------------------------------------------*/
void CalculatePolynome1(double X, double P[], short Np)
{
	int j;

	P[1]=1.0;
	for (j=2;j<=Np;j++)
		P[j]=P[j-1]*X;
}

/*----------------------------------------------------------------------------*\
**                                fit_grid_polyn
**  Input parameters:
**    xin : x_axis of the given spectrum : float array []
**    yin : values of the given spectrum : float array []
**    nin : number of values of the given spectrum, n_in: short, mostly DETECTOR_PIXELS
**    Begin, End : range of values of the desired spectrum, from Begin to End, normally 0 to DETECTOR_PIXELS-1
**    order
**  Output parameters:
**    xres, x_axis of the desired spectrum: float array [TOTAL_DETECTOR_PIXELS]
**    yres, values of the spectrum fitted to the desired spectrum: float array [DETECTOR_PIXELS]
**  Other interfaces:
**  Description:
**    fits a polynome
**    of the selected order through the spectrum
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	31.5.95
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void fit_grid_polyn(const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End, short order)
{
	int loop;
	int loop1;
	double pol_coeff[DETECTOR_PIXELS];
	float xin1 [MAX_AKIMA_POINTS + AKIMA_EXTRA_POINTS]; 

	float offset = 500.0;
	float factor = 5000.0;

	double Lambda_fit;
	double Lambda_fit_pow;

	for (loop = 0; loop < nin; loop++)
		xin1[loop] = (xin[loop] - offset) / factor;
			
	for (loop = 0; loop < DETECTOR_PIXELS; loop++) pol_coeff[loop] = 0.0;

	CalculatePolynome(xin1, yin, nin, pol_coeff, order);

	for (loop = Begin; loop <= End; loop++)
	{
		Lambda_fit = (double) (xres[loop] - offset) / factor;

		yres [loop] = pol_coeff[0];
		Lambda_fit_pow = Lambda_fit;

		for (loop1 = 1; loop1 < order; loop1++)
		{
			yres [loop] += Lambda_fit_pow * pol_coeff[loop1];
			Lambda_fit_pow *= Lambda_fit;
		}
	}
}

