/*----------------------------------------------------------------------------*\
**                              four1.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function twofft
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)four1.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

/*----------------------------------------------------------------------------*\
**                                four1
**  Input parameters:
**    vector data[1..2*nm]
**    short nn, isign
**  Output parameters:
**    Vector data[1..2*nm]
**  Other interfaces: none
**  Description:
**    Replaces data by its discrete Fourier transform, if isign is
**    input as 1; or replaces data by nn times its inverse discrete
**    Fourier transform, if isign is input as -1. data is a complex
**    array of length nm, input as a real array [1..2*nm].
**    nm MUST be an short power of 2.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    27.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void four1(double data[], short nn,short isign)
{
	short n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	double tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=2*mmax;
		theta=6.28318530717959/(isign*mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}

#undef SWAP
