/*----------------------------------------------------------------------------*\
**                                FPA.h
**
**  Class FPA declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _FPA_h
#define _FPA_h 

#pragma ident "@(#)FPA.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "fsr1.h"
#include "sph1.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	FPA_SCALE_FACTORS	(5)

#define	FPA_FILTER_COEFFICIENTS	(100)

#define	FPA_SIZE	(FLOAT_SIZE * FPA_SCALE_FACTORS + SHORT_SIZE + FLOAT_SIZE * FPA_FILTER_COEFFICIENTS)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum { 
    FPA_SCALE_12,
    FPA_SCALE_24,
    FPA_SCALE_30,
    FPA_SCALE_48,
    FPA_SCALE_60
} FPA_Scale_e;


/*============================================================================*\
**  class ...
\*============================================================================*/
typedef struct FPA {
    float ScaleFactor[FPA_SCALE_FACTORS]; /* FPA Scale factors */
    short CoefficientsUsed;	/* Number of filter coefficients used */
    float FilterCoefficient[FPA_FILTER_COEFFICIENTS]; /* Filter coefficients */
    short NoiseSize; 		/* Size of the computed Noise */
    float *Noise;		/* Peltier Noise (averaged Peltier output 
				   trend corrected) */
} FPA_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
void
FPA__Compute(FPA_s* Self, float PeltierData[], short PeltierDataSize);

void 
FPA__Apply(const FPA_s* Self, short GroundPixel, float IntgTime,
	       float Reticon[], short Begin, short End, const char* PixelDate);

FunctionStatus_e
FPA__read(FPA_s* self, FILE* Stream, Bool convert_sun_2_host);

FunctionStatus_e
FPA__write(const FPA_s* self, FILE* Stream);

void
FPA__show(const FPA_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
