/*----------------------------------------------------------------------------*\
**                                FSR1.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)FSR1.c	5.1 02/26/99, Copyright DLR 1998"

#include "fsr1.h"
#include "dataconv.h"
#include "log.h"
#include "debug.h"
#include "pir.h"


#define	SccsId	("@(#)FSR1.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("FSR1")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            FSR1__Sun2Host
**  Input parameters:
**    FSR1			in Sun memory representation
**  Output parameters:
**    FSR1			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the  FSR1 struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FSR1__Sun2Host( FSR1_s *Param)
{
    int	i;

    /* short */
    Param->NoOfHeaderRec = SunShort2Short( (unsigned char*)
					   &Param->NoOfHeaderRec);
    /* long */
    Param->HeaderRecLeng = SunLong2Long( (unsigned char*)
					 &Param->HeaderRecLeng);
    /* short */
    Param->NoOf_FCD_Rec = SunShort2Short( (unsigned char*)
					  &Param->NoOf_FCD_Rec);
    /* long */
    Param->FCD_Size = SunLong2Long( (unsigned char*)
				    &Param->FCD_Size);
    /* short */
    Param->NoOf_PCD_Rec = SunShort2Short( (unsigned char*)
					  &Param->NoOf_PCD_Rec);
    /* long */
    Param->PCD_Size = SunLong2Long( (unsigned char*)
				    &Param->PCD_Size);
    /* short */
    Param->NoOf_SCD_Rec = SunShort2Short( (unsigned char*)
					  &Param->NoOf_SCD_Rec);
    /* long */
    Param->SCD_Size = SunLong2Long( (unsigned char*)
				    &Param->SCD_Size);
    /* short */
    Param->NoOf_MCD_Rec = SunShort2Short( (unsigned char*)
					  &Param->NoOf_MCD_Rec);
    /* long */
    Param->MCD_Size = SunLong2Long( (unsigned char*)
				    &Param->MCD_Size);
    /* short */
    Param->NoOf_Spare_Rec = SunShort2Short( (unsigned char*)
					    &Param->NoOf_Spare_Rec);
    /* long */
    Param->Spare_Size = SunLong2Long( (unsigned char*)
				      &Param->Spare_Size);

    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
	{
	    /* short */
	    Param->NoOfBDR_Rec[ i] = SunShort2Short( (unsigned char*)
						     &Param->NoOfBDR_Rec[ i]);
	    /* long */
	    Param->BDR_Rec_Size[ i] = SunLong2Long( (unsigned char*)
						    &Param->BDR_Rec_Size[ i]);
	}
}

/*----------------------------------------------------------------------------*\
**                                FSR1__read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		FSR1
**
**  Other interfaces:
**
**  Description:
**	Function to read all FSR1 entries from the given 
**	input stream into the FSR1 structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FSR1__read(FSR1_s* self, FILE* file_ptr, Bool convert_sun_2_host)
{
    int i;
    FunctionStatus_e status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** read FSR1 and check if any error occured
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread(&self->NoOfHeaderRec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->HeaderRecLeng, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->NoOf_FCD_Rec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->FCD_Size, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->NoOf_PCD_Rec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->PCD_Size, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->NoOf_SCD_Rec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->SCD_Size, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->NoOf_MCD_Rec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->MCD_Size, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->NoOf_Spare_Rec, 1, FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->Spare_Size, 1, FSR1_SIZE_ENTRY_SIZE, file_ptr);
    for (i = 0; i < BAND_DEFINED; i++)
    {
	NoOfBytes += fread(&self->NoOfBDR_Rec[ BAND_1A + i], 1,
			    FSR1_NUMBER_ENTRY_SIZE, file_ptr);
	NoOfBytes += fread(&self->BDR_Rec_Size[ BAND_1A + i], 1,
			    FSR1_SIZE_ENTRY_SIZE, file_ptr);
    }
    NoOfBytes += fread(&self->NoOfBDR_Rec[ BLIND_1A], 1, 
		       FSR1_NUMBER_ENTRY_SIZE, file_ptr);
    NoOfBytes += fread(&self->BDR_Rec_Size[BLIND_1A], 1,
		       FSR1_SIZE_ENTRY_SIZE, file_ptr);

    for (i = 0; i < STRAYLIGHT_DEFINED; i++)
    {
	NoOfBytes += fread(&self->NoOfBDR_Rec[ STRAYLIGHT_1A + i], 1,
			   FSR1_NUMBER_ENTRY_SIZE, file_ptr);
	NoOfBytes += fread(&self->BDR_Rec_Size[ STRAYLIGHT_1A + i], 1,
			   FSR1_SIZE_ENTRY_SIZE, file_ptr);
    }

    if (NoOfBytes != FSR1_SIZE)
    {
	status = FATAL;
	sprintf(LogBuffer, "FSR1, #read bytes: %d", NoOfBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__166", status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	FSR1__Sun2Host(self);
    }

    return status;
}


/*----------------------------------------------------------------------------*\
**                                FSR1__write
**  Input parameters:
**	Stream		output stream
**	Param		FSR1
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the FSR1 to the given 
**	output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FSR1__write(const FSR1_s *Param, FILE* Stream)
{
    int i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** write FSR1 and check if any error occured
    \*------------------------------------------------------------------------*/
    NoOfBytes += fwrite( &Param->NoOfHeaderRec, 1, sizeof( Param->NoOfHeaderRec),
			 Stream);
    NoOfBytes += fwrite( &Param->HeaderRecLeng, 1, sizeof( Param->HeaderRecLeng),
			 Stream);
    NoOfBytes += fwrite( &Param->NoOf_FCD_Rec, 1, sizeof( Param->NoOf_FCD_Rec),
			 Stream);
    NoOfBytes += fwrite( &Param->FCD_Size, 1, sizeof( Param->FCD_Size),
			 Stream);
    NoOfBytes += fwrite( &Param->NoOf_PCD_Rec, 1, sizeof( Param->NoOf_PCD_Rec),
			 Stream);
    NoOfBytes += fwrite( &Param->PCD_Size, 1, sizeof( Param->PCD_Size),
			 Stream);

    NoOfBytes += fwrite( &Param->NoOf_SCD_Rec, 1, sizeof( Param->NoOf_SCD_Rec),
			 Stream);
    NoOfBytes += fwrite( &Param->SCD_Size, 1, sizeof( Param->SCD_Size),
			 Stream);
    NoOfBytes += fwrite( &Param->NoOf_MCD_Rec, 1, sizeof( Param->NoOf_MCD_Rec),
			 Stream);
    NoOfBytes += fwrite( &Param->MCD_Size, 1, sizeof( Param->MCD_Size),
			 Stream);
    NoOfBytes += fwrite( &Param->NoOf_Spare_Rec, 1,
			 sizeof( Param->NoOf_Spare_Rec), Stream);
    NoOfBytes += fwrite( &Param->Spare_Size, 1, sizeof( Param->Spare_Size),
			 Stream);

    for (i = 0; i < BAND_DEFINED; i++)
	{
	    NoOfBytes += fwrite( &Param->NoOfBDR_Rec[ BAND_1A + i], 1,
				 sizeof( Param->NoOfBDR_Rec[ BAND_1A + i]), Stream);
	    NoOfBytes += fwrite( &Param->BDR_Rec_Size[ BAND_1A + i], 1,
				 sizeof( Param->BDR_Rec_Size[ BAND_1A + i]),
				 Stream);
	}

    NoOfBytes += fwrite( &Param->NoOfBDR_Rec[ BLIND_1A], 1,
			 sizeof( Param->NoOfBDR_Rec[ BLIND_1A]), Stream);
    NoOfBytes += fwrite( &Param->BDR_Rec_Size[BLIND_1A], 1,
			 sizeof( Param->BDR_Rec_Size[BLIND_1A]), Stream);

    for (i = 0; i < STRAYLIGHT_DEFINED; i++)
	{
	    NoOfBytes += fwrite( &Param->NoOfBDR_Rec[ STRAYLIGHT_1A + i], 1,
				 sizeof( Param->NoOfBDR_Rec[ STRAYLIGHT_1A + i]), Stream);
	    NoOfBytes += fwrite( &Param->BDR_Rec_Size[ STRAYLIGHT_1A + i], 1,
				 sizeof( Param->BDR_Rec_Size[ STRAYLIGHT_1A + i]), Stream);
	}

    if (NoOfBytes != FSR1_SIZE)
	{
	    Status = FATAL;
	    sprintf( LogBuffer, "FSR1, #written bytes: %d", NoOfBytes);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__264", Status, LOG_CANNOT_WRITE,
		      LogBuffer, "");
	}

    return Status;
}

/*----------------------------------------------------------------------------*\
**                                FSR1__show
**  Input parameters:
**	Stream		output stream
**	Param		FSR1 Structure
**
**  Output parameters:
**
**  Other interfaces:
**	FSR1 Structure written to the output stream
**
**  Description:
**	Function to write the FSR1 Structure to the output
**	stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FSR1__show(const FSR1_s *Param, FILE * Stream)
{
    int			i;
    /*------------------------------------------------------------------------*\
    ** write FSR1 and check if any error occured
    \*------------------------------------------------------------------------*/
    fprintf( Stream, "NoOfHeaderRec  : %5d\n", Param->NoOfHeaderRec);
    fprintf( Stream, "HeaderRecLeng  : %5d\n", Param->HeaderRecLeng);
    fprintf( Stream, "NoOf_FCD_Rec   : %5d\n", Param->NoOf_FCD_Rec);
    fprintf( Stream, "FCD_Size       : %5d\n", Param->FCD_Size);
    fprintf( Stream, "NoOf_PCD_Rec   : %5d\n", Param->NoOf_PCD_Rec);
    fprintf( Stream, "PCD_Size       : %5d\n", Param->PCD_Size);
    fprintf( Stream, "NoOf_SCD_Rec   : %5d\n", Param->NoOf_SCD_Rec);
    fprintf( Stream, "SCD_Size       : %5d\n", Param->SCD_Size);
    fprintf( Stream, "NoOf_MCD_Rec   : %5d\n", Param->NoOf_MCD_Rec);
    fprintf( Stream, "MCD_Size       : %5d\n", Param->MCD_Size);
    fprintf( Stream, "NoOf_Spare_Rec : %5d\n", Param->NoOf_Spare_Rec);
    fprintf( Stream, "Spare_Size     : %5d\n", Param->Spare_Size);
    for (i = 0; i < BAND_DEFINED; i++)
	{
	    fprintf( Stream,
		     "Band: %3d, NoOfBDR_Rec: %5d, BDR_Rec_Size: %5d\n",
		     BAND_1A + i, Param->NoOfBDR_Rec[ BAND_1A + i],
		     Param->BDR_Rec_Size[ BAND_1A + i]);
	}

    fprintf( Stream,
	     "Blind: %3d, NoOfBlindRec: %5d, BDR_Rec_Size: %5d\n",
	     BLIND_1A, Param->NoOfBDR_Rec[ BLIND_1A ],
	     Param->BDR_Rec_Size[ BLIND_1A]);

    for (i = 0; i < STRAYLIGHT_DEFINED; i++)
	{
	    fprintf( Stream,
		     "Stray.: %3d, NoOfStraylightRec: %5d, BDR_Rec_Size: %5d\n",
		     STRAYLIGHT_1A + i, Param->NoOfBDR_Rec[ STRAYLIGHT_1A + i],
		     Param->BDR_Rec_Size[ STRAYLIGHT_1A + i]);
	}
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 /*----------------------------------------------------------------------------*\
**                                FSR1__GetFilePos
**  Input parameters:
**    FSR1			File structure record of Level 1 Product
**
**  Output parameters:
**    FilePosPCD		File position of PCD
**    FilePosSCD		File position of SCD
**    FilePosMCD		File position of MCD
**    FilePosSD			File position of BDR
**
**  Other interfaces:
**
**  Description:
**    Function to return the file positions of the PCD, SCD, MCD and BDR.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FSR1__GetFilePos( const FSR1_s *FSR1, long *FilePosPCD, long *FilePosSCD,
		  long *FilePosMCD, long *FilePosSD)
{
    *FilePosPCD = PIR_SIZE + FSR1_SIZE +
	FSR1->NoOfHeaderRec * FSR1->HeaderRecLeng +
	FSR1->NoOf_FCD_Rec * FSR1->FCD_Size;
    *FilePosSCD = *FilePosPCD + FSR1->NoOf_PCD_Rec * FSR1->PCD_Size;
    *FilePosMCD = *FilePosSCD + FSR1->NoOf_SCD_Rec * FSR1->SCD_Size;
    *FilePosSD = *FilePosMCD + FSR1->NoOf_MCD_Rec * FSR1->MCD_Size;
}

