/*----------------------------------------------------------------------------*\
**					gdp01_ex.c
**
**  GDP Level 0 to 1: gdp01_ex file
**
**  Version:    5.3
**  Date:       04/18/02
**  Contents:
**    This file contains the program entry point 'main' for the
**    Level 0 to 1 Extracting
**    This file contains the following functions:
**                                printUsage
**                                getBaseName
**                                main
\*----------------------------------------------------------------------------*/

#pragma ident "@(#)gdp01_ex.c	5.3 04/18/02, Copyright DLR 1996-2002"

#include <string.h>
#include <stdlib.h>
#include "defs.h"
#include "log.h"

#include "config.h"



#define	SccsId	("@(#)gdp01_ex.c 5.3 04/18/02, Copyright DLR 1996-2002")

#define	SS_ID	("gdp01_ex")


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/ 

#define	NEW_NAME_FLAG_ALBEDO	(True)

#define	WRITE_SUN_FLAG_ALBEDO	(True)

#define	SUN_EXTRA_FILE_WRITTEN	(False)

#define	DO_WRITE_EPS_FLAG	(True)


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
**  These variables must be global to avoid overhead if the extractor
**  program is compiled with option -DNDEBUG
\*----------------------------------------------------------------------------*/
Debug_s* debug_ptr;
Debug_s* debug_bdr_ptr;
Debug_s* debug_fcd_ptr;
Debug_s* debug_pcd_ptr;


#define	DEBUG_FILENAME	("test_gdp01_ex.dbg")

#define	DEBUG_BDR_FILENAME	("test_gdp01_ex_BDR.dbg")

#define	DEBUG_FCD_FILENAME	("test_gdp01_ex_FCD.dbg")

#define	DEBUG_PCD_FILENAME	("test_gdp01_ex_PCD.dbg")
#endif


/*----------------------------------------------------------------------------*\
**  global variables
\*----------------------------------------------------------------------------*/
float previous_PMD_16[OPTICAL_CHANNELS][DETECTOR_PIXELS];
char previous_PMD_16_time[ASCII_UTC_TIME_LENGTH] = {0, 0, 0};



/*----------------------------------------------------------------------------*\
**                                getBaseName
**  Input parameters:
**    WholeName		the whole name of a file specification
**
**  Output parameters:
**    BaseName		Base name of the given file specitication without
**			extension
**
**  Description:
**    Function to get the base name of a file specification without extension
**    and leading path 
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.5.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
static void
getBaseName(const char *whole_name, char *base_name)
{
    int	i, j;

    /*------------------------------------------------------------------------*\
    ** Search from end to the first '\' to get the basename
    \*------------------------------------------------------------------------*/
    for (i = strlen(whole_name) - 1; (i >= 0) && (whole_name[i] != '/'); i--) 
    {
    }

    /*------------------------------------------------------------------------*\
    ** if found extract the base_name, else base_name is equal the whole_name
    \*------------------------------------------------------------------------*/
    if (i < 0) 
    {
	strcpy(base_name, whole_name);
    }
    else 
    {
	j = 0;
	for (i += 1; i < (short)strlen( whole_name); i++) {
	    base_name[j++] = whole_name[i];
	}
	base_name[j] = 0;
    }

    /*------------------------------------------------------------------------*\
    ** Search from end to the first '.'. If found cut the extention
    \*------------------------------------------------------------------------*/
    for (i = strlen(base_name) - 1;(i >= 0) && ( base_name[ i] != '.'); i--) 
    {
    }
    if (i >= 0) {
	base_name[i] = 0;
    }
}


/*----------------------------------------------------------------------------*\
**                                main
**  Input parameters:
**    argc: Number of arguments
**    argv: Array of strings containing the arguments.
**
**  Output parameters:
**
**  Other interfaces:
**    Input files:
**        input_file: a GDP Level 1 Data Product
**    Output files:
**        output_file: a GDP Level 1 Extracted Data
**        Log and Error Messages (gdp01_ex.err)
**    Temporary files:
**        Debug files (*.dbg) created only in DEBUG mode
**    NOTE: The format of the files is described in "Interface Specification
**            Document of the GDP" ER-IS-DLR-GO-0004
**          The output file names with the -s (split) option is:
**            output_file_0001 to output_file_NNNN
**
**  Description:
**    - Set the Debug with the value given at the command line
**    - Open log and error messages files
**    - Interprete the filter options
**    - Generate the Extract Level 1 Product according the given input parameter
**      and switches (Filter)
**    - A log or error message is written if some problem occur.
**
**  References:
**    <none>
**  Libraries:
**    extract, numrecip, Log, utils
**  Created:    10.1.95
**  Author: 	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
int
main(int argc, char *argv[])
{
    char ExtractingErrorMessagesFileName[MAX_FILENAME_LENGTH];
    FunctionStatus_e Status = SUCCESS;
    Config_s* config_ptr = Config__new();
    Bool log_is_open;
    char LogBuffer[LOG_FIELD_SIZE];

#ifdef DEBUG
    /*------------------------------------------------------------------------*\
    **  Initialise pointers
    \*------------------------------------------------------------------------*/
		debug_ptr = Debug__new();
		debug_bdr_ptr = Debug__new();
		debug_fcd_ptr = Debug__new();
		debug_pcd_ptr = Debug__new();

    /*------------------------------------------------------------------------*\
    **  filter commandline option -debug <level> if it exists
    \*------------------------------------------------------------------------*/
    if ((argc >= 2) && (strcmp(argv[1], "-debug")==0))
    {
	if (argc >=3) 
	{
	    int debug_module_level = atoi(argv[2]);
	    Debug__set(debug_ptr, debug_module_level);
	    Debug__set(debug_bdr_ptr, debug_module_level);
	    Debug__set(debug_fcd_ptr, debug_module_level);
	    Debug__set(debug_pcd_ptr, debug_module_level);
	    argc -= 2;
	    argv[2] = argv[0];
	    argv += 2;
	}
	else 
	{
	    Filter__writeDLR_Header(stderr);
	    fprintf(stderr, "Usage: %s -debug <level>\n", argv[ 0]);
	    return FATAL;
	}
    }    
#endif

    /*------------------------------------------------------------------------*\
    **  parse commandline options and exit, if they are of wrong format
    \*------------------------------------------------------------------------*/
    if (Config__parseUserParameters(config_ptr, argc, argv) != SUCCESS)
    {
	return FATAL;
    }

    /*------------------------------------------------------------------------*\
    **  Extract the file name and build the 
    **  <ExtractingErrorMessagesFileName>.err where error messages are
    **  written to.
    \*------------------------------------------------------------------------*/
    getBaseName(argv[0], ExtractingErrorMessagesFileName);
    strcat(ExtractingErrorMessagesFileName, ".err");

#ifdef DEBUG
    /*------------------------------------------------------------------------*\
    **  Open the DBG Streams
    \*------------------------------------------------------------------------*/
    Debug__open(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, DEBUG_FILENAME);
    Debug__open(debug_bdr_ptr, DBG_EXTRACTING, DEBUG_LEVEL_BDR, 
		DEBUG_BDR_FILENAME);
    Debug__open(debug_fcd_ptr, DBG_EXTRACTING, DEBUG_LEVEL_FCD, 
		DEBUG_FCD_FILENAME);
    Debug__open(debug_pcd_ptr, DBG_EXTRACTING, DEBUG_LEVEL_PCD, 
		DEBUG_PCD_FILENAME);

    if (Debug__toStandardout(debug_ptr))
	log_is_open = True;
    else
	log_is_open = Log__Open(ExtractingErrorMessagesFileName, True);
#else
    log_is_open = Log__Open(ExtractingErrorMessagesFileName, True);
#endif

    if (!log_is_open) 
    {
	Status = FATAL;
	Filter__writeDLR_Header(stderr);
	fprintf(stderr, "Error while initializing the log file. Start the "
		"system again\n");
	return Status;
    }
	
    /*------------------------------------------------------------------------*\
    **  Print the program header
    \*------------------------------------------------------------------------*/
    Config__writeExtractHeader(config_ptr, stdout);

    /*------------------------------------------------------------------------*\
    **  Set bands in class Config
    \*------------------------------------------------------------------------*/
    Config__setSDRBand(config_ptr, BAND_1A, BAND_4);
    Config__setBand(config_ptr, BAND_1A, STRAYLIGHT_2A);

    /*--------------------------------------------------------------------*\
    **  Extract level 1 product
    \*--------------------------------------------------------------------*/
    if (Config__extract(config_ptr, CONVERT_SUN_2_HOST,
			NEW_NAME_FLAG_ALBEDO, WRITE_SUN_FLAG_ALBEDO,
			SUN_EXTRA_FILE_WRITTEN, DO_WRITE_EPS_FLAG)
	!= SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__270", Status,
		 LOG_NUM, "Error extracting level 1 product", "");
    }

#ifdef DEBUG
    /*---------------------------------------------------------------------------*\
    **  Close the DebugStream
    \*---------------------------------------------------------------------------*/
    Debug__close(debug_ptr);
    Debug__close(debug_bdr_ptr);
    Debug__close(debug_fcd_ptr);
    Debug__close(debug_pcd_ptr);


    /*---------------------------------------------------------------------------*\
    **  close logfile if necessary
    \*---------------------------------------------------------------------------*/
    if (!Debug__toStandardout(debug_ptr))
    {
	Log__Close();
    }
#else
    Log__Close();
#endif

    Config__delete(config_ptr);

    if (Status != SUCCESS) 
    {
	terminateExecution(ExtractingErrorMessagesFileName);
    }
    else
    {
	return 0;
    }
}


