/*----------------------------------------------------------------------------*\
**                                GeolocationC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)GeolocationC.c	5.1 02/26/99, Copyright DLR 1998"

#include <string.h>
#include "geolionc.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)GeolocationC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Geolocation")


/*============================================================================*\
**  public operations
\*============================================================================*/
/*----------------------------------------------------------------------------*\
**                                Geolocation__Write
**  Input parameters:
**    Stream		File stream of the output file
**    Geolocation       Geolocation Record of the Product
**
**  Output parameters:
**     Geolocation__Write	SUCCESS	= output ok
**				else	= any error during output
**
**  Other interfaces:
**    Output file specified by fd
**
**  Description:
**    This function writes the Geolocation into the output file.
**    The output is formated according the output format description
**    of the Extract Level 1 Product.
**
**  References:
**    ...
**
**  Libraries:
**    ....
**
**  Created:		16.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GeolocationC__write(const GeolocationC_s *Geolocation, FILE *fd)
{
    FunctionStatus_e Stat = SUCCESS;
    float CenterLatitude;
    float CenterLongitude;
    char ASCII_Time[ 30];
    char LogBuffer[LOG_FIELD_SIZE];

    if (UTC_2_ASCII( Geolocation->DateUTC, ASCII_Time) != SUCCESS) {
	Stat = ERROR;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__64", Stat,
		  LOG_NUM, "converting the UTC time of Geolocation", "");
	strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }
    fprintf( fd, "%s\n", ASCII_Time);
    fprintf( fd, "%6.2f %5.2f %6.2f %5.2f %6.2f %5.2f\n",
	     Geolocation->NorthSunAngle[ 0].Zenith,
	     Geolocation->NorthSunAngle[ 0].Azimuth,
	     Geolocation->NorthSunAngle[ 1].Zenith,
	     Geolocation->NorthSunAngle[ 1].Azimuth,
	     Geolocation->NorthSunAngle[ 2].Zenith,
	     Geolocation->NorthSunAngle[ 2].Azimuth);
    fprintf( fd, "%6.2f %5.2f %6.2f %5.2f %6.2f %5.2f\n",
	     Geolocation->NorthLoS_Angle[ 0].Zenith,
	     Geolocation->NorthLoS_Angle[ 0].Azimuth,
	     Geolocation->NorthLoS_Angle[ 1].Zenith,
	     Geolocation->NorthLoS_Angle[ 1].Azimuth,
	     Geolocation->NorthLoS_Angle[ 2].Zenith,
	     Geolocation->NorthLoS_Angle[ 2].Azimuth);
    fprintf( fd, "%6.2f %5.2f %6.2f %5.2f %6.2f %5.2f\n",
	     Geolocation->SpacecraftSunAngle[ 0].Zenith,
	     Geolocation->SpacecraftSunAngle[ 0].Azimuth,
	     Geolocation->SpacecraftSunAngle[ 1].Zenith,
	     Geolocation->SpacecraftSunAngle[ 1].Azimuth,
	     Geolocation->SpacecraftSunAngle[ 2].Zenith,
	     Geolocation->SpacecraftSunAngle[ 2].Azimuth);
    fprintf( fd, "%6.2f %5.2f %6.2f %5.2f %6.2f %5.2f\n",
	     Geolocation->SpacecraftLoS_Angle[ 0].Zenith,
	     Geolocation->SpacecraftLoS_Angle[ 0].Azimuth,
	     Geolocation->SpacecraftLoS_Angle[ 1].Zenith,
	     Geolocation->SpacecraftLoS_Angle[ 1].Azimuth,
	     Geolocation->SpacecraftLoS_Angle[ 2].Zenith,
	     Geolocation->SpacecraftLoS_Angle[ 2].Azimuth);
    fprintf( fd, "%6.2f %6.2f %1d\n",
	     Geolocation->SatelliteHeight, Geolocation->EarthRadius,
	     Geolocation->SunGlint);

    /*-----------------------------------------------------------------*\
    ** get the CenterCoordinates
    \*-----------------------------------------------------------------*/
    CenterLatitude	= Geolocation->CornerCoord[ 4].Latitude;
    CenterLongitude	= Geolocation->CornerCoord[ 4].Longitude;
    fprintf( fd, "%6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f\n",
	     Geolocation->CornerCoord[0].Latitude,
	     Geolocation->CornerCoord[0].Longitude,
	     Geolocation->CornerCoord[1].Latitude,
	     Geolocation->CornerCoord[1].Longitude,
	     Geolocation->CornerCoord[2].Latitude,
	     Geolocation->CornerCoord[2].Longitude,
	     Geolocation->CornerCoord[3].Latitude,
	     Geolocation->CornerCoord[3].Longitude,
	     CenterLatitude,
	     CenterLongitude);
    if ( ferror( fd) != 0)
	{
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__119", FATAL, LOG_CANNOT_WRITE,
		      "Geolocation record", "");
	    Stat = FATAL;
	}
    return Stat;
}


/*----------------------------------------------------------------------------*\
**                           Geolocation__Sun2Host
**  Input parameters:
**    Geolocation		in Sun memory representation
**  Output parameters:
**    Geolocation		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Geolocation struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
GeolocationC__Sun2Host( GeolocationC_s *Param)
{
  int	i;

  for ( i = 0; i < ANGLES_DEFINED; i++)
  {
    /* float */
    Param->NorthSunAngle[ i].Zenith = SunFloat2Float( (unsigned char*)
		&Param->NorthSunAngle[ i].Zenith);
    /* float */
    Param->NorthSunAngle[ i].Azimuth = SunFloat2Float( (unsigned char*)
		&Param->NorthSunAngle[ i].Azimuth);
  }
  for ( i = 0; i < ANGLES_DEFINED; i++)
  {
    /* float */
    Param->NorthLoS_Angle[ i].Zenith = SunFloat2Float( (unsigned char*)
		&Param->NorthLoS_Angle[ i].Zenith);
    /* float */
    Param->NorthLoS_Angle[ i].Azimuth = SunFloat2Float( (unsigned char*)
		&Param->NorthLoS_Angle[ i].Azimuth);
  }

  for ( i = 0; i < ANGLES_DEFINED; i++)
  {
    /* float */
    Param->SpacecraftSunAngle[ i].Zenith = SunFloat2Float( (unsigned char*)
		&Param->SpacecraftSunAngle[ i].Zenith);
    /* float */
    Param->SpacecraftSunAngle[ i].Azimuth = SunFloat2Float( (unsigned char*)
		&Param->SpacecraftSunAngle[ i].Azimuth);
  }
  for ( i = 0; i < ANGLES_DEFINED; i++)
  {
    /* float */
    Param->SpacecraftLoS_Angle[ i].Zenith = SunFloat2Float( (unsigned char*)
		&Param->SpacecraftLoS_Angle[ i].Zenith);
    /* float */
    Param->SpacecraftLoS_Angle[ i].Azimuth = SunFloat2Float( (unsigned char*)
		&Param->SpacecraftLoS_Angle[ i].Azimuth);
  }

  /* float */
  Param->SatelliteHeight = SunFloat2Float( (unsigned char*)
		&Param->SatelliteHeight);
  /* float */
  Param->EarthRadius = SunFloat2Float( (unsigned char*)
		&Param->EarthRadius);

  for (i = 0; i < GROUND_CORNERS_DEFINED; i++)
  {
    /* float */
    Param->CornerCoord[ i].Latitude = SunFloat2Float(
			(unsigned char*)&Param->CornerCoord[ i].Latitude);
    /* float */
    Param->CornerCoord[ i].Longitude = SunFloat2Float(
			(unsigned char*)&Param->CornerCoord[ i].Longitude);
  }
}

/*----------------------------------------------------------------------------*\
**                           GeolocationC__Read
**  Input parameters:
**	Stream		input stream to read geolocation
**  Output parameters:
**	GeolocationPar	read geolocation
**
**  Other interfaces:
**
**  Description:
**	Function to read all geolocation entries from the given input stream
**	into the geolocation structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GeolocationC__read(GeolocationC_s *GeolocationPar, FILE* Stream,
		  Bool convert_sun_2_host)
{
    long NoOfBytes;
    int i;
    FunctionStatus_e Status = SUCCESS;

    /*------------------------------------------------------------------------*\
     * read Date and Time of Ground Pixel End in UTC format
     \*-----------------------------------------------------------------------*/
    if (UTC__read(&GeolocationPar->DateUTC, Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__243", Status, LOG_CANNOT_READ,
		 "Geolocation UTC time", "");
	return Status;
    }
    NoOfBytes = UTC_SIZE;

    /*------------------------------------------------------------------------*\
    ** read Solar Azimuth and Zenith angles (north)
    \*------------------------------------------------------------------------*/
    for (i = 0;  i < ANGLES_DEFINED; i++)
    {
	NoOfBytes += fread(&GeolocationPar->NorthSunAngle[i].Zenith, 1,
			   sizeof(GeolocationPar->NorthSunAngle[i].Zenith),
			   Stream);
	NoOfBytes += fread(&GeolocationPar->NorthSunAngle[i].Azimuth, 1,
			   sizeof( GeolocationPar->NorthSunAngle[i].Azimuth),
			   Stream);
    }

    /*------------------------------------------------------------------------*\
    ** read Line of Sight Azimuth and Zenith angles (north)
    \*------------------------------------------------------------------------*/
    for (i = 0;  i < ANGLES_DEFINED; i++)
    {
	NoOfBytes += fread(&GeolocationPar->NorthLoS_Angle[i].Zenith, 1,
			   sizeof(GeolocationPar->NorthLoS_Angle[i].Zenith),
			   Stream);
	NoOfBytes += fread(&GeolocationPar->NorthLoS_Angle[i].Azimuth, 1,
			   sizeof(GeolocationPar->NorthLoS_Angle[i].Azimuth),
			   Stream);
    }

    /*------------------------------------------------------------------------*\
    ** read Solar Azimuth and Zenith angles of Spacecraft
    \*------------------------------------------------------------------------*/
    for (i = 0;  i < ANGLES_DEFINED; i++)
    {
	NoOfBytes += fread(&GeolocationPar->SpacecraftSunAngle[i].Zenith, 1,
			   sizeof(GeolocationPar->SpacecraftSunAngle[i].Zenith),
			   Stream);
	NoOfBytes += fread(&GeolocationPar->SpacecraftSunAngle[i].Azimuth, 1,
			   sizeof(GeolocationPar->SpacecraftSunAngle[i].Azimuth)
			   , Stream);
    }
    /*------------------------------------------------------------------------*\
    ** read Line of Sight Azimuth and Zenith angles of Spacecraft
    \*------------------------------------------------------------------------*/
    for (i = 0;  i < ANGLES_DEFINED; i++)
    {
	NoOfBytes += fread(&GeolocationPar->SpacecraftLoS_Angle[i].Zenith, 1,
			   sizeof(GeolocationPar->SpacecraftLoS_Angle[i].Zenith)
			   , Stream);
	NoOfBytes += fread(&GeolocationPar->SpacecraftLoS_Angle[i].Azimuth, 1,
			   sizeof(GeolocationPar->SpacecraftLoS_Angle[i].Azimuth
				  ), Stream);
    }

    /*------------------------------------------------------------------------*\
    ** read Satellite Height
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread(&GeolocationPar->SatelliteHeight, 1,
		       sizeof(GeolocationPar->SatelliteHeight), Stream);
    /*------------------------------------------------------------------------*\
    ** read Earth Radius
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( &GeolocationPar->EarthRadius, 1,
			sizeof( GeolocationPar->EarthRadius), Stream);
    /*------------------------------------------------------------------------*\
    ** read Sun Glint
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( &GeolocationPar->SunGlint, 1,
			sizeof( GeolocationPar->SunGlint), Stream);
    /*------------------------------------------------------------------------*\
    ** read GROUND_CORNERS_DEFINED Corner Coordinates of Ground Pixel
    \*------------------------------------------------------------------------*/
    for ( i = 0;  i < GROUND_CORNERS_DEFINED; i++) 
    {
	NoOfBytes += fread(&GeolocationPar->CornerCoord[i].Latitude, 1,
			   sizeof( GeolocationPar->CornerCoord[i].Latitude),
			   Stream);
	NoOfBytes += fread(&GeolocationPar->CornerCoord[i].Longitude, 1,
			   sizeof(GeolocationPar->CornerCoord[i].Longitude),
			   Stream);
    }

    /*------------------------------------------------------------------------*\
    ** check if written bytes equal to the length of the Geolocation
    \*------------------------------------------------------------------------*/
    if (NoOfBytes != GEOLOCATION_SIZE)
    {
	char LogBuffer[LOG_FIELD_SIZE];
	Status = FATAL;
	sprintf(LogBuffer, "Geolocation information, #read bytes: %d", 
		NoOfBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__337", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	GeolocationC__Sun2Host(GeolocationPar);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                           GeolocationC__Show
**  Input parameters:
**	Stream		output stream
**	GeolocationPar	geolocation structure
**
**  Output parameters:
**
**  Other interfaces:
**	geolocation written to the output stream
**
**  Description:
**	Function to write the geolocation structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
GeolocationC__show(const GeolocationC_s *GeolocationPar, FILE* Stream)
{
  int i;
  char ASCII_Time[ 30];

  fprintf( Stream, "Geolocation\n");
  /*----------------------------------------------------------------------*\
  * write Date and Time of Ground Pixel End in UTC format
  \*----------------------------------------------------------------------*/
  UTC_2_ASCII( GeolocationPar->DateUTC, ASCII_Time);
  fprintf( Stream, "UTC Days : %u, Msecs: %u, ASCII: %s\n",
	GeolocationPar->DateUTC.Days, GeolocationPar->DateUTC.Msecs,
	ASCII_Time);
  /*----------------------------------------------------------------------*\
  ** write Solar Azimuth and Zenith angles
  \*----------------------------------------------------------------------*/
  for ( i = 0;  i < ANGLES_DEFINED; i++)
  {
    fprintf( Stream, "NorthSunAngle Azimuth: %f, Zenith : %f\n",
		GeolocationPar->NorthSunAngle[i].Azimuth,
		GeolocationPar->NorthSunAngle[i].Zenith);
  }
  /*----------------------------------------------------------------------*\
  ** write Line of Sight Azimuth and Zenith angles
  \*----------------------------------------------------------------------*/
  for ( i = 0;  i < ANGLES_DEFINED; i++)
  {
    fprintf( Stream, "NorthLoS_Angle Azimuth: %f, Zenith : %f\n",
		GeolocationPar->NorthLoS_Angle[i].Azimuth,
		GeolocationPar->NorthLoS_Angle[i].Zenith);
  }

  /*----------------------------------------------------------------------*\
  ** write Solar Azimuth and Zenith angles of spacecraft
  \*----------------------------------------------------------------------*/
  for ( i = 0;  i < ANGLES_DEFINED; i++)
  {
    fprintf( Stream, "SpacecraftSunAngle Azimuth: %f, Zenith : %f\n",
		GeolocationPar->SpacecraftSunAngle[i].Azimuth,
		GeolocationPar->SpacecraftSunAngle[i].Zenith);
  }
  /*----------------------------------------------------------------------*\
  ** write Line of Sight Azimuth and Zenith angles of spacecraft
  \*----------------------------------------------------------------------*/
  for ( i = 0;  i < ANGLES_DEFINED; i++)
  {
    fprintf( Stream, "SpacecraftLoS_Angle Azimuth: %f, Zenith : %f\n",
		GeolocationPar->SpacecraftLoS_Angle[i].Azimuth,
		GeolocationPar->SpacecraftLoS_Angle[i].Zenith);
  }

  /*----------------------------------------------------------------------*\
  ** write Satellite Height
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "SatelliteHeight: %f\n",
			 GeolocationPar->SatelliteHeight);
  /*----------------------------------------------------------------------*\
  ** write Earth Radius
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "EarthRadius: %f\n",
			 GeolocationPar->EarthRadius);
  /*----------------------------------------------------------------------*\
  ** write Sun glint
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Sun-glint: %x\n",
			 GeolocationPar->SunGlint);
  /*----------------------------------------------------------------------*\
  ** write GROUND_CORNERS_DEFINED Corner Coordinates of Ground Pixel
  \*----------------------------------------------------------------------*/
  for ( i = 0;  i < GROUND_CORNERS_DEFINED; i++)
  {
    fprintf( Stream, "CornerCoord Latitude : %f, Longitude : %f\n",
		GeolocationPar->CornerCoord[i].Latitude,
		GeolocationPar->CornerCoord[i].Longitude);
  }
}

/*============================================================================*\
**  private operations
\*============================================================================*/

 
