/*----------------------------------------------------------------------------*\
**                                GeolocationC.h
**
**  Class GeolocationC declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _GeolocationC_h
#define _GeolocationC_h

#pragma ident "@(#)GeolocationC.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "utc.h"
#include "coornate.h"
#include "posangle.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/
/* static const short GEOLOCATION_SIZE = 97; */

#define	GEOLOCATION_SIZE	(UTC_SIZE + 4 * (ANGLES_DEFINED * POS_ANGLE_SIZE) + FLOAT_SIZE + FLOAT_SIZE + 1 + GROUND_CORNERS_DEFINED * COORDINATE_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class GeolocationC
\*============================================================================*/
typedef struct GeolocationC {
    UTC_s DateUTC;		/* Date and Time of Ground Pixel End in 
				   UTC format */
    PosAngle_s NorthSunAngle[ANGLES_DEFINED]; /* Solar Zenith and Azimuth angles
						 at Satellite w.r.t North */
    PosAngle_s NorthLoS_Angle[ANGLES_DEFINED]; /* Line of Sight Zenith and 
						  Azimuth angles at Satellite 
						  w.r.t North */
    PosAngle_s SpacecraftSunAngle[ANGLES_DEFINED]; /* Solar Zenith and Azimuth 
						      Angles at Satellite w.r.t
						      Spacecraft */
    PosAngle_s SpacecraftLoS_Angle[ANGLES_DEFINED]; /* Line of Sight Zenith and 
						       Azimuth Angles at
						       Satellite w.r.t 
						       Spacecraft */
    float SatelliteHeight;	/* Satellite Height */
    float EarthRadius;		/* Earth Radius */
    char SunGlint;		/* Possible Sun-glint (0 = no, 1 = yes) */
    Coordinate_s CornerCoord[GROUND_CORNERS_DEFINED]; /* 4 Corner Coordinates 
							 of Ground Pixel */
} GeolocationC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
GeolocationC_s*
GeolocationC__New(FILE *Stream, FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void 
GeolocationC__Delete(GeolocationC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GeolocationC__write(const GeolocationC_s* self, FILE* fd);

FunctionStatus_e
GeolocationC__read(GeolocationC_s* self, FILE* Stream,
		  Bool convert_sun_2_host);

void 
GeolocationC__show(const GeolocationC_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
