/*----------------------------------------------------------------------------*\
**                                GO_IntegrStatus.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)GO_IntegrStatus.c	5.1 02/26/99, Copyright DLR 1998"

#include "go_iatus.h"


#define	SccsId	("@(#)GO_IntegrStatus.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("GO_IntegrStatus")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                GetAverageFactor
**  Input parameters:
**    Band		Band identifier
**    AverageMode	AverageMode Flag
**    IntegrStatus	integration status of GOME
**
**  Output parameters:
**    GetAverageFactor	the factor by which the raw data have to be multiplied.
**
**  Other interfaces:
**
**  Description:
**    Function to get the multiplication faktor of the raw data per band.
**    (co-adding mode)
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
short
GetAverageFactor( const ScienceData_e Band, const Bool AverageMode,
		  const GO_IntegrStatus_s IntegrStatus)
{
    short		Factor;

    /*------------------------------------------------------------------------*\
    ** if in average mode then multiply by the division factors
    \*------------------------------------------------------------------------*/
    if (AverageMode)
    {
	switch (Band2Channel( Band))
	{
	case OPTICAL_CH_2:		/* FPA2 */
	    {
		if (IntegrStatus.FPA2_Division == 1)
		    Factor = 2;
		else
		    Factor = 1;
		break;
	    }
	case OPTICAL_CH_3:		/* FPA3 */
	    {
		if (IntegrStatus.FPA3_Division == 2)
		    Factor = 4;
		else
		    if (IntegrStatus.FPA3_Division == 1)
			Factor = 2;
		    else
			Factor = 1;
		break;
	    }
	case OPTICAL_CH_4:		/* FPA4 */
	    {
		if (IntegrStatus.FPA4_Division == 1)
		    Factor = 2;
		else
		    Factor = 1;
		break;
	    }
	default:	/* FPA1 */
	    {
		Factor = 1;
		break;
	    }
	}
    }
    else
    {
	Factor = 1;
    }

    return Factor;
}


/*============================================================================*\
**  private operations
\*============================================================================*/



