/*----------------------------------------------------------------------------*\
**                                HotPixelC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)HotPixelC.c	5.1 02/26/99, Copyright DLR 1998"

#include "hotpxelc.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)HotPixelC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("HotPixel")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           HotPixelC__Sun2Host
**  Input parameters:
**    HotPixel		in Sun memory representation
**  Output parameters:
**    HotPixel		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the HotPixel struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
HotPixelC__Sun2Host( HotPixelC_s *Param)
{

    /* unsigned short */
    Param->Record = SunShort2Short( (unsigned char*)&Param->Record);
    /* unsigned short */
    Param->Array = SunShort2Short( (unsigned char*)&Param->Array);
    /* unsigned short */
    Param->Pixel = SunShort2Short( (unsigned char*)&Param->Pixel);
}

/*----------------------------------------------------------------------------*\
**                              HotPixelC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	HotPixelPar	read Hot Pixel
**
**  Other interfaces:
**
**  Description:
**	Function to read all Hot Pixel parameter from the given 
**	input stream into the Hot Pixel structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
HotPixelC__read(HotPixelC_s *HotPixelPar, FILE* Stream,
	       Bool convert_sun_2_host)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read hot pixels
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( &HotPixelPar->Record, 1,
			sizeof( HotPixelPar->Record), Stream);
    NoOfBytes += fread( &HotPixelPar->Array, 1,
			sizeof( HotPixelPar->Array), Stream);
    NoOfBytes += fread( &HotPixelPar->Pixel, 1,
			sizeof( HotPixelPar->Pixel), Stream);
    if (NoOfBytes != HOT_PIXEL_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Hot Pixels, #read bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__99", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	HotPixelC__Sun2Host( HotPixelPar);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           HotPixelC__Show
**  Input parameters:
**	Stream		output stream
**	HotPixelPar	Hot Pixel parameter structure
**
**  Output parameters:
**
**  Other interfaces:
**	Hot Pixel parameter structure written to the output stream
**
**  Description:
**	Function to write the Hot Pixel parameter structure to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void HotPixelC__show(const HotPixelC_s *HotPixelPar, FILE* Stream)
{
    /*--------------------------------------------------------------------*\
    **  show fixed pattern parameter
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "\t%d\t%d\t%d\n",
	     HotPixelPar->Record, HotPixelPar->Array, HotPixelPar->Pixel);
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
