/*----------------------------------------------------------------------------*\
**                                InternalPackageHeader.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)InternalPackageHeader.c	5.1 02/26/99, Copyright DLR 1998"

#include "inteader.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)InternalPackageHeader.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("InternalPackageHeader")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                InternalPckHeader2DecFormat
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    Output file ....
**  Description:
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	18.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
InternalPckHeader2DecFormat( InternalPckHeader_s* InternalPckHeader,
				InternalPckHeader_s* OrigInternalPckHeader)
{
  int i;

  OrigInternalPckHeader->UTC_Pck.Days	= 
			Long2DecLong( InternalPckHeader->UTC_Pck.Days);
  OrigInternalPckHeader->UTC_Pck.Msecs	= 
			Long2DecLong( InternalPckHeader->UTC_Pck.Msecs);
  for ( i = 0; i < MPH_6_SIZE + MPH_7_SIZE + MPH_16_SIZE; i++)
    OrigInternalPckHeader->MPH_6_7_16[ i] = InternalPckHeader->MPH_6_7_16[ i];
  for ( i = 0; i < SPH_5_SIZE + SPH_6_SIZE; i++)
    OrigInternalPckHeader->SPH_5_6[ i] = InternalPckHeader->SPH_5_6[ i];  
}


/*----------------------------------------------------------------------------*\
**                                InternalPckHeaderFromDecFormat
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    Output file ....
**  Description:
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	18.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
InternalPckHeaderFromDecFormat( InternalPckHeader_s* OrigInternalPckHeader,
				InternalPckHeader_s* InternalPckHeader)
{
  int i;

  InternalPckHeader->UTC_Pck.Days	= DecLong2Long((unsigned char*)&
			OrigInternalPckHeader->UTC_Pck.Days);
  InternalPckHeader->UTC_Pck.Msecs	= DecLong2Long((unsigned char*)&
			OrigInternalPckHeader->UTC_Pck.Msecs);
  for ( i = 0; i < MPH_6_SIZE + MPH_7_SIZE + MPH_16_SIZE; i++)
    InternalPckHeader->MPH_6_7_16[ i] = OrigInternalPckHeader->MPH_6_7_16[ i];
  for ( i = 0; i < SPH_5_SIZE + SPH_6_SIZE; i++)
    InternalPckHeader->SPH_5_6[ i] = OrigInternalPckHeader->SPH_5_6[ i];  
}


/*----------------------------------------------------------------------------*\
**                                WriteInternalPckHeader
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    Output file ....
**  Description:
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	18.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteInternalPckHeader( FILE* Stream,
			 const InternalPckHeader_s* InternalPckHeader)
{
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  unsigned long		Days;
  unsigned long		Msecs;
  char			spare		= ' ';
  char LogBuffer[LOG_FIELD_SIZE];

  /*----------------------------------------------------------------------*\
  ** write header
  \*----------------------------------------------------------------------*/
  NoOfBytes += fwrite( InternalPckHeader->MPH_6_7_16, 1,
		 sizeof( InternalPckHeader->MPH_6_7_16), Stream);
  NoOfBytes += fwrite( InternalPckHeader->SPH_5_6, 1,
		 sizeof( InternalPckHeader->SPH_5_6), Stream);
/*
  NoOfBytes += fwrite( &InternalPckHeader->spare, 1,
		 sizeof( InternalPckHeader->spare), Stream);
*/
  NoOfBytes += fwrite( &spare, 1, sizeof( spare), Stream);
  /*----------------------------------------------------------------------*\
  ** convert UTC to dec format and write utc
  \*----------------------------------------------------------------------*/
  Days = Long2DecLong( InternalPckHeader->UTC_Pck.Days);
  Msecs = Long2DecLong( InternalPckHeader->UTC_Pck.Msecs);

  NoOfBytes += fwrite( &Days, 1, sizeof( Days), Stream);
  NoOfBytes += fwrite( &Msecs, 1, sizeof( Msecs), Stream);

  /*----------------------------------------------------------------------*\
  ** check header size
  \*----------------------------------------------------------------------*/
  if (NoOfBytes != INTERNAL_PCK_HEADER_SIZE)
  {
    Status = FATAL;
    sprintf( LogBuffer,
	"Internal package header, #written bytes: %d", NoOfBytes);
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__151", Status, LOG_CANNOT_WRITE,
              LogBuffer, "");
  }

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                ReadInternalPckHeader
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    Output file ....
**  Description:
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	18.8.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
ReadInternalPckHeader( FILE* Stream,
			 InternalPckHeader_s* InternalPckHeader)
{
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  char LogBuffer[LOG_FIELD_SIZE];

  /*----------------------------------------------------------------------*\
  ** read 
  \*----------------------------------------------------------------------*/
  NoOfBytes += fread( InternalPckHeader->MPH_6_7_16, 1,
		 sizeof( InternalPckHeader->MPH_6_7_16), Stream);
  NoOfBytes += fread( InternalPckHeader->SPH_5_6, 1,
		 sizeof( InternalPckHeader->SPH_5_6), Stream);
  NoOfBytes += fread( &InternalPckHeader->spare, 1,
		 sizeof( InternalPckHeader->spare), Stream);
  NoOfBytes += fread( &InternalPckHeader->UTC_Pck.Days, 1,
		 sizeof( InternalPckHeader->UTC_Pck.Days), Stream);
  NoOfBytes += fread( &InternalPckHeader->UTC_Pck.Msecs, 1,
		 sizeof( InternalPckHeader->UTC_Pck.Msecs), Stream);

  /*----------------------------------------------------------------------*\
  ** convert from dec long 
  \*----------------------------------------------------------------------*/
  InternalPckHeader->UTC_Pck.Days	= DecLong2Long((unsigned char*)
					&InternalPckHeader->UTC_Pck.Days);
  InternalPckHeader->UTC_Pck.Msecs	= DecLong2Long((unsigned char*)
					&InternalPckHeader->UTC_Pck.Msecs);

  /*----------------------------------------------------------------------*\
  ** check header size
  \*----------------------------------------------------------------------*/
  if (NoOfBytes != INTERNAL_PCK_HEADER_SIZE)
  {
    Status = FATAL;
    sprintf( LogBuffer,
	"Internal package header, #read bytes: %d", NoOfBytes);
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__213", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
  }

  return Status;
}

/*----------------------------------------------------------------------------*\
**                            ShowInternalPckHeader
**  Input parameters:
**	Stream		output file
**	PackageTime	GOME package time
**
**  Output parameters:
**
**  Other interfaces:
**    Output file ....
**  Description:
**    Function to show the GOME pixel time
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.4.94
**  Author:	Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
ShowInternalPckHeader( FILE *Stream, InternalPckHeader_s InternalPckHeader)
{
  int i;
  char			ASCII_Time[ 30];

  /*----------------------------------------------------------------------*\
  * Show Date and Time 
  \*----------------------------------------------------------------------*/
  UTC_2_ASCII( InternalPckHeader.UTC_Pck, ASCII_Time);
  fprintf( Stream, "package time UTC Days : %u, Msecs: %u, ASCII: %s\n",
	InternalPckHeader.UTC_Pck.Days, InternalPckHeader.UTC_Pck.Msecs,
	ASCII_Time);
  fprintf( Stream, "MPH_6_7_16:\n");
  for ( i = 0; i < MPH_6_SIZE + MPH_7_SIZE + MPH_16_SIZE; i++)
  {
    if (( i % 10) == 0)
      fprintf( Stream, "\n i = %4d : ", i);
    fprintf( Stream, "%3u", InternalPckHeader.MPH_6_7_16[ i]);
  }
  fprintf( Stream, "\n");
  fprintf( Stream, "SPH_5_6:\n");
  for ( i = 0; i < SPH_5_SIZE + SPH_6_SIZE; i++)
  {
    if ((i % 10) == 0)
      fprintf( Stream, "\n i = %4d : ", i);
    fprintf( Stream, "%3u", InternalPckHeader.SPH_5_6[ i]);
  }
  fprintf( Stream, "\n");
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
