/*----------------------------------------------------------------------------*\
**                                IntensityC.h
**
**  Class IntensityC declaration
** 
**  Version:    5.2
**  Date:       07/28/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _IntensityC_h
#define _IntensityC_h

#pragma ident "@(#)IntensityC.h	5.2 07/28/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "utc.h"
#include "sciedata.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	PMD_CORNER_POINTS	(7)


#define	INTENSITY_SIZE	(OPTICAL_CHANNELS * DETECTOR_PIXELS * FLOAT_SIZE + OPTICAL_CHANNELS * DETECTOR_PIXELS * FLOAT_SIZE + OPTICAL_CHANNELS * DETECTOR_PIXELS * FLOAT_SIZE + PMD_DEFINED * FLOAT_SIZE + PMD_DEFINED * FLOAT_SIZE + UTC_SIZE)


#define	MIN_VALUE	(1e-20)

#define	MIN_VALUE_INTERPOL_H	(0.6)

#define	MIN_VALUE_MEAN_SUN_REF	(100)

/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class IntensityC
\*============================================================================*/
typedef struct IntensityC {
    float Calibration[OPTICAL_CHANNELS][DETECTOR_PIXELS];
				/* IntensityC calibration parameters */
    float MeanSunReference[OPTICAL_CHANNELS][DETECTOR_PIXELS];
				/* Mean Sun Reference Spectrum */
    float RelativeError[OPTICAL_CHANNELS][DETECTOR_PIXELS];
				/* Relative errors of Mean Sun Reference 
				   Spectrum */
    float MeanPMD[PMD_DEFINED];
				/* PMD Mean Values of the Sun Reference 
				   Spectra */
    float MeanPMD_Wavelength[PMD_DEFINED];
				/* Wavelength of the PMD Mean Values of the Sun
				   Reference Spectra */
    UTC_s Date;			/* UTC Date of the Sun Reference Spectra */
} IntensityC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
IntensityC_s*
IntensityC__New(FILE* Stream, FunctionStatus_e* Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void 
IntensityC__Delete(IntensityC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
void 
IntensityC__Apply(const IntensityC_s* Self, const float IntensityGrid[],
		 const float RadianceResponse[], ScienceData_e Band,
		 float Reticon[], const float LambdaGrid[], short Begin, 
		 short End, const char *PixelDate);

FunctionStatus_e 
IntensityC__ApplyPMD(const IntensityC_s* Self, const float LambdaPMD[], 
		    PMD_Number_e Number, float PMD[], 
		    const char* PixelDate);

void
IntensityC__CalculateAlbedo(const float Irradiance[], 
			   const float IntensityGrid[], ScienceData_e Band,
			   float Reticon[], const float LambdaGrid[], 
			   short Begin, short End, const char *PixelDate);

void 
IntensityC__CalculateSunCalibrated(const IntensityC_s* Self, 
				  ScienceData_e Band, float* SunCalibrated,
				  short Begin, short End);

void
IntensityC__PrepareJumpCorrection(float Pmd[PMD_DEFINED][PMD_SIZE], 
				 const float IntgTime, float S_PMD_basic[],
				 const char *PixelDate);

void
IntensityC__CorrectJumps(const float LambdaGrid[], float Reticon[], 
			const float IntgTime, const short do_correction, 
			float Pmd[PMD_DEFINED][PMD_SIZE], 
			float S_PMD_basic[], float previous_PMD_16[],
			ScienceData_e Band, short Begin, short End, 
			const char *PixelDate);

FunctionStatus_e
IntensityC__read(IntensityC_s* self,  FILE* Stream, Bool convert_sun_2_host);

void
IntensityC__show(const IntensityC_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
