/*----------------------------------------------------------------------------*\
**                                KeyDataCalibrationL1.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)KeyDataCalibrationL1.c	5.1 02/26/99, Copyright DLR 1998"

#include "keydonl1.h"
#include "fcd.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)KeyDataCalibrationL1.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("KeyDataCalibrationL1")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                              KeyDataCalibrationL1__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read KeyData Calibration parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all KeyData Calibration parameter from the given 
**	input stream into the KeyData Calibration parameter structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
KeyDataCalibrationL1__read(KeyDataCalibrationL1_s *Param,  FILE* Stream,
			   Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    long	NoOfBytes		= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** read Eta_Omega
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Eta_Omega, 1, sizeof( Param->Eta_Omega),
			Stream);
    /*------------------------------------------------------------------------*\
    ** read RadianceResponse_Omega
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->RadianceResponse_Omega, 1,
			sizeof( Param->Eta_Omega), Stream);

    if (NoOfBytes != PREFLIGHT_CALIBRATION_L1_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "KeyDataCalibrationL1, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__71", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	KeyDataCalibrationL1__Sun2Host( Param);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           KeyDataCalibrationL1__Show
**  Input parameters:
**	Stream		output stream
**	Param		KeyData Calibration Parameter
**
**  Output parameters:
**
**  Other interfaces:
**	KeyData Calibration Parameter written to the output stream
**
**  Description:
**	Function to write the KeyData Calibration Parameter to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
KeyDataCalibrationL1__show(const KeyDataCalibrationL1_s *Param, FILE* Stream)
{
    int	i;

    /*----------------------------------------------------------------------*\
    ** show Eta_Omega and RadianceResponse_Omega
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Index |\tEta Omega |\tRadiance Response\n");
    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%6d\t%f\t%f\n", i, Param->Eta_Omega[ i],
		 Param->RadianceResponse_Omega[ i]);
    }
}


/*----------------------------------------------------------------------------*\
**                           KeyDataCalibrationL1__Sun2Host
**  Input parameters:
**    KeyDataCalibrationL1		in Sun memory representation
**  Output parameters:
**    KeyDataCalibrationL1		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the KeyDataCalibrationL1 struct
**    from Sun memory representation into the memory representation of the 
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
KeyDataCalibrationL1__Sun2Host( KeyDataCalibrationL1_s *Param)
{
    int	i;

    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	/* float */
	Param->Eta_Omega[ i] = SunFloat2Float( (unsigned char*)&
					       Param->Eta_Omega[ i]);
	/* float */
	Param->RadianceResponse_Omega[ i] = SunFloat2Float( (unsigned char*)&
							    Param->RadianceResponse_Omega[ i]);
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
