/*----------------------------------------------------------------------------*\
**                                KeyDataError.h
**
**  Class KeyDataError (Calculation of the Errors on the Measurements)
**  declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _KeyDataError_h
#define _KeyDataError_h

#pragma ident "@(#)KeyDataError.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"
#include "sciedata.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	KEYDATA_ERRORS_FUNCTIONS	(7)

#define	KEYDATA_ERRORS_PER_CHANNEL	(2)

#define	KEYDATA_ERRORS_SIZE	(FLOAT_SIZE * KEYDATA_ERRORS_FUNCTIONS * OPTICAL_CHANNELS * KEYDATA_ERRORS_PER_CHANNEL + FLOAT_SIZE * OPTICAL_CHANNELS * DETECTOR_PIXELS)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum 
{
	KDE_BSDF = 0,
	KDE_RESPONSE,
	KDE_F2,
	KDE_SMDEP,
	KDE_CHI,
	KDE_ETA,
	KDE_KSI,
	KDE_RESPONSE_F2_SMDEP
} KeyDataError_e;


/*============================================================================*\
**  class KeyDataError
\*============================================================================*/
typedef struct KeyDataError {
	float Budget[KEYDATA_ERRORS_FUNCTIONS][OPTICAL_CHANNELS * KEYDATA_ERRORS_PER_CHANNEL];
		/* Relative error budget factors on KeyData */
	float Response_f2_SMDep[OPTICAL_CHANNELS][DETECTOR_PIXELS];
		/* Relative error budget factors on Response.f2.ScanMirrorDependency */
} KeyDataError_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
const char*
KeyDataError2Str(KeyDataError_e Enum);

float 
KeyDataError__Get(const KeyDataError_s *Self, KeyDataError_e Function,
		  OpticalChannel_e Ch, short Pixel);

FunctionStatus_e
KeyDataError__read(KeyDataError_s* self, FILE* Stream,
		   Bool convert_sun_2_host);

FunctionStatus_e
KeyDataError__write(const KeyDataError_s* self, FILE* Stream);

void
KeyDataError__show(const KeyDataError_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
