/*----------------------------------------------------------------------------*\
**                                LeakageC.h
**
**  Class Leakage declaration
** 
**  Version:    1.3
**  Date:       11/11/98
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Leakage_h
#define _Leakage_h

#pragma ident "@(#)LeakageC.h	1.3 11/11/98, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"
#include "sciedata.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	LEAKAGE_SIZE	(FLOAT_SIZE + FLOAT_SIZE * PMD_DEFINED + FLOAT_SIZE + OPTICAL_CHANNELS * DETECTOR_PIXELS * FLOAT_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class Leakage
\*============================================================================*/
typedef struct LeakageC {
    float ArrayNoise;		/* Detector Array Noise */
    float PMD_Offset[PMD_DEFINED]; /* PMD Offset */
    float PMD_Noise;		/* PMD noise during this measurement */
    float DarkCurrent[OPTICAL_CHANNELS][DETECTOR_PIXELS]; /* Dark Current */
} LeakageC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
LeakageC_s* 
LeakageC__New(FILE *Stream, FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
    void 
    LeakageC__Delete(LeakageC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
void
LeakageC__Apply(const LeakageC_s *Self, ScienceData_e Band, float Reticon[],
	       short Begin, short End, const char *PixelDate);

void 
LeakageC__ApplyPMD(const LeakageC_s *Self, PMD_Number_e Number, float PMD[],
		  const char *PixelDate);

FunctionStatus_e
LeakageC__read(LeakageC_s* self, FILE* Stream, Bool convert_sun_2_host);

void
LeakageC__show(const LeakageC_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
