/*----------------------------------------------------------------------------*\
**                                Level_1_Product.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Level_1_Product.c	5.2 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include <string.h>
#include "leveduct.h"
#include "pcd.h"
#include "smcd.h"
#include "fcd.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)Level_1_Product.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("Level_1_Product")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
extern Debug_s* debug_fcd_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                name
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Level_1_Product_s*
Level_1_Product__new()
{
#ifdef __cplusplus
    Level_1_Product_s* l = new Level_1_Product_s;
#else
    Level_1_Product_s* l = (Level_1_Product_s *) malloc( sizeof(Level_1_Product_s) );
#endif

    l->SPH1.Nref = 0;
    l->SPH1.PIR_Of_SDH = 0;

    l->FCD.Nleak = 0;
    l->FCD.LeakagePar = 0;
    l->FCD.Nhot = 0;
    l->FCD.HotPixelPar = 0;
    l->FCD.Nspec = 0;
    l->FCD.SpectralPar = 0;
    l->FCD.Nang = 0;
    l->FCD.KeyDataCalibrationL1_Par = 0;

    return l;
}


/*----------------------------------------------------------------------------*\
**                                name
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
Level_1_Product__delete(Level_1_Product_s* self)
{
#ifdef __cplusplus
    delete self;
#else
    free(self);
#endif
}


/*----------------------------------------------------------------------------*\
**                                ExtractInfo_Show
**  Input parameters:
**    Stream			output stream of info
**    InputStream		input strem of level 1 product
**    PIR			PIR of Level 1 Product
**    FSR1			File structure record of Level 1 Product
**    SPH1			Specific Product Header of Level 1 Product
**
**  Output parameters:
**    ExtractInfo_Show		SUCCESS	= output of info ok
**				else	= any error occured during output
**
**  Other interfaces:
**    Output file for Info specified by stream
**
**  Description:
**    Function to show a summary of the Level 1 Product.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Level_1_Product__showExtractInfo(const Level_1_Product_s* self, FILE* Stream,
				 FILE* InputStream,
				 Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    int	i;
    char ASCII_Time[ 25];
    char ASCII_timeStart[ 25];
    char ASCII_timeStop[ 25];
    UTC_s TimeFirst_xCD;
    UTC_s UTC_MeasurementPixel;
    long FilePosPCD;
    long FilePosSCD;
    long FilePosMCD;
    long FilePosSD;
    UTC_s TimeFirst_PCD;
    UTC_s TimeLast_PCD;
    UTC_s TimeFirst_SCD;
    UTC_s TimeLast_SCD;
    UTC_s TimeFirst_MCD;
    UTC_s TimeLast_MCD;

    /*----------------------------------------------------------------------*\
    ** Show the PIR of the level 1 product
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "\n");
    fprintf(Stream, "Product Identifier Record\n");
    if (PIR__writeExtractLevel_1_Product(self->PIR, Stream) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__190", Status,
		 LOG_CANNOT_WRITE, "PIR in extract level 1 product", "");
	return Status;
    }
    fprintf(Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** Show the Input data reference of the level 1 product
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Input Data Reference\n");
    for (i = 0; i < self->SPH1.Nref; i++) 
    {
	if (PIR__writeExtractLevel_1_Product(self->SPH1.PIR_Of_SDH[i], Stream)
	    != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__206", Status,
		     LOG_CANNOT_WRITE, 
		     "PIR of SPH1 in extract level 1 product", "");
	    return Status;
	}
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** Show GDP software version
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "GDP Software Version, Level 0 to 1 Processing\n");
    for (i = 0; i < sizeof(self->SPH1.SoftwareVersion); i++) {
	fprintf(Stream, "%c", self->SPH1.SoftwareVersion[i]);
    }
    fprintf( Stream, "\n\n");

    /*----------------------------------------------------------------------*\
    ** Show Key Data Calibration Data Version
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Key Data Calibration Data Version\n");
    for (i = 0; i < sizeof(self->SPH1.KeyDataDataVersion); i++) {
	fprintf(Stream, "%c", self->SPH1.KeyDataDataVersion[i]);
    }
    fprintf( Stream, "\n\n");

    /*----------------------------------------------------------------------*\
    ** Show Product Format Version
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Product Format Version\n %hd\n\n",
	    self->SPH1.ProductFormatVersion);

    /*----------------------------------------------------------------------*\
    ** show time correlation
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Time Correlation Information: Orbit Number, UTC Date, "
	    "Satellite Binary Counter\n");
    if (UTC_2_ASCII(self->SPH1.TimeCor.UTC, ASCII_Time) != SUCCESS) {
	Status = ERROR;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__245", Status, LOG_NUM,
		 "convert UTC date of Time correlation into ASCII string", "");
	ASCII_Time[0] = 0;
    }

    fprintf(Stream, "%05u %s %09u\n\n",
	    self->SPH1.TimeCor.OrbitNumber, ASCII_Time, self->SPH1.TimeCor.SBT_Period);

    /*-----------------------------------------------------------------------*\
    ** get the file pointer positions of PCD, SCD, MCD
    \*-----------------------------------------------------------------------*/
    FSR1__GetFilePos(&(self->FSR1), &FilePosPCD, &FilePosSCD, &FilePosMCD,
		     &FilePosSD);

    /*----------------------------------------------------------------------*\
    ** get time range of PCD
    \*----------------------------------------------------------------------*/
    if (self->FSR1.NoOf_PCD_Rec > 0) 
    {
	if (PCD__getTimeRange(InputStream, PCD__ReadTime, FilePosPCD,
			      self->FSR1.NoOf_PCD_Rec, 
			      self->FSR1.PCD_Size, 
			      &TimeFirst_PCD, &TimeLast_PCD, 
			      convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.2__271", Status,
		     LOG_CANNOT_READ,
		     "time range of PCD from Level 1 Product file", "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** Show the temporal coverage
	\*--------------------------------------------------------------------*/
	UTC_MeasurementPixel.Days = 0;
	UTC_MeasurementPixel.Msecs = (Time_t)(GROUND_PIXEL_TIME * 1000.0);
	UTC_Sub( TimeFirst_PCD, UTC_MeasurementPixel, &TimeFirst_xCD);
	if (UTC_2_ASCII( TimeFirst_xCD, ASCII_timeStart) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__286", Status, LOG_NUM, 
		     "convert UTC date of the first measurement pixel", "");
	    ASCII_timeStart[0] = 0;
	}

	if (UTC_2_ASCII(TimeLast_PCD, ASCII_timeStop) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__294", Status, LOG_NUM,
		     "convert UTC date of the last measurement pixel", "");
	    ASCII_timeStop[ 0] = 0;
	}
	fprintf( Stream, "Temporal Coverage Earth\n");
	fprintf( Stream, "%s to %s\n\n", ASCII_timeStart, ASCII_timeStop);
    }

    /*----------------------------------------------------------------------*\
    ** Show the number of Measurement pixels
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Number of Earthshine Measurement Pixels\n");
    fprintf(Stream, "%1d\n\n", self->FSR1.NoOf_PCD_Rec);

    /*----------------------------------------------------------------------*\
    ** get time range of SCD
    \*----------------------------------------------------------------------*/
    if (self->FSR1.NoOf_SCD_Rec > 0)	
    {
	if (PCD__getTimeRange(InputStream, SMCD__ReadTime, 
			      FilePosSCD,
			      self->FSR1.NoOf_SCD_Rec, self->FSR1.SCD_Size,
			      &TimeFirst_SCD, &TimeLast_SCD,
			      convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.2__320", Status, 
		     LOG_CANNOT_READ,
		     "time range of SCD from Level 1 Product file", "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** Show the temporal coverage
	\*--------------------------------------------------------------------*/
	UTC_MeasurementPixel.Days = 0;
	UTC_MeasurementPixel.Msecs = (Time_t)(GROUND_PIXEL_TIME * 1000.0);
	UTC_Sub( TimeFirst_SCD, UTC_MeasurementPixel, &TimeFirst_xCD);
	if (UTC_2_ASCII( TimeFirst_xCD, ASCII_timeStart) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__335", Status, LOG_NUM,
		     "convert UTC date of the first measurement pixel", "");
	    ASCII_timeStart[ 0] = 0;
	}

	if (UTC_2_ASCII( TimeLast_SCD, ASCII_timeStop) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__343", Status, LOG_NUM,
		      "convert UTC date of the last measurement pixel", "");
	    ASCII_timeStop[ 0] = 0;
	}

	fprintf( Stream, "Temporal Coverage Sun\n");
	fprintf( Stream, "%s to %s\n\n", ASCII_timeStart, ASCII_timeStop);
    }

    /*----------------------------------------------------------------------*\
    ** Show the number of Measurement pixels
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Number of Sun Measurements\n");
    fprintf( Stream, "%1d\n\n", self->FSR1.NoOf_SCD_Rec);

    /*----------------------------------------------------------------------*\
    ** get time range of MCD
    \*----------------------------------------------------------------------*/
    if (self->FSR1.NoOf_MCD_Rec > 0)	
    {
	if (PCD__getTimeRange(InputStream, SMCD__ReadTime, 
			      FilePosMCD,
			      self->FSR1.NoOf_MCD_Rec, self->FSR1.MCD_Size,
			      &TimeFirst_MCD, &TimeLast_MCD, 
			      convert_sun_2_host) != SUCCESS) 
	{
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__370", Status, 
		      LOG_CANNOT_READ,
		      "time range of MCD from Level 1 Product file", "");
	    return Status;
	}
    
	/*--------------------------------------------------------------------*\
	** Show the temporal coverage
	\*--------------------------------------------------------------------*/
	UTC_MeasurementPixel.Days = 0;
	UTC_MeasurementPixel.Msecs = (Time_t)(GROUND_PIXEL_TIME * 1000.0);
	UTC_Sub( TimeFirst_MCD, UTC_MeasurementPixel, &TimeFirst_xCD);
	if (UTC_2_ASCII( TimeFirst_xCD, ASCII_timeStart) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__385", Status, LOG_NUM,
		     "convert UTC date of the first measurement pixel", "");
	    ASCII_timeStart[ 0] = 0;
	}

	if (UTC_2_ASCII( TimeLast_MCD, ASCII_timeStop) != SUCCESS) 
	{
	    Status = ERROR;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__393", Status, LOG_NUM,
		      "convert UTC date of the last measurement pixel", "");
	    ASCII_timeStop[ 0] = 0;
	}

	fprintf( Stream, "Temporal Coverage Moon\n");
	fprintf( Stream, "%s to %s\n\n", ASCII_timeStart, ASCII_timeStop);
    }
  
    /*----------------------------------------------------------------------*\
    ** Show the number of Measurement pixels
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Number of Moon Measurements\n");
    fprintf(Stream, "%1d\n\n", self->FSR1.NoOf_MCD_Rec);

    /*----------------------------------------------------------------------*\
    ** Show total number of Band Data Records per band
    \*----------------------------------------------------------------------*/
    fprintf(Stream, "Total Number of Data Records per Band (Band 1a to 4, "
	    "Blind, Straylight 1a to 2a)\n");

    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) {
	fprintf(Stream, "%-7d ", self->FSR1.NoOfBDR_Rec[i]);
    }
    fprintf( Stream, "\n\n");

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                Extract_ReadStatic
**  Input parameters:
**    Stream			input stream
**
**  Output parameters:
**    PIR			PIR of Level 1 Product
**    FSR1			File structure record of Level 1 Product
**    SPH1			Specific Product Header of Level 1 Product
**    FCD			Fixed calibration record
**    Extract_ReadStatic	SUCCESS	=	Read ok
**				else	=	any error occured during read
**
**  Other interfaces:
**
**  Description:
**    Function to read the static parts of the Level 1 Product
**	 (PIR, FSR1, SPH1, FCD)
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	28.9.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Level_1_Product__readStatic(Level_1_Product_s* self, FILE* file_ptr,
			    Bool convert_sun_2_host)
{
    FunctionStatus_e status = SUCCESS;
    long NoOfReadBytes = 0;
#ifdef DEBUG
    FILE* debug_file_ptr;
    char debug_message[255];
#endif

    /*------------------------------------------------------------------------*\
    ** read the PIR from the product
    \*------------------------------------------------------------------------*/
    if (PIR__read(self->PIR, file_ptr) != SUCCESS)
    {
	status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__466", status,
		 LOG_CANNOT_READ, "PIR from level 1 product", "");
	return status;
    }
#ifdef DEBUG  
    if ( (debug_file_ptr = Debug__getStream(debug_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL)) != 0)
    {
	fprintf(debug_file_ptr, "\nPIR of Level 1 Product");
	PIR__show(self->PIR, debug_file_ptr);
    }
#endif

    /*------------------------------------------------------------------------*\
    ** read FSR1 and check if any error occured
    \*------------------------------------------------------------------------*/
    if (FSR1__read(&(self->FSR1), file_ptr, convert_sun_2_host) != SUCCESS)
    {
	status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__485", status,
		 LOG_CANNOT_READ, "FSR1 from level 1 product", "");
	return status;
    }
#ifdef DEBUG  
    if ( (debug_file_ptr = Debug__getStream(debug_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL)) != 0)
    {
	fprintf(debug_file_ptr, "FSR1 from level 1 product\n");
	FSR1__show(&(self->FSR1), debug_file_ptr);
    }
#endif
    
    /*---------------------------------------------------------------------------*\
    ** read the SPH1
    \*---------------------------------------------------------------------------*/
    if (SPH1__read(&(self->SPH1), file_ptr, convert_sun_2_host) != SUCCESS)
    {
	status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__504", status,
		 LOG_CANNOT_READ, "SPH1 from level 1 product", "");
	return status;
    }
#ifdef DEBUG  
    if ( (debug_file_ptr = Debug__getStream(debug_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL)) != 0)
    {
	fprintf(debug_file_ptr, "SPH1 from Level 1 Product\n");
	SPH1__show(&(self->SPH1), debug_file_ptr);
    }
#endif

    SPH1__checkSoftwareVersion(&(self->SPH1));
    if (SPH1__checkProductVersion(&(self->SPH1)) != SUCCESS)
    {
	return FATAL;
    }

    /*------------------------------------------------------------------------*\
    ** read the FCD
    \*------------------------------------------------------------------------*/
    if (FCD__read(&(self->FCD), file_ptr, convert_sun_2_host) != SUCCESS)
    {
	status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__529", status,
		 LOG_CANNOT_READ, "FCD from level 1 product", "");
	return status;
    }

#ifdef DEBUG  
    sprintf(debug_message, "\nFCD:: Nleak: %d, Nhot: %d, Nspec: %d, Nang: "
	   "%d\n", self->FCD.Nleak, self->FCD.Nhot, self->FCD.Nspec,
	   self->FCD.Nang);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, 
		 "FCD from Level 1 Product\n");
    if ( (debug_file_ptr = Debug__getStream(debug_fcd_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL_FCD)) != 0)
    {
	FCD__show(&(self->FCD), debug_file_ptr, True);
    }
#endif

    /*--------------------------------------------------------------------*\
    ** check the indices
    \*--------------------------------------------------------------------*/
    if ((self->FCD.IndexSpectralCalPar < 0) 
	|| (self->FCD.IndexSpectralCalPar >= self->FCD.Nspec)) 
    {
	status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__555", status, LOG_NUM,
		  "FCD, Index to Spectral Calibration Parameter is out "
		  "of range", "");
	return status;
    }
    if (self->FCD.IndexSpectralCalPar == INVALID_INDEX) {
	status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__562", status, LOG_NUM,
		  "FCD, Index to Spectral Calibration Parameter = "
		  "INVALID_INDEX", "");
	return status;
    }

    return SUCCESS;
}



/*============================================================================*\
**  private operations
\*============================================================================*/

 
