/*----------------------------------------------------------------------------*\
**                                Level_0_FileRec.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Level_0_FileRec.c	5.1 02/26/99, Copyright DLR 1998"

#include "leveerec.h"
#include "dsr.h"
#include "log.h"



#define	SccsId	("@(#)Level_0_FileRec.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Level_0_FileRec")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            Level_0_FileRec__Read
**  Input parameters:
**    Stream			file stream for output
**
**  Output parameters:
**    MPH[ MPH_SIZE]		MPH of Level 0 file
**    SPH[ SPH_SIZE]		SPH of Level 0 file
**    DSR_REC_NO		number of DSR records
**    DSR			Sciecne data record
**
**  Other interfaces:
**
**  Description:
**    This function reads one complete level 0 file record from the input
**    file (MPH, SPH, DSR_REC_NO, DSR)
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Level_0_FileRec__read(char MPH[ MPH_SIZE], char SPH[ SPH_SIZE],
		      char DSR_REC_NO[ DSR_REC_NO_SIZE],
		      char DSR[], FILE *Stream)
{
    FunctionStatus_e Status = SUCCESS;

    /*------------------------------------------------------------------------*\
    ** read the MPH
    \*------------------------------------------------------------------------*/
    if (MPH__read(MPH, Stream) != SUCCESS)
    {
	Status = FATAL;      
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__573", Status, LOG_CANNOT_READ,
		 " MPH", "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** read the SPH
    \*---------------------------------------------------------------------*/
    if (SPH__read(SPH, Stream) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__585", Status, LOG_CANNOT_READ,
		 " SPH", "");
	return Status;
    }
   
    /*-------------------------------------------------------------------*\
    ** read the DSR_REC_NO
    \*-------------------------------------------------------------------*/
    if (DSR_REC_NO__read(DSR_REC_NO, Stream) != SUCCESS)
    {
        Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__597", Status, LOG_CANNOT_READ,
		 " DSR_REC_NO", "");
	return Status;
    }

    /*-----------------------------------------------------------------*\
    ** read the Science package of the current file
    \*-----------------------------------------------------------------*/
    if (fread(DSR, 1, DSR_SIZE, Stream) != DSR_SIZE)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__610", Status,
		 LOG_CANNOT_READ, " Science Data Record", "");
	return Status;
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                            Level_0_FileRec__Write
**  Input parameters:
**    Stream			file stream for output
**    MPH[ MPH_SIZE]		MPH of Level 0 file
**    SPH[ SPH_SIZE]		SPH of Level 0 file
**    DSR_REC_NO		number of DSR records
**    DSR			Sciecne data record
**
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**    This function writes one complete level 0 file record into the output
**    file (MPH, SPH, DSR_REC_NO, DSR)
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Level_0_FileRec__write(const char MPH[ MPH_SIZE], const char SPH[ SPH_SIZE],
		       char DSR_REC_NO[ DSR_REC_NO_SIZE], char DSR[],
		       FILE *Stream)
{
    FunctionStatus_e Status = SUCCESS;

    /*-------------------------------------------------------------------------*\
    ** Write the MPH
    \*-------------------------------------------------------------------------*/
    if (MPH__write(MPH, Stream) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__488", Status, LOG_CANNOT_WRITE,
		 " MPH", "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** write the SPH
    \*---------------------------------------------------------------------*/
    if (SPH__write(SPH, Stream) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__499", Status, LOG_CANNOT_WRITE,
		 " SPH", "");
	return Status;
    }

    /*-------------------------------------------------------------------*\
    ** write the DSR_REC_NO
    \*-------------------------------------------------------------------*/
    if (DSR_REC_NO__write(Stream, DSR_REC_NO) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__510", FATAL, LOG_CANNOT_WRITE,
		 " DSR_REC_NO", "");
	return Status;
    }

    /*-----------------------------------------------------------------*\
    ** write the Science package of the current file
    \*-----------------------------------------------------------------*/
    if (fwrite( DSR, 1, DSR_SIZE, Stream) != DSR_SIZE)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__523", FATAL,
		 LOG_CANNOT_WRITE, " Science Data Record (DSR)", "");
	return Status;
    }

    return SUCCESS;
}


/*============================================================================*\
**  private operations
\*============================================================================*/


