/*----------------------------------------------------------------------------*\
**                              matrix.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the functions: matrix__new, matrix__delete
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)matrix.c	5.1 02/26/99, Copyright DLR 1994"

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include "numrecip.h"
#include "log.h"



#define	SS_ID	("matrix")

/*----------------------------------------------------------------------------*\
**                                matrix__new
**  Input parameters:
**    nrl, nrh: row low and high limits
**    ncl, nch: column low and high limits
**  Output parameters:
**  Other interfaces: none
**  Description:
**    Initialize a matrix [nrl..nrh][ncl..nch]
**  References:
**    Numerical Recipes in C
**  Libraries:
**  Created:    26.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double	**matrix__new(short nrl, short nrh, short ncl, short nch)
{
	short	i;
	double	**m;

	m = (double **)malloc( (unsigned) (nrh-nrl+1)*sizeof(double *) );
	if (!m) 
	{
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_45", FATAL, LOG_CANNOT_ALLOCATE, "for a matrix object", "");
	    exit(-1);
	}
	m -= nrl;

	for(i=nrl; i<=nrh; i++) {
		m[i] = (double *)malloc((unsigned)(nch-ncl+1)*sizeof(double));
		if (!m[i]) {
			Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_53", FATAL, LOG_CANNOT_ALLOCATE, "for a matrix object", "");
			exit(-1);
		}
		m[i] -= ncl;
	}
	return m;
}



/*----------------------------------------------------------------------------*\
**                                matrix__delete
**  Input parameters:
**    m: pointer to a matrix
**    nrl, nrh: row low and high limits
**    ncl, nch: column low and high limits
**  Output parameters:
**  Other interfaces: none
**  Description:
**    Delete the matrix [nrl..nrh][ncl..nch]
**  References:
**    Numerical Recipes in C
**  Libraries:
**  Created:    26.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void	matrix__delete(double  **m, short nrl, short nrh, short ncl, short nch)
{
	short	i;

	for(i=nrh; i>=nrl; i--) free( (char *) (m[i]+ncl) );
	free( (char *) (m+nrl) );
}
