/*----------------------------------------------------------------------------*\
**                                MPH.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)MPH.c	5.1 02/26/99, Copyright DLR 1998"

#include "mph.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)MPH.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("MPH")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            MPH__Extract
**  Input parameters:
**    Stream		file stream for output
**    MPH		MPH of Level 0 file
**  Output parameters:
**    ProductTime	Time of the product of MPH (data package)
**    TimeCor		time correlation of the MPH
**    StateVector	state vector of the MPH
**    MPH_6_7_16	MPH entries of MPH entry 6,7,16
**
**  Other interfaces:
**
**  Description:
**    This function extracts the specified output parameter of the MPH
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MPH__extract(char MPH[MPH_SIZE], UTC_s *ProductTime, TimeCor_s *TimeCor,
	     StateVectorC_s *StateVector,
	     char MPH_6_7_16[MPH_6_SIZE + MPH_7_SIZE + MPH_16_SIZE])
{
    FunctionStatus_e Stat = SUCCESS;
    int i;
    char TimeCorASCII_Time[ MPH_TCOR_UTC_TIME_SIZE + 1];
    char StateVectorASCII_Time[ MPH_UTC_ASCENDING_NODE_SIZE + 1];
    char ProductASCII_Time[ MPH_UTC_BEGIN_PRODUCT_SIZE + 1];
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** get product time
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < MPH_UTC_BEGIN_PRODUCT_SIZE; i++)
	ProductASCII_Time[ i] = MPH[ MPH_UTC_BEGIN_PRODUCT_ENTRY + i];
    ProductASCII_Time[ MPH_UTC_BEGIN_PRODUCT_SIZE] = 0;
    if (ASCII_2_UTC( ProductASCII_Time, ProductTime) != SUCCESS)
    {
	Stat = FATAL;
	sprintf( LogBuffer, "illegal time specification in Product time of MPH, %s",
		 ProductASCII_Time);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__156", FATAL, LOG_NUM,
		  LogBuffer, "");
    }

    /*--------------------------------------------------------------------*\
    ** get the time correlation
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < MPH_TCOR_UTC_TIME_SIZE; i++)
	TimeCorASCII_Time[ i] = MPH[ MPH_TCOR_UTC_TIME_ENTRY + i];
    TimeCorASCII_Time[ MPH_TCOR_UTC_TIME_SIZE] = 0;
    if (ASCII_2_UTC( TimeCorASCII_Time, &TimeCor->UTC) != SUCCESS)
    {
	Stat = FATAL;
	sprintf( LogBuffer,
		 "illegal time specification in time correlation of MPH, %s",
		 TimeCorASCII_Time);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__172", FATAL, LOG_NUM,
		 LogBuffer, "");
    }
    TimeCor->SBT		= DecLong2Long(( unsigned char*)&MPH[
								     MPH_TCOR_SBT_ENTRY]);
    TimeCor->SBT_Period	= DecLong2Long(( unsigned char*)&MPH[
							     MPH_TCOR_SBT_PERIODE_ENTRY]);
    TimeCor->OrbitNumber	= 0;

    /*--------------------------------------------------------------------*\
    ** get the StateVector of ascending node
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < MPH_UTC_ASCENDING_NODE_SIZE; i++)
	StateVectorASCII_Time[ i] = MPH[ MPH_UTC_ASCENDING_NODE_ENTRY + i];
    StateVectorASCII_Time[ MPH_UTC_ASCENDING_NODE_SIZE] = 0;

    /*--------------------------------------------------------------------*\
    ** calculate the utc time of the State Vector
    \*--------------------------------------------------------------------*/
    if (ASCII_2_UTC( StateVectorASCII_Time, &StateVector->DateUTC) != SUCCESS)
    {
	Stat = FATAL;
	sprintf( LogBuffer,
		 "illegal time specification of State Vector of MPH, %s",
		 StateVectorASCII_Time);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__197", FATAL, LOG_NUM,
		 LogBuffer, "");
    }
    StateVector->Position.X	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY]) * TEN_MINUS_2_METER_IN_KM;
    StateVector->Position.Y	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY + MPH_STATE_VECTOR_ENTRY_SIZE]) *
	TEN_MINUS_2_METER_IN_KM;
    StateVector->Position.Z	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY + 2 * MPH_STATE_VECTOR_ENTRY_SIZE]) *
	TEN_MINUS_2_METER_IN_KM;
    StateVector->Velocity.X	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY + 3 * MPH_STATE_VECTOR_ENTRY_SIZE]) *
	TEN_MINUS_5_METER_PER_SEC_IN_KM_PER_SEC;
    StateVector->Velocity.Y	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY + 4 * MPH_STATE_VECTOR_ENTRY_SIZE]) *
	TEN_MINUS_5_METER_PER_SEC_IN_KM_PER_SEC;
    StateVector->Velocity.Z	= (float)DecLong2Long(( unsigned char*)&MPH[
									    MPH_STATE_VECTOR_ENTRY + 5 * MPH_STATE_VECTOR_ENTRY_SIZE]) *
	TEN_MINUS_5_METER_PER_SEC_IN_KM_PER_SEC;
    StateVector->OrbitNumber = 0;

    /*--------------------------------------------------------------------*\
    ** fill internal data package header from MPH
    \*--------------------------------------------------------------------*/
    for (i = 0; i < MPH_6_SIZE; i++)
	MPH_6_7_16[ i] = MPH[ MPH_6_ENTRY + i];
    for (i = 0; i < MPH_7_SIZE; i++)
	MPH_6_7_16[ i + MPH_6_SIZE] = MPH[ MPH_7_ENTRY + i];
    for (i = 0; i < MPH_16_SIZE; i++)
	MPH_6_7_16[ i + MPH_6_SIZE + MPH_7_SIZE] = MPH[ MPH_16_ENTRY + i];

    return Stat;
}


/*----------------------------------------------------------------------------*\
**                            MPH__Read
**  Input parameters:
**    Stream		file stream for output
**  Output parameters:
**    MPH		MPH of Level 0 file
**
**  Other interfaces:
**
**  Description:
**    This function reads the MPH from the input file.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MPH__read(char MPH[ MPH_SIZE], FILE *Stream)
{
    FunctionStatus_e Status = SUCCESS;
    long NrOfBytes = 0;

    /*------------------------------------------------------------------------*\
    ** read the MPH of the current file
    \*------------------------------------------------------------------------*/
    NrOfBytes = fread( MPH, 1, MPH_SIZE, Stream);
    if (NrOfBytes != MPH_SIZE)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1__102", Status, LOG_CANNOT_READ,
		 "MPH", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                            MPH__Write
**  Input parameters:
**    Stream		file stream for output
**    MPH		MPH of Level 0 file
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**    This function writes the MPH into the output file.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MPH__write(const char MPH[ MPH_SIZE], FILE *Stream)
{
    FunctionStatus_e Stat = SUCCESS;
    long NrOfBytes = 0;

    /*------------------------------------------------------------------------*\
    ** write the MPH of the current file
    \*------------------------------------------------------------------------*/
    NrOfBytes = fwrite( MPH, 1, MPH_SIZE, Stream);
    if (NrOfBytes != MPH_SIZE)
    {
	Stat = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__62", Stat, LOG_CANNOT_WRITE,
            	  "MPH", "");
    }

    return Stat;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
