/*----------------------------------------------------------------------------*\
**                                MPH.h
**
**  GDP Level 0 to 1 class MPH (main product header of level 0 tape) declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _MPH_h
#define _MPH_h

#pragma ident "@(#)MPH.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "utc.h"
#include "timecor.h"
#include "stattorc.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	MPH_SIZE	(176)

#define	MPH_PRODUCT_ID_ENTRY	(0)

#define	MPH_PRODUCT_ID_SIZE	(17)

#define	MPH_PRODUCT_TYPE_ENTRY	(17)

#define	MPH_SPACECRAFT_ID_ENTRY	(18)

#define	MPH_UTC_BEGIN_PRODUCT_ENTRY	(19)

#define	MPH_UTC_BEGIN_PRODUCT_SIZE	(24)

#define	MPH_STATION_ID_ENTRY	(43)

#define	MPH_STATION_ID_SIZE	(1)

#define	MPH_6_ENTRY	(44)

#define	MPH_6_SIZE	(2)

#define	MPH_7_ENTRY	(46)

#define	MPH_7_SIZE	(24)

#define	MPH_SPH_SIZE_ENTRY	(70)

#define	MPH_DSR_NO_ENTRY	(74)

#define	MPH_MPH_DSR_SIZE_ENTRY	(78)

#define	MPH_SUBSYS_PROD_GEN_ENTRY	(82)

#define	MPH_SAR_FALGS_ENTRY	(83)

#define	MPH_TCOR_UTC_TIME_ENTRY	(84)

#define	MPH_TCOR_UTC_TIME_SIZE	(24)

#define	MPH_TCOR_SBT_ENTRY	(108)

#define	MPH_TCOR_SBT_PERIODE_ENTRY	(112)

#define	MPH_16_ENTRY	(116)

#define	MPH_16_SIZE	(8)

#define	MPH_THRESHOLD_TABLE_VERSION_ENTRY	(124)

#define	MPH_SPARE_ENTRY	(126)

#define	MPH_SPARE_SIZE	(2)

#define	MPH_UTC_ASCENDING_NODE_ENTRY	(128)

#define	MPH_UTC_ASCENDING_NODE_SIZE	(24)

#define	MPH_STATE_VECTOR_ENTRY	(152)

#define	MPH_NO_OF_STATE_VECTOR_ENTRIES	(6)

#define	MPH_STATE_VECTOR_ENTRY_SIZE	(4)

/*------------------------------------------------------------------------*\
** conversion factors for velocity and position of state vector
\*------------------------------------------------------------------------*/

#define	TEN_MINUS_5_METER_PER_SEC_IN_KM_PER_SEC	(1.0E-08)

#define	TEN_MINUS_2_METER_IN_KM	(1.0E-05)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class MPH
\*============================================================================*/
typedef struct MPH {
    char Product_ID[MPH_PRODUCT_ID_SIZE];
    char Product_Type;
    char Spacecraft_ID;
    char UTC_Begin_Product[ MPH_UTC_BEGIN_PRODUCT_SIZE];
    char Station_ID;
    char ProductConfidenceData[ MPH_6_SIZE];
    char UTC_MPH_Generation[ MPH_7_SIZE];
    unsigned long SPH_Size;
    unsigned long NoOf_DSR;
    unsigned long DSR_Size;
    char SubsysProductGen;
    char SAR_Flags;
    char TCOR_UTC_Time[ MPH_TCOR_UTC_TIME_SIZE];
    unsigned long TCOR_SBT;
    unsigned long TCOR_SBT_Periode;
    char ProcessorSoftwareVersion[ MPH_16_SIZE];
    unsigned short ThresholdTableVersion;
    char Spare[ 2];
    char UTC_AscendingNode[ MPH_UTC_ASCENDING_NODE_SIZE];
    long StateVector[ MPH_NO_OF_STATE_VECTOR_ENTRIES];  
} MPH_s;


/*============================================================================*\
**  public operations of class Spectrum
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MPH__write(const char MPH[ MPH_SIZE], FILE *Stream);

FunctionStatus_e
MPH__read(char MPH[ MPH_SIZE], FILE *Stream);

FunctionStatus_e
MPH__extract(char MPH[ MPH_SIZE], UTC_s *ProductTime, TimeCor_s *TimeCor,
	     StateVectorC_s *StateVector,
	     char MPH_6_7_16[ MPH_6_SIZE + MPH_7_SIZE + MPH_16_SIZE]);

void
MPH__show(const MPH_s *MPH, FILE * Stream);


/*============================================================================*\
**  private operations of class Spectrum
\*============================================================================*/
 

#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
