/*----------------------------------------------------------------------------*\
**                              numrecip.h
**
**  Numerical Recipes in C declaration file
**
**  Version:    5.1
**  Date:       02/26/99
**  Author: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
#ifndef _Numrecip_h
#define _Numrecip_h 1

#pragma ident	"@(#)numrecip.h	5.1 02/26/99, Copyright DLR 1994"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"



#define	NATURAL_SPLINE	(0.99e30)

#define	MAX_AKIMA_POINTS	((4*DETECTOR_PIXELS))

/*----------------------------------------------------------------------------*\
**  Akima Interpolation
\*----------------------------------------------------------------------------*/
void akima_su (double x_arr[], double y_arr[], short n_source, double a_arr[], double b_arr[], double c_arr[], double d_arr[]);
void akima_ex (short n_source, double x_arr[], double y_arr[], double st[]);
void akima_po (short n_source, double x_arr[], double a[], double b[], double c[], double d[], double x, double *y);

/*----------------------------------------------------------------------------*\
**  Parabolic Interpolation and Brent's Method
\*----------------------------------------------------------------------------*/
double brent(double ax, double bx, double cx, double (*f)(double), double tol,
			double *xmin, FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Convolves or deconvolves a real data set
\*----------------------------------------------------------------------------*/
void convlv(double data[], short n, double respns[], short m, short isign, double ans[], FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Interpolates the values of a given spectrum to another wavelength
\*----------------------------------------------------------------------------*/
void fit_grid_akima(const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End);
void fit_grid_spline (const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End);
void fit_grid_linear(const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End);
float LinearInterpolation(float X, float X_left, float X_right, float Y_left, float Y_right);
void spline1( double x[], double y[], short n, double yp1, double ypn, double y2[]);
void splint1(double xa[],double ya[],double y2a[], short n,double x,double *y, FunctionStatus_e * Status);
void CalculatePolynome1(double X, double P[], short Np);
void CalculatePolynome(const float xin[], const float yin[], short nin, double pol_coeff[], short order);
void fit_grid_polyn(const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End, short order);

/*----------------------------------------------------------------------------*\
**  Replaces data by its discrete Fourier transform
\*----------------------------------------------------------------------------*/
void four1(double data[], short nn, short isign);

/*----------------------------------------------------------------------------*\
**  matrix operations
\*----------------------------------------------------------------------------*/
double **matrix__new(short nrl, short nrh, short ncl, short nch);
void matrix__delete(double  **m, short nrl, short nrh, short ncl, short nch);

/*----------------------------------------------------------------------------*\
**  Calculates the Fourier transform of a set of data
\*----------------------------------------------------------------------------*/
void realft(double data[], short n, short isign);

/*----------------------------------------------------------------------------*\
**  Find the root of a function using bisection
\*----------------------------------------------------------------------------*/
double rtbis(double (*func)(double), double x1, double x2, double xacc, FunctionStatus_e *Status);

/*----------------------------------------------------------------------------*\
**  Cubic-spline interpolation
\*----------------------------------------------------------------------------*/
void spline( double x[], double y[], short n, double yp1, double ypn, double y2[]);
void splint(double xa[],double ya[],double y2a[], short n,double x,double *y, FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Backsubstitution routine. Solves A*X = B
\*----------------------------------------------------------------------------*/
void svbksb(double **u,double w[],double **v, short m, short n,double b[],double x[]);

/*----------------------------------------------------------------------------*\
**  Singular value descomposition (SVD). A = U * W * V^T
\*----------------------------------------------------------------------------*/
void svdcmp( double **a, short m, short n, double *w, double **v, FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Backsubstitution routine. Solves A*X = B
\*----------------------------------------------------------------------------*/
void svdfit( double x[], double y[], double sig[], short ndata, double a[], short ma, double **u, double **v, double w[], double *chisq, void (* funcs)(double, double[], short), FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Evaluate the covariance matrix of the fit parameters obtained by svdfit
\*----------------------------------------------------------------------------*/
void svdvar( double **v, short ma, double w[], double **cvm);

/*----------------------------------------------------------------------------*\
**  Computes discrete Fourier transformations for two complex arrays
\*----------------------------------------------------------------------------*/
void twofft(double data1[],double data2[],double fft1[],double fft2[], short n);

/*----------------------------------------------------------------------------*\
**  vector operations
\*----------------------------------------------------------------------------*/
double *vector__new(short nl, short nh);
void  vector__delete(double  *v, short nl, short nh);

/*----------------------------------------------------------------------------*\
**  Find the root of a function using Brent's method
\*----------------------------------------------------------------------------*/
double zbrent(double (*func)(double), double x1, double x2, double tol, FunctionStatus_e * Status);

/*----------------------------------------------------------------------------*\
**  Smooth with triangular convolution
\*----------------------------------------------------------------------------*/
void triangle(float DataIn [DETECTOR_PIXELS], float DataOut [DETECTOR_PIXELS], short KernelHalfSize, short Begin, short End);
void convol(float DataIn[], float DataOut[], short Begin, short End, float Kernel[], short KernelHalfSize);

#ifdef __cplusplus
}               /* end C linkage */
#endif

#endif
