/*----------------------------------------------------------------------------*\
**                                PCD.h
**
**  Class PCD declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _PCD_h
#define _PCD_h 

#pragma ident "@(#)PCD.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sph1.h"
#include "pixelpar.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	PCD_GEOLOCATION_REC_SIZE	(GEOLOCATION_SIZE)

#define	PCD_DARK_CURRENT_AND_NOISE_COR_SIZE	(2 * FLOAT_SIZE)

#define	PCD_INDEX_SPECTRAL_CAL_PAR	(SHORT_SIZE)

#define	PCD_INDEX_LEAKAGE_COR_PAR	(SHORT_SIZE)

#define	PCD_POLARIZATION_PAR_SIZE	(POLARIZATION_SIZE)

#define	PCD_MPH_EXTRACT_SIZE	(MPH_L1_SIZE)

#define	PCD_SPH_EXTRACT_SIZE	(SPH_L1_SIZE)

#define	PCD_INSTRUMENT_HEADER_SIZE	(AUXILIARY_DATA_SIZE * SHORT_SIZE)

#define	PCD_SCIENCE_INDICE_SIZE	(SHORT_SIZE)

#define	PCD_SIZE	(PCD_GEOLOCATION_REC_SIZE + PCD_DARK_CURRENT_AND_NOISE_COR_SIZE + PCD_INDEX_SPECTRAL_CAL_PAR + PCD_INDEX_LEAKAGE_COR_PAR + PCD_POLARIZATION_PAR_SIZE + PCD_MPH_EXTRACT_SIZE + PCD_SPH_EXTRACT_SIZE + PCD_INSTRUMENT_HEADER_SIZE + SCIENCE_DATA_DEFINED * PCD_SCIENCE_INDICE_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum { 
    SEARCH_NOT_FOUND,
    SEARCH_FOUND,
    SEARCH_DOWN,
    SEARCH_UP
} Search_e;


/*============================================================================*\
**  class PCD
\*============================================================================*/
typedef struct PCD {
    GeolocationC_s Geolocation;
    float DarkCurrentCorFactor;
    float DarkCurrentNoiseCorFactor;
    short IndexSpectralCalPar;
    short IndexLeakagePar;
    PolarizationC_s PolarizationPar;
    char MPH_L1[ MPH_L1_SIZE];
    char SPH_L1[ SPH_L1_SIZE];
    GOME_Word_t AuxiliaryData[ AUXILIARY_DATA_SIZE];
    short ScienceIndex[ SCIENCE_DATA_DEFINED];
} PCD_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__ReadTime(FILE *Stream, long FilePos, long PCD_Size, UTC_s *PCD_Time,
	      PCD_s *PCD, Bool CONVERT_SUN_2_HOST);

FunctionStatus_e
PCD__SearchTime(FILE *Stream,
		FunctionStatus_e (*xCD__ReadTime)
		(FILE *Stream, long FilePos, long PCD_Size, UTC_s *PCD_Time, 
		 PCD_s *PCD, Bool convert_sun_2_host),
		long FirstFilePosPCD, short NoOf_PCD_Rec,
		long PCD_Size, UTC_s SearchTime, int MeasurementPixel,
		int *FoundMeasurementPixel, UTC_s *FoundTime,
		Bool convert_sun_2_host);

FunctionStatus_e
PCD__getTimeRange(FILE* Stream, 
  		  FunctionStatus_e (*xCD__ReadTime)
  		  (FILE* Stream, long FilePos, long PCD_Size, 
 		   UTC_s* PCD_Time, PCD_s* PCD, Bool convert_sun_2_host),
  		  long FilePosPCD, short NoOf_PCD_Rec, long PCD_Size,
 		  UTC_s* TimeFirst_PCD, UTC_s* TimeLast_PCD,
		  Bool convert_sun_2_host);

FunctionStatus_e
PCD__GetSpectrumParameter(const SPH1_s *SPH1, const PCD_s *PCD,
			  PixelPar_s *PixelPar);

FunctionStatus_e
PCD__read(PCD_s* self, FILE * Stream, Bool convert_sun_2_host);

void
PCD__show(const PCD_s* slef, FILE * Stream, Bool FullInfo);


FunctionStatus_e
PixelPar__Init(const SPH1_s *SPH1, int MeasurementPixel,
	       ProductType_e ProductType, const PCD_s *xCD, long xCD_Size,
	       PixelPar_s *PixelPar);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
