/*----------------------------------------------------------------------------*\
**                                PIR.h
**
**  Class PIR declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _PIR_h
#define _PIR_h 

#pragma ident "@(#)PIR.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	PIR_SATELLITE_ID_STRING	("E2")

#define	PIR_SENSOR_ID_STRING	("GOM")

#define	PIR_PROC_FAC_ID_STRING	("DP")


#define	PIR_SIZE	(38)

#define	PIR_SATELLITE_ID_ENTRY	(0)

#define	PIR_SATELLITE_ID_SIZE	(2)

#define	PIR_SENSOR_ID_ENTRY	(2)

#define	PIR_SENSOR_ID_SIZE	(3)

#define	PIR_ORBIT_NUMBER_ENTRY	(5)

#define	PIR_ORBIT_NUMBER_SIZE	(5)

#define	PIR_NUMBER_OF_ORBITS_ENTRY	(10)

#define	PIR_NUMBER_OF_ORBITS_SIZE	(4)

#define	PIR_ACQ_FAC_ID_ENTRY	(14)

#define	PIR_ACQ_FAC_ID_SIZE	(2)

#define	PIR_PRODUCT_TYPE_ENTRY	(16)

#define	PIR_PRODUCT_TYPE_SIZE	(5)

#define	PIR_SPACE_ENTRY	(21)

#define	PIR_PROC_FAC_ID_ENTRY	(22)

#define	PIR_PROC_FAC_ID_SIZE	(2)

#define	PIR_PROC_DATE_ENTRY	(24)

#define	PIR_PROC_DATE_SIZE	(8)

#define	PIR_PROC_TIME_ENTRY	(32)

#define	PIR_PROC_TIME_SIZE	(6)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class PIR
\*============================================================================*/
typedef struct PIR {
    char SatelliteId[PIR_SATELLITE_ID_SIZE];
    char SensorId[PIR_SENSOR_ID_SIZE];
    char StartOrbitNumber[PIR_ORBIT_NUMBER_SIZE];
    char ProcessedOrbits[PIR_NUMBER_OF_ORBITS_SIZE];
    char AcquisitionFacilityId[PIR_ACQ_FAC_ID_SIZE];
    char ProductType[PIR_PRODUCT_TYPE_SIZE];
    char Spare[1];
    char ProcessingFacilityId[PIR_PROC_FAC_ID_SIZE];
    char ProcessingDate[PIR_PROC_DATE_SIZE];
    char ProcessingTime[PIR_PROC_TIME_SIZE];
} PIR_s;


/*============================================================================*\
**  public operations of class Spectrum
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PIR__generate(char* PIR, unsigned long OrbitNumber, const char* ProductType,
	      const char* ProcessingInfo);

FunctionStatus_e
PIR__read(char PIR[PIR_SIZE], FILE* file_ptr);

FunctionStatus_e
PIR__write(const char PIR[PIR_SIZE], FILE* Stream);

void
PIR__show(const char PIR[PIR_SIZE], FILE* Stream);

FunctionStatus_e
PIR__writeExtractLevel_1_Product(const char PIR[PIR_SIZE], FILE* file_ptr);


/*============================================================================*\
**  private operations of class ...
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
