/*----------------------------------------------------------------------------*\
**                                PixelPar.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)PixelPar.c	5.1 02/26/99, Copyright DLR 1998"

#include <string.h>
#include "pixelpar.h"
#include "smcd.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)PixelPar.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("PixelPar")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
extern Debug_s* debug_pcd_ptr;
#endif



/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                PixelPar__Show
**  Input parameters:
**    Stream		output stream
**    PixelPar		pixel specific parameter
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**
**  Description:
**    Function to write the data structure PixelPar_s to the output stream
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	7.6.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PixelPar__show(const PixelPar_s *Param, FILE* Stream, ProductType_e ProductType,
	       Bool FullInfo)
{
    int	i;
    char	ASCII_Time[ 30];

    /*------------------------------------------------------------------------*\
    ** Show PixelPar
    \*------------------------------------------------------------------------*/
    if (UTC_2_ASCII( Param->Date, ASCII_Time) != SUCCESS)
    {
	/*
	  Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__75", ERROR,
	  LOG_NUM, "converting the UTC time of Solar Spectrum", "");
	  */
	strcpy(ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }
    if (FullInfo)
    {
	fprintf( Stream, "Measurement Pixel %5d  ", Param->MeasurementPixel);
	fprintf( Stream, "Date %s  %d  %d\n", 
		 ASCII_Time, Param->Date.Days, Param->Date.Days);
    }
    else
    {
	fprintf( Stream, "Measurement Pixel %5d  %s\n",
		 Param->MeasurementPixel, ASCII_Time);
    }
    fprintf( Stream, "Dark Current & Noise Correction Factor\n");
    fprintf( Stream, "%f %f\n\n", Param->DarkCurrentCorFactor,
	     Param->DarkCurrentNoiseCorFactor);

    fprintf( Stream, "Spectral Calibration Index %d\n",
	     Param->IndexSpectralCalPar);
    fprintf( Stream, "Leakage Parameter Index    %d\n",
	     Param->IndexLeakagePar);

    fprintf( Stream, "Science Data Index\n");
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	fprintf( Stream, "%s\t%d\n", ScienceData2Str((ScienceData_e)i),
		 Param->ScienceIndex[ i]);
    }
    fprintf( Stream, "\n");

    switch (ProductType)
    {
    case PR_SUNSHINE:
	{
	    fprintf( Stream, "Sun angles Zenith, Azimuth %f %f\n",
		     Param->Specific.Sun.SunAngle.Zenith,
		     Param->Specific.Sun.SunAngle.Azimuth);
	    fprintf( Stream, "BSDF angles Zenith, Azimuth %f %f\n",
		     Param->Specific.Sun.BSDF_Angle.Zenith,
		     Param->Specific.Sun.BSDF_Angle.Azimuth);
	    if (Param->Specific.Sun.UseInReference)
		fprintf( Stream, "used in sun reference spectrum\n");
	    else
		fprintf( Stream, "not used in sun reference spectrum\n"); 
	    break;
	}
    case PR_MOONSHINE:
	{
	    fprintf( Stream, "Sun angles Zenith, Azimuth %f %f\n",
		     Param->Specific.Moon.SunAngle.Zenith,
		     Param->Specific.Moon.SunAngle.Azimuth);
	    fprintf( Stream, "Moon angles Zenith, Azimuth %f %f\n",
		     Param->Specific.Moon.MoonAngle.Zenith,
		     Param->Specific.Moon.MoonAngle.Azimuth);
	    fprintf( Stream, "IlluminatedFraction %f\n",
		     Param->Specific.Moon.IlluminatedFraction);
	    break;
	}
    case PR_EARTHSHINE:
	{
	    if (FullInfo)
	    {
		GeolocationC__show(&Param->Specific.Earth.Geolocation, Stream);
		fprintf( Stream, "\n");
	    }
	    PolarizationC__show(&Param->Specific.Earth.PolarizationPar, Stream);
	    fprintf( Stream, "\n");
	    break;
	}
    }
}


/*----------------------------------------------------------------------------*\
**                                PixelPar__Get
**  Input parameters:
**    Stream			input stream
**    FilePosPCD		position of the PCD, SCD or MCD
**    PCD_Size			PCD, SCD or MCD size
**    SPH1			specific product header
**    ProductType		product type
**    MeasurementPixel		measurement pixel number
**
**  Output parameters:
**    PixelPar			structure containing the pixel parameter
**
**  Other interfaces:
**
**  Description:
**    Function to get the pixel parameter of the PCD or SCD or MCD into the
**    PixelPar struct. First the PCD or SCD or MCD is read, then the pixel
**    parameter are extracted.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PixelPar__Get(FILE* Stream, long FilePosPCD, long PCD_Size, const SPH1_s *SPH1,
	      ProductType_e ProductType, int MeasurementPixel,
	      PixelPar_s *PixelPar, Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e Status = SUCCESS;
    UTC_s PixelTime;
    PCD_s PCD;
    SMCD_s SMCD;
    PCD_s *xCD = 0;
    char ASCII_Time[ 30];
#ifdef DEBUG
    FILE* debug_file_ptr;
    char debug_message[255];
#endif

    /*----------------------------------------------------------------------*\
    ** read PCD, SCD or MCD
    \*----------------------------------------------------------------------*/
    switch (ProductType) 
    {
    case PR_EARTHSHINE:
	{
	    if(PCD__ReadTime( Stream, FilePosPCD, PCD_Size, &PixelTime, &PCD,
			      CONVERT_SUN_2_HOST) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__205", Status,
			  LOG_CANNOT_READ,
			  "PCD from level 1 product file", "");
		return Status;
	    }
	    xCD	= &PCD;
	    break;
	}
    case PR_SUNSHINE:
    case PR_MOONSHINE:
	{
	    if (SMCD__ReadTime(Stream, FilePosPCD, PCD_Size, &PixelTime,
			       (PCD_s*)&SMCD, CONVERT_SUN_2_HOST) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__220", Status,
			  LOG_CANNOT_READ,
			  "SCD or MCD from level 1 product file", "");
		return Status;
	    }
	    xCD	= (PCD_s*)&SMCD;
	    break;
	}
    default:
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.1__231", Status,
		     LOG_NUM, "Illegal product type", "");
	    return Status;
	}
    }

    /*----------------------------------------------------------------------*\
    ** check if read PCD, SCD or MCD was ok
    \*----------------------------------------------------------------------*/
    if (xCD == 0)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__243", Status, LOG_NUM,
		  "xCD == 0", "");
	return Status;
    }

#ifdef DEBUG
    UTC_2_ASCII( PixelTime, ASCII_Time);
    sprintf(debug_message, "*** Get Spectrum of xCD[ %d] %s Days: %u, "
	   "Msecs: %u\n", MeasurementPixel, ASCII_Time, PixelTime.Days,
	   PixelTime.Msecs);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);

    if ( (debug_file_ptr = Debug__getStream(debug_pcd_ptr, DBG_EXTRACTING, 
					    DEBUG_LEVEL_PCD)) != 0)
    {
	fprintf(debug_file_ptr, "*** Get Spectrum of xCD[ %d] %s Days: "
		"%u, Msecs: %u\n", MeasurementPixel, ASCII_Time,
		PixelTime.Days, PixelTime.Msecs);
	switch (ProductType) 
	{
	case PR_EARTHSHINE: 
	    {
		PCD__show(xCD, debug_file_ptr, True);
		break;
	    }
	case PR_MOONSHINE:
	case PR_SUNSHINE: 
	    {
		SMCD__show((SMCD_s*)xCD, debug_file_ptr, True);
		break;
	    }
	}
    }
#endif
	    
    /*--------------------------------------------------------------------*\
    ** get the pixel specific spectrum parameter
    \*--------------------------------------------------------------------*/
    if (PixelPar__Init( SPH1, MeasurementPixel, ProductType, xCD, 
			PCD_Size, PixelPar) != SUCCESS) 
    {
	Status = FATAL;
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.1__285", Status,
		 LOG_NUM, "PCD, Leakage Calibration Parameter Index "
		 "== INVALID_INDEX", "");
	return Status;
    }
    
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                WriteSpectrumParameterSplit
**  Input parameters:
**    Stream		Output stream of Extract Level 1 Product
**    ProductType	product type
**    PixelPar		pixel specific spectrum parameter
**    Nband		number of Bands of Measurement pixel
**
**  Output parameters:
**    WriteSpectrumParameterSplit	SUCCESS	= output ok
**					FATAL	= any error occcured
**
**  Other interfaces:
**    Output file of specified by the given Stream
**
**  Description:
**    Function to write the Spectrum parameter to given stream in split mode
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteSpectrumParameterSplit(FILE *Stream, ProductType_e ProductType,
			    const PixelPar_s *PixelPar, int Nband)
{
    FunctionStatus_e	Status		= SUCCESS;
    char			ASCII_Time[ 30];

    switch (ProductType) 
    {
	/*--------------------------------------------------------------------*\
	** write the Earthshine spectrum parameter
	\*--------------------------------------------------------------------*/
    case PR_EARTHSHINE: 
	{
	    fprintf(Stream, "Ground Pixel %4d %1d %1d\n",
		    PixelPar->MeasurementPixel + 1,
		    Nband, PixelPar->AuxiliaryPar.SubsetCounter);
	    if (ferror( Stream) != 0) 
	    {
		Status = FATAL;
		Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.1__340", Status, 
			 LOG_CANNOT_WRITE,
			 "spectrum parameter of earth into exract level 1 file",
			 "");
		return Status;
	    }
	    if (GeolocationC__write(&PixelPar->Specific.Earth.Geolocation, 
				   Stream) != SUCCESS) 
	    {
		Status = FATAL;
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__350", Status,
			  LOG_CANNOT_WRITE, "Geolocation record", "");
		return Status;
	    }
	    break;
	}

    /*------------------------------------------------------------*\
    ** write the Sunshine spectrum parameter
    \*------------------------------------------------------------*/
    case PR_SUNSHINE: 
	{
	    if (UTC_2_ASCII( PixelPar->Date, ASCII_Time) != SUCCESS) 
	    {
		Status = ERROR;
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__365", Status,
			  LOG_NUM, "converting the UTC time of Sun Spectrum",
			  "");
		strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
	    }
	    fprintf( Stream, "Sun Measurements %4d %1d\n",
		     PixelPar->MeasurementPixel + 1, Nband);
	    fprintf( Stream, "%s\n", ASCII_Time);
	    fprintf( Stream, "%6.2f %6.2f\n",
		     PixelPar->Specific.Sun.SunAngle.Zenith,
		     PixelPar->Specific.Sun.SunAngle.Azimuth);
	    fprintf( Stream, "%6.2f %6.2f\n",
		     PixelPar->Specific.Sun.BSDF_Angle.Zenith,
		     PixelPar->Specific.Sun.BSDF_Angle.Azimuth);
	    fprintf( Stream, "%1hd\n", PixelPar->Specific.Sun.UseInReference);	
	    if ( ferror(Stream) != 0) 
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__383", Status, LOG_CANNOT_WRITE,
			  "spectrum parameter of sun into exract level 1 file", "");
		return Status;
	    }
	    break;
	}

    /*------------------------------------------------------------*\
    ** write the Moonshine spectrum parameter
    \*------------------------------------------------------------*/
    case PR_MOONSHINE: 
	{
	    if (UTC_2_ASCII( PixelPar->Date, ASCII_Time) != SUCCESS) 
	    {
		Status = ERROR;
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__398", Status,
			  LOG_NUM, "converting the UTC time of Moon Spectrum",
			  "");
		strcpy(ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
	    }
	    fprintf( Stream, "Moon Measurements %4d %1d\n", 
		     PixelPar->MeasurementPixel + 1, Nband);
	    fprintf( Stream, "%s\n", ASCII_Time);
	    fprintf( Stream, "%6.2f %6.2f\n",
		     PixelPar->Specific.Moon.SunAngle.Zenith,
		     PixelPar->Specific.Moon.SunAngle.Azimuth);
	    fprintf( Stream, "%6.2f %6.2f\n",
		     PixelPar->Specific.Moon.MoonAngle.Zenith,
		     PixelPar->Specific.Moon.MoonAngle.Azimuth);
	    fprintf( Stream, "%7.5f\n",
		     PixelPar->Specific.Moon.IlluminatedFraction);
	    if ( ferror(Stream) != 0) 
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__417", Status, 
			  LOG_CANNOT_WRITE,
			  "spectrum parameter of moon into exract level 1 file",
			  "");
		return Status;
	    }
	    break;
	}

    default: 
	{
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__429", Status, LOG_NUM,
		      "illegal ProductType in write spectrum parameter", "");
	    return Status;
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                WriteSpectrumParameter
**  Input parameters:
**    Stream		Output stream of Extract Level 1 Product
**    ProductType	product type
**    PixelPar		pixel specific spectrum parameter
**    Nband		number of Bands of Measurement pixel
**
**  Output parameters:
**    WriteSpectrumParameter		SUCCESS	= output ok
**					FATAL	= any error occcured
**
**  Other interfaces:
**    Output file of specified by the given Stream
**
**  Description:
**    Function to write the Spectrum parameter to given stream
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteSpectrumParameter( FILE *Stream, ProductType_e ProductType,
			const PixelPar_s *PixelPar, int Nband,
			const short PMD_Flag[ PMD_DEFINED])
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i, j;

    /*--------------------------------------------------------------------*\
    ** write the split spectrum parameter
    \*--------------------------------------------------------------------*/
    if ( WriteSpectrumParameterSplit( Stream, ProductType, PixelPar, Nband) !=
	 SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__479", Status, LOG_CANNOT_WRITE,
		  "spectrum parameter into exract level 1 file", "");
	    return Status;
    }

    /*--------------------------------------------------------------------*\
    ** write PMD values
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "PMD");
    for ( i = 0; i < PMD_DEFINED; i++) 
    {
	fprintf( Stream, " %1hd", PMD_Flag[ i]);
    }
    fprintf( Stream, "\n");

    for( j = 0; j < PMD_SIZE; j++)
    {
	for ( i = 0; i < PMD_DEFINED; i++)
	{
	    fprintf( Stream, "%011.5e ", PixelPar->AuxiliaryPar.PMD[ i][ j]);
	}
	fprintf( Stream, "\n");
    }

    if ( ferror( Stream) != 0)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__506", Status,
		  LOG_CANNOT_WRITE,
		  "spectrum parameter (PMD) into exract level 1 file", "");
	    return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                        CheckGap
**  Input parameters: const char *ASCII_Date_prev, const char *ASCII_Date_cur
**  Output parameters: Status
**  Other interfaces:
**  Description:
**    Checks if the time gap between two subsequent ground pixels is bigger than 
**    1.5 sec.
**  References:
**  Libraries:
**  Created:	05 July 1995
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e CheckGap (const char *ASCII_Date_prev, const char *ASCII_Date_cur)
{
    FunctionStatus_e Status = SUCCESS;

    UTC_s	PrevTime;
    UTC_s	CurrentTime;
    UTC_s	DiffTime;
    UTC_s	IntgTime_UTC;

    unsigned long	TimeTol	= 10;	/* Time Tolerance in msec */

    IntgTime_UTC.Days	= 0;
    IntgTime_UTC.Msecs	= (unsigned long)(GROUND_PIXEL_TIME * 1000.0) + TimeTol;

    /* check if one of the times is an empty string */
    if (strlen(ASCII_Date_prev) == 0 || strlen(ASCII_Date_cur) == 0) return ERROR;

    Status = ASCII_2_UTC(ASCII_Date_prev, &PrevTime);

    if (Status == SUCCESS)
	Status = ASCII_2_UTC(ASCII_Date_cur, &CurrentTime);

    if (Status == SUCCESS)
	Status = UTC_Sub( PrevTime, IntgTime_UTC, &DiffTime);

    if (Status == SUCCESS)
    {
	if (UTC_Comp( '<', CurrentTime, DiffTime))
	{
	    /* there is a ground-pixel-gap */
	    Status = ERROR;
	}
    }
	
    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
