/*----------------------------------------------------------------------------*\
**                                PixelIndex.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)PixelIndex.c	5.1 02/26/99, Copyright DLR 1998"


#include "pixendex.h"
#include "log.h"



#define	SccsId	("@(#)PixelIndex.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("PixelIndex")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**  string buffer for log description messages
\*----------------------------------------------------------------------------*/
static char 			LogMoreDescription[ 300];

/*----------------------------------------------------------------------------*\
**                                PixelIndex__Initialize
**  Input parameters:
**    N1		Band separation of Band_2a and Band 2b
**    N2		Band separation of Band_1a and Band 1b
**    Channel		first and last pixel index of optical channel 0 to 3
**    Blind		first and last pixel index of Blind
**    Straylight	first and last pixel index of Straylight of optical
**				channel 0 to 1
**			
**  Output parameters:
**    PixelIndex	Pixel Index of all Science data (Band_1a..Straylight_2a)
**    PixelIndex__Initialize	SUCCESS	= PixelIndex initialized correct
**			else	= it was not possible to initialize PixelIndex
**
**  Other interfaces:
**
**  Description:
**    Function to set the Pixel Indices of all Science data defined.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.5.95
**  Author:	Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PixelIndex__initialize(PixelIndex_s PixelIndex[SCIENCE_DATA_DEFINED],
		       short N1, short N2, short Channel[OPTICAL_CHANNELS * 2],
		       short Blind[ 2], 
		       short Straylight[STRAYLIGHT_DEFINED * 2])		
{
    FunctionStatus_e	Status	= SUCCESS;
    int			Indx, cc;

    PixelIndex[ BAND_1A].First	= Channel[0];
    PixelIndex[ BAND_1A].Last	= N2;  
    PixelIndex[ BAND_1B].First	= N2 + 1;
    PixelIndex[ BAND_1B].Last	= Channel[1];
    PixelIndex[ BAND_2A].First	= Channel[2];
    PixelIndex[ BAND_2A].Last	= N1;  
    PixelIndex[ BAND_2B].First	= N1 + 1;
    PixelIndex[ BAND_2B].Last	= Channel[3];                      
    for (Indx = BAND_3, cc=4; Indx <= BAND_4; Indx++)
    {
	PixelIndex[ Indx].First	= Channel[cc++];
	PixelIndex[Indx].Last	= Channel[cc++];
    }
    PixelIndex[BLIND_1A].First	= Blind[0];
    PixelIndex[BLIND_1A].Last	= Blind[1];
    for ( Indx = STRAYLIGHT_1A, cc=0; Indx <= STRAYLIGHT_2A; Indx++)
    {
	PixelIndex[Indx].First	= Straylight[cc++];
	PixelIndex[Indx].Last	= Straylight[cc++];
    }

    /*--------------------------------------------------------------------*\
    **  Check the PixelIndex Limits
    \*--------------------------------------------------------------------*/
    for (Indx = 0; Indx < SCIENCE_DATA_DEFINED; Indx++)
    {
	if (PixelIndex[ Indx].First < 0)
	{
	    sprintf(LogMoreDescription, "of pixel index in %s [%d ..]",
		    ScienceData2Str( (ScienceData_e)Indx), PixelIndex[ Indx].First);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__96", FATAL,
		      LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		      "Check the initialization file");
	    Status = FATAL;
	}

	if (PixelIndex[ Indx].Last >= DETECTOR_PIXELS)
	{
	    sprintf(LogMoreDescription, "of pixel index in %s [.. %d]",
		    ScienceData2Str( (ScienceData_e)Indx), PixelIndex[ Indx].Last);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__106", FATAL,
		      LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		      "Check the initialization file");
	    Status = FATAL;
	}

	if (PixelIndex[ Indx].First > PixelIndex[ Indx].Last)
	{
	    sprintf(LogMoreDescription, "of pixel index in %s [%d %d]",
		    ScienceData2Str( (ScienceData_e)Indx), PixelIndex[ Indx].First,
		    PixelIndex[ Indx].Last);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__117", FATAL,
		      LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		      "Check the initialization file");
	    Status = FATAL;
	}
    }

    if (PixelIndex[ STRAYLIGHT_1A].First <= PixelIndex[ BLIND_1A].Last)
    {
	sprintf(LogMoreDescription, "of pixel index in %s [%d ..] and %s [.. %d]",
		ScienceData2Str( STRAYLIGHT_1A), PixelIndex[ STRAYLIGHT_1A].First,
		ScienceData2Str( BLIND_1A), PixelIndex[ BLIND_1A].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__129", FATAL,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file");
	Status = FATAL;
    }

    if (PixelIndex[ BAND_1A].First <= PixelIndex[ STRAYLIGHT_1A].Last)
    {
	sprintf(LogMoreDescription, "of pixel index in %s [%d ..] and %s [.. %d]",
		ScienceData2Str( BAND_1A), PixelIndex[ BAND_1A].First,
		ScienceData2Str( STRAYLIGHT_1A), PixelIndex[ STRAYLIGHT_1A].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__140", FATAL,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file");
	Status = FATAL;
    }

    if (PixelIndex[ STRAYLIGHT_1B].First <= PixelIndex[ BAND_1B].Last)
    {
	sprintf(LogMoreDescription, "of pixel index in %s [%d ..] and %s [.. %d]",
		ScienceData2Str( STRAYLIGHT_1B), PixelIndex[ STRAYLIGHT_1B].First,
		ScienceData2Str( BAND_1B), PixelIndex[ BAND_1B].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__151", FATAL,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file");
	Status = FATAL;
    }

    if (PixelIndex[ BAND_2A].First <= PixelIndex[ STRAYLIGHT_2A].Last)
    {
	sprintf( LogMoreDescription, "of pixel index in %s [%d ..] and %s [.. %d]",
		 ScienceData2Str( BAND_2A), PixelIndex[ BAND_2A].First,
		 ScienceData2Str( STRAYLIGHT_2A), PixelIndex[ STRAYLIGHT_2A].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__162", FATAL,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file");
	Status = FATAL;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PixelIndex__Update
**  Input parameters:
**    N1		Band separation of Band_2a and Band 2b
**    N2		Band separation of Band_1a and Band 1b
**			
**  Output parameters:
**    PixelIndex	Pixel Index of Band_1a..Band_2b
**    PixelIndex__Update	SUCCESS	= PixelIndex initialized correct
**			else	= it was not possible to initialize PixelIndex
**
**  Other interfaces:
**
**  Description:
**    Function to set the Pixel Indices of all Science data defined.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.5.95
**  Author:	Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PixelIndex__update(PixelIndex_s *PixelIndex, short N1, short N2)
{
    FunctionStatus_e Status = SUCCESS;

    if ((N2 < PixelIndex[ BAND_1A].First) || (N2 > PixelIndex[ BAND_1B].Last))
    {
	Status = FATAL;
	sprintf(LogMoreDescription, "of N2 (%d) in CHANNEL I [%d %d]",
		N2, PixelIndex[ BAND_1A].First, PixelIndex[ BAND_1B].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__205", Status,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file and/or the RawDataPackage");
    }
    if ((N1 < PixelIndex[BAND_2A].First) || (N1 > PixelIndex[BAND_2B].Last))
    {
	Status = FATAL;
	sprintf(LogMoreDescription, "of N1 (%d) in CHANNEL II [%d %d]",
		N1, PixelIndex[BAND_2A].First, PixelIndex[ BAND_2B].Last);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__214", Status,
		  LOG_RANGE_CHECK_ERROR, LogMoreDescription,
		  "Check the initialization file and/or the RawDataPackage");
    }

    if (Status == SUCCESS)
    {
	PixelIndex[BAND_1A].Last	= N2;
	PixelIndex[BAND_1B].First	= N2 + 1;
	PixelIndex[BAND_2A].Last	= N1;
	PixelIndex[BAND_2B].First	= N1 + 1;
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                                PixelIndex__show
**  Input parameters:
**	Stream		output stream
**	PixelIndex	pixel indices of all science data
**
**  Output parameters:
**
**  Other interfaces:
**    Output file specified by stream
**
**  Description:
**    Function to show the pixel indices of all science data
**
**  References:
**
**  Libraries:
**    ....
**  Created:	20.5.95
**  Author:	Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PixelIndex__show(PixelIndex_s *PixelIndex, FILE *Stream)
{
    int indx;

    fprintf( Stream, "Pixel Index\n");
    for (indx = 0; indx < SCIENCE_DATA_DEFINED; indx++)
    {
	fprintf( Stream, "ScienceData: %3d, First Pixel: %5d, Last Pixel: %5d\n",
		 indx, PixelIndex[ indx].First, PixelIndex[ indx].Last);
    }
}



/*============================================================================*\
**  private operations
\*============================================================================*/

 
