/*----------------------------------------------------------------------------*\
**                                PolarizationC.h
**
**  Class Polarization declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Polarization_h
#define _Polarization_h

#pragma ident "@(#)PolarizationC.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include <math.h>
#include "defs.h"
#include "keydrror.h"
#include "pixendex.h"
#include "p7_getry.h"
#include "posangle.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	POLARIZATION_COEFFICIENTS	(8)

#define	POLARIZATION_SIZE	(FLOAT_SIZE * POLARIZATION_COEFFICIENTS + FLOAT_SIZE * POLARIZATION_COEFFICIENTS + FLOAT_SIZE * POLARIZATION_COEFFICIENTS + FLOAT_SIZE)


#define	BIGNUM	(1.0e+37)

#define	BIGEXP	(700.0)

/*----------------------------------------------------------------------------*\
** Declaration of constants for PolarizationC__AdjustGeometry
** Obsolete since usage of ESA propagator
\*----------------------------------------------------------------------------*/
/*

#define	EARTH_RADIUS	(6370.0)

#define	SAT_HEIGHT	(780.0)

#define	ATM_HEIGHT	(30.0)
*/

/*

#define	RHO_N	(0.0279)
Changed : memo I.Aben/C.Tanzi 4 dec 1998
*/

#define	RHO_N	(0.032)


#define	SMALL	(1.0e-10)


#define	RAD_TO_DEG	((180.0 / M_PI))

#define	DEG_TO_RAD	((M_PI / 180.0))


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/
typedef enum { 
    PC_PMD_1 = 0,
    PC_PMD_2,
    PC_PMD_3,
    PC_OVERLAP_1,
    PC_OVERLAP_2,
    PC_OVERLAP_3,
    PC_P7,
    PC_P7_BAND_1A
} PolarizationCoefficients_e;

typedef enum { 
    ANGLE_1 = 0,
    ANGLE_2,
    ANGLE_3
} Angles_e;


/*============================================================================*\
**  class Polarization
\*============================================================================*/
typedef struct PolarizationC {
    float Coefficient[POLARIZATION_COEFFICIENTS]; /* Polarization
						     coefficients */
    float Wavelength[POLARIZATION_COEFFICIENTS]; /* Wavelength of the
						    corresponding polarization
						    coefficients */
    float Error[POLARIZATION_COEFFICIENTS]; /* Errors of the polarization
					       coefficients */
    float Chi;			/* Plane of polarization angle */
} PolarizationC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
PolarizationC_s*
PolarizationC__New(FILE *Stream, FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void 
PolarizationC__Delete(PolarizationC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
void 
PolarizationC__Apply(const PolarizationC_s *Self, ScienceData_e Band,
		    const float PolSensitivity[], 
		    const float SensitivityGrid[],
		    float Reticon[], const float Lambda[], short Begin, 
		    short End, const char *PixelDate);

FunctionStatus_e
PolarizationC__ApplyPMD(const PolarizationC_s* Self, PMD_Number_e Number,
		       float PMD[], const char *PixelDate);

void
PolarizationC__Compute(const PolarizationC_s *Self, const float Lambda[],
		      float BandCoefficients[], short Begin, short End, 
		      const char *PixelDate);

float
PolarizationC__CalculatePMD_Wavelength(const PixelIndex_s *PixelIndex, 
		      const float Detector[OPTICAL_CHANNELS][DETECTOR_PIXELS], 
		      const float *DetectorGrid[OPTICAL_CHANNELS], 
		      const float *Ksi[OPTICAL_CHANNELS], 
		      FunctionStatus_e *Status);

void 
PolarizationC__SetBands_for_PMD(PMD_Number_e loop_PMD,
			       short BandsForPMD [BAND_DEFINED]);

void
PolarizationC__ComputeCorFak(const PolarizationC_s *PolarizationPar,
			    const float Sensitivity[ DETECTOR_PIXELS],
			    const float SensitivityGrid[ DETECTOR_PIXELS],
			    const float Lambda[ DETECTOR_PIXELS],
			    short Begin, short End,
			    float Correction[ DETECTOR_PIXELS],
			    float BandCoefficients[ DETECTOR_PIXELS],
			    float Eta_Omega[ DETECTOR_PIXELS],
			    const char *PixelDate);

void
PolarizationC__ComputePolFakError(const PolarizationC_s *PolarizationPar,
				 const float Sensitivity[DETECTOR_PIXELS],
				 const float SensitivityGrid[DETECTOR_PIXELS],
				 const float Lambda[ DETECTOR_PIXELS], 
				 ScienceData_e Band, short Begin, short End,
				 const KeyDataError_s *KeyDataError, 
				 float PolCorFak[ DETECTOR_PIXELS],
				 float ErrPolCor[ DETECTOR_PIXELS],
				 const char *PixelDate);

void
PolarizationC__InterpolGeometry (float IntgTime,
				const PosAngle_s SunAngleOrig[], 
				const PosAngle_s LoS_AngleOrig[], 
				PosAngle_s SunAngle[], 
				PosAngle_s LoS_Angle[]);

float
PolarizationC__Calculate_p7(float IntgTime, const P7_Geometry_s* AnglesOrig, 
			   float *p_seven_error, float *ParamChi, 
			   FunctionStatus_e *Status, 
			   const char *PixelDate);

FunctionStatus_e
PolarizationC__Calculate_p7_mean(PolarizationC_s *PolarizationParP7_Mean,
				PolarizationC_s Polarization[], 
				short NoOfPol, char *ASCII_Time);

FunctionStatus_e
PolarizationC__AdjustGeometry(float theta_sat, float theta_0_sat,
			     float phi_phi_0_sat, float earth_radius,
			     float sat_height, float smallval,
			     float atm_height,float * theta, 
			     float *theta_0, float * phi_phi_0,
			     Bool * do_arbitrary, Bool * do_twilight);

FunctionStatus_e
PolarizationC__SeventhPoint(float theta_deg, float theta_0_deg,
			   float phi_minus_phi_0_deg, float rho_n, 
			   float* p_seven, float* p_seven_error, 
			   float* X_s_s, float * Q_I_ratio, float smallval,
			   const char* PixelDate);

FunctionStatus_e
PolarizationC__read(PolarizationC_s* self, FILE* Stream,
		   Bool convert_sun_2_host);

void
PolarizationC__show(const PolarizationC_s* self, FILE* Stream);

void
PolarizationC__Sun2Host(PolarizationC_s *self);


/*============================================================================*\
**  private operations
\*============================================================================*/
double
gdf_function(double x);

void
PolarizationC__GDF(float grad_first, float x0, float p0, float x1, float p1, 
		  const float *Lambda, float *BandCoefficients, 
		  short Begin, short End, const char *PixelDate);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
