/*----------------------------------------------------------------------------*\
**                                QA.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)QA.c	5.2 08/19/02, Copyright DLR 1998"

#include <assert.h>
#include "qa.h"
#include "sfs.h"
#include "log.h"


#define	SccsId	("@(#)QA.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("QA")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                QA__WriteSpectral
**  Input parameters:
**	Stream		output stream
**	Param		FCD structure
**
**  Output parameters:
**
**  Other interfaces:
**	QA output stream (Qualitiy Assurance file *.qad)
**
**  Description:
**	Function to write the first and last wavelength of each channel
**	as a spectral calibration health check.
**
**  References:
**  Libraries:
**  Created:	24.10.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
QA__WriteSpectral( FILE * Stream, const FCD_s *Param)
{
    int	temp, order, channel;
    double	xfac;

    /*--------------------------------------------------------------------*\
    **  show spectral parameter
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "\n\n");
    fprintf( Stream, "First and last wavelength per channel\n\n");
    fprintf( Stream, "Channel_1         Channel_2         Channel_3         Channel_4         \n");
    for ( temp = 0; temp < Param->Nspec; temp++)
    {
	if ( SPECTRAL_FITT_ORDER >= 4)
	{
	    for ( channel = 0; channel < OPTICAL_CHANNELS; channel++)
	    {
		/* first pixel
		fprintf( Stream, "%-8.3f ", Param->SpectralPar[ temp].Coefficient[ channel][ 0] );
		*/

		/* first pixel */
		xfac = (double) 10;

		fprintf( Stream, "%-8.3f ", 
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 0]) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 1] * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 2] * xfac * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 3] * xfac * xfac * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 4] * xfac * xfac * xfac * xfac)
			 );

		/* last pixel */
		xfac = (double) DETECTOR_PIXELS - 11;

		fprintf( Stream, "%-8.3f ", 
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 0]) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 1] * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 2] * xfac * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 3] * xfac * xfac * xfac) +
			 (Param->SpectralPar[ temp].Coefficient[ channel][ 4] * xfac * xfac * xfac * xfac)
			 );
	    }
	    fprintf( Stream, "\n");
	}
	else
	{
	    /* something is wrong, print zeros */
	    for ( channel = 0; channel < OPTICAL_CHANNELS; channel++)
	    {
				/* first pixel */
		fprintf( Stream, "%-8.3f ", 0.0);

				/* last pixel */
		fprintf( Stream, "%-8.3f ", 0.0);
	    }
	    fprintf( Stream, "\n");
	}
    }
}

/*----------------------------------------------------------------------------*\
**                                QA__WriteSun
**  Input parameters:
**    Config_s *Config	Configuration (flags, output stream ...)
**    SDR		Sun Data Record
**    SolarGrid		Wavelength grid for the sun measurement
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**	QA output stream (Qualitiy Assurance file *.qad)
**
**  Description:
**	Function to write the irradiance for later evaluation
**	as a calibration and data health check for Qualitiy Assurance purposes
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	5.11.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
			Adopted from SDR__Write
\*----------------------------------------------------------------------------*/
FunctionStatus_e
QA__WriteSun(const Config_s *Config, const FCD_s *FCD, const SDR_s *SDR,
	     float SolarGrid[ OPTICAL_CHANNELS][ DETECTOR_PIXELS])
{
    FunctionStatus_e Status = SUCCESS;
    int i, j;
    OpticalChannel_e Channel;
    OpticalChannel_e OldChannel;
    float StartWavelSolar[ OPTICAL_CHANNELS];
    float EndWavelSolar[ OPTICAL_CHANNELS];
    long Nsolar[ OPTICAL_CHANNELS];
    char DeadPixel;
    char HotPixel;
    char SaturatedPixel;
    char SpectralCheck;
    short Flag	= 0;
    static Bool QA_HeaderWritten = False;

    /*----------------------------------------------------------------------*\
    ** get the subflags from Quality Flag
    \*----------------------------------------------------------------------*/
    DeadPixel	= (Flag & 0x3);
    HotPixel	= (Flag & 0xC) / (char)0x4;
    SaturatedPixel= (Flag & 0x30) / (char)0x10;
    SpectralCheck	= (Flag & 0xC0) / (char)0x40;

    /*--------------------------------------------------------------------------*\
    ** calculate the length of the channel, determine the wavelength range
    \*--------------------------------------------------------------------------*/
    for ( i = 0; i < OPTICAL_CHANNELS; i++)
	Nsolar[ i] = 0;
    OldChannel = (OpticalChannel_e)-1;
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
    {
	Channel = Band2Channel( (ScienceData_e)i);
	Nsolar[ Channel] += FCD->BCR[ i].EndDetector - FCD->BCR[ i].StartDetector +
	    1;
	if (OldChannel != Channel)
	{
	    OldChannel		= Channel;
	    StartWavelSolar[ Channel]	= SolarGrid[ Channel]
		[ FCD->BCR[ i].StartDetector];
	}
	EndWavelSolar[ Channel]	= SolarGrid[ Channel]
	    [ FCD->BCR[ i].EndDetector];
    }

    /*--------------------------------------------------------------------------*\
    ** write the solar spectrum summary
    \*--------------------------------------------------------------------------*/
    fprintf( Config->QA_ProductStream, "\n\n");
    SFS__Write( Config->QA_ProductStream, Config, FCD);
    fprintf( Config->QA_ProductStream, "\n");

    /*--------------------------------------------------------------------------*\
    ** write the whole solar spectrum information
    \*--------------------------------------------------------------------------*/
    OldChannel = (OpticalChannel_e)-1;
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
	for ( j = FCD->BCR[ i].StartDetector; j <= FCD->BCR[ i].EndDetector; j++)
	{
	    Channel = Band2Channel( (ScienceData_e)i);
	    if (OldChannel != Channel)
	    {
		OldChannel = Channel;
		fprintf( Config->QA_ProductStream,
			 "Channel %01d %08.4f %08.4f %04d %06.4f %1d %1d %1d %1d\n",
			 Channel + 1,
			 StartWavelSolar[ Channel],
			 EndWavelSolar[ Channel],
			 Nsolar[ Channel], 
			 FCD->SpectralPar[ FCD->IndexSpectralCalPar].Error[ Channel],
			 SpectralCheck,
			 SaturatedPixel,
			 HotPixel,
			 DeadPixel);
	    }
	    fprintf( Config->QA_ProductStream,
		     "%08.4f %011.5e %011.5e %011.5e %01d\n",
		     SolarGrid[ Channel][ j],
		     SDR->Irradiance[ Channel][ j],
		     SDR->AbsErrorIrr[ Channel][ j],
		     SDR->RelErrorIrr[ Channel][ j],
		     SDR->Flag[ Channel][ j]);
	    if ( ferror( Config->QA_ProductStream) != 0)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__221", Status, LOG_CANNOT_WRITE,
			  "SDR into exract level 1 file", "");
		return Status;
	    }
	}


    /*--------------------------------------------------------------------------*\
    ** Write file header
    \*--------------------------------------------------------------------------*/
    if (!QA_HeaderWritten) {
	fprintf( Config->QA_ProductStream, "\n");
	fprintf( Config->QA_ProductStream, "\n");

	fprintf( Config->QA_ProductStream, "Mean values of first channel pixels - ");
	fprintf( Config->QA_ProductStream, "Ratios of fractional polarisation values - ");
	fprintf( Config->QA_ProductStream, "Ratios of PMD values and radiances\n\n");
	
	fprintf( Config->QA_ProductStream, "Pixel# ");
	fprintf( Config->QA_ProductStream, "Mean_Ch1 Mean_Ch2 Mean_Ch3 Mean_Ch4 ");
	
	fprintf( Config->QA_ProductStream, "p1/p7    p2/p1    p3/p1    ");
	
	fprintf( Config->QA_ProductStream, "S_PMD1       S_PMD2       S_PMD3 \n");
	fprintf( Config->QA_ProductStream, "                                    ");
	fprintf( Config->QA_ProductStream, "                                    ");
	fprintf(Config->QA_ProductStream, "/Ch2(%-4d)   /Ch3(%-4d)   /Ch4(%-4d)\n\n", 
		Config->Filter.qa_pixelnumber,
		Config->Filter.qa_pixelnumber,
		Config->Filter.qa_pixelnumber);
	
	QA_HeaderWritten = True;
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                                QA__WriteLeakage
**  Input parameters:
**    Config_s *Config	Configuration (flags, output stream ...)
**    PixelPar		pixel specific parameter
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**	QA output stream (Qualitiy Assurance file *.qad)
**
**  Description:
**	Function to write leakage, fpa, fixed and straylight calibrated data
**	as a calibration and data health check for Qualitiy Assurance purposes
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	24.10.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
QA__writeLeakage(Config_s *Config, const EDR_s *EDR, 
		 const FCD_s *FCD)
{
    int index;
    int band;
    int channel;
    float PixelMean[OPTICAL_CHANNELS];
    ScienceData_e Bands[OPTICAL_CHANNELS] = {BAND_1A, BAND_2A, BAND_3, BAND_4};
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------------*\
    ** Check if file is open
    \*--------------------------------------------------------------------------*/
    if (Config->QA_ProductStream == NULL)
    {
	sprintf( LogBuffer, " Quality Assurance file (*.qad) is not open");
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__298", FATAL, LOG_CANNOT_WRITE,
		 LogBuffer, "");
	return FATAL;
    }

    /*------------------------------------------------------------------------*\
    ** Check if data are available (if integration is completed)
    ** same check in QA__WriteEDR !
    \*------------------------------------------------------------------------*/
    if ( (Config->Filter.bandFilter[ BAND_2B]) 
	 && (EDR->PixelPar.ScienceIndex[ BAND_2B] != INVALID_INDEX) 
	 && (EDR->Spectrum.Radiance[OPTICAL_CH_2]
	     [Config->Filter.qa_pixelnumber] > 10e-20)
	 && (Config->Filter.bandFilter[ BAND_3]) 
	 && (EDR->PixelPar.ScienceIndex[ BAND_3] != INVALID_INDEX) 
	 && (EDR->Spectrum.Radiance[OPTICAL_CH_3]
	     [Config->Filter.qa_pixelnumber] > 10e-20) 
	 && (Config->Filter.bandFilter[ BAND_4]) &&
	 (EDR->PixelPar.ScienceIndex[ BAND_4] != INVALID_INDEX) &&
	 (EDR->Spectrum.Radiance[OPTICAL_CH_4]
	  [Config->Filter.qa_pixelnumber] > 10e-20) 
	 && (EDR->ProductType == PR_EARTHSHINE)	) {
    }
    else {
	return SUCCESS;
    }

    /*------------------------------------------------------------------------*\
    ** Write Pixel#
    \*------------------------------------------------------------------------*/
    fprintf( Config->QA_ProductStream, "%-6d ", EDR->PixelPar.MeasurementPixel);

    /*------------------------------------------------------------------------*\
    ** Check if data are available (if integration is completed)
    \*------------------------------------------------------------------------*/
    for (channel = OPTICAL_CH_1; channel <= OPTICAL_CH_4; channel++) {
	/*----------------------------------------------------------------*\
	** Calculate mean value of first pixels
	\*----------------------------------------------------------------*/
	PixelMean[channel] = 0;
	band = Bands[channel];

	if ( (Config->Filter.bandFilter[ band]) &&
	     (EDR->PixelPar.ScienceIndex[ band] != INVALID_INDEX) &&
	     (EDR->ProductType == PR_EARTHSHINE) ) {
	    if (Config->Filter.qa_meannumber > (FCD->BCR[ band].EndDetector - FCD->BCR[ band].StartDetector)) 
		Config->Filter.qa_meannumber = FCD->BCR[ band].EndDetector - FCD->BCR[ band].StartDetector;

	    for (index = FCD->BCR[ band].StartDetector; 
		 index < FCD->BCR[ band].StartDetector + Config->Filter.qa_meannumber; index++)
		PixelMean[channel] += EDR->Spectrum.Radiance[channel][index];

	    PixelMean[channel] /= Config->Filter.qa_meannumber;
	}

	/*----------------------------------------------------------------*\
	** Write mean value of first pixels
	\*----------------------------------------------------------------*/
	fprintf(Config->QA_ProductStream, "%-8.3f ", PixelMean[channel]);
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                QA__WriteEDR
**  Input parameters:
**    Config_s *Config	Configuration (flags, output stream ...)
**    EDR_s *EDR	Earth data record
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**	QA output stream (Qualitiy Assurance file *.qad)
**
**  Description:
**	Function to write calibrated and derived data
**	as a calibration and data health check for Qualitiy Assurance purposes
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	24.10.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
QA__writeEDR( const Config_s *Config, const EDR_s *EDR)
{
    int index;
    int pmd;
    int band;
    float p1, p2, p3, p7;
    float PMDsum[PMD_DEFINED];
    float IntgTime;
    int number_PMD;
    float ratio1 = 0.0;
    float ratio2 = 0.0;
    float ratio3 = 0.0;
    float sza;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------------*\
    ** Check if file is open
    \*--------------------------------------------------------------------------*/
    if (Config->QA_ProductStream == NULL)
    {
	sprintf( LogBuffer, " Quality Assurance file (*.qad) is not open");
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__408", FATAL, LOG_CANNOT_WRITE,
		 LogBuffer, "");
	return FATAL;
    }

    /*--------------------------------------------------------------------------*\
    ** Check if data are available (if integration is completed)
    ** same check in QA__WriteEDR !
    \*--------------------------------------------------------------------------*/
    if (
	(Config->Filter.bandFilter[ BAND_2B]) &&
	(EDR->PixelPar.ScienceIndex[ BAND_2B] != INVALID_INDEX) &&
	(EDR->Spectrum.Radiance[OPTICAL_CH_2][Config->Filter.qa_pixelnumber] > 10e-20) &&

	(Config->Filter.bandFilter[ BAND_3]) &&
	(EDR->PixelPar.ScienceIndex[ BAND_3] != INVALID_INDEX) &&
	(EDR->Spectrum.Radiance[OPTICAL_CH_3][Config->Filter.qa_pixelnumber] > 10e-20) &&

	(Config->Filter.bandFilter[ BAND_4]) &&
	(EDR->PixelPar.ScienceIndex[ BAND_4] != INVALID_INDEX) &&
	(EDR->Spectrum.Radiance[OPTICAL_CH_4][Config->Filter.qa_pixelnumber] > 10e-20) &&

	(EDR->ProductType == PR_EARTHSHINE)
	)
    {
    }
    else
	return SUCCESS;

    /*--------------------------------------------------------------------------*\
    ** Write some ratios of fractional polarisation values
    \*--------------------------------------------------------------------------*/
    p1 = EDR->PixelPar.Specific.Earth.PolarizationPar.Coefficient[ PC_PMD_1];
    p2 = EDR->PixelPar.Specific.Earth.PolarizationPar.Coefficient[ PC_PMD_2];
    p3 = EDR->PixelPar.Specific.Earth.PolarizationPar.Coefficient[ PC_PMD_3];
    p7 = EDR->PixelPar.Specific.Earth.PolarizationPar.Coefficient[ PC_P7];

    if (p1 <= 1.0 && p1 >= 0 && p7 <= 1.0 && p7 >= 0)
	fprintf( Config->QA_ProductStream, "%-7.3f  ", p1 / p7);
    else
	fprintf( Config->QA_ProductStream, "%-7.3f  ", 0.0);

    if (p1 <= 1.0 && p1 >= 0 && p2 <= 1.0 && p2 >= 0)
	fprintf( Config->QA_ProductStream, "%-7.3f  ", p2 / p1);
    else
	fprintf( Config->QA_ProductStream, "%-7.3f  ", 0.0);

    if (p1 <= 1.0 && p1 >= 0 && p3 <= 1.0 && p3 >= 0)
	fprintf( Config->QA_ProductStream, "%-7.3f  ", p3 / p1);
    else
	fprintf( Config->QA_ProductStream, "%-7.3f  ", 0.0);

    /*--------------------------------------------------------------------------*\
    ** Calculate sum of PMD readouts
    \*--------------------------------------------------------------------------*/
    for ( pmd = 0; pmd < PMD_DEFINED; pmd++)
    {
	switch (pmd)
	{
	case PMD_1: band = BAND_2B; break;
	case PMD_2: band = BAND_3; break;
	case PMD_3: band = BAND_4; break;
	default: band = BAND_4;
	}

	IntgTime = EDR->Spectrum.BandRecord[ band].IntgTime;
	number_PMD = (int) (PMD_SIZE * IntgTime / GROUND_PIXEL_TIME);

	PMDsum[pmd] = 0;
	for( index = PMD_SIZE - 1; (index >= PMD_SIZE - number_PMD) 
		 && (index >= 0); index--)
	{
	    assert(index >= 0);
	    PMDsum[pmd] += EDR->PixelPar.AuxiliaryPar.PMD[ pmd][ index];
	}
    }
    /*--------------------------------------------------------------------------*\
    ** Calculate ratios of PMD and reticon values if SZA smaller or equal 
    ** than 96.0
    \*--------------------------------------------------------------------------*/
    sza = EDR->PixelPar.Specific.Earth.Geolocation.NorthSunAngle[1].Zenith;

    if (sza <= 96.0)
    {
      ratio1 = PMDsum[PMD_1] / EDR->Spectrum.Radiance[OPTICAL_CH_2][Config->Filter.qa_pixelnumber];
      ratio2 = PMDsum[PMD_2] / EDR->Spectrum.Radiance[OPTICAL_CH_3][Config->Filter.qa_pixelnumber];
      ratio3 = PMDsum[PMD_3] / EDR->Spectrum.Radiance[OPTICAL_CH_4][Config->Filter.qa_pixelnumber];
    }

    /*--------------------------------------------------------------------------*\
    ** Write ratios
    \*--------------------------------------------------------------------------*/
    fprintf(Config->QA_ProductStream, "%-12.5g %-12.5g %-12.5g \n", ratio1, ratio2, ratio3);

    return SUCCESS;
}



/*============================================================================*\
**  private operations
\*============================================================================*/

 
