/*----------------------------------------------------------------------------*\
**                                QualityCheck.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)QualityCheck.c	5.1 02/26/99, Copyright DLR 1998"

#include "qualheck.h"
#include "filter.h"


#define	SccsId	("@(#)QualityCheck.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("QualityCheck")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                QualityCheck__CheckSaturation
**  Input parameters:
**    Band (1A .. 4)		which band
**    Reticon data		to be checked
**    Begin 			first pixel in 1024 array
**    End 			last pixel in 1024 array
**    date 			of the ground pixel as ASCII string
**  Output parameters:
**    Flags array		indicating whether or not the detector pixel is saturated
**    Bool detected		set true if at least one detector pixel in the band is saturated
**  Other interfaces:
**  Description:
**    Checks all detector pixels for saturation and sets the flag. The saturation
**    limits are given as constants.
**  References:
**  Libraries:
**  Created:	4-July-96
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
QualityCheck__CheckSaturation(ScienceData_e Band, const float * Reticon, 
			      short * Flags, short Begin, short End, 
			      const char *PixelDate)
{
	int loop;
	float SaturationLimit;
	Bool detected = False;

	/*-----------------------------------------------------------------------*\
	** Check for Saturation and flag saturated pixels
	\*-----------------------------------------------------------------------*/
	switch (Band)
	{
		case BAND_1A:
		case BAND_1B:
			SaturationLimit = SATURATION_CHANNEL_1;
			break;
		case BAND_2A:
		case BAND_2B:
			SaturationLimit = SATURATION_CHANNEL_2;
			break;
		case BAND_3:
			SaturationLimit = SATURATION_CHANNEL_3;
			break;
		case BAND_4:
			SaturationLimit = SATURATION_CHANNEL_4;
			break;
		default:
			SaturationLimit = 65535;
	}

	for (loop = Begin; loop <= End; loop++)
	{
		if (Reticon[loop] >= SaturationLimit && (Flags[loop] >= QF_SATURATED || Flags[loop] == QF_GOOD) )
		{
			Flags[loop] = QF_SATURATED;
			detected = True;
		}
	}

	return detected;
}


/*============================================================================*\
**  private operations
\*============================================================================*/


