/*----------------------------------------------------------------------------*\
**                              realft.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function twofft
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)realft.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"

/*----------------------------------------------------------------------------*\
**                                realft
**  Input parameters:
**    Vector data[1..2*nm]
**    int n, isign
**  Output parameters:
**    Vector data[1..2*nm]
**  Other interfaces: none
**  Description:
**    Calculates the Fourier transform of a set of 2n real-valued data
**    points. Replaces this data by the positive frequency half of its
**    comples Fourier Transform. The real-valued first and last
**    components of the complex transform are returned as elements
**    data[1] and data[2] respectively. n must be a power of 2. This
**    routine also calculates the inverse transform of a complex data
**    array if it is the transform of real data (results must be
**    multiplied by 1/n).
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    27.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void realft(double data[], short n, short isign)
{
	short i,i1,i2,i3,i4,n2p3;
	double c1=0.5,c2,h1r,h1i,h2r,h2i;
	double wr,wi,wpr,wpi,wtemp,theta;

	theta=3.141592653589793/(double) n;
	if (isign == 1) {
		c2 = -0.5;
		four1(data,n,1);
	} else {
		c2=0.5;
		theta = -theta;
	}
	wtemp=sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi=sin(theta);
	wr=1.0+wpr;
	wi=wpi;
	n2p3=2*n+3;
	for (i=2;i<=n/2;i++) {
		i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
		h1r=c1*(data[i1]+data[i3]);
		h1i=c1*(data[i2]-data[i4]);
		h2r = -c2*(data[i2]+data[i4]);
		h2i=c2*(data[i1]-data[i3]);
		data[i1]=h1r+wr*h2r-wi*h2i;
		data[i2]=h1i+wr*h2i+wi*h2r;
		data[i3]=h1r-wr*h2r+wi*h2i;
		data[i4] = -h1i+wr*h2i+wi*h2r;
		wr=(wtemp=wr)*wpr-wi*wpi+wr;
		wi=wi*wpr+wtemp*wpi+wi;
	}
	if (isign == 1) {
		data[1] = (h1r=data[1])+data[2];
		data[2] = h1r-data[2];
	} else {
		data[1]=c1*((h1r=data[1])+data[2]);
		data[2]=c1*(h1r-data[2]);
		four1(data,n,-1);
	}
}
