/*----------------------------------------------------------------------------*\
**                              rtbis.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function rtbis
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)rtbis.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "log.h"
#include "numrecip.h"


#define	SS_ID	("rtbis")


#define	JMAX	(40)
/* Maximum allowed number of bisections */ /*----------------------------------------------------------------------------*\
**                                rtbis
**  Input parameters:
**    double (*func)(double)]
**    double x1, x2, xacc
**  Output parameters:
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Using bisection, find the root of a function func known to lie between
**    x1 and x2. The root, returned as rtbis, will be refined until its
**    accuracy is +/- xacc
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    29.7.94
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double rtbis(double (*func)(double), double x1, double x2, double xacc, FunctionStatus_e *Status)
{
	int j;
	double dx,f,fmid,xmid,rtb;

	*Status = SUCCESS;
	f=(*func)(x1);
	fmid=(*func)(x2);
	if (f*fmid >= 0.0)
	{
		/*
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_50", ERROR, LOG_NUMRECIP, "Root must be bracketed for bisection", "");
		*/
		*Status = ERROR;
		return -1;
	}
	rtb = f < 0.0 ? (dx=x2-x1,x1) : (dx=x1-x2,x2);
	for (j=1;j<=JMAX;j++) {
		fmid=(*func)(xmid=rtb+(dx *= 0.5));
		if (fmid <= 0.0) rtb=xmid;
		if (fabs(dx) < xacc || fmid == 0.0) return rtb;
	}
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_61", ERROR, LOG_NUMRECIP, "Too many bisections", "");
	*Status = ERROR;
	return rtb;
}

