/*----------------------------------------------------------------------------*\
**                                ScienceData.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)ScienceData.c	5.1 02/26/99, Copyright DLR 1998"

#include "sciedata.h"
#include "log.h"


#define	SccsId	("@(#)ScienceData.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("ScienceData")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                Band2Channel
**  Input parameters:
**    ScienceData	science data enumerator
**  Output parameters:
**    Band2Channel	returned channel value corresponding to the given
**			science data enumerator
**  Other interfaces:
**  Description:
**    This function to returns the channel value of the given science data
**    enumerator. If the science data enumerator is out of range the function
**    terminates.
**  References:
**  Libraries:
**  Created:    24.5.94
**  Author: Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
OpticalChannel_e
Band2Channel( ScienceData_e ScienceData)
{
    switch (ScienceData)
    {
    case BAND_1A:
    case BAND_1B:
    case BLIND_1A:
    case STRAYLIGHT_1A:
    case STRAYLIGHT_1B:
	return OPTICAL_CH_1;
    case BAND_2A:
    case BAND_2B:
    case STRAYLIGHT_2A:
	return OPTICAL_CH_2;
    case BAND_3:
	return OPTICAL_CH_3;
    case BAND_4:
	return OPTICAL_CH_4;
    default:
	{
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__59", FATAL, LOG_NUM,
		      "enum ScienceData_e out of range", "");
	    terminateExecution("*.log");
	}
    }
}


/*----------------------------------------------------------------------------*\
**                                ScienceData2Str
**  Input parameters:
**    Enum ScienceData enumerator
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    A string representing the ScienceData_e Enum is returned
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    24.1.95
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
const char *ScienceData2Str(ScienceData_e Enum)
{
    static const char *ScienceDataStr[] = { "BAND_1A", "BAND_1B", "BAND_2A", "BAND_2B", "BAND_3", "BAND_4", "BLIND_1A", "STRAYLIGHT_1A", "STRAYLIGHT_1B", "STRAYLIGHT_2A" };
    return ScienceDataStr[Enum];
}


/*============================================================================*\
**  private operations
\*============================================================================*/


