/*----------------------------------------------------------------------------*\
**                                Science.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Science.c	5.1 02/26/99, Copyright DLR 1998"

#include "science.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)Science.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Science")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           Science__Sun2Host
**  Input parameters:
**    Science			in Sun memory representation
**  Output parameters:
**    Science			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Science struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Science__Sun2Host( Science_s *Param, long DataSize)
{
    int	i;

    /* short */
    Param->Flag = SunShort2Short( (unsigned char*)&Param->Flag);
    /* short */
    Param->CalibrationIndx = SunShort2Short( (unsigned char*)
					     &Param->CalibrationIndx);
    /* short */
    Param->KeyDataCalibrationIndx = SunShort2Short( (unsigned char*)
						    &Param->KeyDataCalibrationIndx);
    /* short */
    Param->IntgTime = SunShort2Short( (unsigned char*)
				      &Param->IntgTime);
    for ( i = 0; i < DataSize; i++)
    {
	/* short */
	Param->Data[ i] = SunShort2Short( (unsigned char*)&Param->Data[ i]);
    }
}

/*----------------------------------------------------------------------------*\
**                              Science__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Science Data parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all Science Data parameter from the given 
**	input stream into the Science Data structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Science__read(Science_s *Param, FILE* Stream, long DataSize,
	      Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** read Quality Flags
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->Flag, 1, sizeof( Param->Flag), Stream);

    /*--------------------------------------------------------------------*\
    ** read index to KeyDataCalibration
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->KeyDataCalibrationIndx, 1,
			sizeof( Param->KeyDataCalibrationIndx), Stream);

    /*--------------------------------------------------------------------*\
    ** read index to Calibration record
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->CalibrationIndx, 1,
			sizeof( Param->CalibrationIndx), Stream);

    /*--------------------------------------------------------------------*\
    ** read Integration Time
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->IntgTime, 1, sizeof( Param->IntgTime), Stream);

    /*--------------------------------------------------------------------*\
    ** read data array
    \*--------------------------------------------------------------------*/
/*
  for ( int i = 0; i < DataSize; i++)
  {
  NoOfBytes += fread( &Param->Data[ i], 1, sizeof( Param->Data[ i]), Stream);
  }
  */
    NoOfBytes += fread( Param->Data, 1,
			sizeof( Param->Data[ 0]) * (unsigned int)DataSize, Stream);

    /*--------------------------------------------------------------------*\
    ** check size
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != SCIENCE_STAT_SIZE + DataSize * SCIENCE_DATA_ENTRY_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Science Data record, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__139", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (CONVERT_SUN_2_HOST)
    {
	Science__Sun2Host( Param, DataSize);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           Science__Show
**  Input parameters:
**	Stream		output stream
**	Param		Science Data Structure
**
**  Output parameters:
**
**  Other interfaces:
**	Science Data Structure written to the output stream
**
**  Description:
**	Function to write the Science Data Structure to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Science__show(const Science_s *Param,  FILE* Stream, long DataSize)
{
    int	i;

    fprintf( Stream, "\n*** Science Data\n\n");

    /*--------------------------------------------------------------------*\
    ** show Integration Time, Quality flag, Calibration index
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Quality Flag               : %d\n", Param->Flag);
    fprintf( Stream, "KeyDataCalibration Index : %d\n",
	     Param->KeyDataCalibrationIndx);
    fprintf( Stream, "Calibration Index          : %d\n", Param->CalibrationIndx);
    fprintf( Stream, "IntegrationTime            : %hu\n", Param->IntgTime);

    /*--------------------------------------------------------------------*\
    ** show data array
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "\nIndex Data\n");
    for ( i = 0; i < DataSize; i++)
    {
	fprintf( Stream, "%3d %hx\n", i, Param->Data[ i]);
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
