/*----------------------------------------------------------------------------*\
**                                SDR.c
**
**  Description:
** 
**  Version:    5.3
**  Date:       12/21/01
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SDR.c	5.3 12/21/01, Copyright DLR 1998"

#include <stdlib.h>
#include "sdr.h"
#include "wat2phot.h"
#include "log.h"
#include "edr.h"
#include "bsdf.h"
#include "xcortion.h"
#include "specralc.h"
#include "kuru_ref.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)SDR.c 5.3 12/21/01, Copyright DLR 1998")


#define	SS_ID	("SDR")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/


/*----------------------------------------------------------------------------*\
**                                SDR__Init_XCorr
**  Input parameters:
**      self: a Spectrum_s structure
**
**  Output parameters:
**      self:
**
**  Return value:
**
**  Description:
**      Copy the mean sun spectrum and a template (e.g. Kurucz) spectrum into
**			Spectrum_s structures as input for the X-Correlation routine
**
**  References:
**
**  Libraries:
**      ....
**
**  Created:	07.12.2001
**
**  Author:	Bernd Aberle, DLR/IMF-AP
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
SDR__Init_XCorr(float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS], FCD_s *FCD, SDR_s* SDR,
	Spectrum_s mean_sun[OPTICAL_CHANNELS], Spectrum_s template_spectrum[OPTICAL_CHANNELS][N_TEMPLATE_FILE])
{
    int ipix, ich, itmpl, ikur;
		int istart, iend, npix;
		
    /*--------------------------------------------------------------------------*\
		** sun spectrum
    \*--------------------------------------------------------------------------*/		
		for (ich = 0; ich < OPTICAL_CHANNELS; ich++)
		{
			switch (ich)
			{
				case OPTICAL_CH_1 : 
					istart = FCD->BCR[BAND_1A].StartDetector;
					iend   = FCD->BCR[BAND_1B].EndDetector;
					break;
				case OPTICAL_CH_2 : 
					istart = FCD->BCR[BAND_2A].StartDetector;
					iend   = FCD->BCR[BAND_2B].EndDetector;
					break;
				case OPTICAL_CH_3 : 
					istart = FCD->BCR[BAND_3].StartDetector;
					iend   = FCD->BCR[BAND_3].EndDetector;
					break;
				case OPTICAL_CH_4 : 
					istart = FCD->BCR[BAND_4].StartDetector;
					iend   = FCD->BCR[BAND_4].EndDetector;
					break;
			}

			npix = iend - istart + 1;

    	mean_sun[ich].re_ptr   = (float*) malloc( npix * sizeof(float) );
    	mean_sun[ich].grid_ptr = (float*) malloc( npix * sizeof(float) );

    	mean_sun[ich].im_ptr = 0;
    	mean_sun[ich].number_of_values = npix;
    	mean_sun[ich].has_grid = True;
    	mean_sun[ich].has_constant_sampling_intervall = False;

    	for (ipix = 0; ipix < npix; ipix++)
			{
 				mean_sun[ich].grid_ptr[ipix] = SolarGrid[ich][istart + ipix];
				/* mean_sun[ich].re_ptr[ipix]   = FCD->IntensityPar.MeanSunReference[ich][istart + ipix]; */
				mean_sun[ich].re_ptr[ipix]   = SDR->Irradiance[ich][istart + ipix];
    	}

    	/*--------------------------------------------------------------------------*\
			** kurucz reference spectrum
    	\*--------------------------------------------------------------------------*/
			for (itmpl = 0; itmpl < N_TEMPLATE_FILE; itmpl++)
			{
				ikur = ich * N_TEMPLATE_FILE + itmpl; 
			
    		template_spectrum[ich][itmpl].re_ptr   = (float*) malloc( n_kurucz[ikur] * sizeof(float) );
    		template_spectrum[ich][itmpl].grid_ptr = (float*) malloc( n_kurucz[ikur] * sizeof(float) );

    		template_spectrum[ich][itmpl].im_ptr = 0;
    		template_spectrum[ich][itmpl].number_of_values = n_kurucz[ikur];
    		template_spectrum[ich][itmpl].has_grid = True;
    		template_spectrum[ich][itmpl].has_constant_sampling_intervall = True;

				if (ich == OPTICAL_CH_1 && itmpl == 0)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_1_1[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_1_1[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_1 && itmpl == 1)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_1_2[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_1_2[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_1 && itmpl == 2)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_1_3[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_1_3[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_2 && itmpl == 0)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_2_1[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_2_1[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_2 && itmpl == 1)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_2_2[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_2_2[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_2 && itmpl == 2)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_2_3[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_2_3[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_3 && itmpl == 0)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_3_1[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_3_1[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_3 && itmpl == 1)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_3_2[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_3_2[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_3 && itmpl == 2)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_3_3[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_3_3[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_4 && itmpl == 0)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_4_1[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_4_1[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_4 && itmpl == 1)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_4_2[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_4_2[1][ipix];
    			}
				}
				else if (ich == OPTICAL_CH_4 && itmpl == 2)
				{
    			for (ipix=0; ipix < n_kurucz[ikur]; ipix++)
					{
						template_spectrum[ich][itmpl].grid_ptr[ipix] = kurucz_4_3[0][ipix];
						template_spectrum[ich][itmpl].re_ptr[ipix]   = kurucz_4_3[1][ipix];
    			}
				}
				else
				{
					Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__230", FATAL,
						LOG_NUM, "Unexpected channel/segment combination for Kurucz reference spectrum", "");

					return FATAL;
				}
				
			} /* for (itmpl... */
		} /* for (ich... */
				
    return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**                                SDR__Delete_XCorr
**  Input parameters:
**      self: a Spectrum_s structure
**
**  Output parameters:
**      self:
**
**  Return value:
**
**  Description:
**      Free memory which has been allocated for the X-Correlation routine
**
**  References:
**
**  Libraries:
**      ....
**
**  Created:	07.12.2001
**
**  Author:	Bernd Aberle, DLR/IMF-AP
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
SDR__Delete_XCorr(Spectrum_s mean_sun[OPTICAL_CHANNELS], Spectrum_s template_spectrum[OPTICAL_CHANNELS][N_TEMPLATE_FILE])
{
	int ich, itmpl;

	for (ich = 0; ich < OPTICAL_CHANNELS; ich++)
	{
    free(mean_sun[ich].re_ptr);
    free(mean_sun[ich].grid_ptr);

		for (itmpl = 0; itmpl < N_TEMPLATE_FILE; itmpl++)
    {
			free(template_spectrum[ich][itmpl].re_ptr);
    	free(template_spectrum[ich][itmpl].grid_ptr);
		}
	}

  return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**                                SDR__Show_XCorr
**  Input parameters:
**
**  Output parameters:
**      self:
**
**  Return value:
**
**  Description:
**      Print calculated X-correlation parameters
**
**  References:
**
**  Libraries:
**
**  Created:	07.12.2001
**
**  Author:	Bernd Aberle, DLR/IMF-AP
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
SDR__Show_XCorr(FILE* Stream, XC_Coeff_s* Param)
{
	int i, j;
	
	fprintf(Stream, "\n");
	fprintf(Stream, "X-Correlation coefficients\n");

	for ( i = 0; i < MAX_FITT_ORDER; i++)
	{
		for ( j = 0; j < OPTICAL_CHANNELS; j++)
			fprintf( Stream, "\t%e", Param->Coefficient[ j][ i]);
		fprintf( Stream, "\n");
	}

	fprintf(Stream, "\n");
	fprintf(Stream, "X-Correlation coefficient errors\n");
	for ( i = 0; i < MAX_FITT_ORDER; i++)
	{
		for ( j = 0; j < OPTICAL_CHANNELS; j++)
			fprintf( Stream, "\t%e", Param->CoefficientError[ j][ i]);
		fprintf( Stream, "\n");
	}

	fprintf(Stream, "\n");
	fprintf( Stream, "Spectral Error (standard deviation) \n");
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
		fprintf( Stream, "\t%lf", Param->Error[ j]);
	fprintf( Stream, "\n");
	fprintf( Stream, "\n");
	
	fprintf( Stream, "Maximum spectral Error (standard deviation)\n");
	fprintf( Stream, "\t%lf\n", TOTAL_STDDEV_LIMIT);
	
	return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**                                SDR__Init
**  Input parameters:
**	Config		Extract configuration structure
**	FCD		Fixed Calibration Parameter structure
**
**  Output parameters:
**	SDR		initialized SDR structure
**	SolarGrid	Solar grid belonging to the given SDR
**
**  Other interfaces:
**    <none>
**
**  Description:
**    Function to initialize the Solar Description Record (SDR).
**    The initialization steps are:
**	- calculate the grid of the solar spectrum
**	- calculate the Irradiance.
**	- define the wavelength range of the solar spectrum
**	- convert from watts to photons if all sun specific calibration
**        switches are set
**	- calculate the absolute error, the relative error and the
**	  flags of the Irradiance.
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDR__init(SDR_s* SDR, float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	  const Config_s *Config, FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    FunctionStatus_e XCorrStatus[OPTICAL_CHANNELS] = {SUCCESS, SUCCESS, SUCCESS, SUCCESS};
    int	i, j, iband, ich, ipix, ispectr;
    float SolarAzimuth = 0.0;

		char LogBuffer[LOG_FIELD_SIZE];

		Spectrum_s sun_spectrum[OPTICAL_CHANNELS];
		Spectrum_s template_spectrum[OPTICAL_CHANNELS][N_TEMPLATE_FILE];		
		XC_Coeff_s xcorr_coeffs;

    /*------------------------------------------------------------------------*\
    ** set default values for parameter
    \*------------------------------------------------------------------------*/
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
		{
			for ( j = FCD->BCR[ i].StartDetector; j <= FCD->BCR[ i].EndDetector; j++)
			{
	  	  SDR->Irradiance[ Band2Channel( (ScienceData_e)i)][ j]  = 0.0;
	    	SDR->AbsErrorIrr[ Band2Channel( (ScienceData_e)i)][ j] = 0.0;
	    	SDR->RelErrorIrr[ Band2Channel( (ScienceData_e)i)][ j] = 0.0;
	    	SDR->Flag[ Band2Channel( (ScienceData_e)i)][ j]        = 0;
			}
		}

    /*--------------------------------------------------------------------------*\
    ** calculate the grid of the solar spectrum
    \*--------------------------------------------------------------------------*/
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
		{
			SpectralC__apply( &FCD->SpectralPar[ FCD->IndexSpectralCalPar],
			 (ScienceData_e)i,
			 SolarGrid[ Band2Channel( (ScienceData_e)i)],
			 FCD->BCR[ (ScienceData_e)i].StartDetector,
			 FCD->BCR[ (ScienceData_e)i].EndDetector,
			 FCD->IndexSpectralCalPar);
		}
				
    /*------------------------------------------------------------------------*\
    **  correct mean sun for degradation (aging)
    \*------------------------------------------------------------------------*/
    if (Config->Filter.correctDegradationFlag == True)
    {
      for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
      {
        if (DegradationParameter__Apply(SolarGrid[Band2Channel((ScienceData_e)i)], 
					FCD->IntensityPar.MeanSunReference[Band2Channel((ScienceData_e)i)],
					&(FCD->IntensityPar.Date),
					(ScienceData_e)i,
					FCD->BCR[ (ScienceData_e)i].StartDetector,
					FCD->BCR[ (ScienceData_e)i].EndDetector)
                        != SUCCESS)
        {
          Status = FATAL;
          Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__430", Status,
	    LOG_NUM, "Error while correcting mean sun degradation", "");

          return Status;
        }
      }
    }

    /*------------------------------------------------------------------------*\
    **  correct mean sun for BSDF azimuth dependancy
    \*------------------------------------------------------------------------*/
    if (Config->Filter.correct_BSDF_Flag == True)
    {
      if ( BSDF__CalculateMeanSunAzimuth (&SolarAzimuth, FCD->IntensityPar.Date) != SUCCESS )
      {
        Status = FATAL;
        Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__446", Status,
          LOG_NUM, "azimuth for mean sun reference", "");

        return Status;
      }

      for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
      {
        if (BSDF__CorrectAzimuth(SolarAzimuth,
			(ScienceData_e)i,
			FCD->IntensityPar.MeanSunReference[Band2Channel((ScienceData_e)i)],
			SolarGrid[Band2Channel((ScienceData_e)i)], 
			FCD->BCR[ (ScienceData_e)i].StartDetector,
			FCD->BCR[ (ScienceData_e)i].EndDetector,
			&(FCD->IntensityPar.Date) )
                        != SUCCESS)
        {
          Status = FATAL;
          Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__464", Status,
						LOG_NUM, "Error while correcting mean sun degradation", "");

          return Status;
        }
      }
    }

    /*-------------------------------------------------------------------------*\
    ** calculate the Irradiance
    \*-------------------------------------------------------------------------*/
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
    {
			/*---------------------------------------------------------------------*\
			** divide mean sun reference spectrum by instrument response function)
			\*---------------------------------------------------------------------*/
			IntensityC__CalculateSunCalibrated( &FCD->IntensityPar,
					   (ScienceData_e)i,
					   SDR->Irradiance[ Band2Channel( (ScienceData_e)i)],
					   FCD->BCR[ (ScienceData_e)i].StartDetector,
					   FCD->BCR[ (ScienceData_e)i].EndDetector);
		}

    /*--------------------------------------------------------------------------*\
    ** define the wavelength range of the solar spectrum
    \*--------------------------------------------------------------------------*/
    SDR->StartWavelSolar = SolarGrid[ Band2Channel( Config->SDR_StartBand)]
			[ FCD->BCR[ Config->SDR_StartBand].StartDetector];
    SDR->EndWavelSolar = SolarGrid[ Band2Channel( Config->SDR_EndBand)]
			[ FCD->BCR[ Config->SDR_StartBand].EndDetector];

    /*---------------------------------------------------------------------*\
    **  convert units 
    \*---------------------------------------------------------------------*/
    if ((Config->Filter.calibrationFilter[ CAL_LEAKAGE]) &&
			(Config->Filter.calibrationFilter[ CAL_FIXED]) &&
			(Config->Filter.calibrationFilter[ CAL_STRAYLIGHT]) &&
			(Config->Filter.calibrationFilter[ CAL_NORMALIZE]) &&
			(Config->Filter.calibrationFilter[ CAL_INTENSITY]) &&
			(Config->Filter.calibrationFilter[ CAL_UNITS]))
    {
			for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
				for ( j = FCD->BCR[ i].StartDetector; j <= FCD->BCR[ i].EndDetector; j++)
					SDR->Irradiance[ Band2Channel( (ScienceData_e)i)][ j] = Watt2Photon(
										    SDR->Irradiance[ Band2Channel( (ScienceData_e)i)][ j],
										    SolarGrid[ Band2Channel( (ScienceData_e)i)][ j]);
    }

    /*------------------------------------------------------------------------*\
    **  correct mean sun wavelength using X-Correlation (Fraunhofer lines)
    \*------------------------------------------------------------------------*/
		if (Config->Filter.xCorrelationFlag == True)
		{			
			if ( (SDR__Init_XCorr(SolarGrid, FCD, SDR, sun_spectrum, template_spectrum) == SUCCESS) )
			{
				/*--------------------------------------------------------------------------*\
				** calculate correction polynome and standard deviations
				\*--------------------------------------------------------------------------*/
				for ( ich = (int) OPTICAL_CH_1; ich <= (int) OPTICAL_CH_4; ich++)
				{
					XCorrStatus[ich] = Spectrum__wavelengthCalibration(&(sun_spectrum[ich]), (OpticalChannel_e) ich,
						template_spectrum[ich], &xcorr_coeffs);
				}

				/*--------------------------------------------------------------------------*\
				** output for FCD file : polynome, stddev
				\*--------------------------------------------------------------------------*/
				if (Config->Filter.calFileFlag && Config->FCD_ProductStream != NULL)
				{
					SDR__Show_XCorr(Config->FCD_ProductStream, &xcorr_coeffs);
				}

				/*--------------------------------------------------------------------------*\
				** evaluate polynome and add onto SolarGrid
				** evaluate stand.dev. of polynome and stddev
				\*--------------------------------------------------------------------------*/
				for (iband = Config->SDR_StartBand; iband <= Config->SDR_EndBand; iband++)
				{
					ich = Band2Channel( (ScienceData_e) iband);
					
					if (xcorr_coeffs.Error[ich] <= TOTAL_STDDEV_LIMIT && XCorrStatus[ich] == SUCCESS)
					{
#ifdef DEBUG
						printf("Applying x-correlation for band %d\n", iband);
#endif 

						ispectr = 0;
						for (ipix = FCD->BCR[iband].StartDetector; ipix <= FCD->BCR[iband].EndDetector; ipix++)
						{
							SolarGrid[ich][ipix] -= Spectrum__EvalPolynom_d(
								(double) sun_spectrum[ich].grid_ptr[ispectr], xcorr_coeffs.Coefficient[ich], MAX_FITT_ORDER);
							ispectr++;
						}
					}
				}
				
    		/*--------------------------------------------------------------------------*\
    		** define the wavelength range of the solar spectrum
    		\*--------------------------------------------------------------------------*/
    		SDR->StartWavelSolar = SolarGrid[ Band2Channel( Config->SDR_StartBand)]
					[ FCD->BCR[ Config->SDR_StartBand].StartDetector];
    		SDR->EndWavelSolar = SolarGrid[ Band2Channel( Config->SDR_EndBand)]
					[ FCD->BCR[ Config->SDR_StartBand].EndDetector];

			}			
			SDR__Delete_XCorr(sun_spectrum, template_spectrum);
		}

    /*--------------------------------------------------------------------------*\
    ** calculate the absolute error and the relative error and the flags
    \*--------------------------------------------------------------------------*/
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
		{
			for ( j = FCD->BCR[ i].StartDetector; j <= FCD->BCR[ i].EndDetector; j++)
			{
				SDR->AbsErrorIrr[ Band2Channel( (ScienceData_e)i)][ j] = 0.0;
				SDR->RelErrorIrr[ Band2Channel( (ScienceData_e)i)][ j] = 0.0;
				SDR->Flag[ Band2Channel( (ScienceData_e)i)][ j] = 0;
			}
		}
		
  return Status;
}


/*----------------------------------------------------------------------------*\
**                                SDR__Write
**  Input parameters:
**	Config		Extract configuration structure
**	FCD		Fixed Calibration Parameter structure
**	SDR		Solar Description Record
**	SolarGrid	Solar Grid belonging to the SDR
**
**  Output parameters:
**    SDR__Write	SUCCESS	= output ok
**			else	= any error during output
**
**  Other interfaces:
**    output file of Extract Level 1 Product specified in Config
**
**  Description:
**    Function to write the whole solar spectrum information onto the Level 1 
**    Extracted Product
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDR__write(const SDR_s* SDR, float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	   FILE *Stream, const Config_s *Config, const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    int i, j;
    OpticalChannel_e Channel;
    OpticalChannel_e OldChannel;
    float StartWavelSolar[OPTICAL_CHANNELS];
    float EndWavelSolar[OPTICAL_CHANNELS];
    long Nsolar[OPTICAL_CHANNELS];
    char DeadPixel;
    char HotPixel;
    char SaturatedPixel;
    char SpectralCheck;
    short Flag = 0;

    /*------------------------------------------------------------------------*\
    ** get the subflags from Quality Flag
    \*------------------------------------------------------------------------*/
    DeadPixel = (Flag & 0x3);
    HotPixel = (Flag & 0xC) / (char)0x4;
    SaturatedPixel = (Flag & 0x30) / (char)0x10;
    SpectralCheck = (Flag & 0xC0) / (char)0x40;

    /*------------------------------------------------------------------------*\
    ** calculate the length of the channel, determine the wavelength range
    \*------------------------------------------------------------------------*/
    for (i = 0; i < OPTICAL_CHANNELS; i++)
	Nsolar[i] = 0;
    OldChannel = (OpticalChannel_e)-1;
    for (i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
    {
	Channel = Band2Channel( (ScienceData_e)i);
	Nsolar[Channel] += FCD->BCR[i].EndDetector - FCD->BCR[i].StartDetector
	    + 1;
	if (OldChannel != Channel)
	{
	    OldChannel = Channel;
	    StartWavelSolar[Channel] = SolarGrid[Channel][FCD->BCR[i].StartDetector];
	}
	EndWavelSolar[Channel] = SolarGrid[Channel][FCD->BCR[i].EndDetector];
    }

    /*------------------------------------------------------------------------*\
    ** write the whole solar spectrum information
    \*------------------------------------------------------------------------*/
    OldChannel = (OpticalChannel_e) - 1;
    for (i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
	for (j = FCD->BCR[i].StartDetector; j <= FCD->BCR[i].EndDetector; j++)
	{
	    Channel = Band2Channel( (ScienceData_e)i);
	    if (OldChannel != Channel)
	    {
		OldChannel = Channel;
		fprintf(Stream, "Channel %01d %08.4f %08.4f %04d %06.4f %1d "
			"%1d %1d %1d\n", Channel + 1, StartWavelSolar[ Channel],
			 EndWavelSolar[ Channel], Nsolar[ Channel], 
			 FCD->SpectralPar[FCD->IndexSpectralCalPar].Error[Channel],
			 SpectralCheck, SaturatedPixel, HotPixel, DeadPixel);
	    }
	    fprintf(Stream, "%08.4f %011.5e %011.5e %011.5e %01d\n",
		    SolarGrid[Channel][j], SDR->Irradiance[Channel][j],
		     SDR->AbsErrorIrr[ Channel][j],
		     SDR->RelErrorIrr[ Channel][j],
		     SDR->Flag[ Channel][j]);
	    if (ferror(Stream) != 0)
	    {
		Status = FATAL;
		Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__684", Status,
			 LOG_CANNOT_WRITE, "SDR into exract level 1 file", "");
		return Status;
	    }
	}

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SDR__CalculateErrors
**  Input parameters:
**    FCD			Fixed calibrartion record of the Level 1 Product
**    SolarGrid			grid of Sun Spectrum
**    SDR			Solar description record structure
**
**  Output parameters:
**    SDR			Solar description record structure
**    SDR__CalculateErrors	SUCCESS	= error calculation of Spectrum ok
**				else	= any error occured during error
**						calculation
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the absolute and relative error of the Solar
**    spectrum.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*---------------------------------------------------------------------------*/
FunctionStatus_e
SDR__calculateErrors(SDR_s* SDR, const Config_s *Config, const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    int i, j;
    ScienceData_e Band;
    OpticalChannel_e Ch;
    float RadResp;
    float AbsErrorRadianceResponse;
    float AbsErrorMeanSunRef;
    char LogBuffer[LOG_FIELD_SIZE];
/*
  float			RelErrResponse;
  float			RelErr_F2;
  float			RelErr_SMDEP;
  float			RelErrorRadianceResponse;
  */

    /*---------------------------------------------------------------------*\
    ** calculate absolute and relative error
    \*---------------------------------------------------------------------*/
    for (i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
    {
	Band = (ScienceData_e)i;
	Ch = Band2Channel(Band);
	for (j = FCD->BCR[i].StartDetector; j <= FCD->BCR[ i].EndDetector; j++)
	{
	    /*----------------------------------------------------------------*\
	    ** calculate absolute error of Mean sun reference
	    \*----------------------------------------------------------------*/
	    AbsErrorMeanSunRef = FCD->IntensityPar.RelativeError[ Ch][ j] *
		FCD->IntensityPar.MeanSunReference[ Ch][ j];
	    /*----------------------------------------------------------------*\
	    ** calculate absolute error extract format (E1)
	    \*----------------------------------------------------------------*/
	    SDR->AbsErrorIrr[Ch][j] = FCD->IntensityPar.RelativeError[Ch][j]
		* SDR->Irradiance[Ch][j];
	    /*----------------------------------------------------------------*\
	    ** calculate absolute error extract format (E2)
	    \*----------------------------------------------------------------*/
	    RadResp = FCD->IntensityPar.Calibration[Ch][j];
	    AbsErrorRadianceResponse =
		FCD->KeyDataError.Response_f2_SMDep[Ch][j] / 3.0 * RadResp;
	    if ((float)fabs( (double)RadResp) < MIN_VAL)
	    {
		Status = WARN;
		
#ifdef DEBUG
		if (Debug__isActive(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL))
		{
		    sprintf(LogBuffer, "Cannot calculate Solar Errors of "
			    "Channel %2d Pixel %3d, reason RadResp: %f",
			    Ch + 1, j, RadResp);
		    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__773", Status,
			     LOG_NUM, LogBuffer, "");
		}
#endif
		SDR->RelErrorIrr[ Ch][ j] = UNKNOWN_ERROR_VAL;
	    }
	    else
	    {
		SDR->RelErrorIrr[ Ch][ j] =
		    (float)sqrt((double)
				(1.0/RadResp * 1.0/RadResp * AbsErrorMeanSunRef *
				 AbsErrorMeanSunRef +
				 -FCD->IntensityPar.MeanSunReference[ Ch][ j] /
				 RadResp / RadResp *
				 -FCD->IntensityPar.MeanSunReference[ Ch][ j] /
				 RadResp / RadResp *
				 AbsErrorRadianceResponse *
				 AbsErrorRadianceResponse));
	    }
	}
    }
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SDR__CalculateErrorsAndFlags
**  Input parameters:
**    Config		Extract configuration structure
**    FCD		fixed calibration record of Level 1 Product
**    SDR		Sun description record structure
**
**  Output parameters:
**    SDR		Sun description record structure
**    SDR__CalculateErrorsAndFlags	SUCCESS	= Error calculation ok
**					else	= error calculation not ok
**
**  Other interfaces:
**
**  Description:
**    Calculate the errors of the Irradiance and set the flags of the SDR.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDR__calculateErrorsAndFlags(SDR_s* SDR, const Config_s *Config,
			     const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    int i, j;
    char ASCII_Time[30];

    /*----------------------------------------------------------------------*\
    ** Calculate error of SDR
    \*----------------------------------------------------------------------*/
    if (SDR__calculateErrors(SDR, Config, FCD) != SUCCESS)
    {
	Status = WARN;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__836", Status, LOG_NUM,
		  "Cannot calculate Solar Errors", "");
    }

    /*----------------------------------------------------------------------*\
    ** set flags for all defined bands
    \*----------------------------------------------------------------------*/
    for ( i = Config->SDR_StartBand; i <= Config->SDR_EndBand; i++)
	for ( j = FCD->BCR[ (ScienceData_e)i].StartDetector;
	      j <= FCD->BCR[ (ScienceData_e)i].EndDetector; j++)
	    if ((SDR->Flag[ Band2Channel( (ScienceData_e)i)][ j] == QF_GOOD) ||
		(SDR->Flag[ Band2Channel( (ScienceData_e)i)][ j] > QF_OTHER))
	    {
		if ((SDR->Irradiance[ Band2Channel( (ScienceData_e)i)][ j] < 0.0) ||
		    (FCD->IntensityPar.Calibration[Band2Channel((ScienceData_e)i)][ j] < 0.0) ||
		    (FCD->IntensityPar.MeanSunReference[Band2Channel((ScienceData_e)i)][ j] < 0.0))
		    SDR->Flag[Band2Channel((ScienceData_e)i)][ j] = QF_OTHER;
	    }

    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/


