/*----------------------------------------------------------------------------*\
**                                SDR.h
**
**  Class SDR (Solar Description Record = Sun Spectrum) declaration.
** 
**  Version:    5.3
**  Date:       12/21/01
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _SDR_h
#define _SDR_h 

#pragma ident "@(#)SDR.h	5.3 12/21/01, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "dr.h"
#include "xcortion.h"

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class SDR
\*============================================================================*/
typedef struct SDR {
    float StartWavelSolar;
    float EndWavelSolar;
    float Irradiance[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    float AbsErrorIrr[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    float RelErrorIrr[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    short Flag[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
} SDR_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
SDR__Init_XCorr(float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS], FCD_s *FCD, SDR_s* SDR,
	Spectrum_s mean_sun[OPTICAL_CHANNELS], Spectrum_s template_spectrum[OPTICAL_CHANNELS][N_TEMPLATE_FILE]);

FunctionStatus_e 
SDR__Delete_XCorr(Spectrum_s mean_sun[OPTICAL_CHANNELS], Spectrum_s template_spectrum[OPTICAL_CHANNELS][N_TEMPLATE_FILE]);

FunctionStatus_e 
SDR__Show_XCorr(FILE* Stream, XC_Coeff_s* Param);

FunctionStatus_e
SDR__init(SDR_s* self, float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	  const Config_s *Config, FCD_s *FCD);

FunctionStatus_e
SDR__write(const SDR_s* SDR, float SolarGrid[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	   FILE *Stream, const Config_s *Config, const FCD_s *FCD);

FunctionStatus_e
SDR__calculateErrors(SDR_s* self, const Config_s *Config, const FCD_s *FCD);

FunctionStatus_e
SDR__calculateErrorsAndFlags(SDR_s* self, const Config_s *Config,
			     const FCD_s *FCD);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
