/*----------------------------------------------------------------------------*\
**                                SFS.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SFS.c	5.1 02/26/99, Copyright DLR 1998"

#include <string.h>
#include "sfs.h"
#include "log.h"


#define	SccsId	("@(#)SFS.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("SFS")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                SFS__Write
**  Input parameters:
**	Stream		The file, where to write
**	Config		Extract configuration structure
**	FCD		Fixed Calibration Parameter structure
**
**  Output parameters:
**    SFS__Write	SUCCESS	= output ok
**			else	= any error during output
**
**  Other interfaces:
**    output file of Extract Level 1 Product specified in Config
**
**  Description:
**    Function to write the solar spectrum summary onto the Level 1 
**    Extracted Product
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SFS__Write( FILE *Stream, const Config_s *Config, const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    char ASCII_Time[ 30];

    /*--------------------------------------------------------------------------*\
    ** convert UTC Date of solar spectrum to ASCII
    \*--------------------------------------------------------------------------*/
    if (UTC_2_ASCII( FCD->IntensityPar.Date, ASCII_Time) != SUCCESS)
    {
	Status = ERROR;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__63", Status,
		  LOG_NUM, "converting the UTC time of Solar Spectrum", "");
	strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }

    /*--------------------------------------------------------------------------*\
    ** write the solar spectrum summary
    \*--------------------------------------------------------------------------*/
    fprintf(Stream, "Solar Spectrum %s\n", ASCII_Time);
    if (ferror( Stream) != 0)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__75", Status, LOG_CANNOT_WRITE,
		  "SFS into exract level 1 file", "");
	return Status;
    }
    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
