/*----------------------------------------------------------------------------*\
**                                SMCD.h
**
**  Class SMCD declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _SMCD_h
#define _SMCD_h 

#pragma ident "@(#)SMCD.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "pcd.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	SMCD_DATE_SIZE	(UTC_SIZE)

#define	SMCD_SUN_ANGLE_SIZE	(POS_ANGLE_SIZE)

#define	SMCD_MOON_ANGLE_SIZE	(POS_ANGLE_SIZE)

#define	SMCD_DARK_CURRENT_AND_NOISE_COR_SIZE	(2 * FLOAT_SIZE)

#define	SMCD_USE_IN_REFERENCE_SIZE	(FLOAT_SIZE)

#define	SMCD_INDEX_SPECTRAL_CAL_PAR	(SHORT_SIZE)

#define	SMCD_INDEX_LEAKAGE_COR_PAR	(SHORT_SIZE)

#define	SMCD_MPH_EXTRACT_SIZE	(MPH_L1_SIZE)

#define	SMCD_SPH_EXTRACT_SIZE	(SPH_L1_SIZE)

#define	SMCD_INSTRUMENT_HEADER_SIZE	(AUXILIARY_DATA_SIZE * SHORT_SIZE)

#define	SMCD_SCIENCE_INDICE_SIZE	(SHORT_SIZE)

#define	SMCD_SIZE	(SMCD_DATE_SIZE + SMCD_SUN_ANGLE_SIZE + SMCD_MOON_ANGLE_SIZE + SMCD_DARK_CURRENT_AND_NOISE_COR_SIZE + SMCD_USE_IN_REFERENCE_SIZE + SMCD_INDEX_SPECTRAL_CAL_PAR + SMCD_INDEX_LEAKAGE_COR_PAR + SMCD_MPH_EXTRACT_SIZE + SMCD_SPH_EXTRACT_SIZE + SMCD_INSTRUMENT_HEADER_SIZE + SCIENCE_DATA_DEFINED * SMCD_SCIENCE_INDICE_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class ...
\*============================================================================*/
typedef struct SMCD {
    UTC_s Date;
    PosAngle_s SunAngle;
    PosAngle_s MoonAngle;
    float UseInReference;
    float DarkCurrentCorFactor;
    float DarkCurrentNoiseCorFactor;
    short IndexSpectralCalPar;
    short IndexLeakagePar;
    char MPH_L1[MPH_L1_SIZE];
    char SPH_L1[SPH_L1_SIZE];
    GOME_Word_t AuxiliaryData[ AUXILIARY_DATA_SIZE];
    short ScienceIndex[ SCIENCE_DATA_DEFINED];
} SMCD_s;



/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SMCD__ReadTime(FILE *Stream, long FilePos, long SMCD_Size, UTC_s *SMCD_Time,
	       PCD_s *SMCD, Bool convert_sun_2_host);

FunctionStatus_e
SMCD__GetSpectrumParameter(ProductType_e ProductType, const SPH1_s *SPH1,
			   const SMCD_s *SMCD, PixelPar_s *PixelPar);

FunctionStatus_e
SMCD__read(SMCD_s* self, FILE* Stream, Bool convert_sun_2_host);

void
SMCD__show(const SMCD_s* self, FILE* Stream, Bool FullInfo);

long
xCD__Read(FILE * Stream, char MPH_L1[MPH_L1_SIZE], char SPH_L1[SPH_L1_SIZE],
	  GOME_Word_t AuxiliaryData[AUXILIARY_DATA_SIZE],
	  short ScienceIndex[SCIENCE_DATA_DEFINED]);

/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
