/*----------------------------------------------------------------------------*\
**                                SpectralC.h
**
**  Class Spectral declaration
** 
**  Version:    5.3
**  Date:       04/08/02
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Spectral_h
#define _Spectral_h

#pragma ident "@(#)SpectralC.h	5.3 04/08/02, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "sys.h"
#include "sciedata.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	MAX_SPECTRAL_LINES	(25)

#define	SPECTRAL_FITT_ORDER	(5)

#define	SPECTRAL_SIZE	(DOUBLE_SIZE * OPTICAL_CHANNELS * SPECTRAL_FITT_ORDER + DOUBLE_SIZE * OPTICAL_CHANNELS)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class Spectral
\*============================================================================*/
typedef struct SpectralC {
    double Coefficient[OPTICAL_CHANNELS][SPECTRAL_FITT_ORDER]; /* 4th order 
								  polynomial */
    double Error[OPTICAL_CHANNELS]; /* Spectral Calibration Error */
} SpectralC_s;

/* double CoefficientError[OPTICAL_CHANNELS][SPECTRAL_FITT_ORDER]; Stddev of 
		Coefficient */								  	

/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
SpectralC_s*
SpectralC__New(FILE *Stream, FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
SpectralC__Delete(SpectralC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
float
SpectralC__GetWavelength(const SpectralC_s *Self, OpticalChannel_e Channel,
			short Index);

void
SpectralC__apply(const SpectralC_s* self, ScienceData_e Band,
		float LambdaGrid[], short Begin, short End, short CallIndex);

FunctionStatus_e
SpectralC__read(SpectralC_s* self, FILE* Stream, Bool convert_sun_2_host);

void
SpectralC__show(const SpectralC_s* self, FILE* Stream);


/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**  EvalPolynom: Evaluates a polynom in the point given
\*----------------------------------------------------------------------------*/
float
EvalPolynom_f(float X, const float *Coefficient, short Grad);

double
EvalPolynom_d(double X, const double *Coefficient, short Grad);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
