/*----------------------------------------------------------------------------*\
**                                SPH.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SPH.c	5.1 02/26/99, Copyright DLR 1998"

#include "sph.h"
#include "log.h"


#define	SccsId	("@(#)SPH.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("SPH")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            SPH__Extract
**  Input parameters:
**    Stream		file stream for output
**    SPH		SPH of Level 0 file
**
**  Output parameters:
**    SPH_5_6		entries 5 and 6 ot the SPH
**
**  Other interfaces:
**
**  Description:
**    This function extract the entries 5 and 6 from the SPH
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SPH__extract( char SPH[ SPH_SIZE], char SPH_5_6[ SPH_5_SIZE + SPH_6_SIZE])
{
    FunctionStatus_e	Stat		= SUCCESS;
    int			i;

    for (i = 0; i < SPH_5_SIZE; i++)
	SPH_5_6[ i] = SPH[ SPH_5_ENTRY + i];
    for (i = 0; i < SPH_6_SIZE; i++)
	SPH_5_6[ i + SPH_5_SIZE] = SPH[ SPH_6_ENTRY + i];
}


/*----------------------------------------------------------------------------*\
**                            SPH__Read
**  Input parameters:
**    Stream		file stream for output
**  Output parameters:
**    SPH		SPH of Level 0 file
**
**  Other interfaces:
**
**  Description:
**    This function reads the SPH from the input file
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH__read(char SPH[ SPH_SIZE], FILE *Stream)
{
    FunctionStatus_e Status = SUCCESS;
    long NrOfBytes = 0;

    /*------------------------------------------------------------------------*\
    ** read the SPH of the current file
    \*------------------------------------------------------------------------*/
    NrOfBytes = fread( SPH, 1, SPH_SIZE, Stream);
    if (NrOfBytes != SPH_SIZE)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__304", Status, LOG_CANNOT_READ,
            	  "SPH", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                            SPH__Write
**  Input parameters:
**    Stream		file stream for output
**    SPH		SPH of Level 0 file
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**    This function writes the SPH into the output file
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.6.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH__write(const char SPH[ SPH_SIZE], FILE *Stream)
{
    FunctionStatus_e	Stat		= SUCCESS;
    long			NrOfBytes	= 0;

    /*------------------------------------------------------------------------*\
    ** write the SPH of the current file
    \*------------------------------------------------------------------------*/
    NrOfBytes = fwrite( SPH, 1, SPH_SIZE, Stream);
    if (NrOfBytes != SPH_SIZE)
    {
	Stat = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.1__264", Stat, LOG_CANNOT_WRITE,
            	  "SPH", "");
    }

    return Stat;
}


/*============================================================================*\
**  private operations
\*============================================================================*/


