/*----------------------------------------------------------------------------*\
**                                SPH1.c
**
**  Description:
** 
**  Version:    5.3
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SPH1.c	5.3 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include <string.h>
#include "sph1.h"
#include "pixelpar.h"
#include "pcd.h"
#include "smcd.h"
#include "go_isbit.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)SPH1.c 5.3 08/19/02, Copyright DLR 1998")


#define	SS_ID	("SPH1")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                ERS_Information_Show
**  Input parameters:
**    Stream			output stream of info
**    Config			Extract Configuration structure
**    SPH1			Specific Product Header of Level 1 Product
**
**  Output parameters:
**    ERS_Information_Show	SUCCESS	= output of info ok
**				else	= any error occured during output
**
**  Other interfaces:
**    Input file of Level 1 Product specified in Config
**    Output file for ERS Information specified by stream
**
**  Description:
**    Function to write ERS Information of the SPH1 of the Level 1 Product
**    to the output stream.
**
**  References:
**    This function ...
**  Libraries:
**    ....
**  Created:	7.3.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH1__showERSInformation(const SPH1_s *SPH1, FILE *Stream)
{
    FunctionStatus_e Status = SUCCESS;
    int i;

    /*----------------------------------------------------------------------*\
    ** Show ERS Information
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "ERS Information\n");

    fprintf( Stream,
	     "%05u  %05u  %05u  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f\n",
	     SPH1->ERS_PropagatorState.StateVector.DateUTC.Days,
	     SPH1->ERS_PropagatorState.StateVector.DateUTC.Msecs,
	     SPH1->ERS_PropagatorState.StateVector.OrbitNumber,
	     SPH1->ERS_PropagatorState.StateVector.Position.X,
	     SPH1->ERS_PropagatorState.StateVector.Position.Y,
	     SPH1->ERS_PropagatorState.StateVector.Position.Z,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.X,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.Y,
	     SPH1->ERS_PropagatorState.StateVector.Velocity.Z);

    for (i = 0; i < ATTITUDE_SIZE; i++)
	fprintf( Stream, "%lf ", SPH1->ERS_PropagatorState.ATT[ i]);
    for (i = 0; i < ATTITUDE_SIZE; i++)
	fprintf( Stream, "%lf ", SPH1->ERS_PropagatorState.DATT[ i]);
    fprintf( Stream, "%ld ", SPH1->ERS_PropagatorState.IATT);
    fprintf( Stream, "%ld", SPH1->ERS_PropagatorState.StatusInit);  
    fprintf( Stream, "\n");

    fprintf( Stream, "%lf ", SPH1->ERS_PropagatorState.RTIME);
    for (i = 0; i < KEPLER_STATE_SIZE; i++)
	fprintf( Stream, "%lf ", SPH1->ERS_PropagatorState.XM[ i]);
    fprintf( Stream, "\n");

    if ( ferror( Stream) != 0)
    {
	Status = FATAL;
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__97", Status, LOG_CANNOT_WRITE,
		 "ERS Information", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SPH1__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		SPH1
**
**  Other interfaces:
**
**  Description:
**	Function to read all SPH1 entries from the given 
**	input stream into the SPH1 structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH1__read(SPH1_s* self, FILE* file_ptr, Bool convert_sun_2_host)
{
    int i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read number of input data references, input data references itself,
    ** GDP Software Version and Pre-flight Calibration Data Version
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread(&self->Nref, 1, SPH1_NREF_SIZE, file_ptr);
    if (NoOfReadBytes != SPH1_NREF_SIZE)
    {
	Status = FATAL;
	sprintf(LogBuffer, "SPH1, number of input data references (Nref), "
		"#read bytes: %d", NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4769", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** convert Nref
    \*---------------------------------------------------------------------*/
    if (convert_sun_2_host) {
	self->Nref = SunShort2Short( (unsigned char*)&self->Nref);
    }

    /*----------------------------------------------------------------------*\
    ** create PIRs
    \*----------------------------------------------------------------------*/
    if (self->Nref == 0)
    {
	self->PIR_Of_SDH = 0;
    }
    else
    {
#ifdef __cplusplus
	self->PIR_Of_SDH = new char[ self->Nref][ PIR_SIZE];
#else
	self->PIR_Of_SDH = (char(*)[PIR_SIZE])malloc(sizeof(char) *
						     self->Nref * PIR_SIZE);
#endif
	if (self->PIR_Of_SDH == 0)
	{
	    Status = FATAL;
	    sprintf(LogBuffer, "SPH1, memory for input data reference, "
		    "#bytes to allocate: %d", self->Nref * PIR_SIZE);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4807", Status,
		     LOG_CANNOT_ALLOCATE, LogBuffer, "");
	    return Status;
	}

	/*----------------------------------------------------------------------*\
	** Read input data references (PIRs)
	\*----------------------------------------------------------------------*/
	for (i = 0; i < self->Nref; i++)
	{
	    if (PIR__read(self->PIR_Of_SDH[i], file_ptr) != SUCCESS)
	    {
		Status = FATAL;
		sprintf(LogBuffer, "SPH1, Input Data Reference[%4d]", i);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4823", Status,
			 LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*--------------------------------------------------------------------------*\
    ** Read versions
    \*--------------------------------------------------------------------------*/
    NoOfReadBytes = fread(self->SoftwareVersion, 1, SPH1_GDP_VERSION_SIZE,
			  file_ptr);
    NoOfReadBytes += fread(self->KeyDataDataVersion, 1, SPH1_CAL_VERSION_SIZE,
			   file_ptr);
    NoOfReadBytes += fread(&self->ProductFormatVersion, 1,
			   SPH1_PRODUCT_FORMAT_VERSION_SIZE, file_ptr);
    if (NoOfReadBytes != (SPH1_GDP_VERSION_SIZE + SPH1_CAL_VERSION_SIZE +
			  SPH1_PRODUCT_FORMAT_VERSION_SIZE))
    {
	Status = FATAL;
	sprintf(LogBuffer, "SPH1, SoftwareVersion or KeyDataDataVersion, "
		"#read bytes: %d", NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4853", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------------*\
    ** Read time correlation
    \*--------------------------------------------------------------------------*/
    if (TimeCor__read(&self->TimeCor, file_ptr, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf(LogBuffer, "SPH1, Time correlation, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4873", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }
    
    /*--------------------------------------------------------------------------*\
    ** read PMD_EntryPoint, SubsetCounterEntryPoint
    \*--------------------------------------------------------------------------*/
    NoOfReadBytes = fread(&self->PMD_EntryPoint, 1,
			  SPH1_PMD_ENTRY_POINT_SIZE, file_ptr);
    NoOfReadBytes += fread(&self->SubsetCounterEntryPoint, 1,
			   SPH1_SUBCOUNT_ENTRY_POINT_SIZE, file_ptr);
    NoOfReadBytes += fread(&self->IntegrationStatus_EntryPoint, 1,
			   SPH1_INTG_STATUS_ENTRY_POINT_SIZE, file_ptr);
    NoOfReadBytes += fread(&self->PeltierEntryPoint, 1,
			   SPH1_PELTIER_ENTRY_POINT_SIZE, file_ptr);
    NoOfReadBytes += fread(&self->InstrStatus_2_EntryPoint, 1,
			   SPH1_INSTR_STATUS_2_ENTRY_POINT_SIZE, file_ptr);
    if (NoOfReadBytes != (SPH1_INTG_STATUS_ENTRY_POINT_SIZE
			  + SPH1_PMD_ENTRY_POINT_SIZE 
			  + SPH1_SUBCOUNT_ENTRY_POINT_SIZE 
			  + SPH1_PELTIER_ENTRY_POINT_SIZE
			  + SPH1_INSTR_STATUS_2_ENTRY_POINT_SIZE))
    {
	Status = FATAL;
	sprintf(LogBuffer, "SPH1, auxiliary data entry points, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4904", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------------*\
    ** read PMD conversion faktors
    \*--------------------------------------------------------------------------*/
    for (i = 0; i < PMD_DEFINED; i++) {
	if ( (Status = EngConversion__read(&(self->PMD_EngConv[i]), file_ptr,
					   convert_sun_2_host)) != SUCCESS)
	{
	    sprintf(LogBuffer, "SPH1, PMD[ %4d] Engineering Conversion factors",
		    i);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4927", Status,
		     LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
	}
    }

    /*--------------------------------------------------------------------------*\
    ** read ERS_PropagatorState
    \*--------------------------------------------------------------------------*/
    if ( (Status = ERS_PropagatorState__read(&(self->ERS_PropagatorState), 
					     file_ptr, convert_sun_2_host))
	 != SUCCESS)
    {
	sprintf(LogBuffer, "SPH1, ERS_PropagatorState");
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.2__4950", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	SPH1__Sun2Host(self);
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                SPH1__Show
**  Input parameters:
**	Stream		output stream
**	Param		SPH1 structure
**
**  Output parameters:
**
**  Other interfaces:
**	SPH1 structure written to the output stream
**
**  Description:
**	Function to write the SPH1 structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SPH1__show(const SPH1_s *Param, FILE * Stream)
{
    int			i;

    /*----------------------------------------------------------------------*\
    ** show input data references (PIRs)
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Nref: %3d\n", Param->Nref);
    for ( i = 0; i < Param->Nref; i++)
    {
	PIR__show(Param->PIR_Of_SDH[ i], Stream);
    }

    /*----------------------------------------------------------------------*\
    ** show versions
    \*----------------------------------------------------------------------*/
    fprintf( Stream, " SoftwareVersion     : ");
    for (i = 0; i < sizeof( Param->SoftwareVersion); i++)
	fprintf( Stream, "%c", Param->SoftwareVersion[ i]);
    fprintf( Stream, "\n");

    fprintf( Stream, " KeyDataDataVersion: ");
    for (i = 0; i < sizeof( Param->KeyDataDataVersion); i++)
	fprintf( Stream, "%c", Param->KeyDataDataVersion[ i]);
    fprintf( Stream, "\n");
    fprintf( Stream, " ProductFormatVersion: %hd", Param->ProductFormatVersion);

    /*----------------------------------------------------------------------*\
    ** show time correlation
    \*----------------------------------------------------------------------*/
    TimeCor__show(&Param->TimeCor, Stream);
    
    /*-------------------------------------------------------------------*\
    ** write PMD_EntryPoint, SubsetCounterEntryPoint
    \*-------------------------------------------------------------------*/
    fprintf( Stream, " PMD entry               : %hd\n", Param->PMD_EntryPoint);
    fprintf( Stream, " SubsetCounterEntryPoint : %hd\n",
	     Param->SubsetCounterEntryPoint);
    fprintf( Stream, " IntegrationStatus entry : %hd\n",
	     Param->IntegrationStatus_EntryPoint);
    fprintf( Stream, " PeltierEntryPoint       : %hd\n",
	     Param->PeltierEntryPoint);
    fprintf( Stream, " InstrStatus_2_EntryPoint: %hd\n",
	     Param->InstrStatus_2_EntryPoint);

    /*-------------------------------------------------------------------*\
    ** write PMD conversion faktors
    \*-------------------------------------------------------------------*/
    for (i = 0; i < PMD_DEFINED; i++)
    {
	fprintf( Stream, "PMD %3d Conversion Fak.:\n", i);
	EngConversion__Show( Stream, &Param->PMD_EngConv[ i]);
    }

    /*----------------------------------------------------------------------*\
    ** show ERS_PropagatorState
    \*----------------------------------------------------------------------*/
    ERS_PropagatorState__Show( Stream, &Param->ERS_PropagatorState);
}


/*----------------------------------------------------------------------------*\
**                                FPA__GetPeltier
**  Input parameters:
**    Stream			input stream of product
**    ProductType		product type
**    SPH1			Specific Product Header of Level 1 Product
**    FilePos_xCD		file position of PCD, SCD or MCD
**    xCD_Size			size of PCD, SCD or MCD
**    PeltierDataSize		number of PCD, SCD or MCD records that
**				corresponds to number of peltiers
**
**  Output parameters:
**    PeltierData[]		peltier of channel 0 of all PCD, SCD or MCD
**				records
**
**  Other interfaces:
**
**  Description:
**    Function to get the peltier of CHANNEL 0 from all PCDs, SCDs or MCDs
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	8.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FPA__GetPeltier( FILE *Stream, ProductType_e ProductType, const SPH1_s *SPH1,
		 long FilePos_xCD, long xCD_Size,
		 short PeltierDataSize, float PeltierData[],
		 Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    int			i;
    long			FilePos;
    UTC_s			xCD_Time;
    PCD_s			PCD;
    SMCD_s		SMCD;
    AuxiliaryPar_s	AuxiliaryPar;

    /*-------------------------------------------------------------------------*\
    ** create memory for peltier if PeltierDataSize > 0
    \*-------------------------------------------------------------------------*/
    for (i = 0; i < PeltierDataSize; i++) {
	FilePos = FilePos_xCD + i * xCD_Size;

	/*-----------------------------------------------------------------------*\
	** get the peltier data from the auxiliary data of the PCD
	\*-----------------------------------------------------------------------*/
	if (ProductType == PR_EARTHSHINE) 
	{
	    if (PCD__ReadTime( Stream, FilePos, xCD_Size, &xCD_Time,
			       &PCD, convert_sun_2_host) 
		!= SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__434", Status,
			  LOG_CANNOT_READ, "PCD for getting peltier values", "");
		return Status;
	    }

	    if (SPH1__getAuxiliaryParameter(SPH1, PCD.AuxiliaryData,
					    &AuxiliaryPar) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__443", Status,
			  LOG_NUM, "Cannot get Peltier data from PCD", "");
		return Status;
	    }
	    PeltierData[ i] = AuxiliaryPar.Peltier[ 0];
	}
	else if ((ProductType == PR_SUNSHINE) 
		 || (ProductType == PR_MOONSHINE))
	{
	    /*----------------------------------------------------------------*\
	    ** get the peltier data from the auxiliary data of the SCD or MCD
	    \*----------------------------------------------------------------*/
	    if (SMCD__ReadTime(Stream, FilePos, xCD_Size, &xCD_Time,
			       (PCD_s*)&SMCD, convert_sun_2_host) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__459", Status,
			  LOG_CANNOT_READ, "SMCD for getting peltier values", "");
		return Status;
	    }
	    
	    if (SPH1__getAuxiliaryParameter(SPH1, SMCD.AuxiliaryData,
					    &AuxiliaryPar) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__468", Status,
			  LOG_NUM, "Cannot get Peltier data from SMCD", "");
		return Status;
	    }
	    PeltierData[ i] = AuxiliaryPar.Peltier[ 0];
	}
	else 
	{
	    Status = FATAL;
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__477", Status,
		     LOG_NUM, "Unknown product type", "");
	    return Status;
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                GetAuxiliaryParameter
**  Input parameters:
**    SPH1			SPH1 of Level 1 Product
**    AuxiliaryData		AuxiliaryData of the of Level 1 Product 
**
**  Output parameters:
**    AuxiliaryPar		pixel specific auxiliary data needed for the
**				product parameter
**    GetAuxiliaryParameter	SUCCESS	= Parameter extracted
**				else	= error during parameter extraction
**
**  Other interfaces:
**
**  Description:
**    Function to extract the pixel spectific auxiliary data for the product.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH1__getAuxiliaryParameter(const SPH1_s *SPH1, 
			    const GOME_Word_t *AuxiliaryData,
			    AuxiliaryPar_s *AuxiliaryPar)
{
    FunctionStatus_e Status = SUCCESS;
    int i;
    GOME_Word_t IntegrStatusWord;


    /*----------------------------------------------------------------------*\
    ** get subset counter
    \*----------------------------------------------------------------------*/
    AuxiliaryPar->SubsetCounter = AuxiliaryData[ SPH1->SubsetCounterEntryPoint];

    /*----------------------------------------------------------------------*\
    ** get PMD values
    \*----------------------------------------------------------------------*/
    for (i = 0; i < PMD_SIZE; i++)
    {
	AuxiliaryPar->PMD[ 0][ i] = AuxiliaryData[ 
						  SPH1->PMD_EntryPoint + i * 4];
	AuxiliaryPar->PMD[ 1][ i] = AuxiliaryData[ 
						  SPH1->PMD_EntryPoint + i * 4 + 1];
	AuxiliaryPar->PMD[ 2][ i] = AuxiliaryData[ 
						  SPH1->PMD_EntryPoint + i * 4 + 2];
    }

    /*------------------------------------------------------------------------*\
    ** convert all PMDs
    \*------------------------------------------------------------------------*/
    /*
      for ( i = 0; i < PMD_DEFINED; i++)
      for ( j = 0; j < PMD_SIZE; j++)
      AuxiliaryPar->PMD[ i][ j] = SPH1->PMD_EngConv[ i].Coefficient[ 0] +
      AuxiliaryPar->PMD[ i][ j] * SPH1->PMD_EngConv[ i].Coefficient[ 1];
      */

    /*----------------------------------------------------------------------*\
    ** get Peltier values
    \*----------------------------------------------------------------------*/
    for (i = 0; i < OPTICAL_CHANNELS; i++)
    {
	AuxiliaryPar->Peltier[ i] = AuxiliaryData[ SPH1->PeltierEntryPoint + i];
    }

    /*----------------------------------------------------------------------*\
    ** get average mode and the integration status
    \*----------------------------------------------------------------------*/
    AuxiliaryPar->AverageMode = GetAverageModeExtract( 
						      AuxiliaryData[ SPH1->InstrStatus_2_EntryPoint]);
    IntegrStatusWord = AuxiliaryData[ SPH1->IntegrationStatus_EntryPoint];
    GetIntegrStatus( IntegrStatusWord, &AuxiliaryPar->IntegrStatus);

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                name
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      check the software version of the product with the software version
**      of the software.
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
void
SPH1__checkSoftwareVersion(SPH1_s* self)
{
    char SoftwareVersion_[VERSION_LENGTH];
    char ProductVersion[VERSION_LENGTH]; 
    int idx;

    for (idx = 0; idx < VERSION_LENGTH; idx++)
    {
      SoftwareVersion_[idx] = 0;
      ProductVersion[idx] = 0;
    }

    sprintf(SoftwareVersion_, VERSION_STRING_FORMAT, SOFTWARE_VERSION);
    strncpy(ProductVersion, self->SoftwareVersion, sizeof(self->SoftwareVersion));

    if (strncmp(self->SoftwareVersion, SoftwareVersion_, sizeof(self->SoftwareVersion)) != 0) 
    {
	fprintf(stdout, "\t************************************************************\n");
	fprintf(stdout, "\t** WARNING: The software version used to generate the     **\n");
	fprintf(stdout, "\t**          Level 1 product (%s) is different to the   **\n", ProductVersion);
	fprintf(stdout, "\t**          version of the extraction program (%s)     **\n", SoftwareVersion_);
	fprintf(stdout, "\t************************************************************\n");
    }
}


/*----------------------------------------------------------------------------*\
**                                name
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**       check the product version with the product version of the software
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH1__checkProductVersion(SPH1_s* self)
{
    FunctionStatus_e status = SUCCESS;

    if (self->ProductFormatVersion != PRODUCT_FORMAT_VERSION) 
    {
	status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__659", status, LOG_CANNOT_READ,
		 "level 1 product because the product version of the extract "
		 "software doesn't match the product version of the product"
		 " file", "");
    }
    return status;
}



/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            SPH1__Sun2Host
**  Input parameters:
**    SPH1			in Sun memory representation
**  Output parameters:
**    SPH1			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the SPH1 struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SPH1__Sun2Host( SPH1_s *Param)
{
    /* short */
    Param->ProductFormatVersion =
	SunShort2Short( (unsigned char*)&Param->ProductFormatVersion);

    /* short */
    Param->PMD_EntryPoint = SunShort2Short( (unsigned char*)
					    &Param->PMD_EntryPoint);
    /* short */
    Param->SubsetCounterEntryPoint = SunShort2Short( (unsigned char*)
						     &Param->SubsetCounterEntryPoint);
    /* short */
    Param->IntegrationStatus_EntryPoint =
	SunShort2Short( (unsigned char*)&Param->IntegrationStatus_EntryPoint);

  /* short */
    Param->PeltierEntryPoint = SunShort2Short( (unsigned char*)
					       &Param->PeltierEntryPoint);
    /* short */
    Param->InstrStatus_2_EntryPoint = SunShort2Short( (unsigned char*)
						      &Param->InstrStatus_2_EntryPoint);
}

 
/*----------------------------------------------------------------------------*\
**                                GetAverageModeExtract
**  Input parameters:
**    Value		word representing the bitfield of GOME instrument
**			status containing the Average mode flag
**
**  Output parameters:
**    GetAverageModeExtract	True	= AverageMode
**				False	= no AverageMode
**
**  Other interfaces:
**
**  Description:
**    Function to get average mode flag from the GOME instrument status word
**    containing the average mode flag
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
GetAverageModeExtract(const GOME_Word_t Value)
{
    if ((AVERAGE_MODE_MASK & (unsigned short)Value) == 0)
	return False;
    else
	return True;
}


/*----------------------------------------------------------------------------*\
**                                GetIntegrStatus
**  Input parameters:
**    GO_IntegrStatusBit	pointer to GOME integration status bitfield
**
**  Output parameters:
**    IntegrStatus		integration status
**
**  Other interfaces:
**
**  Description:
**    Function to get integration status from a bitfield
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
GetIntegrStatus(GOME_Word_t IntegrStatusWord, GO_IntegrStatus_s *IntegrStatus)
{
    int	i;

    /* not portable code
       union {
       char			CharVal[ sizeof( GO_IntegrStatusBit_s)];
       GOME_Word_t			ShortVal;
       GO_IntegrStatusBit_s 	Bit;
       } GO_IntegrStatus;
  
       GO_IntegrStatus.ShortVal = IntegrStatusWord;  
       IntegrStatus->FPA4_Division = (short)
       GO_IntegrStatus.Bit.GO_IntegrStatus_FPA4_Div;
       IntegrStatus->FPA3_Division = (short)
       GO_IntegrStatus.Bit.GO_IntegrStatus_FPA3_Div;
       IntegrStatus->FPA2_Division = (short)
       GO_IntegrStatus.Bit.GO_IntegrStatus_FPA2_Div;

       IntegrStatus->BandStat[ 0] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_1;
       IntegrStatus->BandStat[ 1] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_2;
       IntegrStatus->BandStat[ 2] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_3;
       IntegrStatus->BandStat[ 3] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_4;
       IntegrStatus->BandStat[ 4] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_5;
       IntegrStatus->BandStat[ 5] = ( IntegrationStatus_e)
       GO_IntegrStatus.Bit.GO_IntegrStatusBandStat_6;  
       */

    for ( i = 0; i < BAND_DEFINED; i++) 
    {
	IntegrStatus->BandStat[ i] = ( IntegrationStatus_e)0;
    }
    IntegrStatus->FPA4_Division = (short)((IntegrStatusWord & FPA4_DIVISION_MASK) / FPA4_DIVISION_FAC);
    IntegrStatus->FPA3_Division = (short)((IntegrStatusWord & FPA3_DIVISION_MASK) / FPA3_DIVISION_FAC);
    IntegrStatus->FPA2_Division = (short)((IntegrStatusWord & FPA2_DIVISION_MASK) / FPA2_DIVISION_FAC);

}
