/*----------------------------------------------------------------------------*\
**                                SPH1.h
**
**  Class SPH1 declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _SPH1_h
#define _SPH1_h 

#pragma ident "@(#)SPH1.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "engcsion.h"
#include "pir.h"
#include "ers_tate.h"
#include "timecor.h"
#include "auxiypar.h"

/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	SPH_L1_SIZE	(22)

#define	SPH1_NREF_SIZE	(SHORT_SIZE)

#define	SPH1_GDP_VERSION_SIZE	(LEVEL_1_PRODUCT_VERSION_SIZE)

#define	SPH1_CAL_VERSION_SIZE	(LEVEL_1_PRODUCT_VERSION_SIZE)

#define	SPH1_PRODUCT_FORMAT_VERSION_SIZE	(SHORT_SIZE)

#define	SPH1_TIME_COR_SIZE	(TIME_COR_SIZE)

#define	SPH1_INTG_STATUS_ENTRY_POINT_SIZE	(SHORT_SIZE)

#define	SPH1_PMD_ENTRY_POINT_SIZE	(SHORT_SIZE)

#define	SPH1_SUBCOUNT_ENTRY_POINT_SIZE	(SHORT_SIZE)

#define	SPH1_PELTIER_ENTRY_POINT_SIZE	(SHORT_SIZE)

#define	SPH1_INSTR_STATUS_2_ENTRY_POINT_SIZE	(SHORT_SIZE)

#define	SPH1_PMD_ENG_CONV_SIZE	(ENG_CONVERSION_SIZE * PMD_DEFINED)

#define	SPH1_ERS_PROPAGATOR_STATE_SIZE	(ERS_PROPAGATOR_STATE_SIZE)

#define	SPH1_STATIC_SIZE	(SPH1_NREF_SIZE + SPH1_GDP_VERSION_SIZE + SPH1_CAL_VERSION_SIZE + SPH1_PRODUCT_FORMAT_VERSION_SIZE + SPH1_TIME_COR_SIZE + SPH1_PMD_ENTRY_POINT_SIZE + SPH1_SUBCOUNT_ENTRY_POINT_SIZE + SPH1_INTG_STATUS_ENTRY_POINT_SIZE + SPH1_PELTIER_ENTRY_POINT_SIZE + SPH1_INSTR_STATUS_2_ENTRY_POINT_SIZE + SPH1_PMD_ENG_CONV_SIZE + SPH1_ERS_PROPAGATOR_STATE_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class SPH1
\*============================================================================*/
typedef struct SPH1 {
    short Nref;
    char (*PIR_Of_SDH)[PIR_SIZE];
    char SoftwareVersion[LEVEL_1_PRODUCT_VERSION_SIZE];
    char KeyDataDataVersion[LEVEL_1_PRODUCT_VERSION_SIZE];
    short ProductFormatVersion;
    TimeCor_s TimeCor;
    short PMD_EntryPoint;
    short SubsetCounterEntryPoint;
    short IntegrationStatus_EntryPoint;
    short PeltierEntryPoint;
    short InstrStatus_2_EntryPoint;
    EngConversion_s PMD_EngConv[PMD_DEFINED];
    ERS_PropagatorState_s ERS_PropagatorState;
} SPH1_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SPH1__showERSInformation(const SPH1_s* self, FILE* Stream);

FunctionStatus_e
SPH1__read(SPH1_s* self, FILE* file_ptr, Bool convert_sun_2_host);

void
SPH1__show(const SPH1_s *Param, FILE* Stream);

FunctionStatus_e
FPA__GetPeltier(FILE *Stream, ProductType_e ProductType, const SPH1_s *SPH1,
		long FilePos_xCD, long xCD_Size,
		short PeltierDataSize, float PeltierData[],
		Bool CONVERT_SUN_2_HOST);

FunctionStatus_e
SPH1__getAuxiliaryParameter(const SPH1_s* self, const GOME_Word_t *AuxiliaryData,
			    AuxiliaryPar_s *AuxiliaryPar);

void
SPH1__checkSoftwareVersion(SPH1_s* self);

FunctionStatus_e
SPH1__checkProductVersion(SPH1_s* self);


/*============================================================================*\
**  private operations
\*============================================================================*/
void
SPH1__Sun2Host(SPH1_s *Param);

Bool
GetAverageModeExtract(const GOME_Word_t Value);

void
GetIntegrStatus(GOME_Word_t IntegrStatusWord, GO_IntegrStatus_s *IntegrStatus);


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
