/*----------------------------------------------------------------------------*\
**                              splint.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function splint
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)splint.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"
#include "log.h"


#define	SS_ID	("splint")

/*----------------------------------------------------------------------------*\
**                                splint
**  Input parameters:
**    vector xa[1..n], ya[1..n], y2a[1..n]
**    short n
**    double x
**  Output parameters:
**    double *y
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Given arrays xa[1..n] and ya[1..n] wich tabulate a function and
**    given the array y2a[1..n], wich is the output from spline above,
**    and given a value x, this routine returns a cubic-spline
**    interpolated value y.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    11.2.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void splint(double xa[],double ya[],double y2a[], short n,double x,double *y, FunctionStatus_e * Status)
{
	short klo,khi,k;
	double h,b,a;
	void nrerror();

	klo=1;
	khi=n;
	while (khi-klo > 1) {
		k=(khi+klo) >> 1;
		if (xa[k] > x) khi=k;
		else klo=k;
	}
	h=xa[khi]-xa[klo];
	if (h == 0.0) {
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_55", ERROR, LOG_NUMRECIP, "Bad XA input", "");
		*Status = ERROR;
		return;
	}
	a=(xa[khi]-x)/h;
	b=(x-xa[klo])/h;
	*y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}
