/*----------------------------------------------------------------------------*\
**                                StateVectorC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)StateVectorC.c	5.1 02/26/99, Copyright DLR 1998"

#include "stattorc.h"
#include "log.h"
#include "dataconv.h"


#define	SccsId	("@(#)StateVectorC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("StateVector")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           StateVectorC__Sun2Host
**  Input parameters:
**    StateVector		in Sun memory representation
**  Output parameters:
**    StateVector		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the StateVector struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
StateVectorC__Sun2Host( StateVectorC_s *Param)
{

   /* unsigned long */
  Param->OrbitNumber = SunLong2Long( (unsigned char*)&Param->OrbitNumber);
  /* float */
  Param->Position.X = SunFloat2Float( (unsigned char*)&Param->Position.X);
  /* float */
  Param->Position.Y = SunFloat2Float( (unsigned char*)&Param->Position.Y);
  /* float */
  Param->Position.Z = SunFloat2Float( (unsigned char*)&Param->Position.Z);
  /* float */
  Param->Velocity.X = SunFloat2Float( (unsigned char*)&Param->Velocity.X);
  /* float */
  Param->Velocity.Y = SunFloat2Float( (unsigned char*)&Param->Velocity.Y);
  /* float */
  Param->Velocity.Z = SunFloat2Float( (unsigned char*)&Param->Velocity.Z);

}

/*----------------------------------------------------------------------------*\
**                              StateVectorC__Read
**  Input parameters:
**	Stream		input stream to read State Vector
**  Output parameters:
**	StateVec	read State Vector
**
**  Other interfaces:
**
**  Description:
**	Function to read all State Vector parameter from the given 
**	input stream into the State Vector structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
StateVectorC__read(StateVectorC_s *StateVec, FILE* Stream, 
		  Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes;
    long NoOfReadBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** read StateVector
    \*------------------------------------------------------------------------*/
    if (UTC__read(&StateVec->DateUTC,  Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf(LogBuffer, "StateVector UTC time, #read bytes: %d", 
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__104", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }
    NoOfBytes = UTC_SIZE;

    NoOfBytes += fread(&StateVec->OrbitNumber, 1, STATE_VEC_ORBIT_NUMBER,
		       Stream);
    NoOfBytes += fread(&StateVec->Position.X, 1, sizeof( StateVec->Position.X),
		       Stream);
    NoOfBytes += fread(&StateVec->Position.Y, 1, sizeof( StateVec->Position.Y),
		       Stream);
    NoOfBytes += fread(&StateVec->Position.Z, 1, sizeof( StateVec->Position.Z),
		       Stream);
    NoOfBytes += fread(&StateVec->Velocity.X, 1, sizeof( StateVec->Velocity.X),
		       Stream);
    NoOfBytes += fread(&StateVec->Velocity.Y, 1, sizeof( StateVec->Velocity.Y),
		       Stream);
    NoOfBytes += fread(&StateVec->Velocity.Z, 1, sizeof( StateVec->Velocity.Z),
		       Stream);
    if (NoOfBytes != STATE_VECTOR_SIZE) 
    {
	Status = FATAL;
	sprintf( LogBuffer, "StateVector, #read bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__128", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	StateVectorC__Sun2Host( StateVec);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           StateVectorC__Show
**  Input parameters:
**	Stream		output stream
**	StateVec	State vector structure
**
**  Output parameters:
**
**  Other interfaces:
**	State vector structure written to the output stream
**
**  Description:
**	Function to write the State vector structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
StateVectorC__show(const StateVectorC_s *StateVec, FILE* Stream)
{
  char			ASCII_Time[ 30];

  fprintf( Stream, "\n*** StateVector\n");
  /*----------------------------------------------------------------------*\
  * Show Date and Time 
  \*----------------------------------------------------------------------*/
  UTC_2_ASCII( StateVec->DateUTC, ASCII_Time);
  fprintf( Stream, "UTC Days : %u, Msecs: %u, ASCII: %s\n",
	StateVec->DateUTC.Days, StateVec->DateUTC.Msecs,
	ASCII_Time);

  /*----------------------------------------------------------------------*\
  ** show StateVector
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Orbit Number: %u\n", StateVec->OrbitNumber);
  fprintf( Stream, "Position X: %f, Y: %f, Z: %f\n",
	StateVec->Position.X, StateVec->Position.Y, StateVec->Position.Z);
  fprintf( Stream, "Velocity X: %f, Y: %f, Z: %f\n",
	StateVec->Velocity.X, StateVec->Velocity.Y, StateVec->Velocity.Z);
}


/*----------------------------------------------------------------------------*\
**                              StateVectorC__Write
**  Input parameters:
**	Stream		output stream
**	Param		State Vector
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the State Vector to the given output stream
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.4.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
StateVectorC__write(const StateVectorC_s *Param, FILE* Stream)
{
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  char LogBuffer[LOG_FIELD_SIZE];

  /*----------------------------------------------------------------------*\
  ** write StateVector
  \*----------------------------------------------------------------------*/
  NoOfBytes += fwrite( &Param->DateUTC.Days, 1, sizeof( Param->DateUTC.Days),
			Stream);
  NoOfBytes += fwrite( &Param->DateUTC.Msecs, 1, sizeof( Param->DateUTC.Msecs),
			Stream);
  NoOfBytes += fwrite( &Param->OrbitNumber, 1, sizeof( Param->OrbitNumber),
			Stream);
  NoOfBytes += fwrite( &Param->Position.X, 1, sizeof( Param->Position.X),
			Stream);
  NoOfBytes += fwrite( &Param->Position.Y, 1, sizeof( Param->Position.Y),
			Stream);
  NoOfBytes += fwrite( &Param->Position.Z, 1, sizeof( Param->Position.Z),
			Stream);
  NoOfBytes += fwrite( &Param->Velocity.X, 1, sizeof( Param->Velocity.X),
			Stream);
  NoOfBytes += fwrite( &Param->Velocity.Y, 1, sizeof( Param->Velocity.Y),
			Stream);
  NoOfBytes += fwrite( &Param->Velocity.Z, 1, sizeof( Param->Velocity.Z),
			Stream);
  if (NoOfBytes != STATE_VECTOR_SIZE)
  {
    Status = FATAL;
    sprintf( LogBuffer, "StateVector, #written bytes: %d", NoOfBytes);
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__242", Status, LOG_CANNOT_WRITE,
		LogBuffer, "");

  }

  return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
