/*----------------------------------------------------------------------------*\
**                                StateVectorC.h
**
**  Class StateVector declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _StateVector_h
#define _StateVector_h

#pragma ident "@(#)StateVectorC.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "defs.h"
#include "basector.h"
#include "utc.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/
/* static const short STATE_VECTOR_SIZE = 36; */

#define	STATE_VEC_ORBIT_NUMBER	(LONG_SIZE)

#define	STATE_VECTOR_SIZE	(UTC_SIZE + STATE_VEC_ORBIT_NUMBER + BASE_VECTOR_SIZE + BASE_VECTOR_SIZE)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class StateVector
\*============================================================================*/
typedef struct StateVectorC {
    UTC_s DateUTC;		/* Date in UTC format */
    unsigned long OrbitNumber;	/* Orbit number */
    BaseVector_s Position;	/* X, Y, Z */
    BaseVector_s Velocity;	/* dx/dt, dy/dt, dz/dt */
} StateVectorC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
StateVectorC_s*
StateVectorC__New(FILE *Stream, FunctionStatus_e *Status);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void 
StateVectorC__Delete(StateVectorC_s *Self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
StateVectorC__write(const StateVectorC_s *self, FILE* file_ptr);

FunctionStatus_e
StateVectorC__read(StateVectorC_s *self, FILE* file_ptr,
		   Bool convert_sun_2_host);

void
StateVectorC__show(const StateVectorC_s *self, FILE* file_ptr);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif

