/*----------------------------------------------------------------------------*\
**                                StraylightCharacteristic.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)StraylightCharacteristic.c	5.1 02/26/99, Copyright DLR 1998"

#include "strastic.h"
#include "dataconv.h"
#include "log.h"
#include "numrecip.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)StraylightCharacteristic.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("StraylightCharacteristic")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            StraylightCharacteristic__Sun2Host
**  Input parameters:
**    StraylightCharacteristic	in Sun memory representation
**  Output parameters:
**    StraylightCharacteristic	in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the StraylightCharacteristic struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
StraylightCharacteristic__Sun2Host( StraylightCharacteristic_s *Param)
{
    int	i, j;

    for ( j = 0; j < OPTICAL_CHANNELS; j++)
    {
	/* float */
	Param->UniformLevel[ j] = SunFloat2Float( (unsigned char*)
						  &Param->UniformLevel[ j]);
	for ( i = 0; i < STRAYLIGHT_GHOSTS; i++)
	{
	    /* short */
	    Param->Ghost[ j][ i].Symmetry = SunShort2Short( (unsigned char*)
							    &Param->Ghost[ j][ i].Symmetry);
	    /* short */
	    Param->Ghost[ j][ i].Centre = SunShort2Short( (unsigned char*)
							  &Param->Ghost[ j][ i].Centre);
	    /* float */
	    Param->Ghost[ j][ i].Defocusing = SunFloat2Float( (unsigned char*)
							      &Param->Ghost[ j][ i].Defocusing);
	    /* float */
	    Param->Ghost[ j][ i].Energy = SunFloat2Float( (unsigned char*)
							  &Param->Ghost[ j][ i].Energy);
	}
    }

    /* short */
    Param->TriangleConvolutionWidth = SunShort2Short( (unsigned char*)
						      &Param->TriangleConvolutionWidth);
}

/*----------------------------------------------------------------------------*\
**                                StraylightCharacteristic__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		Straylight Characteristic
**
**  Other interfaces:
**
**  Description:
**	Function to read all Straylight Characteristic entries from the given 
**	input stream into the Straylight Characteristic structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
StraylightCharacteristic__read(StraylightCharacteristic_s *Param, FILE * Stream,
			       Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    int	i, j;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read UniformLevel
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread(Param->UniformLevel, 1,
		       sizeof( Param->UniformLevel), Stream);
    /*----------------------------------------------------------------------*\
    ** read Ghost
    \*----------------------------------------------------------------------*/
    for ( i = 0; i < OPTICAL_CHANNELS; i++)
	for ( j = 0; j < STRAYLIGHT_GHOSTS; j++)
	{
	    NoOfBytes += fread(&Param->Ghost[ i][ j].Symmetry, 1,
			       sizeof( Param->Ghost[ i][ j].Symmetry), Stream);
	    NoOfBytes += fread(&Param->Ghost[ i][ j].Centre, 1,
			       sizeof( Param->Ghost[ i][ j].Centre), Stream);
	    NoOfBytes += fread(&Param->Ghost[ i][ j].Defocusing, 1,
			       sizeof(Param->Ghost[i][j].Defocusing), Stream);
	    NoOfBytes += fread(&Param->Ghost[ i][ j].Energy, 1,
			       sizeof( Param->Ghost[ i][ j].Energy), Stream);
	}

    NoOfBytes += fread( &Param->TriangleConvolutionWidth, 1,
			sizeof( Param->TriangleConvolutionWidth), Stream);

    /*----------------------------------------------------------------------*\
    ** check length
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != STRAYLIGHT_CHARACTERISTIC_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Straylight Characteristic, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__151", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	StraylightCharacteristic__Sun2Host( Param);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                StraylightCharacteristic__Show
**  Input parameters:
**	Stream		output stream
**	Param		Straylight Characteristic structure
**
**  Output parameters:
**
**  Other interfaces:
**	Straylight Characteristic structure written to the output stream
**
**  Description:
**	Function to write the Straylight Characteristic structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
StraylightCharacteristic__show(const StraylightCharacteristic_s *Param,
			       FILE * Stream)
{
    int	i, j;

    fprintf( Stream, "Straylight Characteristic (Channel 1 2 3 4)\n");
    /*----------------------------------------------------------------------*\
    ** print UniformLevel
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Uniform Level");
    for ( i = 0; i < OPTICAL_CHANNELS; i++)
	fprintf( Stream, "\t%f ", Param->UniformLevel[ i]);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** print Ghost
    \*----------------------------------------------------------------------*/
    for ( j = 0; j < STRAYLIGHT_GHOSTS; j++)
    {
	fprintf( Stream, "Symmetry %1d", j);
	for ( i = 0; i < OPTICAL_CHANNELS; i++)
	    fprintf( Stream, "\t%d", Param->Ghost[ i][ j].Symmetry);
	fprintf( Stream, "\n");
    }
    for ( j = 0; j < STRAYLIGHT_GHOSTS; j++)
    {
	fprintf( Stream, "Centre %1d", j);
	for ( i = 0; i < OPTICAL_CHANNELS; i++)
	    fprintf( Stream, "\t%d", Param->Ghost[ i][ j].Centre);
	fprintf( Stream, "\n");
    }
    for ( j = 0; j < STRAYLIGHT_GHOSTS; j++)
    {
	fprintf( Stream, "Defocusing %1d", j);
	for ( i = 0; i < OPTICAL_CHANNELS; i++)
	    fprintf( Stream, "\t%f", Param->Ghost[ i][ j].Defocusing);
	fprintf( Stream, "\n");
    }
    for ( j = 0; j < STRAYLIGHT_GHOSTS; j++)
    {
	fprintf( Stream, "Energy %1d", j);
	for ( i = 0; i < OPTICAL_CHANNELS; i++)
	    fprintf( Stream, "\t%f", Param->Ghost[ i][ j].Energy);
	fprintf( Stream, "\n");
    }
    fprintf( Stream, "TriangleConvolutionWidth %2hd\n",
	     Param->TriangleConvolutionWidth);

}


/*----------------------------------------------------------------------------*\
**                                StrayLight__Correction
**  Input parameters:
**    StraylightCharacteristic_s (Energy levels of Ghost and Uniform Straylight,
**    pixels of symmetry ...)
**    Channel (1..4)
**    Reticon data (1024)
**    IntgTimeA, IntgTimeB two Integration times for that channel (may be different)
**    PixelIndizes
**    date (string)
**  Output parameters:
**    Straylight corrected Reticon data (1024)
**  Other interfaces:
**  Description:
**      Calculation and application of Straylight Correction
**  References:
**  Libraries:
**  Created:	22.7.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void StrayLight__Correction(const StraylightCharacteristic_s *Self,
			    OpticalChannel_e Channel, float Reticon[], float IntgTimeA, float IntgTimeB,
			    const PixelIndex_s PixelIndex[], const char *PixelDate)
{
    int loop;
    int Band_loop;
    ScienceData_e Band;
    short ChannelBands [MAX_BANDS_PER_CHANNEL];
    float IntgTime [MAX_BANDS_PER_CHANNEL];
    float Energy;

    int loop_right;
    int loop_left;
    int start;
    short Ghost_Number;
    short KernelHalfSize;

    int Begin, End;

    float Sum;
    int Number;
    float MeanValue;
    float UniformStraylight = 0.0;
    float BandUniformStraylight;

    float GhostStraylight [DETECTOR_PIXELS];
    float SmoothedGhostStraylight [DETECTOR_PIXELS];
    float TotalStraylight [DETECTOR_PIXELS];
    float Detector [DETECTOR_PIXELS];

#ifdef DEBUG
    float InputDetector [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];
#endif


    /*-----------------------------------------------------------------------*\
    ** Initialize
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
    {
	GhostStraylight [loop] = 0.0;
	SmoothedGhostStraylight [loop] = 0.0;
	TotalStraylight [loop] = 0.0;
	Detector [loop] = 0.0;
    }

    /*-----------------------------------------------------------------------*\
    ** Organize the bands and integration times in this channel
    \*-----------------------------------------------------------------------*/
    for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
    {
	ChannelBands [Band_loop] = -1; 
	IntgTime [Band_loop] = -1.0;
    }


    switch (Channel)
    {
    case OPTICAL_CH_1 : 
	{
	    ChannelBands[0] = BAND_1A; 
	    ChannelBands[1] = BAND_1B;
	    IntgTime[0] = IntgTimeA; 
	    IntgTime[1] = IntgTimeB;
	    break;
	}
    case OPTICAL_CH_2 : 
	{
	    ChannelBands[0] = BAND_2A; 
	    ChannelBands[1] = BAND_2B;
	    IntgTime[0] = IntgTimeA;
	    IntgTime[1] = IntgTimeB;
	    break;
	}
    case OPTICAL_CH_3 : 
	{
	    ChannelBands[0] = BAND_3;
	    IntgTime[0] = IntgTimeA; 
	    break;
	}
    case OPTICAL_CH_4 : 
	{
	    ChannelBands[0] = BAND_4;
	    IntgTime[0] = IntgTimeA; 
	    break;
	}
    }


    /*-----------------------------------------------------------------------*\
    ** Save Input for debug output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
	InputDetector [loop] = 0.0;

    if (Debug__isActive(debug_ptr, DBG_STRAYLIGHT, DBG_LEVEL_0))
    {
	for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
	{
	    if (ChannelBands [Band_loop] >= 0 && IntgTime [Band_loop] > 0.0)
	    {
		Band = (ScienceData_e) ChannelBands [Band_loop];
		for (loop = PixelIndex[Band].First;
		     loop <= PixelIndex[Band].Last; loop++)
		{
		    InputDetector [loop] = Reticon [loop];
		}
	    }
	}
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Divide Detector by integration time
    \*-----------------------------------------------------------------------*/
    for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
    {
	if (ChannelBands [Band_loop] >= 0 && IntgTime [Band_loop] > 0.0)
	{
	    Band = (ScienceData_e) ChannelBands [Band_loop];
	    for (loop = PixelIndex[Band].First; loop <= PixelIndex[Band].Last; loop++)
	    {
		Detector [loop] = Reticon [loop] / IntgTime [Band_loop];
	    }
	}
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate uniform straylight
    ** Build mean value of input signal
    \*-----------------------------------------------------------------------*/
    Sum = 0.0;
    Number = 0;
    UniformStraylight = 0.0;

    if (Self->UniformLevel[Channel] > 0.0)
    {
	for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
	{
	    if (ChannelBands [Band_loop] >= 0 && IntgTime [Band_loop] > 0.0)
	    {
		Band = (ScienceData_e) ChannelBands [Band_loop];
		for (loop = PixelIndex[Band].First; loop <= PixelIndex[Band].Last; loop++)
		{
		    Sum += Detector [loop];
		    Number++;
		}
	    }
	}

	MeanValue = Sum / (float) Number;
	UniformStraylight = MeanValue * ( Self->UniformLevel[Channel] );
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate uniform straylight from Straylight bands
    ** Build mean value of Straylight bands
    \*-----------------------------------------------------------------------*/

    /*-----------------------------------------------------------------------*\
    ** currently not used 

    Sum = 0.0;
    Number = 0;
    BandUniformStraylight = 0.0;

    switch (Channel)
    {
    case OPTICAL_CH_1 : 
    Begin = PixelIndex[STRAYLIGHT_1A].First; End = PixelIndex[STRAYLIGHT_1A].Last; 
    Begin = PixelIndex[STRAYLIGHT_1B].First; End = PixelIndex[STRAYLIGHT_1B].Last; 
    Begin = PixelIndex[BAND_1B].Last; End = PixelIndex[BAND_1B].Last; 
    break;
    case OPTICAL_CH_2 : 
    Begin = PixelIndex[STRAYLIGHT_2A].First; End = PixelIndex[STRAYLIGHT_2A].Last; 
    Begin = PixelIndex[BAND_2A].First; End = PixelIndex[BAND_2A].First; 
    break;
    case OPTICAL_CH_3 : 
    Begin = PixelIndex[BAND_3].First; End = PixelIndex[BAND_3].First; 
    break;
    case OPTICAL_CH_4 : 
    Begin = PixelIndex[BAND_4].First; End = PixelIndex[BAND_4].First; 
    break;
    default : 
    Begin = 0; End = 0; 
    break;		
    }

    for (loop = Begin; loop <= End; loop++)
    {
    Sum += Detector [loop];
    Number++;
    }

    BandUniformStraylight = Sum / (float) Number;
    \*-----------------------------------------------------------------------*/

    /*-----------------------------------------------------------------------*\
    ** Calculate ghost straylight
    \*-----------------------------------------------------------------------*/
    for (Ghost_Number = 0; Ghost_Number < STRAYLIGHT_GHOSTS; Ghost_Number++)
    {
	Energy = (Self->Ghost [Channel] [Ghost_Number] ).Energy / 100.0;
	start = (Self->Ghost [Channel] [Ghost_Number] ).Symmetry;

	if (Energy > 0.0)
	{
	    /* first calculate pixel of symmetry */
	    GhostStraylight [start] += (Detector [start] * Energy);

	    /* then run to the left and to the right through the detector, starting at the pixel of symmetry */

	    loop_left = start - 1;
	    loop_right = start + 1;

	    while (loop_right < DETECTOR_PIXELS && loop_left >= 0)
	    {
		GhostStraylight [loop_right] += Detector [loop_left] * Energy;
		GhostStraylight [loop_left] += Detector [loop_right] * Energy;
		
		loop_left--;
		loop_right++;
	    }
	}
    }

    /*-----------------------------------------------------------------------*\
    ** Smooth ghost straylight
    \*-----------------------------------------------------------------------*/
    /* KernelHalfSize = 5; */
    KernelHalfSize = 3;
    KernelHalfSize = Self->TriangleConvolutionWidth;

    for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
    {
	if (ChannelBands [Band_loop] >= 0 && IntgTime [Band_loop] > 0.0)
	{
	    Band = (ScienceData_e) ChannelBands [Band_loop];

	    /* this is a triangle convolution */
	    triangle( GhostStraylight, SmoothedGhostStraylight,
		      KernelHalfSize, PixelIndex[Band].First, PixelIndex[Band].Last);
	}
    }

    /*-----------------------------------------------------------------------*\
    ** Add uniform and ghost straylight
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
	TotalStraylight [loop] = UniformStraylight + SmoothedGhostStraylight [loop];
    /* TotalStraylight [loop] = BandUniformStraylight + SmoothedGhostStraylight [loop]; */

    /*-----------------------------------------------------------------------*\
    ** Subtract straylight from Detector
    \*-----------------------------------------------------------------------*/
    for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; Band_loop++)
    {
	if (ChannelBands [Band_loop] >= 0 && IntgTime [Band_loop] > 0.0)
	{
	    Band = (ScienceData_e) ChannelBands [Band_loop];
	    for (loop = PixelIndex[Band].First; loop <= PixelIndex[Band].Last; loop++)
		Reticon [loop] -= (TotalStraylight [loop] * IntgTime [Band_loop]);
	}
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("StrayLight__Correction", 
					       PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_STRAYLIGHT, DBG_LEVEL_0))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Channel == OPTICAL_CH_1)
	    {
		fprintf(out, "\nIntegration times : ");
		for (Band_loop = 0; Band_loop < MAX_BANDS_PER_CHANNEL; 
		     Band_loop++)
		{
		    if ( (ChannelBands [Band_loop] >= 0 )
			&& (IntgTime [Band_loop] > 0.0) )
		    {
			fprintf(out, "Band %d : %f  ", ChannelBands[Band_loop],
				IntgTime[Band_loop]);
		    }
		}
		fprintf(out, "\n");

		fprintf(out, 
			"Channel    Pixel      Input      Input/IT   UnifStrayl UnifStrBa  GhostStr   SmooGS     TotalStr   OutputRet  \n");
		fprintf(out, 
			"[]         []         [BU]       [BU/s]     [BU/s]     [BU/s]     [BU/s]     [BU/s]     [BU/s]     [BU]       \n\n");
	    }
	
	    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g\n", 
			Channel+1, 
			loop + (Channel * DETECTOR_PIXELS), 
			DEBUG_WIDTH, DEBUG_PRECISION, InputDetector[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Detector[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, UniformStraylight, 
			DEBUG_WIDTH, DEBUG_PRECISION, BandUniformStraylight, 
			DEBUG_WIDTH, DEBUG_PRECISION, GhostStraylight [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, SmoothedGhostStraylight [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, TotalStraylight [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon [loop]);
	    }
		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_358", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}


/*============================================================================*\
**  private operations
\*============================================================================*/


