/*----------------------------------------------------------------------------*\
**                                StraylightCharacteristic.h
**
**  Class StraylightCharacteristic declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _StraylightCharacteristic_h
#define _StraylightCharacteristic_h

#pragma ident "@(#)StraylightCharacteristic.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include "strahost.h"
#include "pixendex.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	STRAYLIGHT_GHOSTS	(2)


#define	STRAYLIGHT_CHARACTERISTIC_SIZE	(FLOAT_SIZE * OPTICAL_CHANNELS + OPTICAL_CHANNELS * STRAYLIGHT_GHOSTS * STRAYLIGHT_GHOST_SIZE + SHORT_SIZE)


#define	MAX_BANDS_PER_CHANNEL	(5)


/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class StraylightCharacteristic
\*============================================================================*/
typedef struct StraylightCharacteristic {
    float UniformLevel[OPTICAL_CHANNELS]; /* Uniform straylight level */
    StraylightGhost_s Ghost[OPTICAL_CHANNELS][STRAYLIGHT_GHOSTS];
				/* Symmetrical and asymmetrical ghosts 
				   characteristics */
    short TriangleConvolutionWidth; /* Width of the triangle convolution window
				       used in straylight */
} StraylightCharacteristic_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
void 
StrayLight__Correction(const StraylightCharacteristic_s *Self,
		       OpticalChannel_e Channel, float Reticon[],
		       float IntgTimeA, float IntgTimeB,
		       const PixelIndex_s PixelIndex[],
		       const char *PixelDate); 

FunctionStatus_e
StraylightCharacteristic__read(StraylightCharacteristic_s* self, FILE* file_ptr, 
			       Bool convert_sun_2_host);

void
StraylightCharacteristic__show(const StraylightCharacteristic_s* self,
			       FILE* file_ptr);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
