/*----------------------------------------------------------------------------*\
**                                SunReference.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SunReference.c	5.1 02/26/99, Copyright DLR 1998"

#include "sunrence.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)SunReference.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("SunReference")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           SunReference__Sun2Host
**  Input parameters:
**    SunReference		in Sun memory representation
**  Output parameters:
**    SunReference		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the SunReference struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SunReference__Sun2Host( float SunReferenceSpec[ OPTICAL_CHANNELS] 
			[ DETECTOR_PIXELS])
{
    int	i, j;

    for ( j = 0; j < OPTICAL_CHANNELS; j++)
	for ( i = 0; i < DETECTOR_PIXELS; i++)
	{
	    SunReferenceSpec[ j][ i] =  SunFloat2Float( (unsigned char*)
							&SunReferenceSpec[ j][ i]);
	}
}

/*----------------------------------------------------------------------------*\
**                              SunReference__Read
**  Input parameters:
**	Stream			input stream
**  Output parameters:
**	SunReferenceSpec	read Sun Reference Spectrum
**
**  Other interfaces:
**
**  Description:
**	Function to read the Sun Reference Spectrum from the given 
**	input stream 
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SunReference__read(float SunReferenceSpec[OPTICAL_CHANNELS][DETECTOR_PIXELS],
		   FILE* Stream, Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read Sun Reference Spectrum
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( SunReferenceSpec, 1,
			sizeof( SunReferenceSpec[0][0]) * OPTICAL_CHANNELS * DETECTOR_PIXELS,
			Stream);
    /*----------------------------------------------------------------------*\
    ** error check
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != SUN_REF_SPECTRUM_MEAS_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "SunReference Spectrum, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__102", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (CONVERT_SUN_2_HOST)
    {
	SunReference__Sun2Host( SunReferenceSpec);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           SunReference__Show
**  Input parameters:
**	Stream			output stream
**	SunReferenceSpec	Sun Reference Spectrum
**
**  Output parameters:
**
**  Other interfaces:
**	Sun Reference Spectrum written to the output stream
**
**  Description:
**	Function to write the Sun Reference Spectrum to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SunReference__show(const float
		   SunReferenceSpec[OPTICAL_CHANNELS][DETECTOR_PIXELS],
		   FILE* Stream)
{
    int	i, j;

    /*----------------------------------------------------------------------*\
    ** show Sun Reference Spectrum
    \*----------------------------------------------------------------------*/
    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", SunReferenceSpec[ j][ i]);
	fprintf( Stream, "\n");
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/


