/*----------------------------------------------------------------------------*\
**                              svbksb.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function svbksb
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)svbksb.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"

/*----------------------------------------------------------------------------*\
**                                svbksb
**  Input parameters:
**    Matrix u[1..m][1..n]
**    Vector w[1..n]
**    Matrix v[1..n][1..n]
**    short m, n
**    Vector b[1..m] input right-hand side
**  Output parameters:
**    Vector x[1..n] output solution vector
**  Other interfaces: none
**  Description:
**    Backsubstitution routine. Solves A*X = B for a vector X.
**    A is specified by the arrays u, w and v as returned by svdcmp
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    12.2.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void svbksb(double **u,double w[],double **v, short m, short n,double b[],double x[])
{
	short jj,j,i;
	double s,*tmp;

	tmp=vector__new(1,n);
	for (j=1;j<=n;j++) {
		s=0.0;
		if (w[j]) {
			for (i=1;i<=m;i++) s += u[i][j]*b[i];
			s /= w[j];
		}
		tmp[j]=s;
	}
	for (j=1;j<=n;j++) {
		s=0.0;
		for (jj=1;jj<=n;jj++) s += v[j][jj]*tmp[jj];
		x[j]=s;
	}
	vector__delete(tmp,1,n);
}
