/*----------------------------------------------------------------------------*\
**                              svdvar.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function svdvar
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)svdvar.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"

/*----------------------------------------------------------------------------*\
**                                svbvar
**  Input parameters:
**    Matrix v[1..ma][1..ma]
**    int ma
**    Vector w[1..ma]
**  Output parameters:
**    Matrix cvm[1..ma][1..ma]
**  Other interfaces: none
**  Description:
**    To evaluate the covariance matrix cvm of the fit ma
**    parameters obtained by svdfit, call this routine wiht matrixes
**    v and w as returned from svdfit
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    14.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void svdvar( double **v, short ma, double w[], double **cvm)
{
	short k,j,i;
	double sum,*wti;

	wti=vector__new(1,ma);
	for (i=1;i<=ma;i++) {
		wti[i]=0.0;
		if (w[i]) wti[i]=1.0/(w[i]*w[i]);
	}
	for (i=1;i<=ma;i++) {
		for (j=1;j<=i;j++) {
			for (sum=0.0,k=1;k<=ma;k++) sum += v[i][k]*v[j][k]*wti[k];
			cvm[j][i]=cvm[i][j]=sum;
		}
	}
	vector__delete(wti,1,ma);
}
