/*----------------------------------------------------------------------------*\
**                                Sys.c
**
**  Description:
** 
**  Version:    5.3
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Sys.c	5.3 08/19/02, Copyright DLR 1998"

#include "sys.h"


#define	SccsId	("@(#)Sys.c 5.3 08/19/02, Copyright DLR 1998")


#define	SS_ID	("Sys")


/*============================================================================*\
**  public operations
\*============================================================================*/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#ifndef __MSDOS__

#include <ctype.h>
#include <unistd.h>

#ifndef _AIX /* Solaris and others */
#ifndef linux /* Solaris and others */

#ifdef _INCLUDE_HPUX_SOURCE
#include <sys/sysinfo.h>	/* HP */
#else
#include <sys/systeminfo.h>	/* Solaris */
#endif

#endif /* linux */
#endif /* _AIX */

#endif /* __MSDOS__ */


#pragma ident	"@(#)system.c	4.5 10/28/97, Copyright DLR 1994"


/*----------------------------------------------------------------------------*\
**                                Sys__GetHostName
**  Input parameters:
**    <none>
**  Output parameters:
**    Hostname the name of the host machine
**  Other interfaces:
**    <none>
**  Description:
**    Get the Hostname for the current processor
**    NOTE: use gethostname instead of sysinfo for BSD compatible systems
**          int gethostname( char *name, int namelen);
**  References:
**    <none>
**  Libraries:
**    This function is operating system and machine dependent
**  Created:	30.5.95
**  Author:	Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
Sys__GetHostName(char *Hostname, size_t MaxSize)
{
#ifdef _INCLUDE_HPUX_SOURCE
        strncpy(Hostname, "HP_UX", MaxSize);
#elif __MSDOS
        strncpy(Hostname, "MSDOS", MaxSize);
#elif _AIX
    if (gethostname(Hostname, strlen(Hostname)-1) == -1)
    {
	strncpy(Hostname, "UNKNOW HOST MACHINE NAME", MaxSize);
	return False;
    }
#elif linux
    if (gethostname(Hostname, strlen(Hostname)-1) == -1)
    {
	strncpy(Hostname, "UNKNOW HOST MACHINE NAME", MaxSize);
	return False;
    }
#else	/* Solaris and others */
    if (sysinfo(SI_HOSTNAME, Hostname, MaxSize) == -1)
    {
	strncpy(Hostname, "UNKNOW HOST MACHINE NAME", MaxSize);
	return False;
    }
#endif /* __MSDOS__ */
    return True;
}


/*----------------------------------------------------------------------------*\
**                                Sys__GetPid
**  Input parameters:
**    <none>
**  Output parameters:
**    Pid the process ID of the calling process
**  Other interfaces:
**    <none>
**  Description:
**    Get the Hostname
**  References:
**    <none>
**  Libraries:
**    This function is operating system and machine dependent
**  Created:	30.5.95
**  Author:	Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
Sys__GetPid(long *Pid)
{
#ifdef __MSDOS__
    *Pid = -1;
#else	/* Solaris and others */
    if ((*Pid = getpid()) == -1)
    {
	return False;
    }
#endif /* __MSDOS__ */
    return True;
}



/*----------------------------------------------------------------------------*\
**                                Sys__GetDate
**  Input parameters:
**    <none>
**  Output parameters:
**    Hostname the name of the host machine
**  Other interfaces:
**    <none>
**  Description:
**    Get the current Date using the format DD MM YYYY HH:MM:SS
**  References:
**    <none>
**  Libraries:
**    This function is operating system and machine dependent
**  Created:	30.5.95
**  Author:	Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
Bool
Sys__GetDate(char *Date, size_t MaxSize)
{
    Bool Result = True;

#ifdef __MSDOS__
    strcpy(Date, "DD-MMM-YYYY HH:MM:SS"); /* TBD Set the correct Date */
#else	/* Solaris and others */
    char *EnvPtr;
    time_t Clock=time(NULL);

    /*------------------------------------------------------------------------*\
    ** Save the value of the environment variable 'TZ', set it to the default
    ** value (MET) get the local time and restore the original 'TZ' value
    \*------------------------------------------------------------------------*/
    EnvPtr = getenv( "TZ");
    if (putenv("TZ=") != 0)
    {
	Result = False;
    }
    if (strftime(Date, MaxSize, "%d-%h-%Y %T", localtime(&Clock)) != 0)
    {
	Date[4] = toupper(Date[4]);
	Date[5] = toupper(Date[5]);
    }
    else
    {
	strcpy(Date, "DD-MMM-YYYY HH:MM:SS");
	Result = False;
    }
    if (EnvPtr != NULL)
    {
	EnvPtr -=3;
	if (putenv(EnvPtr) != 0)
	{
	    Result = False;
	}
    }
#endif /* __MSDOS__ */
    return Result;
}


#ifndef __MSDOS__
#ifndef _AIX
#ifndef __ALPHA
#endif
#endif
#endif


/*----------------------------------------------------------------------------*\
**                                terminateExecution
**  Input parameters:
**    <none>
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    This function is called when FATAL errors are detected. A error message
**    is printed in stderr and exit(1) is executed.
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	24.2.95
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void
terminateExecution(char* log_file_name)
{
    long pid;     
    Sys__GetPid(&pid);
    fprintf(stderr, "FATAL problems found during the execution. See the "
	    "messages in the log file '%s' (Pid %d) for a detailed "
	    "description\n", log_file_name, pid);
    exit (1);
}


/*----------------------------------------------------------------------------*\
**                                FunctionStatus2Str
**  Input parameters:
**    Enum FunctionStatus enumerator
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    A string representing the FunctionStatus_e Enum is returned
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    24.1.95
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
const char*
FunctionStatus2Str(FunctionStatus_e Enum)
{
    static const char *FunctionStatusStr[] = {"INFO", "WARNING", "SUCCESS",
					      "ERROR", "FATAL"};
    return FunctionStatusStr[Enum];
}



/*============================================================================*\
**  private operations
\*============================================================================*/

 
