/*----------------------------------------------------------------------------*\
**                                TimeCor.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)TimeCor.c	5.1 02/26/99, Copyright DLR 1998"

#include "timecor.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)TimeCor.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("TimeCor")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            TimeCor__Sun2Host
**  Input parameters:
**    TimeCor			in Sun memory representation
**  Output parameters:
**    TimeCor			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the TimeCor struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
TimeCor__Sun2Host( TimeCor_s *Param)
{
   /* long */
  Param->OrbitNumber = SunLong2Long( (unsigned char*)&Param->OrbitNumber);
   /* long */
  Param->SBT = SunLong2Long( (unsigned char*)&Param->SBT);
   /* long */
  Param->SBT_Period = SunLong2Long( (unsigned char*)&Param->SBT_Period);
}

/*----------------------------------------------------------------------------*\
**                                TimeCor__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	TimeCor		Time Correlation
**
**  Other interfaces:
**
**  Description:
**	Function to read all Time Correlation entries from the given 
**	input stream into the Time Correlation structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
TimeCor__read(TimeCor_s* self, FILE* file_ptr, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** read time correlation
    \*------------------------------------------------------------------------*/
    NoOfBytes = fread(&self->OrbitNumber, 1, TIME_COR_ORBIT_NUMBER_SIZE,
		      file_ptr);
    if ( (Status = UTC__read(&(self->UTC), file_ptr, convert_sun_2_host))
	 != SUCCESS)
    {
	sprintf(LogBuffer, "Time Correlation UTC time");
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__91", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }
    NoOfBytes += UTC_SIZE;

    NoOfBytes += fread(&self->SBT, 1, TIME_COR_SBT_SIZE, file_ptr);
    NoOfBytes += fread(&self->SBT_Period, 1, TIME_COR_SBT_PERIOD, file_ptr);
    if (NoOfBytes != TIME_COR_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer, "Time Correlation, #read bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__102", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	TimeCor__Sun2Host(self);
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                TimeCor__Write
**  Input parameters:
**	Stream		output stream
**	TimeCor		Time Correlation
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the Time Correlation to the given 
**	output
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
TimeCor__write(const TimeCor_s *TimeCor, FILE * Stream)
{
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  char LogBuffer[LOG_FIELD_SIZE];

  /*----------------------------------------------------------------------*\
  ** write time correlation
  \*----------------------------------------------------------------------*/
  NoOfBytes += fwrite( &TimeCor->OrbitNumber, 1,
		 sizeof( TimeCor->OrbitNumber), Stream); 
  NoOfBytes += fwrite( &TimeCor->UTC.Days, 1,
		 sizeof( TimeCor->UTC.Days), Stream);
  NoOfBytes += fwrite( &TimeCor->UTC.Msecs, 1, sizeof( TimeCor->UTC.Msecs),
				Stream);
  NoOfBytes += fwrite( &TimeCor->SBT, 1, sizeof( TimeCor->SBT),
				Stream);
  NoOfBytes += fwrite( &TimeCor->SBT_Period, 1,
			 sizeof( TimeCor->SBT_Period), Stream);
  if (NoOfBytes != TIME_COR_SIZE)
  {
    Status = FATAL;
    sprintf( LogBuffer, "Time Correlation, #written bytes: %d", NoOfBytes);
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__161", Status,
		 LOG_CANNOT_WRITE, LogBuffer, "");
  }

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                TimeCor__Show
**  Input parameters:
**	Stream		output stream
**	TimeCor		Time correlation structure
**
**  Output parameters:
**
**  Other interfaces:
**	Time correlation structure written to the output stream
**
**  Description:
**	Function to write the Time correlation structure to the output stream
**	in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
TimeCor__show(const TimeCor_s *TimeCor, FILE* Stream)
{
  char			ASCII_Time[ 30];

  /*----------------------------------------------------------------------*\
  * Show Date and Time 
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "TimeCor->OrbitNumber: %u\n", TimeCor->OrbitNumber); 
  UTC_2_ASCII( TimeCor->UTC, ASCII_Time);
  fprintf( Stream, "TimeCor->UTC   Days : %u, Msecs: %u, ASCII: %s\n",
	TimeCor->UTC.Days, TimeCor->UTC.Msecs, ASCII_Time);
  fprintf( Stream, "TimeCor->SBT        : %u\n", TimeCor->SBT);
  fprintf( Stream, "TimeCor->SBT_Period : %u\n", TimeCor->SBT_Period);
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
