/*----------------------------------------------------------------------------*\
**                              twofft.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function twofft
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)twofft.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"

/*----------------------------------------------------------------------------*\**                                twofft
**  Input parameters:
**    vector data1[1..n], data2[1..n]
**    int n
**  Output parameters:
**    vector fft1[1..n], fft2[1..n]
**  Other interfaces: none
**  Description:
**    Given two real input arrays data1[1..n] and data2[1..n]
**    this routine calls four1 and returns two complex output arrays,
**    fft1 and fft2, each of complex length n (i.e. real dimensions
**    [1..2n]), which contain the discrete Fourier transforms of the
**    respective datas. n MUST be an integer power of 2.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    27.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void twofft(double data1[],double data2[],double fft1[],double fft2[], short n)
{
	short nn3,nn2,jj,j;
	double rep,rem,aip,aim;

	nn3=1+(nn2=2+n+n);
	for (j=1,jj=2;j<=n;j++,jj+=2) {
		fft1[jj-1]=data1[j];
		fft1[jj]=data2[j];
	}
	four1(fft1,n,1);
	fft2[1]=fft1[2];
	fft1[2]=fft2[2]=0.0;
	for (j=3;j<=n+1;j+=2) {
		rep=0.5*(fft1[j]+fft1[nn2-j]);
		rem=0.5*(fft1[j]-fft1[nn2-j]);
		aip=0.5*(fft1[j+1]+fft1[nn3-j]);
		aim=0.5*(fft1[j+1]-fft1[nn3-j]);
		fft1[j]=rep;
		fft1[j+1]=aim;
		fft1[nn2-j]=rep;
		fft1[nn3-j] = -aim;
		fft2[j]=aip;
		fft2[j+1] = -rem;
		fft2[nn2-j]=aip;
		fft2[nn3-j]=rem;
	}
}
