/*----------------------------------------------------------------------------*\
**                                UTC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)UTC.c	5.1 02/26/99, Copyright DLR 1998"

#include <stdio.h>
#include <sys/types.h>
#ifndef __MSDOS__
#include <sys/uio.h>
#include <unistd.h>
#endif
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "utc.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)UTC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("UTC")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                utc2date
**  Input parameters:
**      utc_date        :       date in UTC format
**
**  Output parameters:
**      year	        :       year  ( e.g. 1993 )
**      mon	        :       month 
**      day	        :       day 
**      hour	        :       hour 
**      minute	        :       minute 
**      sec        	:       real second containing milliseconds as broken part
**    
**  Other interfaces:
**   
**  Description:
**	To compute unix date from utc date a offset ( days between 01/01/50 
**	(UTC reference) and 31/12/69 (UNIX reference)) is subtracted from utc days 
**	and the full seconds are added. So the unix reference second since 01.01.1970 
**	is computed. To compute ascii date from unix reference the full seconds 
**	contained in given milliseconds are added to the seconds and then the unix 
**	routine gmtime() is used to evaluate the ascii date
**
**  References:
**    
**  Libraries:
**   
**  Created:    24.2.94
**  Author:     Thomas Ruppert, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
FunctionStatus_e
utc2date(UTC_s utc_date, short *year , short *mon, short *day,
	 short *hour, short *min,float *sec)
{

	time_t		unix_secs;
	struct tm	*time_struct;

        unix_secs = (utc_date.Days - DAY_OFFSET) * SECS_OF_DAY + (time_t) (utc_date.Msecs / 1000);

/*----------------------------------------------------------------------------*\
**      call gmtime to get tm structure containing the ascii date
\*----------------------------------------------------------------------------*/
        time_struct = gmtime(&unix_secs);


        *year = time_struct->tm_year + 1900;
        *mon  = time_struct->tm_mon + 1;
        *day  = time_struct->tm_mday;
        *hour = time_struct->tm_hour;
        *min  = time_struct->tm_min;
        *sec  = (float) time_struct->tm_sec + (float) (utc_date.Msecs % 1000) / 1000.0;

        return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                date2utc
**  Input parameters:
**      year	        :       year  ( e.g. 1993 )
**      mon	        :       month 
**      day	        :       day 
**      hour	        :       hour 
**      minute	        :       minute 
**      sec        	:       real second containing milliseconds as broken part
**
**  Output parameters:
**      utc_date        :       date in UTC format
**    
**  Other interfaces:
**   
**  Description:
**	The utc date is computed by using the unix function mktime() to get seconds
**	from 1/1/1970 out of ascii date. Then utc days are computed by extracting 
**	days since 1/1/1970 and adding the offset ( days between 1/1/1950 and 1/1/1970)
**	To get milliseconds the remaining seconds are computed an the broken part of
**	input seconds are added.
**
**  References:
**    
**  Libraries:
**   
**  Created:    24.2.94
**  Author:     Thomas Ruppert, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
FunctionStatus_e
date2utc(short year , short mon, short day , short hour, short min,float sec,
	 UTC_s *utc_date)
{
        time_t  u_secs;
        int  FullSecs;
	struct tm time_str;

/*----------------------------------------------------------------------------*\
**	put time zone 0 in environment to prevent mktime from using local time
\*----------------------------------------------------------------------------*/

        putenv("TZ=0");

        FullSecs = (int) sec;

        time_str.tm_year       = (int)year - 1900;
        time_str.tm_mon        = (int)mon - 1;
        time_str.tm_mday       = (int)day;
        time_str.tm_hour       = (int)hour;
        time_str.tm_min        = (int)min;
        time_str.tm_sec        = FullSecs;
        time_str.tm_isdst      = -1;

        u_secs = mktime(&time_str);

	if ( u_secs == -1 ) return FATAL;

        utc_date->Days = u_secs / SECS_OF_DAY + DAY_OFFSET;
        utc_date->Msecs = u_secs %  SECS_OF_DAY * 1000 + (sec - FullSecs) * 1000;

        return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**				UTC_Add
** Inputs:
**   utc1	UTC time
**   utc2	UTC time
**
** Outputs:
**   utc	result utc
**		SUCCESS	= time added
**
** Description:
**   Procedure to add two UTC times
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
UTC_Add( UTC_s utc1, UTC_s utc2, UTC_s* utc)
{
    utc->Msecs = (utc1.Msecs + utc2.Msecs) % (unsigned long)86400000LU;
    utc->Days = utc1.Days + utc2.Days + (utc1.Msecs + utc2.Msecs) / 
	(unsigned long)86400000LU;
    return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**				UTC_Sub
** Inputs:
**   utc1	UTC time
**   utc2	UTC time
**
** Outputs:
**   utc	result UTC
**   return	SUCCESS	= time substracted
** Description:
**   Procedure to sub two UTC times
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
UTC_Sub( UTC_s utc1, UTC_s utc2, UTC_s* utc)
{
    utc->Msecs = utc1.Msecs - utc2.Msecs;
    utc->Days = utc1.Days - utc2.Days;
    if (utc->Msecs > (unsigned long)86400000LU)
    {
	utc->Msecs = (unsigned long)86400000LU + utc->Msecs;
	utc->Days = utc->Days - 1;
    }
    return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**				UTC_Comp
** Inputs:
**   CompOperator	= compare to equal
**			> compare to greater
**			< compare to less than
**   utc1	UTC time
**   utc2	UTC time
**
** Outputs:
**   return	True condition true
**		False condition false
** Description:
**   Procedure to compare two utc times
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
\*----------------------------------------------------------------------------*/
Bool
UTC_Comp( char CompOperator, UTC_s utc1, UTC_s utc2)

{
    switch (CompOperator)
    {
	/*-----------------------------------------------------------------------*\
	** check if utc1 is equal than utc2
	\*-----------------------------------------------------------------------*/
    case '=':
	{
	    if ((utc1.Days == utc2.Days) && (utc1.Msecs == utc2.Msecs))
		return True;
	    else
		return False;
	    break;
	}
    /*-----------------------------------------------------------------------*\
    ** check if utc1 is greater than utc2
    \*-----------------------------------------------------------------------*/
    case '>':
	{
	    if (utc1.Days > utc2.Days)
		return True;
	    else
		if (utc1.Days < utc2.Days)
		    return False;
		else
		    if (utc1.Msecs > utc2.Msecs)
			return True;
		    else
			return False;
	    break;
	}
    /*-----------------------------------------------------------------------*\
    ** check if utc1 is lesser than utc2
    \*-----------------------------------------------------------------------*/
    case '<':
	{
	    if (utc1.Days < utc2.Days)
		return True;
	    else
		if (utc1.Days > utc2.Days)
		    return False;
		else
		    if (utc1.Msecs < utc2.Msecs)
			return True;
		    else
			return False;
	    break;
	}
    }
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.7__170", FATAL, LOG_NUM,
              "hard error: illegal Compare Operator used", "");
    exit( -1);
    return False;
}

/*----------------------------------------------------------------------------*\
**				UTC_2_tm
** Inputs:
**   utc	ESA UTC time since 01.01.1950 00:00:00
**
** Outputs:
**   tm_time	time in tm-struct
**   return	FATAL	= time not converted
**		SUCCESS	= time converted
**
** Description:
**   Procedure to convert a ESA UTC time into a tm-struct time.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
UTC_2_tm(UTC_s utc, struct tm* tm_time)
{
    time_t	secs_since;
    struct tm	*tm_ptr;

    secs_since = (time_t)(( utc.Days - (unsigned long)7305LU) *
			  (unsigned long)86400LU + utc.Msecs / (unsigned long)1000LU);
    tm_ptr = gmtime( &secs_since);
    if ( tm_ptr != NULL)
    {
	memcpy( tm_time, tm_ptr, sizeof(struct tm) );
	return SUCCESS;
    }
    else
    {
	return FATAL;
    }
}

/*----------------------------------------------------------------------------*\
**				tm_2_UTC
** Inputs Parameter:
**   tm_time	time in tm--struct within
**                 01.01.1970 00:00:00 < tm_time < 19.01.2038 03:14:07
** Outputs Parameter:
**   utc	ESA UTC time since 01.01.1950 00:00:00
**   return	FATAL	= time not converted
**		SUCCESS	= time converted
**
** Description:
**   Procedure to convert a time given in a tm-Struct into the GMT ESA UTC time.
**   The ranges of the tm-Struct values must be valid. If not the procedure
**   returns FATAL.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.5.94
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
tm_2_UTC( struct tm tm_time, UTC_s* utc)

{
    time_t		secs_since;		/* typedef long time_t */
    FunctionStatus_e	Stat;
    char*			EnvVal;
    static char*		TZ_GMT = "TZ=GMT";


    Stat = FATAL;

    /*----------------------------------------------------------------------*\
    ** save the value of thr environment variable TZ
    \*----------------------------------------------------------------------*/
    EnvVal = getenv( "TZ");

    /*----------------------------------------------------------------------*\
    ** set environment variable 'TZ' to GMT and get the time in seconds
    \*----------------------------------------------------------------------*/
    utc->Days = 0;
    utc->Msecs = 0;

    if (putenv( TZ_GMT) == 0)
    {
	/*--------------------------------------------------------------------*\
	** calculate seconds since 1950
	\*--------------------------------------------------------------------*/
	secs_since = mktime(&tm_time);

	/*--------------------------------------------------------------------*\
	** calculate the UTC time
	\*--------------------------------------------------------------------*/
	if (secs_since != -1)
	{
	    utc->Days = secs_since / 86400L + 7305L;
	    utc->Msecs = ( secs_since % 86400L) * 1000L;
	    Stat = SUCCESS;
	}
    }

    /*----------------------------------------------------------------------*\
    ** restore the original value of environment variable TZ
    \*----------------------------------------------------------------------*/
    if (EnvVal != NULL)
    {
	EnvVal -= 3;
	if( putenv( EnvVal) != 0)
	{
	    Stat = FATAL;
	}
    }

    return Stat;
}

/*----------------------------------------------------------------------------*\
**		                 UTC_2_ASCII
** Inputs Parameter:
**   UTC_TC_Date	ESA UTC time between 01-JAN-1970 00:00:00.000 to
**                                         31-DEC-2037 23:59:59.999
** Outputs Parameter:
**   ASCII_DateTime	ASCII time between 01-JAN-1970 00:00:00.000 to
**                                         31-DEC-2037 23:59:59.999
**   return		FATAL	= any error occured during conversion
**			SUCCESS	= time converted to the ASCII String
**
** Description:
**   Converts the ESA utc time into a ASCII time with the following format:
**   dd-mmm-yyyy hh:mm:ss.sss into UTC time
**   An Error occurs if the UTC time isn't:
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	16.9.94
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
UTC_2_ASCII( UTC_s UTC, char* ASCII_Time)
{
    /*------------------------------------------------------------------------*\
    ** array for months names
    \*------------------------------------------------------------------------*/
    static char*	mon[ 12] = {
	"JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
	"JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

    struct tm		tm_time;
    FunctionStatus_e	Stat;
    char			Buffer[ 100];
    int			NoOfWrittenChar;

    Stat	= SUCCESS;
    /*------------------------------------------------------------------------*\
    ** convert UTC to tm struct
    \*------------------------------------------------------------------------*/
    if( UTC_2_tm( UTC, &tm_time) != SUCCESS)
    {
	Stat = FATAL;
    }
    else
    {
	/*----------------------------------------------------------------------*\
	** convert tm to ASCII
	\*----------------------------------------------------------------------*/
	NoOfWrittenChar = sprintf( Buffer, "%.2d-%3s-%.4d %.2d:%.2d:%.2d.%.3d",
				   tm_time.tm_mday,
				   mon[ tm_time.tm_mon],
				   tm_time.tm_year + 1900,
				   tm_time.tm_hour,
				   tm_time.tm_min,
				   tm_time.tm_sec,
				   UTC.Msecs - (unsigned long)tm_time.tm_sec * 1000LU -
				   (unsigned long)tm_time.tm_min * 60000LU -
				   (unsigned long)tm_time.tm_hour * 3600000LU);
	if (NoOfWrittenChar != 24)
	{
	    Stat = FATAL;
	}
	else
	{
	    /*--------------------------------------------------------------------*\
	    ** conversion ok, so return ASCII time
	    \*--------------------------------------------------------------------*/
	    Buffer[ 24] = 0;
	    strcpy( ASCII_Time, Buffer);
	}
    }
    return Stat;
}

/*----------------------------------------------------------------------------*\
**		                 ASCII_2_UTC
** Inputs Parameter:
**   ASCII_DateTime	ASCII time between 01-JAN-1970 00:00:00.000 to
**                                         31-DEC-2037 23:59:59.999
** Outputs Parameter:
**   UTC_TC_Date	ESA UTC time date
**   return		FATAL	= any error occured during conversion
**			SUCCESS	= time converted to the UTC time
**
** Description:
**   Converts the ascii time of form dd-mmm-yyyy hh:mm:ss.sss into UTC time
**   days and msec since the 1.1.1950 00:00:00. Errors can occur if:
**     - the year isn't within the given range 1970 <= year < 2038
**     - illegal string of month 
**     - other time syntax errors
**   In any of these error cases the procedure return with FATAL and resets the
**   UTC_TC_Days and UTC_TC_Msec to zero.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.5.94
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
ASCII_2_UTC( const char *ASCII_DateTime, UTC_s *UTC_TC_Date )
{
    /*------------------------------------------------------------------------*\
    ** array for months names
    \*------------------------------------------------------------------------*/
    static char	*mon[ 12] = {
	"JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
	"JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

    FunctionStatus_e Stat;
    struct tm	DateTime;
    UTC_s		UTC_Time;

    char		mmm[4];			/* input month (string) */
    char		ASCII_Date[12];		/* date part of input string */
    char		ASCII_Time[13];		/* time part of input string */
    double	Sec;
    double	MillisecRest;
    int		mmm_idx;
    char 		LogMoreDescription[ 80];


    Stat = FATAL;
    /*------------------------------------------------------------------------*\
    ** decomposition of input string into its date and time part
    \*------------------------------------------------------------------------*/
    strncpy(ASCII_Date, ASCII_DateTime, 11);
    ASCII_Date[11] = '\0';
    strcpy(ASCII_Time, &ASCII_DateTime[12]);
    ASCII_Time[12] = '\0';

  /*------------------------------------------------------------------------*\
  ** decomposition of date and time string into numbers into the tm struct
  \*------------------------------------------------------------------------*/
    sscanf(ASCII_Date, " %2d %*c %3s %*c %4d ", &DateTime.tm_mday, mmm,
	   &DateTime.tm_year);
    sscanf(ASCII_Time,"%2d %*c %2d %*c %lf", &DateTime.tm_hour, &DateTime.tm_min,
	   &Sec);
    if ((DateTime.tm_year >= 1970) && (DateTime.tm_year < 2038))
    {
	DateTime.tm_year = DateTime.tm_year - 1900;
	DateTime.tm_sec = (int)Sec;
	DateTime.tm_wday = 0;
	DateTime.tm_yday = 0;
	DateTime.tm_isdst = 0;
	MillisecRest = (Sec - (double)DateTime.tm_sec)*1000.0L;

	/*------------------------------------------------------------------------*\
	** decomposition of date and time string into numbers into the tm struct
	\*------------------------------------------------------------------------*/
	mmm_idx = 0;
	while ((strcmp(mon[mmm_idx], mmm) != 0) && (mmm_idx < 11))
	{
	    ++mmm_idx;
	}

	if ( strcmp(mon[mmm_idx], mmm) == 0)
	{
	    /*----------------------------------------------------------------------*\
	    ** initialize month
	    \*----------------------------------------------------------------------*/
	    DateTime.tm_mon = mmm_idx;
	    /*----------------------------------------------------------------------*\
	    ** convert tm struct to UTC time and return utc time
	    \*----------------------------------------------------------------------*/
	    if (tm_2_UTC(DateTime, &UTC_Time) == SUCCESS)
	    {
		UTC_TC_Date->Days = UTC_Time.Days;
		UTC_TC_Date->Msecs = UTC_Time.Msecs + (Time_t)(MillisecRest);
		Stat = SUCCESS;
	    }
	    else
	    {
		sprintf( LogMoreDescription,
			 "error convert to UTC time (time: %s)", ASCII_DateTime);
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.7__476", FATAL, LOG_NUM,
			  LogMoreDescription, "");
		UTC_TC_Date->Days = 0;
		UTC_TC_Date->Msecs = 0;
	    }
	}
	else
	{
	    sprintf( LogMoreDescription,
		     "error in month (time: %s)", ASCII_DateTime);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.7__486", FATAL, LOG_NUM,
		      LogMoreDescription, "");
	    UTC_TC_Date->Days = 0;
	    UTC_TC_Date->Msecs = 0;
	}
    }
    else
    {
	sprintf( LogMoreDescription,
		 "year out of range (time: %s)", ASCII_DateTime);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.7__496", FATAL, LOG_NUM,
		  LogMoreDescription, "");
	UTC_TC_Date->Days = 0;
	UTC_TC_Date->Msecs = 0;
    }
    return Stat;
}

/*----------------------------------------------------------------------------*\
**                            Pixel2_UTC
**  Input parameters:
**    UTC_Conv		utc time refered to given SBT
**    SBT		satelite binary counter refered to the given utc time
**				UTC_Conv
**    SBT_Period        periode in nanosecs. of one satelite binary counter
**				increment
**    PixelTime		the satelite binary counter from which the corresponding
**				utc time has to be calculated
**
**  Output parameters:
**    DateUTC		the utc time corresponding to the given PixelTime
**    Pixel2_UTC	FATAL	= illegal time correlation so time not ok
**			else	= correct time is calculated
**
**  Other interfaces:
**    Output file ....
**  Description:
**    Converts the pixel time into the UTC time with the help of the following
**    input parameter:
**       UTC_Conv		The reference UTC time to the SBT
**       SBT			satelite binary time according to the given 
**					UTC_Conv time
**       SBT_Period		satelite periode in nanosec.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.5.94
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Pixel2_UTC( UTC_s UTC_Conv, unsigned long SBT,
	    unsigned long SBT_Period, unsigned long PixelTime, UTC_s* DateUTC)
{
    FunctionStatus_e	Stat;
    double		MsecsPerDay = (double)MSECS_PER_DAY;
    double		TicksPerDay;
    double		MaxTicksFromTimeCor;
    double		SBT_Diff;
    double		UTC_DP;

    UTC_s			UTC_TC_Date;

    /*--------------------------------------------------------------------------*\
    ** get the UTC time of the time correlation
    \*--------------------------------------------------------------------------*/
    Stat = SUCCESS;
    UTC_TC_Date = UTC_Conv;

    TicksPerDay		= ( MsecsPerDay/SBT_Period) * 1000000.0L;
    MaxTicksFromTimeCor	= MAX_DAYS_FROM_TIME_COR * TicksPerDay;
    SBT_Diff		= ( double)PixelTime - ( double)SBT;
  
    /*------------------------------------------------------------------------*\
    ** determine these cases with absolute value SBT_Diff < TickPerDay( number
    ** ticks corresponding to one day)
    \*------------------------------------------------------------------------*/
    if (fabs( SBT_Diff) > MaxTicksFromTimeCor)
    {
	/*------------------------------------------------------------------------*\
	** check SBT_Diff via the wrap around value. So calculate the complement
	** and check the absolute value of the SBT_diff again    
	\*------------------------------------------------------------------------*/
	if (SBT_Diff < 0.0)
	{
	    SBT_Diff = ( double)0xffffffffLU + SBT_Diff;
	}
	else
	{
	    SBT_Diff = SBT_Diff - ( double)0xffffffffLU;
	}
    }

    /*------------------------------------------------------------------------*\
    ** and check the absolute value of the SBT_diff again. If greater than
    ** MaxTicksFromTimeCor print error message illegal timecorrelation file 
    \*------------------------------------------------------------------------*/
    if (fabs( SBT_Diff) > MaxTicksFromTimeCor)
    {
	Stat = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.7__587", Stat, LOG_NUM,
		  "illegal time correlation", "");
	/*----------------------------------------------------------------------*\
	** take the origin SBT_Diff
	\*----------------------------------------------------------------------*/
	SBT_Diff = ( double)PixelTime - ( double)SBT;

    }

    UTC_DP = UTC_TC_Date.Days * MsecsPerDay + UTC_TC_Date.Msecs +
	SBT_Diff * SBT_Period / 1000000.0L;

    DateUTC->Days = (Time_t)(UTC_DP / MsecsPerDay);
    DateUTC->Msecs = (Time_t)(fmod(UTC_DP, MsecsPerDay));

    return Stat;
}

/*----------------------------------------------------------------------------*\
**				GMT_GetLocal_UTC
** Inputs Parameter:
** Outputs Parameter:
**   tm_time	GMT in tm structure
**   return	FATAL	= time not ok
**		SUCCESS	= time ok
**
** Description:
**   Function to get the Current GMT time in tm structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GMT_GetLocal_tm( struct tm* tm_time)
{
    FunctionStatus_e	Stat		= SUCCESS;
    time_t		SecSince;		/* typedef long time_t */
    struct tm*		Time;
    char*			EnvVal;
    static char*		TIME_ZONE	= "TZ=GMT";

    /*----------------------------------------------------------------------*\
    ** save the value of thr environment variable TZ
    \*----------------------------------------------------------------------*/
    EnvVal = getenv( "TZ");

    /*----------------------------------------------------------------------*\
    ** set environment variable 'TZ' to "" and get the time in seconds
    \*----------------------------------------------------------------------*/
    if ( putenv( TIME_ZONE) == 0)
    {
	/*----------------------------------------------------------------------*\
	** get the local time
	\*----------------------------------------------------------------------*/
	SecSince = time(NULL);
	Time = localtime( &SecSince);
	*tm_time = *Time;
    }
    else
    {
	Stat = FATAL;
    }

    /*----------------------------------------------------------------------*\
    ** restore the origin value of environment variable TZ
    \*----------------------------------------------------------------------*/
    if (EnvVal != NULL)
    {
	EnvVal -= 3;
	if( putenv( EnvVal) != 0)
	{
	    Stat = FATAL;
	}
    }

    return Stat;
}

/*----------------------------------------------------------------------------*\
**				GMT_GetLocal_UTC
** Inputs Parameter:
** Outputs Parameter:
**   utc	GMT ESA UTC time since 01.01.1950 00:00:00
**   return	FATAL	= time not ok
**		SUCCESS	= time ok
**
** Description:
**   Procedure to get the current GMT time in UTC format. The  procedure
**   returns FATAL if the GMT time cannot be returned.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	20.6.95
**  Author:		M.Wolfmueller, DLR/WT-DA-SE
**
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GMT_GetLocal_UTC( UTC_s* utc)
{
    FunctionStatus_e	Stat		= SUCCESS;
    struct tm*		Time;
    time_t		SecSince;		/* typedef long time_t */
    char*			EnvVal;
    static char*		TIME_ZONE	= "TZ=";

    /*----------------------------------------------------------------------*\
    ** save the value of thr environment variable TZ
    \*----------------------------------------------------------------------*/
    EnvVal = getenv( "TZ");

    /*----------------------------------------------------------------------*\
    ** set environment variable 'TZ' to "" and get the time in seconds
    \*----------------------------------------------------------------------*/
    if ( putenv( TIME_ZONE) == 0)
    {
	/*----------------------------------------------------------------------*\
	** get the local time
	\*----------------------------------------------------------------------*/
	SecSince = time(NULL);
	Time = localtime( &SecSince);

	/*----------------------------------------------------------------------*\
	** convert the local time in tm structure into the GMT UTC time
	\*----------------------------------------------------------------------*/
	Stat = tm_2_UTC( *Time, utc);
    }
    else
    {
	Stat = FATAL;
    }

    /*----------------------------------------------------------------------*\
    ** restore the origin value of environment variable TZ
    \*----------------------------------------------------------------------*/
    if (EnvVal != NULL)
    {
	EnvVal -= 3;
	if( putenv( EnvVal) != 0)
	{
	    Stat = FATAL;
	}
    }

    return Stat;
}

/*----------------------------------------------------------------------------*\
**                              UTC_2_MJD50
** Inputs Parameter:
**   UTC_TC_Date	ESA UTC time date
** Outputs Parameter:
**   MJD50		time in MJD50 format used in ERS propagator
**   return		SUCCESS	= time converted to the MJD50 time
**
** Description:
**   Converts the ESA UTC time to time in MJD50 format used in ERS propagator
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:    20.6.95
**  Author:             T. Ruppert , DLR/WT-DA-BS
**
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
UTC_2_MJD50(UTC_s UTC, double *MJD50)
{
    *MJD50 = (double) UTC.Days + ( (double) UTC.Msecs /  (double) MSECS_PER_DAY );
	
    return(SUCCESS);
}

/*----------------------------------------------------------------------------*\
**                              MJD50_2_UTC
** Inputs Parameter:
**   MJD50		time in MJD50 format used in ERS propagator
** Outputs Parameter:
**   UTC_TC_Date	ESA UTC time date
**   return		SUCCESS	= time converted to the UTC time
**
** Description:
**   Converts the time in MJD50 format use in ERS propagator to ESA UTC time 
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Libraries:
**    ....
**  Created:    20.6.95
**  Author:             T. Ruppert , DLR/WT-DA-BS
**
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MJD50_2_UTC(double MJD50,UTC_s *UTC)
{
    UTC->Days = (long) MJD50;

    UTC->Msecs = (long) ( (MJD50 - (double) UTC->Days) * (double) MSECS_PER_DAY );

    return(SUCCESS);
}


/*----------------------------------------------------------------------------*\
**                            UTC__Sun2Host
**  Input parameters:
**    UTC			in Sun memory representation
**  Output parameters:
**    UTC			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the UTC struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
UTC__Sun2Host( UTC_s *UTC)
{
    /* long */
    UTC->Days = SunLong2Long( (unsigned char*)&UTC->Days);
    /* long */
    UTC->Msecs = SunLong2Long( (unsigned char*)&UTC->Msecs);
}

/*----------------------------------------------------------------------------*\
**                                UTC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	UTC		UTC time
**
**  Other interfaces:
**
**  Description:
**	Function to read UTC time entries from the given 
**	input stream into the UTC structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
UTC__read(UTC_s* self, FILE* file_ptr, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** read UTC time
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread(&(self->Days), 1, UTC_DAYS_SIZE, file_ptr);
    NoOfBytes += fread(&(self->Msecs), 1, UTC_MSECS_SIZE, file_ptr);

    if (NoOfBytes != UTC_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "UTC Time, #read bytes: %d", NoOfBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__994", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	UTC__Sun2Host(self);
    }

    return SUCCESS;
}


/*============================================================================*\
**  private operations
\*============================================================================*/


