/*----------------------------------------------------------------------------*\
**                                UTC.h
**
**  Class UTC declaration
** 
**  Version:    5.1
**  Date:       02/26/99
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _UTC_h
#define _UTC_h 

#pragma ident "@(#)UTC.h	5.1 02/26/99, Copyright DLR 1998"

#ifdef __cplusplus
extern "C"                      /* External C definitions */
{
#endif


#include <time.h>
#include "defs.h"
#include "sys.h"


/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

#define	UTC_DAYS_SIZE	(LONG_SIZE)

#define	UTC_MSECS_SIZE	(LONG_SIZE)

#define	UTC_SIZE	(UTC_DAYS_SIZE + UTC_MSECS_SIZE)

#define	ASCII_UTC_TIME_LENGTH	(25)


#define	MAX_DAYS_FROM_TIME_COR	(7)


#define	HOURS_OF_DAY	(24)

#define	MINUTES_OF_DAY	(HOURS_OF_DAY * 60)

#define	SECS_OF_DAY	(MINUTES_OF_DAY * 60)

#define	MSECS_PER_DAY	(SECS_OF_DAY * 1000)

/* days between 01/01/50 (UTC reference) */
/* and 31/12/69 (UNIX reference) */


#define	DAY_OFFSET	(7305)

/*----------------------------------------------------------------------------*\
**  enums
\*----------------------------------------------------------------------------*/


/*============================================================================*\
**  class UTC
\*============================================================================*/
typedef struct UTC {
    Time_t Days;		/* Days since 1st January 1950 */
    Time_t Msecs;		/* Milliseconds since midnight */
} UTC_s;


/*============================================================================*\
**  public operations
\*============================================================================*/
 
/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/

FunctionStatus_e
utc2date(UTC_s utc_date, short *year , short *mon, short *day , short *hour, 
	 short *min,float *sec);

FunctionStatus_e
date2utc(short year , short mon, short day , short hour, short min, float sec,
	 UTC_s *utc_date);

FunctionStatus_e
UTC_Add(UTC_s utc1, UTC_s utc2, UTC_s* utc);

FunctionStatus_e 
UTC_Sub(UTC_s utc1, UTC_s utc2, UTC_s* utc);

Bool
UTC_Comp(char CompOperator, UTC_s utc1, UTC_s utc2);

FunctionStatus_e 
UTC_2_tm(UTC_s utc, struct tm* tm_time);

FunctionStatus_e
tm_2_UTC(struct tm tm_time, UTC_s* utc);

FunctionStatus_e 
UTC_2_ASCII(UTC_s UTC, char* ASCII_Time);

FunctionStatus_e
ASCII_2_UTC(const char *ASCII_DateTime, UTC_s *UTC_TC_Date);

/*----------------------------------------------------------------------------*\
**  Function to get the Current GMT time in tm structure
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GMT_GetLocal_tm(struct tm* tm_time);

/*----------------------------------------------------------------------------*\
**  Function to get the Current GMT time in UTC structure
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GMT_GetLocal_UTC(UTC_s* utc);

/*----------------------------------------------------------------------------*\
**  Function to convert the binary satelite counter into the corresponding utc
**  time
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Pixel2_UTC(UTC_s UTC_Conv, unsigned long SBT, unsigned long SBT_Period,
	   unsigned long PixelTime, UTC_s* DateUTC);

/*----------------------------------------------------------------------------*\
**  Function to convert UTC time into MJD50 time
\*----------------------------------------------------------------------------*/
FunctionStatus_e
UTC_2_MJD50(UTC_s UTC, double *MJD50);

/*----------------------------------------------------------------------------*\
**  Function to convert MJD50 time into UTC time
\*----------------------------------------------------------------------------*/
FunctionStatus_e
MJD50_2_UTC(double MJD50, UTC_s *UTC);

FunctionStatus_e
UTC__read(UTC_s* self, FILE* file_ptr, Bool convert_sun_2_host);


/*============================================================================*\
**  private operations
\*============================================================================*/


#ifdef __cplusplus
}                                                        /* end C linkage */
#endif
 
#endif
 
