/*----------------------------------------------------------------------------*\
**                              vector.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the functions: vector__new, vector__delete
\*----------------------------------------------------------------------------*/
#pragma ident   "@(#)vector.c	5.1 02/26/99, Copyright DLR 1994"

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include "numrecip.h"
#include "log.h"



#define	SS_ID	("vector")

/*----------------------------------------------------------------------------*\
**                                vector__new
**  Input parameters:
**    nrl, nrh: row low and high limits
**    ncl, nch: column low and high limits
**  Output parameters:
**  Other interfaces: none
**  Description:
**    Initialize a vector [nl..nh]
**  References:
**    Numerical Recipes in C
**  Libraries:
**  Created:    26.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double	*vector__new(short nl, short nh)
{
	double	*v;

	v = (double *)malloc( (unsigned)(nh-nl+1)*sizeof(double) );
	if (!v) 
	{
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_44", FATAL, LOG_CANNOT_ALLOCATE, "for a vector object", "");
	    exit(-1);
	}
	return v-nl;
}



/*----------------------------------------------------------------------------*\
**                                vector__delete
**  Input parameters:
**    m: pointer to a vector
**    nrl, nrh: row low and high limits
**    ncl, nch: column low and high limits
**  Output parameters:
**  Other interfaces: none
**  Description:
**    Delete the vector [nl..nh]
**  References:
**    Numerical Recipes in C
**  Libraries:
**  Created:    26.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void	vector__delete(double  *v, short nl, short nh)
{
	free((char *) (v+nl));
}

