/*----------------------------------------------------------------------------*\
**                              wat2phot.c
**
**  GDP Level 0 to 1 program: radiance (irradiance) conversion function
**
**  Version:    4.1
**  Date:       29 Jul 1996
**  Contents:
**    This file contains the following debug functions:
**      float Watt2Photon(float Watt, float Wavelength)
**      float Photon2Watt(float Photon, float Wavelength)
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)wat2phot.c	4.1 29 Jul 1996, Copyright DLR 1994"

#include <string.h>
#include "wat2phot.h"

/*----------------------------------------------------------------------------*\
**                             Watt2Photon
**  Input parameters:
**    FuncName the initial part of the debug function name
**    Date a string with date in UTC format
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Appends and returns the FuncName and the UTC Date 
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    06.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float
Watt2Photon(float Watt, float Wavelength)
{
    return Watt * (CONV_FACTOR * Wavelength);
}


/*----------------------------------------------------------------------------*\
**                             Photon2Watt
**  Input parameters:
**    FuncName the initial part of the debug function name
**    Date a string with date in UTC format
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Appends and returns the FuncName and the UTC Date 
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    06.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float
Photon2Watt(float Photon, float Wavelength)
{
    return Photon / (CONV_FACTOR * Wavelength);
}


/*----------------------------------------------------------------------------*\
**                             PhotonUnits__Get
**  Input parameters:
**    <none>
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Return a string with the Photon units
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    06.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
const char*
PhotonUnits__Get()
{
    static char *Units = "Wavelength [nm], Irradiance [photons/s.cm^2.nm], "
	"Radiance [photons/s.sr.cm^2.nm]";
    return Units;
}


/*----------------------------------------------------------------------------*\
**                             WattUnits__Get
**  Input parameters:
**    <none>
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Return a string with the Watt units
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    06.9.95
**  Author:		Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
const char*
WattUnits__Get()
{
    static char *Units = "Wavelength [nm], Irradiance [mW/m^2.nm], "
	"Radiance [mW/sr.m^2.nm]";
    return Units;
}

