/*----------------------------------------------------------------------------*\
**                                XCorrelation.c
**
**  Description
** 
**  Version:    1.4
**  Date:       10/24/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/

#define	SccsId	("@(#)XCorrelation.c 1.4 10/24/02, Copyright DLR 1998")


#define	SS_ID	("gdp01_ex")

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "fftw.h"
#include "xcortion.h"
#include "defs.h"
#include "numrecip.h"
#include "log.h"

/*============================================================================*\
**  public operations of class Spectrum
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                             Spectrum__
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Create a new empty Spectrum object with re_ptr and im_ptr allocated,
**      grid_ptr set to 0, number_of_values set to parameter number_of_values,
**      has_constant_sampling_intervall set to true.
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	22.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__(Spectrum_s* self, int number_of_values)
{
#ifdef __cplusplus  
  self->re_ptr = new float[number_of_values];
  self->im_ptr = new float[number_of_values];
#else
  self->re_ptr = (float*)malloc(number_of_values*(sizeof(float)));
  self->im_ptr = (float*)malloc(number_of_values*(sizeof(float)));
#endif
  self->number_of_values = number_of_values;
  self->has_grid = False;
  self->grid_ptr = 0;
  self->has_constant_sampling_intervall = True;
  
  return self;
}


/*----------------------------------------------------------------------------*\
**                            Spectrum__readGridRe
**  Input parameters:
**      self:
**      file_ptr: a stream
**      has_constant_sampling_intervall: indicates, if the grid in the stream 
**                                       has  constant sampling rate
**
**  Output parameters:
**      self:
**
**  Return value:
**
**  Description:
**      Read a Spectrum from a stream that has two colums, a grid value in the 
**      first and the real part of a complex value in the second
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	27.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__readGridRe(Spectrum_s* self, FILE* file_ptr, 
		     Bool has_constant_sampling_intervall)
{
  FunctionStatus_e Status = SUCCESS;
  int number_of_values;
  int n;
  float dummy1;
  float dummy2;
  int i;
  
  number_of_values = 0;
  n = fscanf(file_ptr, "%f%f", &dummy1, &dummy2);
  while (n != EOF) 
    {
      n = fscanf(file_ptr, "%f%f", &dummy1, &dummy2);
      number_of_values ++;
    }
  if (number_of_values <= 1)
    {
      Status = FATAL;
      return Status;
    }
  
#ifdef __cplusplus
  self->re_ptr = new float[number_of_values];
  self->grid_ptr = new float[number_of_values];
#else
  self->re_ptr = (float*)malloc(number_of_values*(sizeof(float)));
  self->grid_ptr = (float*)malloc(number_of_values*(sizeof(float)));
#endif
  
  self->im_ptr = 0;
  self->number_of_values = number_of_values;
  self->has_grid = True;
  self->has_constant_sampling_intervall = has_constant_sampling_intervall;
  
  rewind(file_ptr);
  for (i=0; i < number_of_values; i++) {
    n = fscanf(file_ptr, "%f%f", &self->grid_ptr[i], &self->re_ptr[i]);
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                            Spectrum__readGridReFromFile
**  Input parameters:
**      self:
**      filename: filename of the file containing grid and re values
**      has_constant_sampling_intervall: indicates, if the grid in the stream 
**                                       has  constant sampling rate
**
**  Output parameters:
**      self:
**
**  Return value:
**
**  Description:
**      Read a Spectrum from a file that has two colums, a grid value in the 
**      first and the real part of a complex value in the second
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	27.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__readGridReFromFile(Spectrum_s* self, const char* filename, 
			     Bool has_constant_sampling_intervall)
{
  FILE* file_ptr;
  
  if ((file_ptr = fopen(filename, "r")) == 0) {
    return ERROR;
  }
  
  Spectrum__readGridRe(self, file_ptr, has_constant_sampling_intervall);
  
  fclose(file_ptr);
  
  return SUCCESS;
  
} 


/*----------------------------------------------------------------------------*\
**                             Spectrum__re
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	22.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__re(Spectrum_s* self, int number_of_values)
{
  
#ifdef __cplusplus
  self->re_ptr = new float[number_of_values];
#else
  self->re_ptr = (float*)malloc(number_of_values*(sizeof(float)));
#endif
  self->im_ptr = 0;
  self->number_of_values = number_of_values;
  self->has_grid = False;
  self->grid_ptr = 0;
  self->has_constant_sampling_intervall = True;
  
  return self;
}

 
/*----------------------------------------------------------------------------*\
**                             Spectrum__destructor
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__destructor(Spectrum_s* self)
{
#ifdef __cplusplus
  delete[] self->re_ptr;
  delete[] self->im_ptr;
#else
  free(self->re_ptr);
  free(self->im_ptr);
#endif
  self->number_of_values = 0;
  if (self->has_grid == True) {
#ifdef __cplusplus
    delete[] self->grid_ptr;
#else
    free(self->grid_ptr);
#endif
    self->has_grid = False;
    self->has_constant_sampling_intervall = True;
  }
  
  return self;
}


/*----------------------------------------------------------------------------*\
**                                Spectrum__write
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Write 2 or 3 columns to stream, the fist contains the grid if
**      self->has_grid equals true
**      and the second (first) and thrird (second) column the 
**      corresponding read and imaginary (if self->im_ptr not equals zero)
**      values.
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	16.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__write(const Spectrum_s* self, FILE* stream)
{
  int i;
  
  for (i = 0; i<self->number_of_values; i++) {
    if (fprintf(stream, "%f\t%f\t%f\n", 
		(self->has_grid == False) ? 0.0 : self->grid_ptr[i],
		(self->re_ptr == 0)       ? 0.0 : self->re_ptr[i],
		(self->im_ptr == 0)       ? 0.0 : self->im_ptr[i]) < 0) {
      return ERROR;
    }
  }
  return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**                    accessor operations
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	15.7.98
**
**  Author:	Christian Rehn, DLR/DFD-SE
**
\*----------------------------------------------------------------------------*/
int
Spectrum__getNumberOfValues(const Spectrum_s* self)
{
  return self->number_of_values;
}

const float*
Spectrum__getRePtr(const Spectrum_s* self)
{
#ifdef __cplusplus
  return const_cast<const float*>(self->re_ptr);
#else
  return (const float*)self->re_ptr;
#endif
}

const float*
Spectrum__getImPtr(const Spectrum_s* self)
{
#ifdef __cplusplus
  return const_cast<const float*>(self->im_ptr);
#else
  return (const float*)self->im_ptr;
#endif
}

const float*
Spectrum__getGridPtr(const Spectrum_s* self)
{
  if(self->has_grid == True)
  {
#ifdef __cplusplus
  return const_cast<const float*>(self->grid_ptr);
#else
  return (const float*)self->grid_ptr;
#endif
  }
	return NULL;
}

Bool
Spectrum__hasGrid(const Spectrum_s* self)
{
  return self->has_grid;
}

Bool
Spectrum__hasConstantSamplingInterval(const Spectrum_s* self)
{
  return self->has_constant_sampling_intervall;
}

float
Spectrum__getSamplingInterval(const Spectrum_s* self)
{
  return self->grid_ptr[1] - self->grid_ptr[0];
}

/*----------------------------------------------------------------------------*\
**                    Spectrum__convoluteNormalizedGaussian
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	15.7.98
**
**  Author:	Christian Rehn, DLR/DFD-SE
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__convoluteNormalizedGaussian(Spectrum_s* self, 
				      int number_of_additional_points) 
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  float j;
  float k;
  float l;
  float* kernel;
  float* kernel2;
  float* new_ptr;
  
  if(self->has_constant_sampling_intervall != True)
    {
      Status = FATAL;
      return Status;
    }
  
  /*------------------------------------------------------------------------*\
  **  create kernel
  \*------------------------------------------------------------------------*/
#ifdef __cplusplus  
  kernel = new float[number_of_additional_points];
  kernel2 = new float[(3*number_of_additional_points+1)];
#else
  kernel = (float*)malloc(number_of_additional_points*(sizeof(float)));
  kernel2 = (float*)malloc((3*number_of_additional_points+1)*(sizeof(float)));
#endif
  l = number_of_additional_points * number_of_additional_points;
  k = 1 / sqrt(M_PI * l);
  for (i=0; i < number_of_additional_points; i++) {
#ifdef __cplusplus
    j = static_cast<float>(i);
#else
    j = (float)i;
#endif
    kernel[i] = k * exp( -(j - number_of_additional_points/2)
			 * (j - number_of_additional_points/2) / l); 
  }
  
  for (i=0;i<(3*number_of_additional_points+1);i++) {
#ifdef __cplusplus
    j = static_cast<float>(i);
#else
    j = (float)i;
#endif
    kernel2[i] = k*exp(-(j-3*number_of_additional_points/2)
		       *(j-3*number_of_additional_points/2)/(l/(4*log(2))));
  }	
  
  /*------------------------------------------------------------------------*\
  **  triangular convolution, the width of the triangle is
  **  defined by number_of_additional_points/2
  \*------------------------------------------------------------------------*/
#ifdef __cplusplus
  new_ptr = new float[self->number_of_values];
#else
  new_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
#endif
  convol(self->re_ptr, new_ptr, 0, self->number_of_values-1, 
	 kernel2, (3*number_of_additional_points+1)/2);
#ifdef __cplusplus
  delete[] self->re_ptr;
#else
  free(self->re_ptr);
#endif
  self->re_ptr = new_ptr;
  
  if (self->im_ptr != 0) {
#ifdef __cplusplus
    new_ptr = new float[self->number_of_values];
#else
    new_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
#endif
    convol(self->im_ptr, new_ptr, 0, self->number_of_values-1, 
	   kernel2, (3*number_of_additional_points+1)/2);
#ifdef __cplusplus
    delete[] self->im_ptr;
#else
    free(self->im_ptr);
#endif
    self->im_ptr = new_ptr;
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                            Spectrum__resampleSpline
**  Input parameters:
**      self:
**      grid_ptr: grid of x-values for resampling
**      number_of_values: number of x-values in grid_ptr
**      has_constant_sampling_intervall: true if grid_ptr has constant
**                                       sampling intervalls, else false
**
**  Output parameters:
**      self: the resampled spectrum
**
**  Return value:
**      The fractional number of original pixels within the resampling 
**      intervall, i.e. the intervall of the
**      new grid divided by the number of values of the Spectrum self
**      within this intervall before resampling.
**      This is needed to compute the cross correlation error.
**
**  Description:
**      Resample intervall to a new resolution of grid new_wavelength_grid
**      using a spline.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	26.1.94
**  Author:	Christian Rehn, DLR/DFD-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__resampleSpline(Spectrum_s* self,	
			 const float* grid_ptr, int number_of_values,
			 Bool has_constant_sampling_intervall,
			 float* fractional_number_of_values)
{
  FunctionStatus_e Status = SUCCESS;
  float fractional_number_of_data;
  float* new_grid_ptr;
  float* new_re_ptr;
  float* new_im_ptr;
  
  if((self->has_grid != True) || (number_of_values <= 1))
    {
      Status = FATAL;
      return Status;
    }


  fractional_number_of_data = *fractional_number_of_values;
  Status = Spectrum__fractionalNumberOfPoints(self, grid_ptr, number_of_values,
					      &fractional_number_of_data);
  if(Status != SUCCESS)
    {
      Status = FATAL;
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__591", Status,
	       LOG_NUM, "error while calculate fractional_number_of_values", "");
      return Status;
    }
  *fractional_number_of_values =  fractional_number_of_data;
#ifdef DEBUG    
  printf("fractional_number_of_values = %f \n", 
	 *fractional_number_of_values);
#endif 
  
#ifdef __cplusplus
  new_grid_ptr = new float[number_of_values];
  new_re_ptr = new float[number_of_values];
#else
  new_grid_ptr = (float*)malloc(number_of_values*(sizeof(float)));
  new_re_ptr = (float*)malloc(number_of_values*(sizeof(float)));
#endif
  
  if (self->im_ptr != 0) {
#ifdef __cplusplus
    new_im_ptr = new float[number_of_values];
#else
    new_im_ptr = (float*)malloc(number_of_values*(sizeof(float)));
#endif
  }
  memcpy(new_grid_ptr, grid_ptr, number_of_values * sizeof(*grid_ptr));
  
  
  fit_grid_spline(self->grid_ptr, self->re_ptr, self->number_of_values, 
		  new_grid_ptr, new_re_ptr, 0, number_of_values-1);
  
  if (self->im_ptr != 0) {
    fit_grid_spline(self->grid_ptr, self->im_ptr, self->number_of_values, 
		    new_grid_ptr, new_im_ptr, 0, number_of_values-1);
#ifdef __cplusplus
    delete[] self->im_ptr;
#else
    free(self->im_ptr);
#endif
    self->im_ptr = new_im_ptr;
  }
  
#ifdef __cplusplus
  delete[] self->grid_ptr;
  delete[] self->re_ptr;
#else
  free(self->grid_ptr);
  free(self->re_ptr);
#endif
  
  self->grid_ptr = new_grid_ptr;
  self->re_ptr = new_re_ptr;
  
  self->number_of_values = number_of_values;
  self->has_constant_sampling_intervall = has_constant_sampling_intervall;
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                            Spectrum__cutEdges
**  Input parameters:
**    ....
**  Output parameters:
**    ....
**  Other interfaces:
**    Output file ...
**  Description:
**      Cut both edges of an intervall by number_of_points each.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	16.7.98
**  Author:	Christian Rehn, DLR/DFD-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__cutEdges(Spectrum_s* self, int number_of_points)
{
  FunctionStatus_e Status = SUCCESS;
  float* new_grid_ptr;
  float* new_re_ptr;
  float* new_im_ptr;
  
  if((number_of_points*2 > self->number_of_values - 2)||
     (self->has_grid != True))
    {
      Status = FATAL;
      return Status;
    }

 
  self->number_of_values -= 2 * number_of_points;
  
#ifdef __cplusplus
  new_grid_ptr = new float[self->number_of_values];
#else
  new_grid_ptr =(float*)malloc(self->number_of_values*(sizeof(float)));
#endif
  memcpy(new_grid_ptr, self->grid_ptr + number_of_points, 
	 self->number_of_values * sizeof(*(self->grid_ptr))); 
#ifdef __cplusplus
  delete[] self->grid_ptr;
#else
  free(self->grid_ptr);
#endif
  
  self->grid_ptr = new_grid_ptr;
  
#ifdef __cplusplus
  new_re_ptr = new float[self->number_of_values];
#else
  new_re_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
#endif
  memcpy(new_re_ptr, self->re_ptr + number_of_points, 
	 self->number_of_values * sizeof(*(self->re_ptr))); 
#ifdef __cplusplus
  delete[] self->re_ptr;
#else
  free(self->re_ptr);
#endif
  
  self->re_ptr = new_re_ptr;
  
  if (self->im_ptr != 0) {
#ifdef __cplusplus
    new_im_ptr = new float[self->number_of_values];
#else
    new_im_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
#endif
    memcpy(new_im_ptr, self->im_ptr + number_of_points, 
	   self->number_of_values * sizeof(*(self->im_ptr))); 
#ifdef __cplusplus
    delete[] self->im_ptr;
#else
    free(self->im_ptr);
#endif
    self->im_ptr = new_im_ptr;
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                          Spectrum__maxRe
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      get the maximum radiance value
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
float
Spectrum__maxRe(const Spectrum_s* self)
{
  int i;
  float maximum = self->re_ptr[0];
  for (i=1; i < self->number_of_values; i++) {
    if (self->re_ptr[i] > maximum) {
      maximum = self->re_ptr[i];
    }
  }
  return maximum;
}


/*----------------------------------------------------------------------------*\
**                            Spectrum__scale
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**       scale the radiance values according to the value of scale_factor
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	20.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__scale(Spectrum_s* self, float scale_factor)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  
  if(scale_factor == 0)
    {
      Status = FATAL;
      return Status;
    }
  
  for (i=0; i<self->number_of_values; i++) {
    self->re_ptr[i] /= scale_factor;
  }
  
  if (self->im_ptr != 0) {
    for (i=0; i<self->number_of_values; i++) {
      self->im_ptr[i] /= scale_factor;
    }
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                       Spectrum__continuumCorrectionPolyfit
**  Input parameters:
**      self:
**      degree: degree of polynomial
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function performs a least-square polynomial fit
**      and makes a continuum correction with this polynome.
**
**  Created:	27.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__continuumCorrectionPolyfit(Spectrum_s* self, int degree)
{
  FunctionStatus_e Status =SUCCESS;
  double* coefficients_ptr;
  float help;
  int i;
  int j;
  
  if(degree < 0)
    {
      Status = FATAL;
      return Status;
    }
  if(self->has_grid != True)
    {
      Status = FATAL;
      return Status;
    }

  /*------------------------------------------------------------------------*\
  **  calculate the coefficients of a fittingpolynome of degree 
  \*------------------------------------------------------------------------*/
#ifdef __cplusplus
  coefficients_ptr = new double[degree+1];
#else
  coefficients_ptr = (double*)malloc((degree+1)*(sizeof(double)));
#endif
  CalculatePolynome(self->grid_ptr, self->re_ptr, self->number_of_values, 
		    coefficients_ptr, degree+1);
  
  for (i=0; i < self->number_of_values; i++) {	
    help = 0.0;
    for (j=0; j <= degree; j++) {
      help += coefficients_ptr[j] * pow(self->grid_ptr[i], j);
    }
    self->re_ptr[i] -= help;
    if (self->im_ptr != 0) {
      self->im_ptr[i] -= help;
    }
  }
  
#ifdef __cplusplus
	delete[] coefficients_ptr;
#else
	free(coefficients_ptr);
#endif
  return Status;
}


/*----------------------------------------------------------------------------*\
**                             Spectrum__apodise
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function apodizes the ends of the spectrum with a cosine bell
**      The cosine bell starts rising from zero at 1 and falls to zero at
**      self->number_of_values
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__apodise(Spectrum_s* self, float fraction)
{
  int nsquash;
  int nsquash1;
  int nsquash2;
  float fact;
  float factor;
  int i;
  
#ifdef __cplusplus
  nsquash1 = fraction * static_cast<float>(self->number_of_values);
#else
  nsquash1 = fraction * (float)self->number_of_values;
#endif
  nsquash2 = self->number_of_values / 2;
  nsquash = (nsquash1 > nsquash2) ? nsquash2 : nsquash1;
  
  if (nsquash <= 1) {
    return ERROR;
  }
  
  fact = M_PI / (nsquash - 1.0);
  
  for (i=0; i < nsquash; i++) {
    factor = 0.5 * (1.0 - cos(fact * i));
    self->re_ptr[i] *= factor;
    self->re_ptr[self->number_of_values -1 -i] *= factor;
    if (self->im_ptr != 0) {
      self->im_ptr[i] *= factor;
      self->im_ptr[self->number_of_values -1 -i] *= factor;
    }
  }
  
  return SUCCESS;
} 


/*----------------------------------------------------------------------------*\
**                          Spectrum__fourierTransform
**  Input parameters:
**      self: pointer to Spectrum object
**
**  Output parameters:
**      self: 
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function applys a discrete fourier transformation to self.
**      Attention: - the distance between the points on the wavelength grid 
**                   must be constant 
**                   (self->has_constant_sampling_intervall equals true)
**                 - self->number_of_values must be a power of 2
**
**  References:
**      Numerical Recipes in C, FFT
**      Using IDL, Chapter 14, Signal Processing
**
**  Libraries:
**      ....
**
**  Created:	21.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__fourierTransform(Spectrum_s* self)
{
  FunctionStatus_e Status = SUCCESS;
  if(self->has_constant_sampling_intervall != True)
    {
      Status = FATAL;
      return Status;
    }
  Spectrum__fft(self, -1);

  return Status;
}


/*----------------------------------------------------------------------------*\
**                      Spectrum__inverseFourierTransform
**  Input parameters:
**      self: pointer to Spectrum object
**
**  Output parameters:
**      self: 
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function applys a discrete fourier transformation to self.
**      Attention: - the distance between the points on the wavelength grid 
**                   must be constant 
**                   (self->has_constant_sampling_intervall equals true)
**                 - self->number_of_values must be a power of 2
**
**  References:
**      Numerical Recipes in C, FFT
**      Using IDL, Chapter 14, Signal Processing
**
**  Libraries:
**      ....
**
**  Created:	21.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__inverseFourierTransform(Spectrum_s* self)
{
  FunctionStatus_e Status = SUCCESS;
  if(self->has_constant_sampling_intervall != True)
    {
      Status = FATAL;
      return Status;
    }
  Spectrum__fft(self, 1);
  return Status;
}

/*----------------------------------------------------------------------------*\
**                        Spectrum__crossCorrelate
**  Input parameters:
**      self: a fourier transformed spectrum
**      second_spectrum_ptr: a fourier transformed spectrum
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function performs a cross correlation between the two
**      fourier transformed spectra self and second_spectrum_ptr.
**      Attention: - im_ptr must not be zero (because the spectra are 
**                   fourier transformed, their im_ptrs are usually
**                   not zero). 
**                 - number_of_values must agree between spectra
**                 - number_of_values must be a power of 2
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	23.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__crossCorrelate(Spectrum_s* self, 
			 const Spectrum_s* second_spectrum_ptr,
			 float norm_factor)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  int nop;
  int nop_div_2;
  float tmp;
  
  if((self->has_constant_sampling_intervall != True)
	  && (second_spectrum_ptr->has_constant_sampling_intervall != True)) 
    {
      Status = FATAL;
      return Status;
    }
  if(second_spectrum_ptr->number_of_values != self->number_of_values)
    {
      Status = FATAL;
      return Status;
    }
  if( (self->re_ptr == 0) && (self->im_ptr == 0)
      && (second_spectrum_ptr->re_ptr == 0) 
      && (second_spectrum_ptr->im_ptr == 0) )
    {
      Status = FATAL;
      return Status;
    }

  Spectrum__complexMultiply(self, second_spectrum_ptr);
  
  Spectrum__inverseFourierTransform(self);
  
  nop = self->number_of_values;
  
  for (i=0; i < nop; i++) {
    self->re_ptr[i] *= nop/norm_factor;
    self->im_ptr[i] *= nop/norm_factor;
  }	
  
  
		
  if ((nop % 2)== 0)
    {
      nop_div_2 = nop / 2;
      for (i=0; i < nop_div_2; i++) {
	tmp = self->re_ptr[i];
	self->re_ptr[i] = self->re_ptr[nop_div_2 + i];
	self->re_ptr[nop_div_2 + i] = tmp;
	tmp = self->im_ptr[i];
	self->im_ptr[i] = self->im_ptr[nop_div_2 + i];
	self->im_ptr[nop_div_2 + i] = tmp;
      }
    }
  else 
    {
      nop+=1;
      nop_div_2 = nop/2;
      for (i=0; i < nop_div_2-1; i++) {
	tmp = self->re_ptr[i];
	self->re_ptr[i] = self->re_ptr[nop_div_2 + i];
	self->re_ptr[nop_div_2 + i] = tmp;
	tmp = self->im_ptr[i];
	self->im_ptr[i] = self->im_ptr[nop_div_2 + i];
	self->im_ptr[nop_div_2 + i] = tmp;
      }
    }	
  
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                             Spectrum__findPeak
**  Input parameters:
**      self:
**      fractional_number_of_points: fractional number of points of spectrum
**                                   before resampling to resampling grid
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__findShiftAndPeak(const Spectrum_s* self, float* shift_ptr,
			   float* peak_height_ptr) 
{
  FunctionStatus_e Status =SUCCESS;
  int i;
  int bound;
  int upper_bound;
  int lower_bound;
  int peak_index;
  float abs_minimum_value;
  float gravity_centre;
  float gravity_offset;	
  float signal_sum;
  float x;
  int nov_div_2;
  float sampling_intervall;
  
  if((self->has_constant_sampling_intervall != True) || (self->number_of_values < 2) ||
     (self->has_grid != True))
    {
      Status = FATAL;
      return Status;
    }

  *peak_height_ptr = self->re_ptr[0];
  peak_index = 0;
  for (i=1; i < self->number_of_values; i++) {
    if (self->re_ptr[i] > *peak_height_ptr) {
      *peak_height_ptr = self->re_ptr[i];
      peak_index = i;
    }
  }
  
  /*------------------------------------------------------------------------*\
  **  find the index of the first maximum and go number_of_values points
  **  to the left and to the right
  \*------------------------------------------------------------------------*/
  bound = (self->number_of_values/10 > 10) ? self->number_of_values/10 : 10;
  upper_bound = (peak_index + bound > self->number_of_values-1) 
    ? self->number_of_values-1 : peak_index + bound;
  lower_bound = (peak_index - bound < 0) ? 0 : peak_index - bound;
  
  abs_minimum_value = self->re_ptr[lower_bound];
  for (i=lower_bound+1; i <= upper_bound; i++) {
    if (self->re_ptr[i] < abs_minimum_value) {
      abs_minimum_value = self->re_ptr[i];
    }
  }
  if (abs_minimum_value < 0.0) {
    abs_minimum_value = - abs_minimum_value;
  }
  
  gravity_centre = 0.0;
  signal_sum = 0.0;
  for (i=lower_bound; i <= upper_bound; i++) {
    x = (self->re_ptr[i] + abs_minimum_value) / *peak_height_ptr;
    gravity_centre += i * x;
    signal_sum += x;
  }
  gravity_centre /= signal_sum;
  
#ifdef DEBUG
  printf("peak_index = %d\tpeak_height = %f \n", peak_index, *peak_height_ptr);
  printf("gravity_centre = %f \n", gravity_centre);
#endif
  
  gravity_offset = gravity_centre - peak_index;
  nov_div_2 = self->number_of_values / 2;
  sampling_intervall = self->grid_ptr[1] - self->grid_ptr[0];
  *shift_ptr = (peak_index - nov_div_2 + gravity_offset) 
    * sampling_intervall;
  /* #ifdef DEBUG
     printf("peak = %f\twavelength shift [nm] = %f\n", 
     *peak_height_ptr, *shift_ptr);
     #endif */
  return Status;
}


/*----------------------------------------------------------------------------*\
**                         Spectrum__crossCorrelationError
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__crossCorrelationError(const Spectrum_s* self, float shift, 
				float peak_height, 
				float fractional_number_of_values,
				float* error)
{
  FunctionStatus_e Status = SUCCESS;
  float* x_ptr;
  float* x1_ptr;
  float* x2_ptr;
  float* y1_ptr;
  float* y2_ptr;
  int i;
  float sampling_intervall;
  float sigma;
  float y1_minus_y2;
  float ratio;
  int nov_div_2;
  
  if((self->has_grid != True) || (self->has_constant_sampling_intervall != True) ||
     (self->number_of_values <= 1))
    {
      Status = FATAL;
      return Status;
    }
 
#ifdef __cplusplus
  x_ptr = new float[self->number_of_values];
  x1_ptr = new float[self->number_of_values];
  x2_ptr = new float[self->number_of_values];
  y1_ptr = new float[self->number_of_values];
  y2_ptr = new float[self->number_of_values];
#else
  x_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
  x1_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
  x2_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
  y1_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
  y2_ptr = (float*)malloc(self->number_of_values*(sizeof(float)));
#endif
  sampling_intervall = self->grid_ptr[1] - self->grid_ptr[0];
  nov_div_2 = self->number_of_values / 2;
  for (i=0; i < self->number_of_values; i++) {
    x_ptr[i] = (i - nov_div_2) * sampling_intervall;
    x1_ptr[i] = (i - nov_div_2) * sampling_intervall + shift;
    x2_ptr[self->number_of_values - i - 1] = 
      shift - (i - nov_div_2) * sampling_intervall;
  }
  fit_grid_akima(x_ptr, self->re_ptr, self->number_of_values,
		 x1_ptr, y1_ptr, 0, self->number_of_values - 1);
  fit_grid_akima(x_ptr, self->re_ptr, self->number_of_values,
		 x2_ptr, y2_ptr, 0, self->number_of_values - 1);
#ifdef __cplusplus
  delete[] x_ptr;
  delete[] x1_ptr;
  delete[] x2_ptr;
#else
  free(x_ptr);
  free(x1_ptr);
  free(x2_ptr);
#endif
  
  sigma = 0.0;
  for (i=0; i < self->number_of_values; i++) {
    y1_minus_y2 = y1_ptr[i] - y2_ptr[self->number_of_values - i - 1];
    sigma += y1_minus_y2 * y1_minus_y2;
  }
  sigma = sqrt(sigma / (2.0 * self->number_of_values));
  ratio = peak_height / (sigma * sqrt(2.0));
  /* #ifdef DEBUG
     printf("sigma = %f\tratio = %f\n", sigma, ratio);
     #endif */
  
#ifdef __cplusplus
  delete[] y1_ptr;
  delete[] y2_ptr;
#else
  free(y1_ptr);
  free(y2_ptr);
#endif
  *error = fractional_number_of_values;
  *error /= (8.0 * BINMAX * (1.0 + ratio));
  return Status;
}

/*----------------------------------------------------------------------------*\
**                         Spectrum__GetTemplateFileNumber
**  Input parameters:
**     template_spectrum
**     start_pixel ;  Start pixel of Sun Spectrum
**
**  Output parameters:
**      
**
**  Return value:
**      template_number: Number of Template Spectrum (1,2,3)
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      
**
**  References:
**      <none>
**
**  Libraries:
**      <none>
**
**  Created:	18.12.2001
**
**  Author:	Sandra Hummel, DLR/IMF-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__GetTemplateFileNumber(int start_pixel,
				int channel,int *template_number)
{
  FunctionStatus_e Status = SUCCESS;

  if(start_pixel + WINDOW_SIZE[channel] >= TEMPLATE_BORDERS[2])
    {
      Status = ERROR;
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__1413", Status,
		   LOG_NUM, "Start Pixel is out of Template Range", "");
      return Status;
    }
  if(start_pixel < TEMPLATE_BORDERS[0]) *template_number = 0;
  else if (start_pixel < TEMPLATE_BORDERS[1]) *template_number = 1;
  else if (start_pixel < TEMPLATE_BORDERS[2]) *template_number = 2;
  
  return Status;
}

/*----------------------------------------------------------------------------*\
**                         Spectrum__GetTemplateWindowSize
**  Input parameters:
**     self:
**     template_ptr
**
**  Output parameters:
**      window_ptr: pointer to the Spectrum object containing the window
**
**  Return value:
**      A new allocated Spectrum object containing the intervall cutted out of
**      template_ptr is returned.
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Cut a window out of the spectrum object template_ptr.
**      The Window starts with the first point i, where template_ptr->re_ptr is less than
**      self->re_ptr, and end with the last point j, where template_ptr->re_ptr ist greater
**      than self->re_ptr.
**
**  References:
**      <none>
**
**  Libraries:
**      <none>
**
**  Created:	12.12.2001
**
**  Author:	Sandra Hummel, DLR/IMF-AP
**
\*----------------------------------------------------------------------------*/
int Spectrum__GetTemplateWindowSize(Spectrum_s* self,const Spectrum_s* template_ptr,
				    int pixel_start,int interval_size,int additional_pixel,
				    Spectrum_s* window_ptr)
{
  float x=0.0;
  float y=0.0;
  int i=0,j=0;
  float a=0.0;
  int number_of_template_pixel=0;

  x=(self->grid_ptr[pixel_start]-template_ptr->grid_ptr[0])/0.01;
  i=floor(x)-additional_pixel;
 
  y=(self->grid_ptr[pixel_start+interval_size]-template_ptr->grid_ptr[0])/0.01;
  j=floor(y)+additional_pixel+2;
  
  number_of_template_pixel= j-i;
 
#ifdef DEBUG 
  printf("Template Window from %f nm to %f nm \n",template_ptr->grid_ptr[i],template_ptr->grid_ptr[j]);  
#endif

  Spectrum__cutWindow(template_ptr,i,j,window_ptr);
  
  return number_of_template_pixel;
}


/*----------------------------------------------------------------------------*\
**                        Spectrum__wavelengthCalibrate
**  Input parameters:
**      self:
**      self_channel: the channel number of the self spectrum
**      template_spectrum_ptr:
**      template_window_pixels: window size in pixels of template spectrum
**      start_wavelength: wavelength in nm where first window starts
**      end_wavelength: wavelength in nm greater or equal to the end of the
**                      last window
**      overlap: the ovelap of two following windows in nm
**
**  Output parameters:
**      ....
**
**  Return value:
**
**
**  Description:
**      ....
**
**  Created:	28.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/

FunctionStatus_e
Spectrum__wavelengthCalibration(Spectrum_s* self, OpticalChannel_e self_channel,
				const Spectrum_s template_spectrum_ptr[N_TEMPLATE_FILE],
				XC_Coeff_s* xcorr_coeffs)	   
{
  FunctionStatus_e Status = SUCCESS;
  Spectrum_s gome_window;
  Spectrum_s template_window;
  int i=0,iptr=0;
  int template_window_size=0;
  int template_number = 0;
  int interval_size = WINDOW_SIZE[self_channel];
  int overlap = WINDOW_OVERLAP[self_channel];
  int step_size = interval_size - overlap;    
  int number_of_windows = 0;
  int additional_points_template_window 
    = ADDITIONAL_PIXEL_TEMPLATE_WINDOW[self_channel]; 
  /* additional points left and right of the
     template window */
  int pixel_start=0,pixel_end=0;
  float start_wavelength=0.0;
  float end_wavelength=0.0;
  float a=0.0;
  float* window_size_nm;
  float* wv_center_ptr;
  float* shift_ptr;
  float* error_ptr;
  char TEMP[80];


  number_of_windows = NUMBER_OF_WINDOWS[self_channel];


  for(iptr=0;iptr<N_TEMPLATE_FILE;iptr++)
    {
      if((template_spectrum_ptr->has_constant_sampling_intervall != True) ||
	 (number_of_windows < 0))
	{
	  Status = FATAL;
	  return Status;
	}
    }
  
#ifdef __cplusplus
  shift_ptr = new float[number_of_windows];
  error_ptr = new float[number_of_windows];
  wv_center_ptr = new float[number_of_windows];
	window_size_nm = new float[number_of_windows];
#else
  shift_ptr = (float*)malloc(number_of_windows*(sizeof(float)));
  error_ptr = (float*)malloc(number_of_windows*(sizeof(float)));
  wv_center_ptr = (float*)malloc(number_of_windows*(sizeof(float)));
	window_size_nm = (float*)malloc(number_of_windows*(sizeof(float)));
#endif 

	for(i=0;i<number_of_windows;i++)
		{
			shift_ptr[i]=0.0;
			error_ptr[i]=0.0;
			wv_center_ptr[i]=0.0;
			window_size_nm[i]=0.0;
		}
	pixel_start = START_PIXEL[self_channel][0];
	pixel_end = START_PIXEL[self_channel][NUMBER_OF_WINDOWS[self_channel]-1];
	start_wavelength = self->grid_ptr[START_PIXEL[self_channel][0]];
	end_wavelength = self->grid_ptr[START_PIXEL[self_channel][NUMBER_OF_WINDOWS[self_channel]-1]];
#ifdef DEBUG
	printf("***********************************************************\n");
	printf("Starting XC for Channel %d ...\n",self_channel+1);
  printf("start_pixel: %d \t end_pixel: %d \n",
	 pixel_start,pixel_end);
  printf("start_wv: %f nm \t end_wv: %f nm \n",start_wavelength,
	 end_wavelength);
  printf("Number of Points GOME: %d \n",self->number_of_values);
  printf("Number of Windows = %d \n",number_of_windows);
#endif
  
  for(i=0;i<number_of_windows;i++)
    /*for(i=0;i<2;i++)*/
    {

      pixel_start=START_PIXEL[self_channel][i];
      if(pixel_start == 0)
	{
	  Status = ERROR;
	  return Status;
	}
      start_wavelength = self->grid_ptr[pixel_start];
      
 	    window_size_nm[i] = self->grid_ptr[i+1]-self->grid_ptr[1];
      
#ifdef DEBUG
      printf("-----\n");
      printf("Window %d from Pixel %d to %d \n",i,pixel_start,pixel_start + interval_size-1);
      printf("Window %d from %f nm to %f nm \n", i, 
	     start_wavelength, self->grid_ptr[pixel_start + interval_size -1]);
#endif 
      Status = Spectrum__GetTemplateFileNumber(pixel_start,
					 self_channel,&template_number);

      template_window_size = Spectrum__GetTemplateWindowSize(self,
							     &template_spectrum_ptr[template_number],
							     pixel_start,interval_size-1,
							     3*additional_points_template_window/2,
							     &template_window);
     
      
      Status = Spectrum__cutWindow(self,pixel_start,
			  pixel_start+interval_size,
			  &gome_window);
      
      if(Status !=SUCCESS)
	{
	  Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__1625", Status,
		   LOG_NUM, "Cannot cut Window out of Sun Spectrum", "");
	  return Status;
	}
      Status = Spectrum__wavelengthCalibrateWindow(&gome_window, &template_window, 
					  additional_points_template_window,
					  CONTINUUM_CORRECTION_DEGREE,
					  &shift_ptr[i], &error_ptr[i],&wv_center_ptr[i]);
      
      if(Status !=SUCCESS)
	{
	  Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__1636", Status,
		   LOG_NUM, "Cross Correlation failed", "");
	  return Status;
	}
#ifdef DEBUG
      printf("%9.5f%9.5f\t%9.5f\t%d\t%d\t%d \n",
	     shift_ptr[i],error_ptr[i],(self->grid_ptr[i+1]-self->grid_ptr[i]),
	     interval_size,pixel_start,pixel_start+interval_size-1);
#endif

      Spectrum__destructor(&gome_window);
      Spectrum__destructor(&template_window);
      
 
      /*pixel_start+=step_size;*/
    }

  Status = Spectrum__fitWavelengthCalibrationResults(xcorr_coeffs,shift_ptr,
						     error_ptr,wv_center_ptr,window_size_nm,
						     self_channel,number_of_windows);
  if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__1658", Status,
	       LOG_NUM, "Cannot calculate Polynom", "");
      return Status;
    }
  
#ifdef __cplusplus
  delete[] shift_ptr;
  delete[] error_ptr;
  delete[] wv_center_ptr;
	delete[] window_size_nm;
#else
  free(shift_ptr);
  free(error_ptr);
  free(wv_center_ptr);
	free(window_size_nm);
#endif
  return Status;
}


/*============================================================================*\
**  private operations of class Spectrum
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            Spectrum__cutWindow
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	..98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__cutWindow(const Spectrum_s* self, int i, int j, 
		    Spectrum_s* window_ptr)
{
  FunctionStatus_e Status = SUCCESS;
  int k;
  
  if((i < 0) && (j > self->number_of_values) )
    {
      Status = FATAL;
      return Status;
    }
  
#ifdef __cplusplus
  window_ptr->re_ptr = new float[j - i];
  window_ptr->grid_ptr = new float[j - i];
#else
  window_ptr->re_ptr = (float*)malloc((j-i)*(sizeof(float)));
  window_ptr->grid_ptr = (float*)malloc((j-i)*(sizeof(float)));
#endif
  
  window_ptr->im_ptr = 0;
  window_ptr->number_of_values = j - i;
  window_ptr->has_grid = True;
  window_ptr->has_constant_sampling_intervall = 
    self->has_constant_sampling_intervall;
  for (k = i; k < j; k++) {
    window_ptr->re_ptr[k - i] = self->re_ptr[k];
    if (self->im_ptr != 0) {
      window_ptr->im_ptr[k - i] = self->im_ptr[k];
    }
    window_ptr->grid_ptr[k - i] = self->grid_ptr[k];
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                         Spectrum__fractionalNumberOfPoints
**  Input parameters:
**      self:
**      grid_ptr:
**      number_of_values:
**
**  Return value:
**      The fractional number of values of self->grid_ptr within the intervall
**      described by grid_ptr.
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	24.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__fractionalNumberOfPoints(const Spectrum_s* self,
				   const float* grid_ptr, int number_of_values,
				   float* fractional_number_of_values)
{
  FunctionStatus_e Status =SUCCESS;
  int i;
  int start_index;
  int end_index;
  float delta_bottom;
  float delta_top;
  
  if(self->has_grid != True)
    {
      Status = FATAL;
      return Status;
    }
  
  /*------------------------------------------------------------------------*\
  **  find values of and their indices of self within grid_ptr
  \*------------------------------------------------------------------------*/
  *fractional_number_of_values = 0.0;
  start_index = 0;
  for (i=0; (i < self->number_of_values) 
	 && (self->grid_ptr[i] <= grid_ptr[number_of_values-1]); i++) {
    if (self->grid_ptr[i] >= grid_ptr[0]) {
      *fractional_number_of_values += 1.0;
    }
    else {
      start_index++;
    }
  }
  if (self->grid_ptr[self->number_of_values - 1] 
      < grid_ptr[number_of_values - 1]) {
    end_index = self->number_of_values - 1;
  }
  else {
    end_index = i - 1;	
  }
  
  /*------------------------------------------------------------------------*\
  **  get the distances to the first values in self->grid_ptr not
  **  within grid_ptr.
  **  If none exists, the distance to the next value within grid_ptr is used.
  \*------------------------------------------------------------------------*/
  if (start_index == 0) {
    delta_bottom = self->grid_ptr[1] - self->grid_ptr[0];
  }
  else {
    delta_bottom = self->grid_ptr[start_index] 
      - self->grid_ptr[start_index - 1];
  }
  if (end_index == self->number_of_values - 1) {
    delta_top = self->grid_ptr[self->number_of_values - 1] 
      - self->grid_ptr[self->number_of_values - 2];
  }
  else {
    delta_top = self->grid_ptr[end_index + 1] 
      - self->grid_ptr[end_index];
  }
  
  *fractional_number_of_values += (self->grid_ptr[start_index] - grid_ptr[0])
    / delta_bottom;
  *fractional_number_of_values += (grid_ptr[number_of_values - 1] 
				  - self->grid_ptr[end_index]) / delta_top;
  return Status;
}


/*----------------------------------------------------------------------------*\
**                          Spectrum__fft
**  Input parameters:
**      self: pointer to Spectrum object
**
**  Output parameters:
**      self: 
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      This function applys a discrete fourier transformation to self.
**      If isign is -1 a forward fourier transformation is performed.
**      If isign is 1 a inverse fourier transformation is performed.
**      Attention: - the distance between the points on the wavelength grid 
**                   must be constant 
**                   (self->has_constant_sampling_intervall equals true)
**
**  References:
**      FFTW
**      
**  Libraries:
**      ...
**
**  Created:	4.12.2001
**
**  Author:	Sandra Hummel, DLR/IMF-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__fft(Spectrum_s* self,int isign)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  fftw_complex *data_ptr,*new_data_ptr;
  fftw_plan p;
  FILE *file_stream=NULL;

  if((self->has_constant_sampling_intervall != True) || !((isign == -1) || (isign == 1)))
    {
      Status = FATAL;
      return Status;
    }

#ifdef __cplusplus
	data_ptr = new fftw_complex[self->number_of_values];
	new_data_ptr = new fftw_complex[self->number_of_values];
#else		
  data_ptr = (fftw_complex *)malloc(self->number_of_values*sizeof(fftw_complex));
  new_data_ptr = (fftw_complex *)malloc(self->number_of_values*sizeof(fftw_complex));
#endif
  
  for(i=0;i<self->number_of_values;i++)
    {
      data_ptr[i].re=self->re_ptr[i];
      data_ptr[i].im=(self->im_ptr != 0) ? self->im_ptr[i] : 0.0;
    }
  
  if (isign ==-1)
    {
      p=fftw_create_plan(self->number_of_values,FFTW_FORWARD,FFTW_ESTIMATE);
    }
  else
    {
      p=fftw_create_plan(self->number_of_values,FFTW_BACKWARD,FFTW_ESTIMATE);
    }
  fftw_one(p,data_ptr,new_data_ptr);
  fftw_destroy_plan(p);
  
  if(self->im_ptr == 0)
    {
      self->im_ptr = (float*)malloc(self->number_of_values*sizeof(float));
    }
  
  for(i=0;i<self->number_of_values;i++)
    {
      self->re_ptr[i] = new_data_ptr[i].re/self->number_of_values;
      self->im_ptr[i] = new_data_ptr[i].im/self->number_of_values;
    }
#ifdef __cplusplus
	delete[] data_ptr;
	delete[] new_data_ptr;
#else
  free(data_ptr);
  free(new_data_ptr);
#endif

#ifdef DEBUG
  file_stream = fopen("four1_gome.dbg", "w");
  Spectrum__write(self, file_stream);
  fclose(file_stream);
#endif

  return Status;
}


/*----------------------------------------------------------------------------*\
**                    Spectrum__complexMultiplication 
**  Input parameters:
**      self:
**      second_im_ptr:
**
**  Output parameters:
**      result_ptr: the result of complex multiplication of self and 
**                  second_spectrum_ptr
**
**  Description:
**      Complex multiplication of two spectra, the result is written to
**      result_ptr.
**      Attention: - number of values must be equal at the three spectra
**                 - re_ptr and im_ptr must not be zero at the three spectra
**
**  Created:	22.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__complexMultiplication(const Spectrum_s* self, 
				const Spectrum_s* second_spectrum_ptr,
				Spectrum_s* result_ptr)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  
  if(((second_spectrum_ptr->number_of_values != self->number_of_values) 
     && (result_ptr->number_of_values != self->number_of_values)) ||
     ((self->re_ptr == 0) && (self->im_ptr == 0)
      && (second_spectrum_ptr->re_ptr == 0) 
      && (second_spectrum_ptr->im_ptr == 0)
      && (result_ptr->re_ptr == 0) && (result_ptr->im_ptr == 0)))
    {
      Status = FATAL;
      return Status;
    }
 
  for (i=0; i < self->number_of_values; i++) {
    result_ptr->re_ptr[i] = self->re_ptr[i] 
      * second_spectrum_ptr->re_ptr[i]
      - self->im_ptr[i] * second_spectrum_ptr->im_ptr[i];
    result_ptr->im_ptr[i] = self->re_ptr[i] 
      * second_spectrum_ptr->im_ptr[i]
      + self->im_ptr[i] * second_spectrum_ptr->re_ptr[i];
  }
  
  result_ptr->has_constant_sampling_intervall = True;
  return Status;
}


/*----------------------------------------------------------------------------*\
**                    Spectrum__complexMultiply
**  Input parameters:
**      self:
**      second_im_ptr:
**
**  Output parameters:
**      self: the result of complex multiplication of self and 
**            second_spectrum_ptr
**
**  Description:
**      Complex multiplication of two spectra.
**      Attention: - number of values must be equal at the two spectra
**                 - re_ptr and im_ptr must not be zero at the two spectra
**
**  Created:	23.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__complexMultiply(Spectrum_s* self, 
			  const Spectrum_s* second_spectrum_ptr)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  float re;
  
  if((second_spectrum_ptr->number_of_values != self->number_of_values)||
     ((self->re_ptr == 0) && (self->im_ptr == 0)
	  && (second_spectrum_ptr->re_ptr == 0) 
	  && (second_spectrum_ptr->im_ptr == 0)))
    {
      Status = FATAL;
      return Status;
    }

  for (i=0; i < self->number_of_values; i++) {
    re = self->re_ptr[i] * second_spectrum_ptr->re_ptr[i]
      - self->im_ptr[i] * second_spectrum_ptr->im_ptr[i];
    self->im_ptr[i] = self->re_ptr[i] * second_spectrum_ptr->im_ptr[i]
      + self->im_ptr[i] * second_spectrum_ptr->re_ptr[i];
    self->re_ptr[i] = re;
  }
  
  return Status;
}


/*----------------------------------------------------------------------------*\
**                              Spectrum__conjugate
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      Compute the complex conjugate of the Spectrum object self.
**      self->state must be INVERSEWAVELENGTH_DOMAIN
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	22.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__conjugate(Spectrum_s* self)
{
  FunctionStatus_e Status = SUCCESS;
  int i;
  
  if(self->im_ptr == 0)
    {
      Status = FATAL;
      return Status;
    }
  
  for (i=0; i < self->number_of_values; i++) {
    self->im_ptr[i] = -self->im_ptr[i];
  }
  
  return Status;
}





/*----------------------------------------------------------------------------*\
**                       Spectrum__wavelengthCalibrateWindow
**  Input parameters:
**      self:
**      template_window_ptr:
**      additional_points_template_window:
**
**  Output parameters:
**      shift_ptr:
**      error_ptr:
**
**  Other interfaces:
**      Some output files for debugging are written, if DEBUG is defined.
**
**  Description:
**      Comute a cross correlation between the self and template_window_ptr
**      Spectrum objects. The shift and error are returned.
**
**  Created:	27.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__wavelengthCalibrateWindow(Spectrum_s* self,
				    Spectrum_s* template_window_ptr, 
				    int additional_pixel_template_window,
				    int continuum_correction_polynim_degree,
				    float* shift_ptr, float* error_ptr,float* wv_center_ptr)
{
  FunctionStatus_e Status = SUCCESS;
  const float* self_re_ptr;
  const float* template_re_ptr;
  float fractional_number_of_self_points=0.0;
  float sum_self_re;
  float sum_template_re;
  float norm_factor;
  FILE* file_stream;
  float* hp_filter_ptr;
  float* lp_filter_ptr;
  int filter_length;
  int i;
  float peak_height=0.0;
  float shift=0.0;
  float error=0.0;
  FILE *f=NULL;
  
  if((self->has_grid != True) || (template_window_ptr->has_grid != True))
     {
       Status = FATAL;
       return Status;
     }

#ifdef DEBUG
  file_stream = fopen("input_gome.dbg", "w");
  Spectrum__write(self, file_stream);
  fclose(file_stream);
#endif
  
#ifdef DEBUG
  file_stream = fopen("input_template.dbg", "w");
  Spectrum__write(template_window_ptr, file_stream);
  fclose(file_stream);
#endif
  /*------------------------------------------------------------------------*\
  **  compute center wavelength
  \*------------------------------------------------------------------------*/

  *wv_center_ptr =
    (self->grid_ptr[self->number_of_values-1]-self->grid_ptr[0])/2.0+self->grid_ptr[0];
  /*------------------------------------------------------------------------*\
  **  convolute - cut edges - correct continuum - apodise
  \*------------------------------------------------------------------------*/
  
  /*i=template_window_ptr->number_of_values;
    Status = Spectrum__convoluteNormalizedGaussian(template_window_ptr, 
    additional_pixel_template_window);
    
    #ifdef DEBUG
    file_stream = fopen("convoluted_template.dbg","w");
    Spectrum__write(template_window_ptr,file_stream);
    fclose(file_stream);
    #endif    
    if(Status !=SUCCESS)
    {
    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2179", Status,
    LOG_NUM, "Cannot convolute Template Spectrum", "");
    return Status;
    }*/
    
  Status = Spectrum__cutEdges(template_window_ptr,3*additional_pixel_template_window/2);
  
#ifdef DEBUG
  file_stream = fopen("cutEdges_template.dbg","w");
  Spectrum__write(template_window_ptr,file_stream);
  fclose(file_stream);
#endif
  if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2193", Status,
	       LOG_NUM, "Cannot cut additional Pixel", "");
      return Status;
    }

  
  if(self->has_grid != True)
    {
      Status = FATAL;
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2202", Status,
	       LOG_NUM, "Window of Sun Spectrum has no grid", "");
      return Status;
    }
  
  Status = Spectrum__resampleSpline(self,
				    Spectrum__getGridPtr(template_window_ptr),
				    Spectrum__getNumberOfValues(template_window_ptr),
				    Spectrum__hasConstantSamplingInterval(template_window_ptr),
				    &fractional_number_of_self_points);

#ifdef DEBUG
  file_stream = fopen("resampled_gome.dbg", "w");
  Spectrum__write(self, file_stream);
  fclose(file_stream);
#endif 
  if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2220", Status,
	       LOG_NUM, "Cannot Resample Sun Spectrum", "");
      return Status;
    }

  Status = Spectrum__scale(self, Spectrum__maxRe(self));
    if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2228", Status,
	       LOG_NUM, "Cannot scale Sun Spectrum", "");
      return Status;
    }
  Status = Spectrum__scale(template_window_ptr, 
		  Spectrum__maxRe(template_window_ptr));
    if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2236", Status,
	       LOG_NUM, "Cannot scale Template Pixel", "");
      return Status;
    }
  Status = Spectrum__continuumCorrectionPolyfit(self, 
						continuum_correction_polynim_degree);
#ifdef DEBUG
    file_stream = fopen("continuum_gome.dbg", "w");
    Spectrum__write(self, file_stream);
    fclose(file_stream);
#endif 
    if(Status !=SUCCESS)
    {
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2249", Status,
	       LOG_NUM, "Cannot make Continuum Correction for Sun Spectrum", "");
      return Status;
    }
  
    Status = Spectrum__continuumCorrectionPolyfit(template_window_ptr, 
					 continuum_correction_polynim_degree); 
#ifdef DEBUG
    file_stream = fopen("continuum_template.dbg", "w");
    Spectrum__write(template_window_ptr, file_stream);
    fclose(file_stream);
#endif
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2263", Status,
		 LOG_NUM, "Cannot make Continuum Correction for Template Spectrum", "");
	return Status;
      }  

    Status = Spectrum__apodise(self, APODISE_FRACTION);

#ifdef DEBUG
    file_stream = fopen("apodised_gome.dbg", "w");
    Spectrum__write(self, file_stream);
    fclose(file_stream);
#endif 

    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2278", Status,
		 LOG_NUM, "Cannot apodise Sun Spectrum", "");
	return Status;
      }

    Status = Spectrum__apodise(template_window_ptr, APODISE_FRACTION);
    

#ifdef DEBUG
    file_stream = fopen("apodised_template.dbg", "w");
    Spectrum__write(template_window_ptr, file_stream);
    fclose(file_stream);
#endif 

    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2294", Status,
		 LOG_NUM, "Cannot apodise Template Spectrum", "");
	return Status;
      } 
    /*------------------------------------------------------------------------*\
    **  compute norm_factor for later use
    \*------------------------------------------------------------------------*/
    self_re_ptr = Spectrum__getRePtr(self);
    template_re_ptr = Spectrum__getRePtr(template_window_ptr);
    sum_self_re = 0.0;
    sum_template_re = 0.0;
    for (i=0; i < Spectrum__getNumberOfValues(self); i++) {
      sum_self_re += self_re_ptr[i] * self_re_ptr[i];
      sum_template_re += template_re_ptr[i] * template_re_ptr[i];
    }	    
    norm_factor = sqrt(sum_self_re * sum_template_re);
    
    /*------------------------------------------------------------------------*\
    **  Fourier filtering
    \*------------------------------------------------------------------------*/
    Status = Spectrum__fourierTransform(self);

#ifdef DEBUG
    file_stream = fopen("four_gome.dbg", "w");
    Spectrum__write(self, file_stream);
    fclose(file_stream);
#endif
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2323", Status,
		 LOG_NUM, "Cannot Fourier Transform Sun Spectrum", "");
	return Status;
      }

    Status = Spectrum__fourierTransform(template_window_ptr);    


#ifdef DEBUG
    file_stream = fopen("four_template.dbg", "w");
    Spectrum__write(template_window_ptr, file_stream);
    fclose(file_stream);
#endif
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2338", Status,
		 LOG_NUM, "Cannot Fourier Transform Template Spectrum", "");
	return Status;
      }

    Status = Spectrum__conjugate(template_window_ptr);

#ifdef DEBUG
    file_stream = fopen("conjfour_template.dbg", "w");
    Spectrum__write(template_window_ptr, file_stream);
    fclose(file_stream);
#endif
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2352", Status,
		 LOG_NUM, "Cannot conjugate Template Spectrum", "");
	return Status;
      }
    /*------------------------------------------------------------------------*\
    **  Cross correlate
    \*------------------------------------------------------------------------*/
    Status = Spectrum__crossCorrelate(self, template_window_ptr, norm_factor);


#ifdef DEBUG
    file_stream = fopen("window_xcorrelated.dbg", "w");
    Spectrum__write(self, file_stream);
    fclose(file_stream);
#endif
    
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2370", Status,
		 LOG_NUM, "Cross Correlation failed", "");
	return Status;
      }   
    
    Status = Spectrum__findShiftAndPeak(self, &shift, &peak_height);
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2378", Status,
		 LOG_NUM, "Cannot find Shift and Peaks", "");
	return Status;
      }
 
    Status = Spectrum__crossCorrelationError(self, shift, peak_height,
					    fractional_number_of_self_points,
					    &error);
    
    if(Status !=SUCCESS)
      {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2389", Status,
		 LOG_NUM, "Cannot calculate Cross Correlation Error", "");
	return Status;
      }
    /* #ifdef DEBUG
       printf("shift [nm] = %f\tpeak_height = %f\terror [nm] = %f\n", shift,
	  peak_height, error);
	  #endif  */
    

    *shift_ptr = shift;
    *error_ptr = error;
    return Status;
} 


/*----------------------------------------------------------------------------*\
**                                Spectrum__fitWavelengthCalibrationResults
**  Input parameters:
**      ....
**
**  Output parameters:
**      ....
**
**  Return value:
**
**  Other interfaces:
**      Output file ...
**
**  Description:
**      ....
**
**  References:
**      This function is described in the Module ...
**
**  Libraries:
**      ....
**
**  Created:	28.7.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__fitWavelengthCalibrationResults(XC_Coeff_s* wv_shift,
					  float* shift_ptr,float* error_ptr,
					  float* wv_center_ptr,float* window_size_nm,int channel,int n_data)
{
  FunctionStatus_e Status = SUCCESS;
  short ma=POLYNOM_DEGREE[channel]+1;
  double *X,*Y,*W,*Sign;
  double *a;
  double **U,**V;
  double ChisQ;
  double **cvm;
  float *yobs;
  float *diff,*diff2;
  float stddev=0.0;
  int i=0,j=0,k=0;

	/*------------------------------------------------------------*\
	**  cut out shifts greater than window_size (nm)
	\*------------------------------------------------------------*/
	for(i=0;i<n_data;i++)
  {
	  if((shift_ptr[i]>=(window_size_nm[i]/2.0)) || 
		  (shift_ptr[i]<=-window_size_nm[i]/2.0))
		  {
			  ++k;
					  for(j=i;j<n_data-1;j++)
					  {
						  shift_ptr[j]=shift_ptr[j+1];
						  error_ptr[j]=error_ptr[j+1];
						  wv_center_ptr[j]=wv_center_ptr[j+1];
						  window_size_nm[j]=window_size_nm[j+1];
					  }
			  if(i < (n_data-k)) --i;	
			  else ++i;
		  }
  }
	n_data-=k;	

  /* catch crash in vector allocation */
  if (n_data <= 0) return ERROR;

  X=vector__new(1,n_data);
  Y=vector__new(1,n_data);
  Sign=vector__new(1,n_data);
  W=vector__new(1,ma);
  a=vector__new(1,ma);
  U=matrix__new(1,n_data,1,ma);
  V=matrix__new(1,ma,1,ma);
  cvm=matrix__new(1,ma,1,ma);
  diff=(float*)malloc(n_data*sizeof(float));
  diff2=(float*)malloc(n_data*sizeof(float));
  yobs=(float*)malloc(n_data*sizeof(float));
  wv_shift->Error[channel]=0.0;
  for(i=0;i<MAX_FITT_ORDER;i++)
    {
      wv_shift->Coefficient[channel][i]=0.0;
      wv_shift->CoefficientError[channel][i]=0.0;
    }
  for(i=1;i<=ma;i++) 
    {
      a[i]=0.0;
    }
  for(i=0;i<n_data;i++) 
    {
      diff[i]=0.0;
      diff2[i]=0.0;
      yobs[i]=0.0;
    }
  for(i=1;i<=n_data;i++) 
    {
      X[i]=0.0;
      Y[i]=0.0;
      Sign[i]=0.0;
    }
  
  for(i=1;i<=n_data;i++)
    {
      X[i]=(wv_center_ptr[i-1]-OFFSET)/PARAMETER;
      Y[i]=shift_ptr[i-1];
      Sign[i]=error_ptr[i-1];
    }

  svdfit(X,Y,Sign,n_data,a,ma,U,V,W,&ChisQ,CalculatePolynome1,&Status);
  if(Status != SUCCESS)
    {
      Status = FATAL;
      Log__Msg(LOG_EXTRACT_DATA, SS_ID, "1.4__2519", Status,
	       LOG_NUM, "Cannot fit Polynom", "");
      return Status;
    }
  svdvar(V,ma,W,cvm);
  for(i=0;i<ma;i++) 
    {
      wv_shift->Coefficient[channel][i]=a[i+1];
      wv_shift->CoefficientError[channel][i]=sqrt(cvm[i+1][i+1]);
#ifdef DEBUG
      printf("Polynom Parameter of degree %d  (std.dev): %e (%e) \n",
	     i,wv_shift->Coefficient[channel][i],wv_shift->CoefficientError[channel][i]);
#endif                                             
      /*mean+=a[i+1];*/
    }
  /*mean/=ma;*/
  /*for(i=0;i<ma;i++) 
    {
    diff[i]=(mean-wv_shift->Coefficient[channel][i])*(mean-wv_shift->Coefficient[channel][i]);
    wv_shift->Error[channel]+=diff[i];
    }
    wv_shift->Error[channel]=sqrt(wv_shift->Error[channel]/(ma-));*/
  for(i=0;i<n_data;i++)
    {
      yobs[i]+=wv_shift->Coefficient[channel][0];
      for(j=1;j<ma;j++)
	yobs[i]+=wv_shift->Coefficient[channel][j]*pow(X[i+1],j);
    }
  for(i=0;i<n_data;i++)
    {
      diff[i]=((yobs[i]-Y[i+1])/Sign[i+1])*((yobs[i]-Y[i+1])/Sign[i+1]);
      diff2[i]=(yobs[i]-Y[i+1])*(yobs[i]-Y[i+1]);
      wv_shift->Error[channel]+=diff[i];
      stddev+=diff2[i];
    }
  wv_shift->Error[channel]+=sqrt(wv_shift->Error[channel]/(n_data-ma));
  stddev+=sqrt(stddev/(n_data-ma));
#ifdef DEBUG
  printf("Rel. std. dev. of fit: %f \n",wv_shift->Error[channel]);
  printf("Abs. std. dev. of fit: %f \n",stddev);
#endif

  vector__delete(X,1,n_data);
  vector__delete(Y,1,n_data);
  vector__delete(Sign,1,n_data);
  vector__delete(W,1,n_data);
  vector__delete(a,1,MAX_FITT_ORDER);
  matrix__delete(U,1,n_data,1,ma);
  matrix__delete(V,1,ma,1,ma);
  matrix__delete(cvm,1,ma,1,ma);
  free(diff);
  free(diff2);
  free(yobs);
  return Status;
}

/*----------------------------------------------------------------------------*\
**                                Spectrum__EvalPolynom_d
**  Input parameters:
**    X: The point to evaluate the polynom
**    Coefficient: Describe the polynom
**    Grad: grad of the polynom
**  Output parameters:
**  Other interfaces:
**  Description:
**    This function evaluates a polynom of grad Grad described by Coefficient 
**    in the value x
**  References:
**  Libraries:
**  Created:    22.3.94
**  Author: 	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double
Spectrum__EvalPolynom_d(double X, const double *Coefficient, short Grad)
{
    double Result = 0.0, Mult = 1.0;
    short i;
    
    X = (X - OFFSET)/PARAMETER;
    for (i=0; i<Grad; i++)
    {
        Result += Coefficient[i]*Mult;
        Mult *= X;
    }
    /* if(Result > ??) Result = 0.0; */ /*geht leider nur fuer alle Kanaele gleich */
    return Result;
}


