/*----------------------------------------------------------------------------*\
**                                XCorrelation.h
**
**  GDP Level 0 to 1 class Spectrum declaration
** 
**  Version:    1.7
**  Date:       01/14/02
**  Author:     Christian Rehn, DLR/DFD-AP
\*----------------------------------------------------------------------------*/
#ifndef _Spectrum_h_
#define _Spectrum_h_

#ifdef __cplusplus
extern "C"
{
#endif
  
#include "defs.h"
#include "assert.h"
#include "sys.h"
#include "sciedata.h"
  
/*----------------------------------------------------------------------------*\
**  constants
\*----------------------------------------------------------------------------*/

static int ADDITIONAL_PIXEL_TEMPLATE_WINDOW[4] = {18, 18, 36, 36};

#define	ADDITIONAL_PIXEL_GOME_WINDOW	(1)


#define	BINMAX	(10.0)
/* half height of FT(Crosscorrelation) */ 

#define	APODISE_FRACTION	(0.25)


#define	HF_FRAC_LOW_PASS_FILTER	(0.6)

#define	ORDER_LOW_PASS_FILTER	(4.0)


#define	LF_BIN_HIGH_PASS_FILTER	(3.0)

#define	ORDER_HIGH_PASS_FILTER	(8.0)


#define	CONTINUUM_CORRECTION_DEGREE	(2)
/* degree of the polynom for 					continuum correction in wavelength
					calibration */


#define	OFFSET	(500.0)

#define	PARAMETER	(5000.0)


#define	TOTAL_STDDEV_LIMIT	(999.0)
/* 10.0 ?? */   /* const float TOTAL_STDDEV_LIMIT[4] = {999.0,999.0,999.0,999.0};*/ /*Kanalabh. Fehler*/

static int NUMBER_OF_WINDOWS[4] = {12,16,20,16};

static int START_PIXEL[4][20] = { 
  { 81, 84,180,183,195,294,297,300,408,411,531,534,0,0,0,0,0,0,0,0},
  { 43, 70,199,202,226,253,394,397,502,505,523,526,703,706,730,757,0,0,0,0,},
  {135,150,152,155,220,355,412,436,439,472,591,599,658,660,765,768,824,827,915,918},
  {151,157,234,255,350,352,378,425,597,599,755,758,779,782,902,907,0,0,0,0} };


#define	MAX_FITT_ORDER	(6)
static int POLYNOM_DEGREE[4] = {1,1,3,3};

static int pixel_lo[4] = {0,13,60,90};
static int pixel_ho[4] = {694,840,1013,1023};

static int WINDOW_SIZE[4] = {44,44,50,50};
static int WINDOW_OVERLAP[4] = {34,34,45,45};


#define	N_TEMPLATE_FILE	(3)

static int TEMPLATE_BORDERS[N_TEMPLATE_FILE] = {348,649,1023};

/*============================================================================*\
**  class Spectrum
\*============================================================================*/
typedef struct Spectrum_s
{
  float* re_ptr;
  float* im_ptr;		/* must be set to 0 if only real values exist */
  int number_of_values;

  Bool has_grid;
  float* grid_ptr;		/* only reasonable if has_grid == true */

  Bool has_constant_sampling_intervall; /* always true, if has_grid == false
					 */ 
} Spectrum_s;

/*============================================================================*\
**  class Spectral
\*============================================================================*/
typedef struct XC_Coeff_s{
  double Coefficient[OPTICAL_CHANNELS][MAX_FITT_ORDER]; /* 4th order 
								polynomial */
  double CoefficientError[OPTICAL_CHANNELS][MAX_FITT_ORDER]; /*Stddev of 
								    Coefficient */  
  double Error[OPTICAL_CHANNELS]; /* Spectral Calibration Error */
} XC_Coeff_s;

/*============================================================================*\
**  public operations of class Spectrum
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**  constructors
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__new(int number_of_values);


/*----------------------------------------------------------------------------*\
**  destructors
\*----------------------------------------------------------------------------*/
void
Spectrum__delete(Spectrum_s* self);

Spectrum_s*
Spectrum__destructor(Spectrum_s* self);


/*----------------------------------------------------------------------------*\
**  accessor operations 
\*----------------------------------------------------------------------------*/

int
Spectrum__getNumberOfValues(const Spectrum_s* self);

const float*
Spectrum__getRePtr(const Spectrum_s* self);

const float*
Spectrum__getImPtr(const Spectrum_s* self);

const float*
Spectrum__getGridPtr(const Spectrum_s* self);

Bool
Spectrum__hasGrid(const Spectrum_s* self);

Bool
Spectrum__hasConstantSamplingInterval(const Spectrum_s* self);

float
Spectrum__getSamplingInterval(const Spectrum_s* self);



/*----------------------------------------------------------------------------*\
**  other operations 
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__(Spectrum_s* self, int number_of_values);

Spectrum_s*
Spectrum__re(Spectrum_s* self, int number_of_values);

FunctionStatus_e 
Spectrum__readGridRe(Spectrum_s* self, FILE* file_ptr, 
		     Bool has_constant_sampling_intervall); 

FunctionStatus_e 
Spectrum__readGridReFromFile(Spectrum_s* self, const char* filename, 
			     Bool has_constant_sampling_intervall); 


/*----------------------------------------------------------------------------*\
**  output to stream
\*----------------------------------------------------------------------------*/
FunctionStatus_e 
Spectrum__write(const Spectrum_s* self, FILE* stream);


/*----------------------------------------------------------------------------*\
**  cut a window out of a spectrum
\*----------------------------------------------------------------------------*/

int Spectrum__GetTemplateWindowSize(Spectrum_s* self,const Spectrum_s* template_ptr,
				  int pixel_start,int interval_size,int additional_pixel,
				  Spectrum_s* window_ptr);

/*----------------------------------------------------------------------------*\
**  convolution
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__convoluteNormalizedGaussian(Spectrum_s* self,
				int number_of_additional_points);

/*Spectrum_s*
  Spectrum__convoluteSlit(Spectrum_s* self, int number_of_additional_points,
  float* slit_parameter_ptr, int slit_parameter_length);*/


/*----------------------------------------------------------------------------*\
**  resampling
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__resampleSpline(Spectrum_s* self,	
			 const float* grid_ptr, int number_of_values,
			 Bool has_constant_sampling_intervall,
			 float* fractional_number_of_values);


/*----------------------------------------------------------------------------*\
**  cut the edges of an intervall by a number of points
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__cutEdges(Spectrum_s* self, int number_of_points);


/*----------------------------------------------------------------------------*\
**  get maximum value
\*----------------------------------------------------------------------------*/
float
Spectrum__maxRe(const Spectrum_s* self);

float
Spectrum__maxIm(const Spectrum_s* self);


/*----------------------------------------------------------------------------*\
**  scale
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__scale(Spectrum_s* self, float scale_factor);


/*----------------------------------------------------------------------------*\
**  continuum correction
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__continuumCorrectionPolyfit(Spectrum_s* self, int degree);


/*----------------------------------------------------------------------------*\
**  apodise
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__apodise(Spectrum_s* self, float fraction);


/*----------------------------------------------------------------------------*\
**  fourier transformation and inverse fourier transformation 
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__fourierTransform(Spectrum_s* self);

FunctionStatus_e
Spectrum__inverseFourierTransform(Spectrum_s* self);


/*----------------------------------------------------------------------------*\
**  low or high pass filtering
\*----------------------------------------------------------------------------*/
Spectrum_s*
Spectrum__filter(Spectrum_s* self, float* filter_ptr, int filter_length);


/*----------------------------------------------------------------------------*\
**  cross correlation
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__crossCorrelate(Spectrum_s* self, 
			 const Spectrum_s* second_spectrum_ptr,
			 float norm_factor);

/*Spectrum_s*
  Spectrum__crossCorrelation(const Spectrum_s* self, 
  const Spectrum_s* second_spectrum_ptr,
  Spectrum_s* result_spectrum_ptr,
  float norm_factor);*/

/*----------------------------------------------------------------------------*\
**  shift and peak finding
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__findShiftAndPeak(const Spectrum_s* self, float* shift_ptr,
			   float* peak_height_ptr);

/*----------------------------------------------------------------------------*\
**  compute error of cross correlation
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__crossCorrelationError(const Spectrum_s* self, float shift, 
				float peak_height, 
				float fractional_number_of_values,
				float* error);


/*----------------------------------------------------------------------------*\
**  wavelength calibration
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Spectrum__wavelengthCalibration(Spectrum_s* self, OpticalChannel_e self_channel,
				     const Spectrum_s template_spectrum_ptr[N_TEMPLATE_FILE],
				     XC_Coeff_s* xcorr_coeffs);
/*============================================================================*\
**  private operations of class Spectrum
\*============================================================================*/
FunctionStatus_e
Spectrum__GetTemplateFileNumber(int start_pixel,
				int channel,int *template_number);

FunctionStatus_e
Spectrum__cutWindow(const Spectrum_s* self, int i, int j, 
		    Spectrum_s* window_ptr);

FunctionStatus_e
Spectrum__fractionalNumberOfPoints(const Spectrum_s* self,
				   const float* grid_ptr, int number_of_values,
				   float* fractional_number_of_values );

FunctionStatus_e
Spectrum__fft(Spectrum_s* self, int isign);

FunctionStatus_e
Spectrum__complexMultiplication(const Spectrum_s* self, 
				const Spectrum_s* second_spectrum_ptr,
				Spectrum_s* result_ptr);

FunctionStatus_e
Spectrum__complexMultiply(Spectrum_s* self, 
			  const Spectrum_s* second_spectrum_ptr);

FunctionStatus_e
Spectrum__conjugate(Spectrum_s* self);

/*float*
  Spectrum__createFilter(const Spectrum_s* self, float cut_off, float order, 
  float* filter_ptr);*/

FunctionStatus_e
Spectrum__wavelengthCalibrateWindow(Spectrum_s* self,
				    Spectrum_s* template_window_ptr, 
				    int additional_pixel_template_window,
				    int continuum_correction_polynom_degree,
				    float* shift_ptr, float* error_ptr,float* wv_center_ptr);

FunctionStatus_e
Spectrum__fitWavelengthCalibrationResults(XC_Coeff_s* wv_shift,
					  float* shift_ptr,float* error_ptr,
					  float* wv_center_ptr,float* window_size_nm,
					  int channel,int n_data);

double
Spectrum__EvalPolynom_d(double X, const double *Coefficient, short Grad);
 
#ifdef __cplusplus
}
#endif

#endif

